#include "Analysis.h"
#include <iostream.h>
#include <iomanip.h>
#include <values.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

// Use STL
#include <list.h>
#include "showseq.h"


void Analysis::adaptRect2ullr (int &x1, int &y1, int &x2, int &y2, \
                               unsigned int maxWidth, unsigned int maxHeight) const {

  Point upperleft, lowerright;

  upperleft.x = (x1 < x2) ? x1 : x2;
  upperleft.y = (y1 < y2) ? y1 : y2;
  lowerright.x = (x1 > x2) ? x1 : x2;
  lowerright.y = (y1 > y2) ? y1 : y2;

  // check size of window
  if (x1 > (int) maxWidth)  { x1 = maxWidth;  };
  if (y1 > (int) maxHeight) { y1 = maxHeight; };
  if (x2 > (int) maxWidth)  { x2 = maxWidth;  };
  if (y2 > (int) maxHeight) { y2 = maxHeight; };

  if (upperleft.x < 0) {
    x1 = 0;
  } else {
    x1 = upperleft.x;
  };
  if (upperleft.y < 0) {
    y1 = 0;
  } else {
    y1 = upperleft.y;
  };

  if (lowerright.x < 0) {
    x2 = 0;
  } else {
    x2 = lowerright.x;
  };
  if (lowerright.y < 0) {
    y2 = 0;
  } else {
    y2 = lowerright.y;
  };

};


unsigned int Analysis::getNumberOfPixels(const Image* pic) {
  return (pic->getWidth()*pic->getHeight());
};


unsigned int Analysis::getNumberOfBlackPixels(const Image* pic) {

  unsigned int result = 0;

  for (unsigned int x = 0; x < pic->getWidth(); x++) {
    for (unsigned int y = 0; y < pic->getHeight(); y++) {
      if (pic->getGreyscale(x,y) == 0) {
        result++;
      };
    };
  };

  return result;

};


unsigned int Analysis::getNumberOfWhitePixels(const Image* pic) {

  unsigned int result = 0;

  for (unsigned int x = 0; x < pic->getWidth(); x++) {
    for (unsigned int y = 0; y < pic->getHeight(); y++) {
      if (pic->getGreyscale(x,y) == 255) {
        result++;
      };
    };
  };

  return result;

};


double Analysis::dist(Point a, double x, double y) {

  double dx, dy;
  dx = ( (double)a.x - x);
  dy = ( (double)a.y - y);

  return sqrt((dx*dx)+(dy*dy));  

};


Point Analysis::getCenterOfBlackPixels(const Image* pic, Rectangle window, unsigned int limit = 0) {

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());

  Point result;
  unsigned int sum_x = 0;
  unsigned int sum_y = 0;
  unsigned int sum_p = 0;

  //Point punkte[getNumberOfPixels(pic)];
  Point punkte[(window.x2-window.x1)*(window.y2-window.y1)];
  unsigned int i = 0; 

  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      if (pic->getGreyscale(x,y) <= limit) {
        punkte[i].x = x;
        punkte[i].y = y;
        i++;
        sum_x += x;
        sum_y += y;
        sum_p++;
      };
    };
  };

  if (sum_p == 0) {
    result.x = result.y = 0;
  } else {
    result.x = sum_x / sum_p;
    result.y = sum_y / sum_p;
  };


  double abweichung = 0.0;
  for (unsigned int j = 0; j < i; j++) {
    abweichung += dist(result, punkte[j].x, punkte[j].y); 
  };

  cout << "Abweichung " << sqrt(abweichung) << endl;

  return result;
  
};


Point Analysis::getCenterOfBlackPixelsMedian(const Image* pic, Rectangle window, unsigned int limit = 0) {

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());

  Point result;
  unsigned int maxPoints = (window.x2-window.x1)*(window.y2-window.y2);
  list<unsigned int> x_werte(maxPoints);
  list<unsigned int> y_werte(maxPoints); 
  unsigned int anzPunkte = 0;

  cout << "Punkte: ";

  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      if (pic->getGreyscale(x,y) <= limit) {
        x_werte.push_back(x);
        y_werte.push_back(y);

        cout << "(" << x << "/" << y << ") ";

        anzPunkte++;
      };
    };
  };
  cout << endl;

  x_werte.sort();
  list<unsigned int>::iterator Ix = x_werte.begin();
  advance(Ix, x_werte.size()/2);
  result.x = *Ix;

  y_werte.sort();
  list<unsigned int>::iterator Iy = y_werte.begin();
  advance(Iy, y_werte.size()/2);
  result.y = *Iy;

  cout << "Median: (" << result.x << "/" << result.y << ")" << endl;

  return result;
  
};



unsigned int Analysis::getNumberOfBlackPixels(const Image* pic, \
                                              Rectangle window) {

  // get number all Pixels that are black (0)
  return getNumberOfDarkPixels(pic, window, 0);

};


unsigned int Analysis::getNumberOfDarkPixels(const Image* pic, \
                                             Rectangle window, \
                                             unsigned int limit) {

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());

  unsigned int result = 0;

  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      if (pic->getGreyscale(x,y) <= limit) {
        result++;
      };
    };
  };

  return result;

};



void Analysis::addDetectedPixels(const Image* inpic, \
                                 Image* outpic, Rectangle window) {

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, inpic->getWidth(), inpic->getHeight());

  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      if (inpic->getGreyscale(x,y) == 0) {
        outpic->setGreyscale(x,y,0);
      };
    };
  };

};




Point Analysis::getCenterOfBlackPixelsIterative(const Image* pic, \
                                                Rectangle window, \
                                                unsigned int repeats, \
                                                unsigned int limit = 0) {

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());

  Point result = getCenterOfBlackPixels(pic, window, limit);
  double resx = result.x;
  double resy = result.y;

  cout << "Base center: (" << resx << "/" << resy << ")" << endl;

  // do the iterations
  for (unsigned int i = 1; i <= repeats; i++) {

    // Faktor fuer Gewichtung der Distanz bestimmen
    double factor = 0.0;
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {

        if (pic->getGreyscale(x,y) <= limit) {
          Point pos; pos.x = x; pos.y = y;
          factor += 1.0 / (dist(pos,resx,resy) + 1.0);
        };

      };
    };
    factor = 1.0 / factor;    
    

    // Punkte verrechnen
    double long posx, posy;
    posx = posy = 0.0;
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {

        if (pic->getGreyscale(x,y) <= limit) {
          Point pos; pos.x = x; pos.y = y;
          double weight = factor / (dist(pos,resx,resy) + 1.0);
          posx += (weight * (double) x);
          posy += (weight * (double) y);
        };

      };
    };

    resx = posx;
    resy = posy;

    cout << "[Analysis::getCenterOfBlackPixelsIterative] Iteration = " << i << ", Point = (" << resx << "/" << resy << "), Factor = " << factor << endl;

  };

  result.x = (unsigned int) resx;
  result.y = (unsigned int) resy;
  return result;
  
};



Point Analysis::getCenterOfBlackPixelsGreyscale(const Image* pic, \
                                                Rectangle window, \
                                                unsigned int limit = 255) {

  Point result;

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());


  // Calculate sum of all selected greylevels
  float sum = 0.0;
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {

      float value = pic->getGreyscale(x,y);
      if ((unsigned int) value <= limit) {
        sum+= (255.0 - value);
      }; 

    };
  };

  // original greylevels above limit are ignored
  // weight of a pixel -> greylevel / sum
  if (sum <= 0.0) {
    result.x = result.y = 0;
    return result;
  };    

  // calculate center using the weights
  float resx, resy;
  resx = resy = 0.0;
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {

      float value = pic->getGreyscale(x,y);
      if ((unsigned int) value <= limit) {

        resx += ((255.0 - value)/sum) * (float) x;
        resy += ((255.0 - value)/sum) * (float) y;

      }; 

    };
  };

  cout << "[Analysis::getCenterOfBlackPixelsGreyscale] Factor: " << sum << "  Point (" << resx << "/" << resy << ")" << endl;


  result.x = (int) resx; 
  result.y = (int) resy;
  return result;

};



Point Analysis::getCenterOfBlackPixelsGreyscaleIterative(const Image* pic, \
         Rectangle window, unsigned int repeats, unsigned int limit = 255) {

  Point result;

  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, pic->getWidth(), pic->getHeight());

  // Calculate sum of all selected greylevels
  float greysum = 0.0;
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {

      float value = pic->getGreyscale(x,y);
      if ((unsigned int) value <= limit) {
        greysum += (255.0 - value);
      };

    };
  };

  // original greylevels above limit are ignored
  // weight of a pixel -> greylevel / greysum
  if (greysum <= 0.0) {
    result.x = result.y = 0;
    return result;
  };    

  // calculate center using the weights
  float resx, resy;
  resx = resy = 0.0;
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {

      float value = pic->getGreyscale(x,y);
      if ((unsigned int) value <= limit) {

        resx += ((255.0 - value)/greysum) * (float) x;
        resy += ((255.0 - value)/greysum) * (float) y;

      }; 

    };
  };

  cout << "[Analysis::getCenterOfBlackPixelsGreyscaleIterative] Base Factor: " << greysum << "  Base Point (" << resx << "/" << resy << ")" << endl;


  // do the iterations
  for (unsigned int i = 1; i <= repeats; i++) {

    // Faktor fuer Gewichtung der Distanz bestimmen
    double factor = 0.0;
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {

        if (pic->getGreyscale(x,y) <= limit) {
          Point pos; pos.x = x; pos.y = y;
          factor += 1.0 / (dist(pos,resx,resy) + 1.0);
        };

      };
    };
    factor = 1.0 / factor;    
    cout << "factor: " << factor << endl;


    // Korrektur der beiden Gewichte (Distanz & Graustufen) berechnen
    double correction = 0.0;
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {
        
        unsigned int value = pic->getGreyscale(x,y);
        if (value <= limit) {

          Point pos; pos.x = x; pos.y = y;
          correction += (factor / (dist(pos,resx,resy) + 1.0)) * \
                        ((255.0 - value)/greysum);

        };

      };
    };
    correction = 1.0 / correction;
    

    // Punkte verrechnen
    double long posx, posy;
    posx = posy = 0.0;
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {

        unsigned int value = pic->getGreyscale(x,y);
        if (value <= limit) {
          Point pos; pos.x = x; pos.y = y;
          double weight = factor / (dist(pos,resx,resy) + 1.0);
          posx += (correction * (255.0 - value)/greysum * weight) * (double) x;
          posy += (correction * (255.0 - value)/greysum * weight) * (double) y;
        };

      };
    };

    resx = posx;
    resy = posy;

    cout << "[Analysis::getCenterOfBlackPixelsGreyscaleIterative] Iteration = " << i << ", Point = (" << resx << "/" << resy << "), Factor = " << correction << endl;

  };


  result.x = (int) resx; 
  result.y = (int) resy;
  return result;

};










