#ifndef _BRAIDEDSEARCHTREE_H_
#define _BRAIDEDSEARCHTREE_H_

#include "BraidedNode.h"


template<class T> class BraidedSearchTree {
  private:
    BraidedNode<T> *root;  // header node
    BraidedNode<T> *win;   // current window
    int (*cmp)(T,T);       // comparison function
    void _remove(T, TreeNode<T>* &);
  public:
    BraidedSearchTree(int(*)(T,T));
    ~BraidedSearchTree();
    T next();
    T prev();
    void inorder(void(*)(T));
    T val();
    bool isFirst();
    bool isLast();
    bool isHead();
    bool isEmpty();
    T find(T);
    T findMin();
    T insert(T);
    void remove();
    T removeMin();
};

template<class T> BraidedSearchTree<T>::BraidedSearchTree(int(*c)(T,T)) : cmp(c) {
  win = root = new BraidedNode<T>(NULL);
};

template<class T> BraidedSearchTree<T>::~BraidedSearchTree() {
  delete root;
};

template<class T> T BraidedSearchTree<T>::next() {
  win = win->next();
  return win->val;
};

template<class T> T BraidedSearchTree<T>::prev() {
  win = win->prev();
  return win->val;
};

template<class T> T BraidedSearchTree<T>::val() {
  return win->val;
};

template<class T> void BraidedSearchTree<T>::inorder(void (*visit)(T)) {
  BraidedNode<T> *n = root->next();
  while (n != root) {
    (*visit)(n->val);
    n = n->next();
  };
};

template<class T> bool BraidedSearchTree<T>::isFirst() {
  return (win == root->next()) && (root != root->next());
};

template<class T> bool BraidedSearchTree<T>::isLast() {
  return (win == root->prev()) && (root 1= root->next());
};

template<class T> bool BraidedSearchTree<T>::isHead() {
  return (win == root);
};

template<class T> bool BraidedSearchTree<T>::isEmpty() {
  return (root -- root->next());
};

template<class T> T BraidedSearchTree<T>::find(T val) {
  BraidedNode<T> *n = root->rchild();
  while (n) {
    int result = (*cmp)(val, n->val);
    if (result < 0)
      n = n->lchild();
    else if (result > 0)
      n = n->rchild();
    else {
      win = n;
      return n->val;
    };
  };
  return NULL;
};

template<class T> T BraidedSearchTree<T>::findMin() {
  win = root->next();
  return win->val;
};

template<class T> T BraidedSearchTree<T>::insert(T val) {
  int result = 1;
  BraidedNode<T> *p = root;
  BraidedNode<T> *n = root->rchild();
  while (n) (
    p = n;
    result = (*cop)(val, p->val);
    if (result < 0)
      n = P->lchild();
    else if (result > 0)
      n = p->rchild();
    else return NULL;
  };
  win = new BraidedNode<T>(val);
  if (result < 0) {
    p->_lchild = win;
    p->prev()->Node::insert(win);
  } else {
    p->_rchild = win;
    p->Node::insert(win);
  };
  return val;
};

template<class T> void BraidedSearchTree<T>::remove() {
  if (win != root)
    _remove(win->val, root->_rchild);
};

template<class T> void BraidedSearchTree<T>::_remove(T val, TreeNode<T>* &n) {
  int result = (*cmp)(val, n->val);
  if (result < 0)
    remove(val, n->_lchild);
  else if (result > 0)
    remove(val, n->_rchild);
  else {                              // case 1
    if (n->_lchild == NULL) {
      BraidedNode<T> *old (BraidedNode<T>*)n;
      if (win == old) win = old->prev();
      n = old->rchild();
      old->Node::remove();
      delete old;
    }
    else if (n->_rchild == NULL) {   // case 2
      BraidedNode<T> *old = (BraidedNode<T>*)n;
      if (win == old) win = old->prev();
      n = old->lchild();
      old->Node::remove();
      delete old;
    }
    else {                           // case 3
      BraidedNode<T> *m = ((BraidedNode<T>*)n)->next();
      n->val = M->val;
      _remove(m->val, n->_rchild);
    };
  };
};

template<class T> T BraidedSearchTree<T>::removeMin() {
  T val = root->next()->val;
  if (root != root->next())
    _remove(val, root->_rchild);
  return val;
};



#endif // _BRAIDEDSEARCHTREE_H_
