/*
 * ColorTypes.C 
 *
 * Copyright (C) 1995, Christoph Streit <streit@iam.unibe.ch>
 *                     Stephan Amann <amann@iam.unibe.ch>
 *                     University of Berne, Switzerland
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 * -----------------------------------------------------------------------------
 *  $Id: ColorTypes.C,v 1.2 1995/08/03 12:15:24 streit Exp $
 * -----------------------------------------------------------------------------
 */
 
#include "ColorTypes.h"

//__________________________________________________________________________ RGB

void RGB::setValue(float c1, float c2, float c3)
{
  setRed(c1);
  setGreen(c2);
  setBlue(c3);
}

void RGB::setValue(float value)
{
  setRed(value);
  setGreen(value);
  setBlue(value);
}

void RGB::setValue(bool value)
{
  if (value == false)
    setValue((float)0.0);
  else
    setValue((float)1.0);
}

void RGB::getValue(float& c1, float& c2, float& c3) const
{
  c1 = getRed();
  c2 = getGreen();
  c3 = getBlue();
}

void RGB::getValue(float& value) const
{
  value = brightness();
}

void RGB::getValue(bool& value) const
{
  if (brightness() <= 0.5)
    value = false;
  else
    value = true;
}

//_________________________________________________________________________ Grey

void Grey::setValue(float c1, float c2, float c3)
{
  myValue = Color(c1, c2, c3).brightness();
}

void Grey::setValue(float value)
{
  myValue = value;
}

void Grey::setValue(bool value)
{
  if (value == false)
    myValue = 0;
  else
    myValue = 1;
}

void Grey::getValue(float& c1, float& c2, float& c3) const
{
  c1 = c2 = c3 = myValue;
}

void Grey::getValue(float& value) const
{
  value = myValue;
}

void Grey::getValue(bool& value) const
{
  if (myValue <= 0.5)
    value = false;
  else
    value = true;
}

//___________________________________________________________________________ BW

void BW::setValue(float c1, float c2, float c3)
{
  setValue(Color(c1, c2, c3).brightness());
}

void BW::setValue(float value)
{
  if (value <= 0.5)
    myValue = false;
  else
    myValue = true;
}

void BW::setValue(bool value)
{
  myValue = value;
}

void BW::getValue(float& c1, float& c2, float& c3) const
{
  if (myValue == false)
    c1 = c2 = c3 = 0.0;
  else
    c1 = c2 = c3 = 1.0;
}

void BW::getValue(float& value) const
{
  if (myValue == false)
    value = 0;
  else
    value = 1;
}

void BW::getValue(bool& value) const
{
  value = myValue;
}
