#include "DelaunayTriangulation.h"
#include <stdlib.h>
#include <values.h>
#include <iostream.h>


DelaunayTriangulation::DelaunayTriangulation() { };

DelaunayTriangulation::~DelaunayTriangulation() { };


void DelaunayTriangulation::updateFrontier(Dictionary<Edge*> &frontier, Point2D &a, Point2D &b) {
  Edge *e = new Edge(a, b);
  if (frontier.find(e)) {
    frontier.remove(e);
  } else {
    e->flip();
    frontier.insert(e);
  };
};


void swap(Point2D &a, Point2D &b) {
  Point2D t = a;
  a = b;
  b = t;
};


Edge* DelaunayTriangulation::hullEdge(Point2D s[], int n) {
  int m = 0;
  for(int i = 1; i < n; i++) {
    if (s[i] < s[m]) m = i;
  };
  swap(s[0], s[m]);
  m = 1;
  for(int j = 2; j < n; j++) {
    int c = s[j].classify(s[0], s[m]);
    if ((c == Point2D::LEFT) || (c == Point2D::BETWEEN)) {
      m = j;
    };
  };
  return new Edge(s[0], s[m]);
};


Polygon* DelaunayTriangulation::triangle(Point2D &a, Point2D &b, Point2D &c) {
  Polygon *t = new Polygon;
  t->insert(a);
  t->insert(b);
  t->insert(c);
  return t;
};


int edgeCmp(Edge *a, Edge *b) {
  if (a->org < b->org) return -1;
  if (a->org > b->org) return 1;
  if (a->dest < b->dest) return -1;
  if (a->dest > b->dest) return 1;
  return 0;
};


bool DelaunayTriangulation::mate(Edge &e, Point2D s[], int n, Point2D &p) {
  Point2D *bestp = NULL;
  double t, bestt = MAXDOUBLE;
  Edge f = e;
  f.rot();         // f is the perpendicular bisector of e
  for (int i = 0; i < n; i++) {
    if (s[i].classify(e) == Point2D::RIGHT) {
      Edge g(e.dest, s[i]);
      g.rot();
      f.intersect(g, t);
      if (t < bestt) {
        bestp = &s[i];
        bestt = t;
      };
    };
  };
  if (bestp) {
    p = *bestp;
    return true;
  };
  return false;
};


List<Polygon*> *DelaunayTriangulation::apply(Point2D s[], int n) {

  Point2D p;
  List<Polygon*> *triangles = new List<Polygon*>;
  Dictionary<Edge*> frontier(edgeCmp);
  Edge *e = hullEdge(s, n);
  frontier.insert(e);
  while(!frontier.isEmpty()) {
    e = frontier.removeMin();
    if (mate(*e, s, n, p)) {
      updateFrontier(frontier, p, e->org);
      updateFrontier(frontier, e->dest, p);
      triangles->insert(triangle(e->org, e->dest, p));

  cout << "triangle ";

    };
  delete e;
  };
  return triangles;
};








