#include "DetectorGreyscale.h"
#include <math.h>
#include <iostream.h>



DetectorGreyscale::DetectorGreyscale(const Rectangle tmpRect, const unsigned int tmpThreshold = 0) {
  myThreshold = tmpThreshold;
  myRectangle = tmpRect;
};


Point DetectorGreyscale::apply(const Image* source, const Rectangle window) {

  Point result;

  // Calculate sum of all selected greylevels
  float sum = 0.0;
  for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
    if ((x >= window.x1) && (x <= window.x2)) {
      for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
        if ((y >= window.y1) && (y <= window.y2)) {
          // original greylevels above limit are ignored
          if (source->getGreyscale(x,y) <= myThreshold) {
            float value = (float) source->getGreyscale(x,y);
            #ifdef DEBUG
            cout << "[DetectorGreyscale::apply] ";
            cout << "Pixel: (" << x << "/" << y << ") -> ";
            cout << (int) value << endl; 
            #endif
            sum+= (255.0 - value);
          };
	};
      };
    };
  };


  // weight of a pixel -> greylevel / sum
  if (sum <= 0.0) {
    result.x = result.y = -1;
    cout << "[DetectorGreyscale::apply] Greyscale Center: (" << result.x << "/" << result.y << ")" << endl;
    return result;
  }; 


  // calculate center using the weights
  float resx, resy;
  resx = resy = 0.0;

  for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
    if ((x >= window.x1) && (x <= window.x2)) {
      for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
        if ((y >= window.y1) && (y <= window.y2)) {
          // original greylevels above limit are ignored
          if (source->getGreyscale(x,y) <= myThreshold) {
            float value = (float) source->getGreyscale(x,y);
            resx += ((255.0 - value)/sum) * (float) x;
            resy += ((255.0 - value)/sum) * (float) y;
          };
	};
      };
    };
  };

  result.x = (int) (resx+0.5); 
  result.y = (int) (resy+0.5);
  cout << "[DetectorGreyscale::apply] Greyscale Center: (";
  cout << result.x << "/" << result.y << ")" << endl;

  return result;

};










