#include "DetectorGreyscaleIterative.h"
#include <math.h>
#include <iostream.h>



DetectorGreyscaleIterative::DetectorGreyscaleIterative(const Rectangle tmpRect, const unsigned int tmpThreshold = 0) {
  myThreshold = tmpThreshold;
  myRectangle = tmpRect;
  myIterations = 3;
};


void DetectorGreyscaleIterative::setIterations(unsigned int itrs) {
  myIterations = itrs;
};


Point DetectorGreyscaleIterative::apply(const Image* source, const Rectangle window) {

  Point result;


  // First calculate Base center using Greyscales
  // Calculate sum of all selected greylevels
  float sum = 0.0;
  for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
    if ((x >= window.x1) && (x <= window.x2)) {
      for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
        if ((y >= window.y1) && (y <= window.y2)) {
          // original greylevels above limit are ignored
          if (source->getGreyscale(x,y) <= myThreshold) {
            float value = (float) source->getGreyscale(x,y);
            #ifdef DEBUG
            cout << "[DetectorGreyscale::apply] ";
            cout << "Pixel: (" << x << "/" << y << ") -> ";
            cout << (int) value << endl;
            #endif 
            sum+= (255.0 - value);
          };
	};
      };
    };
  };

  // weight of a pixel -> greylevel / sum
  if (sum <= 0.0) {
    result.x = result.y = -1;
    cout << "[DetectorGreyscaleIterative::apply] Greyscale Center Iterative: (" << result.x << "/" << result.y << ")" << endl;
    return result;
  }; 

  // calculate center using the weights
  float resx, resy;
  resx = resy = 0.0;
  for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
    if ((x >= window.x1) && (x <= window.x2)) {
      for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
        if ((y >= window.y1) && (y <= window.y2)) {
          // original greylevels above limit are ignored
          if (source->getGreyscale(x,y) <= myThreshold) {
            float value = (float) source->getGreyscale(x,y);
            resx += ((255.0 - value)/sum) * (float) x;
            resy += ((255.0 - value)/sum) * (float) y;
          };
	};
      };
    };
  };
  result.x = (int) resx;
  result.y = (int) resy;  
  #ifdef DEBUG
  cout << "[DetectorGreyscale::apply] Greyscale Center Iterative Base: (";
  cout << result.x << "/" << result.y << ")" << endl;
  #endif



  // do the iterations
  for (unsigned int i = 1; i <= myIterations; i++) {

    // Faktor fuer Gewichtung der Distanz bestimmen
    double factor = 0.0;
    for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
      if ((x >= window.x1) && (x <= window.x2)) {
        for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
          if ((y >= window.y1) && (y <= window.y2)) {
            // original greylevels above limit are ignored
            if (source->getGreyscale(x,y) <= myThreshold) {
              Point pos; pos.x = x; pos.y = y;
              factor += 1.0 / (dist(pos,resx,resy) + 1.0);
            };
          };
        };
      };
    };
    factor = 1.0 / factor;    


    // Korrektur der beiden Gewichte (Distanz & Graustufen) berechnen
    double correction = 0.0;
    for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
      if ((x >= window.x1) && (x <= window.x2)) {
        for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
          if ((y >= window.y1) && (y <= window.y2)) {
            // original greylevels above limit are ignored
            unsigned int value = source->getGreyscale(x,y);
            if (value <= myThreshold) {
              Point pos; pos.x = x; pos.y = y;
              correction += (factor / (dist(pos,resx,resy) + 1.0)) * \
                            ((255.0 - value)/sum);
            };
          };
        };
      };
    };
    correction = 1.0 / correction;
    

    // Punkte verrechnen
    double long posx, posy;
    posx = posy = 0.0;
    for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
      if ((x >= window.x1) && (x <= window.x2)) {
        for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
          if ((y >= window.y1) && (y <= window.y2)) {
            // original greylevels above limit are ignored
            unsigned int value = source->getGreyscale(x,y);
            if (value <= myThreshold) {
              Point pos; pos.x = x; pos.y = y;
              double weight = factor / (dist(pos,resx,resy) + 1.0);
              posx += (correction * (255.0 - value)/sum * weight) * (double) x;
              posy += (correction * (255.0 - value)/sum * weight) * (double) y;
            };
          };
        };
      };
    }; 
    resx = posx;
    resy = posy;

    #ifdef DEBUG
    cout << "[DetectorGreyscale::apply] Greyscale Center Iteration" << i << ":  (";
    cout << resx << "/" << resy << ")" << endl;
    #endif

  };

  result.x = (int) (resx+0.5);
  result.y = (int) (resy+0.5);
  cout << "[DetectorGreyscale::apply] Greyscale Center Iterative: (";
  cout << result.x << "/" << result.y << ")" << endl;

  return result;

};

