#include "DetectorIterative.h"
#include <math.h>
#include <iostream.h>
#include "DetectorSimple.h"



DetectorIterative::DetectorIterative(const Rectangle tmpRect, const unsigned int tmpThreshold = 0) {
  myThreshold = tmpThreshold;
  myRectangle = tmpRect;
  myIterations = 3;
};


void DetectorIterative::setIterations(unsigned int itrs) {
  myIterations = itrs;
};


Point DetectorIterative::apply(const Image* source, const Rectangle window) {

  DetectorSimple myDetectorSimple(myRectangle, myThreshold);
  Point result = myDetectorSimple.apply(source, window);
  if (result.valid()) {
    #ifdef DEBUG
    cout << "[DetectorIterative::apply] Iterative Center Base: (";
    cout << result.x << "/" << result.y << ")" << endl;
    #endif
  } else {
    cout << "[DetectorIterative::apply] Iterative Center: (";
    cout << result.x << "/" << result.y << ")" << endl;
    return result;
  };



  // do the iterations
  float resx = (float) result.x;
  float resy = (float) result.y;
  for (unsigned int i = 1; i <= myIterations; i++) {

    // Faktor fuer Gewichtung der Distanz bestimmen
    double factor = 0.0;
    for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
      if ((x >= window.x1) && (x <= window.x2)) {
        for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
          if ((y >= window.y1) && (y <= window.y2)) {
            // original greylevels above limit are ignored
            if (source->getGreyscale(x,y) <= myThreshold) {
              Point pos; pos.x = x; pos.y = y;
              factor += 1.0 / (dist(pos,resx,resy) + 1.0);
            };
          };
        };
      };
    };
    factor = 1.0 / factor;    
   

    // Punkte verrechnen
    double long posx, posy;
    posx = posy = 0.0;
    for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
      if ((x >= window.x1) && (x <= window.x2)) {
        for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
          if ((y >= window.y1) && (y <= window.y2)) {
            // original greylevels above limit are ignored
            unsigned int value = source->getGreyscale(x,y);
            if (value <= myThreshold) {
              Point pos; pos.x = x; pos.y = y;
              double weight = factor / (dist(pos,resx,resy) + 1.0);
              posx += (weight * (double) x);
              posy += (weight * (double) y);
            };
          };
        };
      };
    }; 
    resx = posx;
    resy = posy;

    #ifdef DEBUG
    cout << "[DetectorIterative::apply] Center Iteration" << i << ":  (";
    cout << resx << "/" << resy << ")" << endl;
    #endif

  };

  result.x = (int) (resx+0.5);
  result.y = (int) (resy+0.5);
  cout << "[DetectorIterative::apply] Center Iterative: (";
  cout << result.x << "/" << result.y << ")" << endl;

  return result;

};




