#include "DetectorNeighbour.h"
#include <math.h>

// Use STL
#include <vector.h>
#include <list.h>
#include <values.h>


DetectorNeighbour::DetectorNeighbour(const Rectangle tmpRect, const unsigned int tmpThreshold = 0) {
  myThreshold = tmpThreshold;
  myRectangle = tmpRect;
};


Point DetectorNeighbour::apply(const Image* source, const Rectangle window) {

  vector<Point> punkte(0);
  vector<long> distsum(0); 

  // read in the points
  for (int x = myRectangle.x1; x < myRectangle.x2; x++) {
    if ((x >= window.x1) && (x <= window.x2)) {
      for (int y = myRectangle.y1; y < myRectangle.y2; y++) {
        if ((y >= window.y1) && (y <= window.y2)) {
          if (source->getGreyscale(x,y) <= myThreshold) {
            #ifdef DEBUG
            cout << "[DetectorNeighbour::apply] ";
            cout << "Pixel: (" << x << "/" << y << ") -> ";
            cout << (int) source->getGreyscale(x,y) << endl; 
            #endif
            punkte.push_back(Point(x,y));
            distsum.push_back(0);
          };
	};
      };
    };
  };

  // calculate the sum of the distance square for 
  // every point to all the other points
  for (unsigned int i = 0; i < punkte.size(); i++) {
    long sum = 0;
    for (unsigned int j = 0; j < punkte.size(); j++) {
      sum += ((punkte[i].x-punkte[j].x)*(punkte[i].x-punkte[j].x));
      sum += ((punkte[i].y-punkte[j].y)*(punkte[i].y-punkte[j].y));
    };

    // analyse the 8-neighbourhood
    // the point itself is counted, too
    int neighbours = 0;
    for (int xx = punkte[i].x-1; xx <= punkte[i].x+1; xx++) {
      for (int yy = punkte[i].y-1; yy <= punkte[i].y+1; yy++) {
        if (source->getGreyscale(xx,yy) <= myThreshold) {
          neighbours++;
	};        
      };
    };

    // prefer points with more neighbours
    distsum[i] = sum / neighbours;
    // because point itself is counted, too
    // it's divided at least by 1
    cout << i << " (" << punkte[i].x << "/" << punkte[i].y << "): " << sum << " / " << neighbours << " = " << distsum[i] << endl;
  };


  // Detect point with smallest distance from the others
  int index = -1;
  long minsum = MAXLONG;
  vector<unsigned int> respoints(0);
  for (unsigned int i = 0; i < distsum.size(); i++) {
    if (distsum[i] == minsum) {
      respoints.push_back(i);
    } else {
      if (distsum[i] < minsum) {
	while (respoints.size() > 0) { respoints.pop_back(); };
        respoints.push_back(i);
        minsum = distsum[i];
        index = i;
      };
    };
  };

  cout << "Result Points: ";
  for (unsigned int i = 0; i < respoints.size(); i++) {
    cout << "(" << punkte[respoints[i]].x << "/" << punkte[respoints[i]].y << ") ";
  };
  cout << endl;


  if (index >= 0) {
    if (respoints.size() > 1) {
      // more than 1 result -> take the nearest to the center of the rect
      int idx = -1;
      long mind = MAXLONG;
      Point center((window.x1+window.x2)/2,(window.y1+window.y2)/2);
      cout << "Center: (" << center.x << "/" << center.y << ")" << endl;
      for (unsigned j = 0; j < respoints.size(); j++) {
        long dist = ((punkte[respoints[j]].x-center.x)*(punkte[respoints[j]].x-center.x));
        dist += ((punkte[respoints[j]].y-center.y)*(punkte[respoints[j]].y-center.y));
        if (dist < mind) {
          mind = dist;
          idx = j;
	};
      };
      cout << "[DetectorNeighbour::apply] Result: (";
      cout << punkte[respoints[idx]].x << "/" << punkte[respoints[idx]].y;
      cout << "), Index " << index << ", Dist " << distsum[respoints[idx]] << ", Centerdist " << mind << endl;
      return punkte[respoints[idx]];
    } else {
      // one result -> return this one
      cout << "[DetectorNeighbour::apply] Result: (";
      cout << punkte[index].x << "/" << punkte[index].y;
      cout << "), Index " << index << ", Dist " << distsum[index] << endl;
      return punkte[index];
    };
  } else {
    cout << "[DetectorNeighbour::apply] Result: NO POINT" << endl;
    return Point();
  };
};














