#include "Drawing.h"
#include <iostream.h>


// Draws a line in the first octant using Bresenham algorithm
void Drawing::line(Image* img, const Point p1, const Point p2, const Color c, const Image* mask = NULL) const {

  int x, y, dx, dy, error, c1, c2;

  // Punkte umrechnen in 1. Oktankten
  int x1, y1, x2, y2, oktant;

  // 1. Beschraenke Richtung auf Oktanten rechts von y-Achse (1,2,7,8)
  if (p1.x <= p2.x) {
    x1 = p1.x;
    x2 = p2.x;
    y1 = p1.y;
    y2 = p2.y;
  } else {
    x1 = p2.x;
    x2 = p1.x;
    y1 = p2.y;
    y2 = p1.y;
  }; 

  // 2. Bestimme den Oktanten
  float steigung = 0.0;
  if ((x2-x1) == 0) {
    steigung = 100.0;   // senkrechte Linie -> "unendlich"
  } else {
    steigung = ((float (y2-y1))/ (float (x2-x1)));
  };

  // cout << (y2-y1) << "/" << (x2-x1) << " -> Steigung " << steigung;

  if (steigung < 0.0) {
    if (steigung < -1.0) {
      oktant = 7;
    } else {
      oktant = 8;
    };
  } else {
    if (steigung > 1.0) {
      oktant = 2;
    } else {
      oktant = 1;
    };
  };

  // cout << " ... Oktant: " << oktant << endl;

  // 3. Transformieren in 1. Oktant 
  switch(oktant) {
  case 1:
    // x1/y1 sind ok
    // x2/y2 sind ok
    break;
  case 2:
    // spiegeln an Winkelhalbierenden x-y-Achse
    swap(x1,y1);
    swap(x2,y2);
    break;
  case 7:
    // spiegeln an x-Achse
    y1 *= -1;
    y2 *= -1;
    // spiegeln an Winkelhalbierenden x-y-Achse
    swap(x1,y1);
    swap(x2,y2);
    break;
  case 8:
    // spiegeln an x-Achse
    y1 *= -1;
    y2 *= -1;
    break;
  default:
    cout << "[Drawing::line] ERROR: Illeagal Octant (";
    cout << oktant << ") detected." << endl;
    return;
  };


  // Init Bresenham
  x = x1;
  dx = x2 - x1;
  y = y1;
  dy = y2 - y1;
  c1 = 2 * dy;
  error = c1 - dx;
  c2 = error - dx;


  while (x <= x2) {
    // Punkt zeichnen abhaengig von Oktant 
    switch(oktant) {
    case 1:
      //img->setColor(x,y,c);
      point(img, Point(x,y), c, mask);
      break;
    case 2:
      // spiegeln an Winkelhalbierenden x-y-Achse
      //img->setColor(y,x,c);
      point(img, Point(y,x), c, mask);
      break;
    case 7:
      // spiegeln an x-Achse und Winkelhalbierenden x-y-Achse
      //img->setColor(y,-x,c);
      point(img, Point(y,-x), c, mask);
      break;
    case 8:
      // spiegeln an x-Achse
      //img->setColor(x,-y,c);
      point(img, Point(x,-y), c, mask);
      break;
    default:
      // nichts tun
      break;
    };
    x++;
    if (error < 0) {
      error += c1;
    } else {
      y++;
      error += c2;
    };
  };

};


void Drawing::point(Image* img, const Point p, const Color c, const Image* mask = NULL) const {

  if (mask != NULL) {
    if (mask->getGreyscale(p.x, p.y) > 0) {

      cout << "(" << p.x << "/" << p.y << ") outside mask" << endl;

      img->setColor(p.x, p.y, Color(255,0,0));
    } else {
      img->setColor(p.x, p.y, c);
    };
  } else {
    img->setColor(p.x,p.y,c);
  };
};


void Drawing::rectangle(Image* img, const Rectangle rect, const Color c) const {

  line(img, Point(rect.x1, rect.y1), Point(rect.x2, rect.y1), c);
  line(img, Point(rect.x2, rect.y1), Point(rect.x2, rect.y2), c);
  line(img, Point(rect.x2, rect.y2), Point(rect.x1, rect.y2), c);
  line(img, Point(rect.x1, rect.y2), Point(rect.x1, rect.y1), c);
 
};




void Drawing::swap(int &i1, int &i2) const {
  int temp;
  temp = i1;
  i1 = i2;
  i2 = temp;
};




void Drawing::cross(Image* img, const Point p, const Color c, const unsigned int size = 1) const {

  if (size <= 1) {
    img->setColor(p.x, p.y, c);
    img->setColor(p.x-1, p.y, c);
    img->setColor(p.x+1, p.y, c);
    img->setColor(p.x, p.y-1, c);
    img->setColor(p.x, p.y+1, c);
  } else {
    line(img, Point(p.x-size, p.y), Point(p.x+size, p.y), c);
    line(img, Point(p.x, p.y-size), Point(p.x, p.y+size), c);
  };

};












