#include "Edge.h"
#include <values.h>


   
Edge::Edge(Point2D &_org, Point2D &_dest) : org(_org), dest(_dest) { };

Edge::Edge() : org(Point2D(0,0)), dest(Point2D(1,0)) { };

Edge &Edge::rot() {
  Point2D m = 0.5 * (org + dest);
  Point2D v = dest - org;
  Point2D n(v.y, -v.x);
  org = m - 0.5 * n;
  dest = m + 0.5 * n;
  return *this;
};

Edge &Edge::flip() {
  return rot().rot();
};

Point2D Edge::point(double t) {
  return Point2D(org + t * (dest - org));
};

double dotProduct(const Point2D &p, const Point2D &q) {
  return (p.x * q.x + p.y * q.y);
};

int Edge::intersect(Edge &e, double &t) {
  Point2D a = org;
  Point2D b = dest;
  Point2D c = e.org;
  Point2D d = e.dest;
  Point2D n = Point2D((d-c).y, (c-d).x);
  double denom = dotProduct(n, b-a);
  if (denom == 0.0) {
    int aclass = org.classify(e);
    if ((aclass==Point2D::LEFT) || (aclass==Point2D::RIGHT))
      return PARALLEL;
    else
      return COLLINEAR;
  };
  double num = dotProduct(n, a-c);
  t = -num / denom;
  return SKEW;
};

int Edge::cross(Edge &e, double &t) {
  double s;
  int crossType = e.intersect(*this, s);
  if ((crossType==COLLINEAR) || (crossType==PARALLEL))
    return crossType;
  if ((s < 0.0) || (s > 1.0))
    return SKEW_NO_CROSS;
  intersect(e, t);
  if ((0.0 <= t) && (t <= 1.0))
    return SKEW_CROSS;
  else
    return SKEW_NO_CROSS;
};

double Point2D::distance(Edge &e) {
  Edge ab = e;
  ab.flip().rot();              // rotate ab 90 degrees counter-clockwise
  Point2D n(ab.dest - ab.org);  // n = vector perpendicular to e
  n = (1.0 / n.length()) * n;   // normalize n
  Edge f(*this, *this + n);     // f = n, positioned at this Point2D
  double t;
  f.intersect(e, t);            // t = signed distance along f
                                // at which f crosses edge e
  return t;
};

bool Edge::isVertical() {
  return (org.x == dest.x);
};

double Edge::slope() {
  if (org.x != dest.x) return (dest.y - org.y) / (dest.x - org.x);
  return MAXDOUBLE;
};

double Edge::y(double x) {
  return slope() * (x - org.x) + org.y;
};


