#include "Filter.h"
#include <iostream.h>
#include <iomanip.h>
#include <values.h>
#include <math.h>
#include <algo.h>
#include <vector.h>
#include <time.h>
#include "Histogram.h"

// #include "FilterOld.h"



void Filter::applyRotate180(const Image* inbild, Image* outbild) {

  int myBreite = inbild->getWidth();
  int myHoehe = inbild->getHeight();

  switch (inbild->getPlanes()) {
  case 3:
    // bewegen ueber Bild
    for(int y = 0; y < myHoehe; y++) {
      for(int x = 0; x < myBreite; x++) {
        outbild->setColor(x,y,inbild->getColor(myBreite-x,myHoehe-y));
      };
    };
    break;
  case 1:
    // bewegen ueber Bild
    for(int y = 0; y < myHoehe; y++) {
      for(int x = 0; x < myBreite; x++) {
        outbild->setGreyscale(x,y,inbild->getGreyscale(myBreite-x,myHoehe-y));
      };
    };
    break;
  default:
    cerr << "[Filter::applyRotate180] " << inbild->getPlanes();
    cerr << " planes are not supported." << endl;
    cerr.flush();
    exit(0);
  };

};




// The Filters below turn the pixels that match a criteria to white



void Filter::applyBrightToWhite(const Image *original, Image *destination, byte limit) {

  byte White = 255;

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {
      if (original->getGreyscale(x,y) > limit) {
        destination->setGreyscale(x,y,White);
      };
    };
  };

};


void Filter::applyBlackAndWhite(const Image *original, Image *destination, byte limit) {

  byte White = 255;
  byte Black = 0;

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {
      if (original->getGreyscale(x,y) >= limit) {
        destination->setGreyscale(x,y,White);
      } else {
        destination->setGreyscale(x,y,Black);
      };
    };
  };

};



// The following filter turns everyting dark to Black
void Filter::applyDarkToBlack(const Image *original, Image *destination, byte limit) {

  byte Black = 0;    // Black

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {
      if (original->getGreyscale(x,y) <= limit) {
        destination->setGreyscale(x,y,Black);
      };
    };
  };

};


void Filter::applyGreyscale(const Image *original, Image *destination) {

  byte grey;

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {

      grey = original->getGreyscale(x,y);
      destination->setGreyscale(x,y,grey);
    };
  };

};



void Filter::applyDespeckle(const Image *original, Image *destination, \
                            byte regionSize = 3, byte bwLimit = 255) {

  byte grey;
  byte White = 255;
  int WhitePixels;
  byte halfsize = regionSize / 2;

  for(int y = 0; y < (int) original->getHeight(); y++) {
   for(int x = 0; x < (int) original->getWidth(); x++) {

    // only check areas around darker pixels
    grey = original->getGreyscale(x,y);
    if ( grey < bwLimit) {
            
      WhitePixels = 0;
      for (int yy = y - halfsize; yy <= y + halfsize; yy++) {
        for (int xx = x - halfsize; xx <= x + halfsize; xx++) {
          grey = original->getGreyscale(xx,yy);
          if ( grey >= bwLimit) {
            WhitePixels++;
          }; 
	};
      };

      // alle Pixel in der Region um die Mitte muessen weiss sein
      if (WhitePixels == (regionSize*regionSize) - 1 ) {
        destination->setGreyscale(x,y,White);
      };

    };
   
   };
  };

};



Image8* Filter::applyDeltaImage(const Image *last, const Image *actual, const Rectangle bounds) {

  Image8 *delta = new Image8(actual->getWidth(), actual->getHeight());
  delta->setWhite();

  byte pixel1, pixel2, outpixel;
  int d;

  for(int y = bounds.y1; y <= bounds.y2; y++) {
    for(int x = bounds.x1; x <= bounds.x2; x++) {

      pixel1 = last->getGreyscale(x,y);
      pixel2 = actual->getGreyscale(x,y);

      d = (int) pixel1 - pixel2;
      if (d <= 0) { outpixel = 255; }
             else { outpixel = 255 - d; };

      delta->setGreyscale(x,y,outpixel);

      };
    };

  return delta;

};


void Filter::applyDeltaImageCorrection(Image* dest, const Image* image1, \
                                       const Image* image2, \
                                       const Rectangle bounds, int limit = 0) {

  // generate Delta image between Image1 and Image2
  // and use it (only) to correct the destination image

  byte outpixel;
  int d, p1, p2, out;

  for(int y = bounds.y1; y <= bounds.y2; y++) {
    for(int x = bounds.x1; x <= bounds.x2; x++) {

      p1 = (int) image1->getGreyscale(x,y);
      p2 = (int) image2->getGreyscale(x,y);
      d = p1 - p2;
 
      // if delta-value is smaller than 0 eliminate this part
      if (d < (int) limit) {
        out = (int) dest->getGreyscale(x,y) - d; 
        outpixel = (byte) out;
        if (out <  0 ) { outpixel =  0 ; };
        if (out > 255) { outpixel = 255; };
        dest->setGreyscale(x,y,outpixel);
      };

    };
  };

};



void Filter::applyDeltaImageCorrected(const Image *lastImage, const Image *actualImage, Image* lastDelta, Image* actualDelta, int limit = 0) {


  byte pixel1, pixel2, outpixel;
  int d;

  for(int y = 0; y < (int) lastImage->getHeight(); y++) {
    for(int x = 0; x < (int) lastImage->getWidth(); x++) {

      pixel1 = lastImage->getGreyscale(x,y);
      pixel2 = actualImage->getGreyscale(x,y);

      d = (int) (pixel1 - pixel2);
 
      // if delta-value is smaller than 0 eliminate this part 
      // in the actual and the last delta-Image
      if (d < limit) {
        outpixel = 255;
        actualDelta->setGreyscale(x,y,outpixel);
        lastDelta->setGreyscale(x,y,outpixel);
      } else {
        d = (int) pixel1 - pixel2;
        if (d <= 0) { outpixel = 255; }
               else { outpixel = 255 - d; };
        actualDelta->setGreyscale(x,y,outpixel);
      };

    };
  };

};



void Filter::applyBWQualify(const Image *original, Image *destination, \
                            unsigned int RegionSize, unsigned int bwLimit, \
                            float BlackRatioLimit) {

  time_t start = time(NULL);

  int myWidth = original->getWidth();
  int myHeight = original->getHeight();

  // int changes = 0;

  // Status wird groesser gewaehlt damit die
  // Region um Bildpixel immer enthalten ist
  byte status[(myHeight+RegionSize)*(myWidth+RegionSize)];

  byte White, inpixel;
  White = 255;  // White
  unsigned int AnzBlack, AnzWhite;

  // Status initialisieren
  unsigned int pos;
  unsigned int myWidthRegion = myWidth + RegionSize;
  byte HalfSize = RegionSize / 2;
  for(int y = -HalfSize; y < myHeight+HalfSize; y++) {
    for(int x = -HalfSize; x < myWidth+HalfSize; x++) {
  
      inpixel = original->getGreyscale(x,y);      

      // Weiss, Schwarz oder Farbe
      pos = (x+HalfSize) + ((y+HalfSize)*myWidthRegion);
      if ( inpixel >= bwLimit ) {
        status[pos] = 1;
      } else {
        status[pos] = 0;
      };

    };
  };


  // time_t mitte = time(NULL);


  // bewegen ueber Bild
  for(int y = HalfSize; y < myHeight + HalfSize; y++) {
    for(int x = HalfSize; x < myWidth + HalfSize; x++) {

      AnzBlack = AnzWhite = 0;

      // bewegen ueber Region um Pixel
      // Haufigkeiten von Schwarz und Weiss bestimmen
      // 
      for (int yy = y - HalfSize; yy <= y + HalfSize; yy++) {
        for (int xx = x - HalfSize; xx <= x + HalfSize; xx++) {

          // Weiss, Schwarz oder Farbe
          pos = xx + (yy*myWidthRegion);
          if ( status[pos] == 1 ) {
            AnzWhite++;
	  } else {
            AnzBlack++;
	  };

          // AnzWhite += status[pos];

        };
      };

      // AnzBlack = RegionSize*RegionSize - AnzWhite;

      // Auswertung des Verhaeltnisses zwischen Hell und Dunkel
      //
      // Vergleich von (Black / White) mit bwRatio
      if ( ( (float) AnzBlack / (RegionSize*RegionSize) ) <= BlackRatioLimit) {
	// leave the pixel as it was
      } else {
        destination->setGreyscale(x-HalfSize,y-HalfSize,White);
        // changes++;
      };
           
    };
  };



  time_t ende = time(NULL);

  // cout << "[Filter::applyBWQualify] " << changes << " changes in " << mitte - start << " + " << ende - mitte << " = " << ende - start << " seconds" << endl;
  cout << "[Filter::applyBWQualify] " << ende - start << " seconds" << endl;

};



void Filter::applyDeltaImageRedCorrected(const Image *lastImage, const Image *actualImage, Image* lastDelta, Image* actualDelta) {


  byte pixel1, pixel2;
  Color outpixel;
  int d;

  for(int y = 0; y < (int) actualImage->getHeight(); y++) {
    for(int x = 0; x < (int) actualImage->getWidth(); x++) {

      pixel1 = lastImage->getGreyscale(x,y);
      pixel2 = actualImage->getGreyscale(x,y);

      d = (int) pixel1 - pixel2;

      // if delta-value is smaller than 0 paint it red
      if (d < 0) {
 
        outpixel.r = 255;
        outpixel.g = 255 - d;
        outpixel.b = 255 - d;
        actualDelta->setColor(x,y,outpixel);
        lastDelta->setColor(x,y,outpixel);

      } else {
        d = (int) pixel1 - pixel2;
        if (d <= 0) { outpixel.r = outpixel.g = outpixel.b = 255; }
               else { outpixel.r = outpixel.g = outpixel.b = 255 - d; };
        actualDelta->setColor(x,y,outpixel);
      };

    };
  };

};




void Filter::applyMatrixCut(const Image *inbild, Image *outbild, Matrix<int> mat, float faktor, int bias) {

  int myBreite = inbild->getWidth();
  int myHoehe = inbild->getHeight();
  Color f;
  int summe_r, summe_g, summe_b;
  Color pixel;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      summe_r = 0; summe_g = 0; summe_b = 0;

      // bewegen ueber Matrix
      int xx, yy;
      for (unsigned int z = 0; z < mat.Zeilen(); z++) {
        for (unsigned int s = 0; s < mat.Spalten(); s++) {
          xx = x + s - (mat.Spalten() / 2);
          yy = y + z - (mat.Zeilen() / 2);
          pixel = inbild->getColor(xx,yy);      
          summe_r += pixel.r * mat[s][z];
          summe_g += pixel.g * mat[s][z];
          summe_b += pixel.b * mat[s][z];
        };
      };
      summe_r = int (summe_r * faktor) + bias;
      summe_g = int (summe_g * faktor) + bias;
      summe_b = int (summe_b * faktor) + bias;
      if (summe_r > 255) { summe_r = 255; };
      if (summe_r < 0)   { summe_r = 0; };
      if (summe_g > 255) { summe_g = 255; };
      if (summe_g < 0)   { summe_g = 0; };
      if (summe_b > 255) { summe_b = 255; };
      if (summe_b < 0)   { summe_b = 0; };

      f.r = summe_r;
      f.g = summe_g;
      f.b = summe_b;
      outbild->setColor(x,y,f);   
    };
  };

};




void Filter::applyBrightDarkGrey(const Image* original, Image* destination, byte brightLimit, byte darkLimit, byte scaleGrey = false) {

  byte Black = 0;    // Black
  byte White = 255;  // White
  int input;        // actual Pixel value
  byte correctedPixel;

  // Check that brightLimit > darkLimit
  byte lowerL, upperL;
  if (darkLimit <= brightLimit) {
    lowerL = darkLimit;
    upperL = brightLimit;
  } else {
    lowerL = brightLimit;
    upperL = darkLimit;
  };

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {

      input = original->getGreyscale(x,y);
      if (input <= lowerL) {
        // dark to black
        destination->setGreyscale(x,y,Black);
      } else { 
        if (input > upperL) {
          // bright to white
          destination->setGreyscale(x,y,White);
	} else {
          // leave it or scale it
          if (scaleGrey == false) {
            destination->setGreyscale(x,y,(byte) input);
	  } else {
            input -= lowerL;
            input *= (int) (255.0 / (float) (upperL - lowerL) );
            if (input > 255) {
              correctedPixel = 255;
	    } else if (input < 0) {
              correctedPixel = 0;
	    } else {
              correctedPixel = (byte) input;
	    };
            destination->setGreyscale(x,y,correctedPixel);
	  };
          
	};
      };
    
    };
  };



};



// merges the pixels of the source into the 
// destination image, the darker pixel is taken
void Filter::applyMergeImages(const Image *src, Image* dest, Rectangle tmpRect) {

  //int myBreite = src->getWidth();
  //int myHoehe = src->getHeight();
  byte greyscale;

  // bewegen ueber Bild
  //for(int y = 0; y < myHoehe; y++) {
  //  for(int x = 0; x < myBreite; x++) {
  for(int y = tmpRect.y1; y <= tmpRect.y2; y++) {
    for(int x = tmpRect.x1; x <= tmpRect.x2; x++) {

      // take the darker pixel
      greyscale = src->getGreyscale(x,y);
      if (greyscale < 150) {
        greyscale = (byte) ( (float) greyscale / 1.5 );
      };
      if (greyscale < dest->getGreyscale(x,y)) {
        dest->setGreyscale(x,y,greyscale);
      };

    };
  };

};



// merges the pixels of the source into the destination image
// the wights are indicated with the two int variables
void Filter::applyMergeImages(const Image *src, Image* dest, \
                              int countSrc, int countDest) {

  int myBreite = src->getWidth();
  int myHoehe = src->getHeight();
  unsigned long value;
  byte greyscale;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {

      value  = (countSrc * src->getGreyscale(x,y));
      value += (countDest * dest->getGreyscale(x,y)); 

      greyscale = (byte) ( value / (countSrc + countDest) );

      dest->setGreyscale(x,y,greyscale); 

    };
  };


};



void Filter::applySubtractImage(Image *ori, const Image* remove, const Rectangle bounds) {

  int value, diff;
  byte greyscale;

  // bewegen ueber Bild
  for(int y = bounds.y1; y <= bounds.y2; y++) {
    for(int x = bounds.x1; x <= bounds.x2; x++) {

      diff = 255 - (int) remove->getGreyscale(x,y);
      value  = ori->getGreyscale(x,y) + diff;
      value = (value < 0) ? 0 : value;
      value = (value > 255) ? 255 : value;
      greyscale = (byte) value;

      ori->setGreyscale(x,y,greyscale);

    };
  };


};



void Filter::applySubtractImageMasked(Image *ori, const Image* mask, const Image* values) {

  int myBreite = ori->getWidth();
  int myHoehe = ori->getHeight();
  int value, diff;
  byte greyscale;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {

      // if pixels aren't white there will 
      // be a subtraction at this position 
      if (mask->getGreyscale(x,y) < 255) {
        diff = 255 - (int) values->getGreyscale(x,y);
        value  = ori->getGreyscale(x,y) + diff;
        value = (value < 0) ? 0 : value;
        value = (value > 255) ? 255 : value;
        greyscale = (byte) value;

        ori->setGreyscale(x,y,greyscale);
 
      };

    };
  };


};



void Filter::createExpandedMask(const Image *ori, Image8 *dest, Rectangle window, unsigned int expandsteps = 1) {

  // Zielbild weiss fuellen
  dest->setWhite();

  // Otsu in Fenster berechnen
  Histogram myHistogram;
  myHistogram.generate(ori, window);
  unsigned int otsu = myHistogram.votsu();

  // Fensterbereich S/W wandeln und in Ziel kopieren
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      if (ori->getGreyscale(x,y) <= otsu) {
        dest->setGreyscale(x,y,0);
      };
    };
  };

  // Fensterbereich verdicken    
  Image* input = ori->copy();
  Image* output = dest->copy();
  for (unsigned int i = 1; i <= expandsteps; i++) {
    
    // Bildausschnitt verarbeiten
    for (int x = window.x1; x < window.x2; x++) {
      for (int y = window.y1; y < window.y2; y++) {

        unsigned int summe = 0;
        for (int xx = x - 1; xx <= x + 1; xx++) {
          for (int yy = y - 1; yy <= y + 1; yy++) {
            if (input->getGreyscale(xx,yy) == 0) {
              summe++;
	    };
	  };
	};
 
        if (summe > 0) {
          output->setGreyscale(x,y,0);
	};

      };
    };
    
    // Quelle lschen, Ziel wird neue Quelle
    delete input;
    input = output->copy();

  };

  delete input;
  dest->adoptImage8(output->getImage8());
  delete output;

};



void Filter::applyCutOutMask(Image* dest, const Image8* mask) {

  for (unsigned int x = 0; x < mask->getWidth(); x++) {
    for (unsigned int y = 0; y < mask->getHeight(); y++) {
      if (mask->getGreyscale(x,y) != 0) {
        dest->setGreyscale(x,y,255);
      };
    };
  };

};



void Filter::applyExpandGreyscale(const Image* ori, Image* dest, \
                                  const Rectangle bounds) {

  unsigned int darkest = 0;
  unsigned int sum = 0;
  unsigned int result;

  for (int x = bounds.x1; x <= bounds.x2; x++) {
    for (int y = bounds.y1; y <= bounds.y2; y++) {

        sum = 0;
        darkest = 255;

        for (int xx = x-1; xx <= x+1; xx++) {
          for (int yy = y-1; yy <= y+1; yy++) {

            unsigned int grey = ori->getGreyscale(xx,yy);
            sum += grey;
            if (grey < darkest) {
              darkest = grey;
	    };

	  };
        };

        // result = (sum / 9.0);        
        // result = darkest;
        result = (unsigned int) ( ((sum/9.0) + (double) darkest) / 2.0);
 
        dest->setGreyscale(x,y, (byte) result);

    }; // for y
  }; // for x

};



void Filter::applyBrightness(Image* img, const Rectangle bounds, int delta) {

  for (int x = bounds.x1; x <= bounds.x2; x++) {
    for (int y = bounds.y1; y <= bounds.y2; y++) {

      unsigned int grey = img->getGreyscale(x,y);
      if (grey < 255) {
        grey = grey + delta;
      };  
      if (grey > 255) {
        grey = 255;
      };
      img->setGreyscale(x,y, (byte) grey);

    }; // for y
  }; // for x

};







