#ifndef _FILTER_H_
#define _FILTER_H_

#include "Elements.h"
#include "Point.h"
#include "Rectangle.h"
#include "Image.h"
#include "Image8.h"
#include "Image24.h"
#include "matrix.h"

class Filter {

public:
  //Filter();
  //~Filter();

  // anwenden einer Matrix auf ein Bild
  // die Outputwerte werden in das Intervall 0..255 skaliert
  // Image applyMatrixScaled(Image, Matrix<int>, float Faktor);

  // anwenden einer Matrix auf ein Bild
  // die Outputwerte werden auf das Intervall 0..255
  // zugeschnitten. Der Parameter bias verschiebt die Werte vor dem cut
  // Image applyMatrixCut(Image, Matrix<int>, float Faktor, int bias);

  // Image applyMedian(Image, unsigned int Ordnung);
  // Image applyColorKill(Image, byte limit, byte killvalue);

  // Image applyColorSelectHSV(Image, unsigned int hue, byte limit, byte killvalue);
  // Image applyColorUnSelectHSV(Image, unsigned int hue, byte limit, byte killvalue);
  // Image applyReduceDepth(Image, unsigned char newDepth);
  // Image applyBlackAndWhite(Image, byte limit);



  void applyRotate180(const Image*, Image*);


  // The following Filters turn only the Pixels to White
  // that match the criterias defined inside of the Filter
  void applyBrightToWhite(const Image*, Image*, byte limit);
  void applyBWQualify(const Image*, Image*, unsigned int RegionSize, \
                      unsigned int bwLimit, float BlackRatioLimit);

  // The following filter turns everyting dark to Black
  void applyDarkToBlack(const Image*, Image*, byte limit);

  void applyBlackAndWhite(const Image*, Image*, byte limit);
  void applyGreyscale(const Image*, Image*);

  void applyBrightDarkGrey(const Image*, Image*, byte brightLimit, byte darkLimit, byte scaleGrey = false);

  void applyDespeckle(const Image*, Image*, \
                      byte regionSize = 3, byte bwLimit = 255);

  Image8* applyDeltaImage(const Image*, const Image*, const Rectangle bounds);
  void applyDeltaImageCorrection(Image*, const Image*, const Image*, \
                                 const Rectangle bounds, const int limit = 0);
  void applyDeltaImageCorrected(const Image*, const Image*, Image*, Image*, int limit = 0);
  void applyDeltaImageRedCorrected(const Image*, const Image*, Image*, Image*);

  void applyMatrixCut(const Image*, Image*, Matrix<int>, float Faktor, int bias);


//  void applyColorUnSelectHSV(const Image*, Image*, unsigned int hue, byte limit);
//  void applyRegionQualify(const Image*, Image*, byte RegionSize, \
//                          byte BlackLimit, byte WhiteLimit, \
//                          byte maxColor, byte minWhite, byte maxBlack);
//  void applyInvertImage(const Image*, Image*);
//  void applyConvertGreyscale(const Image*, Image8*);

  void addDetectedPixelsGreyscale(const Image*, Image*);

  void applyMergeImages(const Image*, Image*, int coutnSrc, int countDest);
  void applyMergeImages(const Image*, Image*, Rectangle);
  void applySubtractImage(Image*, const Image*, const Rectangle bounds);
  void applySubtractImageMasked(Image*, const Image*, const Image*);

  void applyExpandGreyscale(const Image*, Image*, \
                            const Rectangle bounds);
  void applyBrightness(Image*, const Rectangle bounds, int delta);

  void createExpandedMask(const Image*, Image8*, Rectangle, unsigned int expandsteps = 1);
  void applyCutOutMask(Image*, const Image8*);

private:
  int *tmpBild;

  inline void rgb2hsv(byte r, byte g, byte b, float& h, float& s, float& v);
  void hsv2rgb(float h, float s, float v, byte &r, byte &g, byte &b);
  inline float maxRGB(float r, float g, float b);
  inline float minRGB(float r, float g, float b);

};

#endif


