
Image Filter::applyMatrixScaled(Image inbild, Matrix<int> mat, float faktor) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  tmpBild = new int[myBreite*myHoehe*3];

  int max_r, max_g, max_b;
  max_r = max_g = max_b = -MAXINT + 1;
  int min_r, min_g, min_b;
  min_r = min_g = min_b = MAXINT;
  int summe_r, summe_g, summe_b;
  Color pixel;
  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      summe_r = 0; summe_g = 0; summe_b = 0;

      // bewegen ueber Matrix
      int xx, yy;
      for (unsigned int z = 0; z < mat.Zeilen(); z++) {
        for (unsigned int s = 0; s < mat.Spalten(); s++) {
          xx = x + s - (mat.Spalten() / 2);
          yy = y + z - (mat.Zeilen() / 2);
          pixel = inbild.getPixel(xx,yy);      
          summe_r += pixel.r * mat[s][z];
          summe_g += pixel.g * mat[s][z];
          summe_b += pixel.b * mat[s][z];
        };
      };
      summe_r = int (summe_r * faktor);
      summe_g = int (summe_g * faktor);
      summe_b = int (summe_b * faktor);
      if (summe_r > max_r) { max_r = summe_r; };
      if (summe_r < min_r) { min_r = summe_r; };
      if (summe_g > max_g) { max_g = summe_g; };
      if (summe_g < min_g) { min_g = summe_g; };
      if (summe_b > max_b) { max_b = summe_b; };
      if (summe_b < min_b) { min_b = summe_b; };

      int pixelStart = 3*(x + myBreite*y);
      tmpBild[pixelStart+0] = summe_r;
      tmpBild[pixelStart+1] = summe_g;
      tmpBild[pixelStart+2] = summe_b;
    };
  };
  int dist_r = max_r - min_r;
  int dist_g = max_g - min_g;
  int dist_b = max_b - min_b;
  float scale_r = 255.0 / dist_r; 
  float scale_g = 255.0 / dist_g; 
  float scale_b = 255.0 / dist_b; 

  // verschobene & skalierte Werte wieder zuordnen
  Color f;
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      int pixelStart = 3*(x + myBreite*y);
      f.r = (byte) ((tmpBild[pixelStart+0] - min_r) * scale_r);
      f.g = (byte) ((tmpBild[pixelStart+1] - min_g) * scale_g);
      f.b = (byte) ((tmpBild[pixelStart+2] - min_b) * scale_b);
      outbild.setPixel(x,y,f);
    };
  };  

  return outbild;
};


Image Filter::applyMatrixCut(Image inbild, Matrix<int> mat, float faktor, int bias) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  Color f;
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  int summe_r, summe_g, summe_b;
  Color pixel;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      summe_r = 0; summe_g = 0; summe_b = 0;

      // bewegen ueber Matrix
      int xx, yy;
      for (unsigned int z = 0; z < mat.Zeilen(); z++) {
        for (unsigned int s = 0; s < mat.Spalten(); s++) {
          xx = x + s - (mat.Spalten() / 2);
          yy = y + z - (mat.Zeilen() / 2);
          pixel = inbild.getPixel(xx,yy);      
          summe_r += pixel.r * mat[s][z];
          summe_g += pixel.g * mat[s][z];
          summe_b += pixel.b * mat[s][z];
        };
      };
      summe_r = int (summe_r * faktor) + bias;
      summe_g = int (summe_g * faktor) + bias;
      summe_b = int (summe_b * faktor) + bias;
      if (summe_r > 255) { summe_r = 255; };
      if (summe_r < 0)   { summe_r = 0; };
      if (summe_g > 255) { summe_g = 255; };
      if (summe_g < 0)   { summe_g = 0; };
      if (summe_b > 255) { summe_b = 255; };
      if (summe_b < 0)   { summe_b = 0; };

      f.r = summe_r;
      f.g = summe_g;
      f.b = summe_b;
      outbild.setPixel(x,y,f);   
    };
  };

  return outbild;
};


Image Filter::applyMedian(Image inbild, unsigned int Ordnung) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  Color pixel, neu;
  vector<int> r(Ordnung);
  vector<int> g(Ordnung);
  vector<int> b(Ordnung);
  int index;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      // bewegen ueber Ausschnitt
      int l = (int) (sqrt(Ordnung) / 2);
      index = 0;

      for (int yy = y-l; yy <= y+l; yy++) {
        for (int xx = x-l; xx <= x+l; xx++) {
          pixel = inbild.getPixel(xx,yy);      
          r[index] = pixel.r;
          g[index] = pixel.g;
          b[index] = pixel.b;
          index++;
        };
      };

      sort(r.begin(), r.end());
      sort(g.begin(), g.end());
      sort(b.begin(), b.end());
      neu.r = r[Ordnung/2];
      neu.g = g[Ordnung/2];
      neu.b = b[Ordnung/2];
      outbild.setPixel(x,y,neu);   
    };

  };

  return outbild;
};



Image Filter::applyColorKill(Image inbild, byte limit, byte killvalue) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  Color alt, neu;
  neu.r = neu.g = neu.b = killvalue;
  int d_rg, d_rb, d_bg, delta;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {

      alt = inbild.getPixel(x,y);
      if (alt.r > alt.g) { d_rg = alt.r - alt.g; }
                    else { d_rg = alt.g - alt.r; };      
      if (alt.r > alt.b) { d_rb = alt.r - alt.b; }
                    else { d_rb = alt.b - alt.r; };      
      if (alt.b > alt.g) { d_bg = alt.b - alt.g; }
                    else { d_bg = alt.g - alt.b; };      
      delta = d_rg + d_rb + d_bg;
      if (delta > (int) limit) { outbild.setPixel(x,y,neu); }
                          else { outbild.setPixel(x,y,alt); }

    };
  };

  return outbild;
};


Image Filter::applyBlackAndWhite(Image inbild, byte limit) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  Color alt, black, white;
  black.r = black.g = black.b = 0;
  white.r = white.g = white.b = 255;
  int luminance;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {

      alt = inbild.getPixel(x,y);
      luminance = alt.r + alt.g + alt.b;
      if (luminance < 3*limit) {
        outbild.setPixel(x,y,black);
      } else {
        outbild.setPixel(x,y,white);
      };
    };
  };

  return outbild;

};


inline float Filter::maxRGB(float r, float g, float b) {
  float res;
  res = (r > g) ? r : g;
  res = (res > b) ? res : b;
  return res;
};

inline float Filter::minRGB(float r, float g, float b) {
  float res;
  res = (r < g) ? r : g;
  res = (res < b) ? res : b;
  return res;
};

inline void Filter::rgb2hsv(byte red, byte green, byte blue, float& h, float& s, float& v) {

  float rd, gd, bd, mi, temp;
  float r, g, b;
  r = red / 255.0;
  g = green / 255.0;
  b = blue / 255.0;

  v = maxRGB(r, g, b);
  mi = minRGB(r, g, b);
  temp = v - mi;

  if (v > 0) { s = temp / v; }
        else { s = 0; };

  if (s > 0) {
    rd = (v - r) / temp;
    gd = (v - g) / temp;
    bd = (v - b) / temp;

    if (v == r) { if (mi == g) { h = 5 + bd ; }
                          else { h = 1 - gd; }
    } else {
    if (v == g) { if (mi == b) { h = 1 + rd; }
                          else { h = 3 - bd; }
                } else {
    if (v == b) { if (mi == r) { h = 3 + gd; }
                          else { h = 5- rd; }
                }
    } }

    if (h < 6) { h *= 60; }
          else { h = 0; }

  };

};

void Filter::hsv2rgb(float h, float s, float v, byte &red, byte &green, byte &blue) {

  float r, g, b;
  float t1, t2, t3, fract;
  int i;

  if (s == 0) { r = g = b = 0; }
  else {
    h /= 60;
    i = (int) i;
    fract = h - i;
    t1 = v * (1 - s);
    t2 = v * (1 - s*fract);
    t3 = v * (1 - s*(1-fract));
    switch (i) {
    case 0:
      r = v;
      g = t3;
      b = t1;
      break;
    case 1:
      r = t2;
      g = v;
      b = t1;
      break;
    case 2:
      r = t1;
      g = v;
      b = t3;
      break;
    case 3:
      r = t1;
      g = t2;
      b = v;
      break;
    case 4:
      r = t3;
      g = t1;
      b = v;
      break;
    case 5:
      r = v;
      g = t1;
      b = t2;
      break;
    };
  };

  red = (byte) r * 255;
  green = (byte) g * 255;
  blue = (byte) b * 255;

};


Image Filter::applyColorSelectHSV(Image inbild, unsigned int hue, byte limit, byte killvalue) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  Color alt, neu;
  neu.r = neu.g = neu.b = killvalue;
  float h, s, v;
  byte delta;

  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {

      alt = inbild.getPixel(x,y);
      rgb2hsv(alt.r, alt.g, alt.b, h, s, v);

      delta = (h > hue) ? (byte) (h-hue) : (byte) (hue-h);

      if (delta > limit) {
        outbild.setPixel(x,y,neu);
      } else {
        outbild.setPixel(x,y,alt);
      };
    };
  };

  return outbild;

};


Image Filter::applyReduceDepth(Image inbild, unsigned char newDepth) {

  int myBreite = inbild.getWidth();
  int myHoehe = inbild.getHeight();
  // Image *outbild = new Image(myBreite, myHoehe);
  Image outbild(myBreite, myHoehe);
  
  int divisor = (int) (256 / pow(2,newDepth));

  cout << divisor << "  " << pow(2,newDepth) << endl;

  Color inpixel, outpixel;
  // bewegen ueber Bild
  for(int y = 0; y < myHoehe; y++) {
    for(int x = 0; x < myBreite; x++) {
      inpixel = inbild.getPixel(x,y);

      outpixel.r = ( (int) (inpixel.r / divisor)) * divisor;
      outpixel.g = ( (int) (inpixel.g / divisor)) * divisor;
      outpixel.b = ( (int) (inpixel.b / divisor)) * divisor;

      
/*

      cout << "(" << x << "/" << y << ")  ";
  cout << "IN: " << (int) inpixel.r << "/" << (int) inpixel.g << "/" << (int) inpixel.b << "/"; 
  cout << "   OUT: " << (int) outpixel.r << "/" << (int) outpixel.g << "/" << (int) outpixel.b << "/"; 
  cout << endl;
      
*/

      outbild.setPixel(x,y,outpixel);
    };
  };

  return outbild;

};



void Filter::applyColorUnSelectHSV(const Image *original, Image *destination, unsigned int hue, byte limit) {

  int changes = 0;
  Color inpixel, outpixel;
  outpixel.r = outpixel.g = outpixel.b = 255;  // White
  float h, s, v;
  byte delta;

  // bewegen ueber Bild
  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {


      // bewegen ueber Region um Pixel
      // Haufigkeiten von Farbe, Schwarz und Weiss bestimmen
      // 

      int HalfSize = 2;      // 5*5 Region

      unsigned int red, green, blue;
      red = green = blue = 0;
      for (int yy = y - HalfSize; yy <= y + HalfSize; yy++) {
        for (int xx = x - HalfSize; xx <= x + HalfSize; xx++) {

          inpixel = original->getPixel(xx,yy);      
          red += inpixel.r;
          green += inpixel.g;
          blue += inpixel.b;

        };
      };

      inpixel.r = red / 25;
      inpixel.g = green / 25;
      inpixel.b = blue / 25;


      // inpixel = original->getPixel(x,y);
      rgb2hsv(inpixel.r, inpixel.g, inpixel.b, h, s, v);

      delta = (h > hue) ? (byte) (h-hue) : (byte) (hue-h);

      if (delta <= limit) {
        destination->setPixel(x,y,outpixel);
        changes++;
      } else {
        // let the pixel be as it was
      };
    };
  };

  cout << "[Filter::applyColorUnSelectHSV] " << changes << " changes" << endl;

};



void Filter::applyRegionQualify(const Image *original, Image *destination, \
                                byte RegionSize, byte BlackLimit, \
                                byte WhiteLimit, byte maxColor, \
                                byte minWhite, byte maxBlack) {

  Color Black, White, inpixel;
  White.r = White.g = White.b = 255;  // White
  Black.r = Black.g = Black.b = 0;    // Black
  unsigned int AnzBlack, AnzWhite, AnzColor;

  byte HalfSize = RegionSize / 2;

  // bewegen ueber Bild
  //  for(int y = 0; y < original->getHeight(); y++) {
  //  for(int x = 0; x < original->getWidth(); x++) {
  for(int y = 50; y < 150; y++) {
    for(int x = 100; x < 200; x++) {

      AnzBlack = AnzWhite = AnzColor = 0;

      // bewegen ueber Region um Pixel
      // Haufigkeiten von Farbe, Schwarz und Weiss bestimmen
      // 

      for (int yy = y - HalfSize; yy <= y + HalfSize; yy++) {
        for (int xx = x - HalfSize; xx <= x + HalfSize; xx++) {

          inpixel = original->getPixel(xx,yy);      

          // Weiss, Schwarz oder Farbe
          if ( inpixel.r + inpixel.g + inpixel.b >= 3*WhiteLimit ) {
            AnzWhite++;
	  } else if ( inpixel.r + inpixel.g + inpixel.b <= 3*BlackLimit ) {
            AnzBlack++;
	  } else {
            AnzColor++;
	  };
 
        };
      };

      cout << x << " / " << y << ": B: " << AnzBlack << "  W: " << AnzWhite << "  C: " << AnzColor;

      // Auswertung der Haufigkeiten
      if ((AnzWhite >= minWhite) && (AnzBlack <= maxBlack) && (AnzColor <= maxColor)) {
	// leave the pixel as it was
        cout << "   ->  leave it"<< endl;
      } else {
        destination->setPixel(x,y,White);
        cout << "   ->  White" << endl;
      };

      cout.flush();
      

      /*
      inpixel = original->getPixel(x,y);
          if ( inpixel.r + inpixel.g + inpixel.b >= 3*WhiteLimit ) {
            destination->setPixel(x,y,White);
	  } else if ( inpixel.r + inpixel.g + inpixel.b <= 3*BlackLimit ) {
            destination->setPixel(x,y,Black);
	  } else {
            AnzColor++;
	  };
      */


    };
  };

};


void Filter::applyInvertImage(const Image *original, Image *destination) {

  Color inpixel, outpixel;

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {
      inpixel = original->getPixel(x,y);
      outpixel.r = 255 - inpixel.r;
      outpixel.g = 255 - inpixel.g;
      outpixel.b = 255 - inpixel.b;
      destination->setPixel(x,y,outpixel);
      };
    };
};


void Filter::applyConvertGreyscale(const Image *original, Image8 *destination) {

  Color inpixel;
  byte grey;

  for(int y = 0; y < (int) original->getHeight(); y++) {
    for(int x = 0; x < (int) original->getWidth(); x++) {
      inpixel = original->getPixel(x,y);
      grey = (inpixel.r + inpixel.g + inpixel.b) / 3;
      destination->setPixel(x,y,grey);
    };
  };

};
