/*
 * Geometry2D.h
 *
 * Copyright (C) 1995, Stephan Amann <amann@iam.unibe.ch>
 *                     Christoph Streit <streit@iam.unibe.ch>
 *                     University of Berne, Switzerland
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 * -----------------------------------------------------------------------------
 *  $Id: Geometry2D.h,v 1.2 1996/01/18 18:15:20 streit Exp $
 * -----------------------------------------------------------------------------
 */

#ifndef _Geometry2D_H
#define _Geometry2D_H

#include "PrimitiveTypes.h"
#include "Vector2D.h"
#include "List.h"

//___________________________________________________________________ Geometry2D

class Geometry2D {
public:
  enum OrientType { CW=-1, CCW=1, ZERO=0 };

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Constructors, destructors, assignment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
public:
  Geometry2D();                                 // No instances.
  // Geometry2D(const Geometry2D&);             // Use default version.

public:
  // ~Geometry2D();                             // Use default version.

private:
  // Geometry2D& operator=(const Geometry2D&);  // No assignments.
  
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Methods of class Geometry2D
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
public:
  static OrientType orient(Real px1, Real py1,
                           Real px2, Real py2,
                           Real px3, Real py3);

  static bool intersect(Real px1, Real py1,
                        Real px2, Real py2,
                        Real px3, Real py3,
                        Real px4, Real py4);

  static bool pointInTriangle(Real px,  Real py,
                              Real px1, Real py1,
                              Real px2, Real py2,
                              Real px3, Real py3);
                              
public:
  struct Triangle {
    Triangle() {}
    Triangle(long index1, long index2, long index3)
    : myIndex1(index1), 
      myIndex2(index2), 
      myIndex3(index3)
    {}
    
    long myIndex1, myIndex2, myIndex3;
  };

  static List<Triangle>* createPolygonTriangulation(List<Vector2D>& vertices);
  // Generate triangulation of convex/concave polygon. The result is a list
  // triangels given by indexes to the provided vertex list.
  //
  // Return value is NULL if an error occured or an invalid polygon was provided.
  //
  // Orginal source code:
  //   ftp://ftp.cs.unc.edu/pub/users/narkhede/triangulation.tar.gz
  //
  // Reference:
  //   R. Seidel, 
  //   A simple and Fast Randomized Algorithm for Computing Trapezoidal 
  //   Decompositions and for Triangulating Polygons,
  //   Computational Geometry Theory & Applications, 1991.
  
};

#endif // _Geometry2D_H

