#!/usr/local/bin/perl

# Program
$program = "src/mpeg2points";


# read command line
($configfile,$rowparameter,$rowstart,$rowend,$rowstep) = ("","", 1, 1, 1);
if ($#ARGV == 0) {
  $configfile = $ARGV[0];
} elsif ($#ARGV == 3) {
  ($rowparameter,$rowstart,$rowend,$rowstep) = @ARGV;
} elsif ($#ARGV == 4) {
  ($configfile,$rowparameter,$rowstart,$rowend,$rowstep) = @ARGV;
} else {
  ($configfile,$rowparameter,$rowstart,$rowend,$rowstep) = ("","", 1, 1, 1);
};
if ($configfile eq "") {
  # set default values
  $word = "";
  $report_dir = "results/";
  $report_name = "mpeg4";
  $report_details = "";
  $source_dir = "mpeg/4/";
  $source_name = "test";
  $source_digits = 4;
  $window = "14 22 150 63";
  $start = 25;
  $end = 185;
  $winsize = 60;
  $lookahead = 2;
  $otsuvariance = 75;
  $intermin = 5;
  $intermax = 10;
  $intergap = 2;
  $masksize = 3;
  $area = 1.0;
  @methods = ("simple", "grey", "median", "greyiterative", "iterative", "neighbour");
} elsif (($configfile eq "-help")||($configfile eq "--help")||($configfile eq "/?")) {
  # print usage
  print "\n";
  print "Usage: HTMLreport.pl [configfile] [rowparameter start stop step]\n";
  print "  configfile    File holding general configs for processing\n";
  print "  rowparameter  Parametername to change for the output rows\n";
  print "                winsize, lookahead, otsuvariance, area\n";
  print "                intermin, intermax, intergap or masksize\n";
  print "    start       Start value of rowparameter\n";
  print "    stop        End value of rowparameter\n";
  print "    step        Step value of rowparameter\n";
  print "\n";
  exit;
} else {
  # read config file
  open(CONFIG, $configfile) || die("config file ($configfile) not found.\n");
  while (<CONFIG>) {
    if ((substr($_,0,1) ne '#') && (substr($_,0,1) ne ' ') && ($_ ne "\n")) {
      ($variable, $values) = split("=", $_);
      if (substr($values, length($values)-1,1) !~ "\d") {
        # remove trailing newline
        $values = substr($values, 0, length($value)-1);
      };
      @values = split(",",$values);
      if ($variable eq 'methods') {
        # methods has to be list but single value is allowed !!!
        if ($#values > 0) {
          # list of values
	  eval "\@$variable = \@values";
          print "\@$variable = @values\n";
        } else {
          # single value
          eval "\@$variable = ($values[0])";
          print "\@$variable = ($values[0])\n";
        };
      } elsif ($#values > 0) {
        # list of values
	eval "\@$variable = \@values";
        print "\@$variable = @values\n";
      } else {
        # single value
        eval "\$$variable = '$values[0]'";
        print "\$$variable = '$values[0]'\n";
      };
    };
  };
  close CONFIG;
};



# Load sharing using rsh (Config is read from host.cfg)
open (HOSTS, "./hosts.cfg") || print "host.cfg not found. Processing is restricted to local machine.\n";
@hosts = ();
@speed = ();
@tasks = ();
@actualload = ();
@targetload = ();
while (<HOSTS>) {
  if (substr($_,0,1) ne '#') {
    ($hostname, $hostspeed, $hosttasks) = split(",", $_);
    if (substr($hosttasks, length($hosttasks)-1,1) !~ "\d") {
      # remove trailing newline
      $hosttasks = substr($hosttasks, 0, length($hosttasks)-1);
    };
    push @hosts, $hostname;
    push @speed, $hostspeed;
    push @tasks, $hosttasks;
    push @actualload, 0;
    push @targetload, 0;
  };
};
close HOSTS;
# be sure to have at least the local machine in the config
if ($#hosts < 0) {
  @hosts = ("localhost");
  @speed = (1);
  @tasks = (1);
  @actualload = (0);
  @targetload = (0);
};
$nexthost = 0;

 


# the following commandline parameters can be 
# variable and are used to generate the rows
#
# Parameters                      Examples
# lookahead start stop step      lookahead 1 5 1
# winsize start stop step        winsize 5 75 10
# otsuvariance start stop step   otsuvariance 70 130 10
# intermin start stop step       intermin 3 7 1
# intermax start stop stop       intermax 8 12 1
# intergap start stop stop       intergap 2 6 1
# masksize start stop step       masksize 1 5 1
# area start stop step           area 0.05 0.2 0.05




# 1. Determinate the variable Parameter for the rows
if ($rowparameter ne '') {
  $report_details = "_" . $rowparameter . "-" . $rowstart . "-" . $rowend . "-" . $rowstep;
} else {
  $report_details = "";
};
system "mkdir $report_dir";
system "mkdir $report_dir$report_name$report_details/";
open (HTML, ">$report_dir$report_name$report_details/"."report.html");
@rowheader = @methods;
print HTML "<HTML>\n<HEAD><TITLE>HTML Report - $report_name$report_details</TITLE></HEAD>\n<BODY>\n";
print HTML "<H1>HTML Report - $report_name$report_details</H1>\n";
# display used parameters
print HTML "Word: $word<BR>\n";
print HTML "Source: $source_dir, $source_name, $source_digits<BR>\n";
print HTML "Window: $window<BR>\n";
print HTML "Intervall: $start - $end<BR>\n";
if ($rowparameter ne 'lookahead') {
  print HTML "Lookahead: $lookahead<BR>\n";
};
if ($rowparameter ne 'winsize') {
  print HTML "Windowsize: $winsize<BR>\n";
};
if ($rowparameter ne 'otsuvariance') {
  print HTML "Otsuvariance: $otsuvariance<BR>\n";
};
if ($rowparameter ne 'intermin') {
  print HTML "Interpolate min: $intermin<BR>\n";
};
if ($rowparameter ne 'intermax') {
  print HTML "Interpolate max: $intermax<BR>\n";
};
if ($rowparameter ne 'intergap') {
  print HTML "Interpolate gap: $intergap<BR>\n";
};
if ($rowparameter ne 'masksize') {
  print HTML "Masksize: $masksize<BR>\n";
};
if ($rowparameter ne 'area') {
  print HTML "Area Ratio: $area<BR>\n";
};
print HTML "<P>\n";
print HTML "<TABLE border=2>\n<TR><TD></TD>";
while(@rowheader) {
    $header = pop(@rowheader);
    print HTML "<TD>$header</TD>";
};
print HTML "</TR>\n";
print "\n";


# 2. Zeile fuer Zeile, Spalte fuer Spalte verarbeiten
@dirs = ();  # save dirs to display later ...
$job = 0;
$jobs = ($#methods+1) * ((($rowend-$rowstart)/$rowstep)+1);
print "$jobs jobs are distributed over " . ($#hosts+1) ." hosts\n";
$speedsum = 0;
for($i = 0; $i <= $#speed; $i++) {
  $speedsum += $speed[$i];
};
for($i = 0; $i <= $#speed; $i++) {
  $targetload[$i] = ($speed[$i]/$speedsum)*$jobs;
  print "$targetload[$i] job(s) planned to run on $hosts[$i] (Speed = $speed[$i], max. $tasks[$i] tasks)\n";
};
# initialize shell scripts for the hosts
for ($i = 0; $i <= $#hosts; $i++) {
  open(JOB, ">$report_dir$report_name$report_details/$hosts[$i].job");
  print JOB "# Job for $hosts[$i]\n#\n";
  print JOB "set path = (\$path /home/fcg/bin)\n";
  close JOB;
};
print "\n";
for ($i = $rowstart; $i <= $rowend; $i = $i + $rowstep) {
    print HTML "<TR><TD>$rowparameter<BR>$i</TD>";

    # set the special paramters to the stepping value
    if ($rowparameter eq 'lookahead') {
        $lookahead = $i;      
    } elsif ($rowparameter eq 'winsize') {
	$winsize = $i;
    } elsif ($rowparameter eq 'otsuvariance') {
	$otsuvariance = $i;
    } elsif ($rowparameter eq 'intermin') {
	$intermin = $i;
    } elsif ($rowparameter eq 'intermax') {
	$intermax = $i;
    } elsif ($rowparameter eq 'intergap') {
	$intergap = $i;
    } elsif ($rowparameter eq 'masksize') {
	$masksize = $i;
    } elsif ($rowparameter eq 'area') {
	$area = $i;
    } else {
        # no special parameters set
    };

    @rowmethods = @methods;
    while (@rowmethods) {
       $method = pop(rowmethods);
       $directory  = "$method" . "_";
       $directory .= "look" . $lookahead . "_"; 
       $directory .= "otsu" . $otsuvariance . "_";
       $directory .= "ipol" . $intermin . "-" . $intermax . "-" . $intergap . "_";
       $directory .= "win" . $winsize . "_";
       $directory .= "mask" . $masksize . "_";
       $directory .= "area" . $area . "/";
       push @dirs, $directory;

       print HTML "<TD>";
       print HTML "<IMG SRC=\"$directory"."imagePoint.gif\">";
       print HTML "<BR>";
       print HTML "<IMG SRC=\"$directory"."imageLine.gif\">";
       print HTML "<BR>";
       print HTML "<IMG SRC=\"$directory"."imageLineI.gif\">";
       print HTML "</TD>";

       $cmdline  = "$program -mpeg $source_dir $source_name $source_digits -win $window ";
       $cmdline .= "-mask $masksize -winsize $winsize -lookahead $lookahead ";
       $cmdline .= "-interval $start $end -word '$word' ";
       $cmdline .= "-area $area -otsu $otsuvariance -method $method ";
       $cmdline .= "-interpolate $intermin $intermax $intergap ";
       $cmdline .= "-result $report_dir$report_name$report_details/$directory";

       $job++;

       # find first Host with free resources       
       while ($actualload[$nexthost] >= $targetload[$nexthost]) {
         $nexthost++;
         if ($nexthost > $#hosts) {
	   $nexthost = 0;
	 };
       };

       # run job on this host
       $actualload[$nexthost]++;
       if ($hosts[$nexthost] eq "localhost") {
         print "job $job launched on local machine ($method, $rowparameter = $i)\n";
       } else {
         print "job $job launched on remote machine $hosts[$nexthost] ($method, $rowparameter = $i)\n";     
       };

       # append new job to job-file
       open(JOB, ">>$report_dir$report_name$report_details/$hosts[$nexthost].job");
       print JOB "$cmdline > $report_dir$report_name$report_details/job.$job";
       if ($actualload[$nexthost] % $tasks[$nexthost] == 0) {
	 print JOB "\n";
       } else {
	 print JOB " &\n";
       };
       close JOB;

       # switch to next host
       $nexthost++;
       if ($nexthost > $#hosts) {
         $nexthost = 0;
       };
    
    };

    print HTML "</TR>\n";

};


# include Applet into File
print HTML "</TABLE>\n\n";
print HTML "<HR>\n";
print HTML "<applet code=\"WriteApplet\" codebase=\"/java/\" align=\"baseline\" width=\"800\" height=\"400\">\n";
print HTML "<param name=\"URLBase\" value=\"http://localhost/$report_dir$report_name$report_details/\">\n";
for($i = 0; $i <= $#dirs; $i++) {
  print HTML "<param name=\"DetectionMethod$i\" value=\"$dirs[$i]\">\n";
};
print HTML "</applet>\n";
print HTML "</BODY></HTML>\n";
close HTML;


# launch jobs on the different hosts
print "\n";
for($i = 0; $i <= $#speed; $i++) {
  print "$actualload[$i] jobs running on $hosts[$i] (Speed = $speed[$i], Targetload = $targetload[$i])\n";
};
for ($i = 0; $i <= $#hosts; $i++) {
  if ($hosts[$i] eq "localhost") {
    system "csh $report_dir$report_name$report_details/$hosts[$i].job > $report_dir$report_name$report_details/job.$hosts[$i] \n";   # &
  } else {
    system "rsh $hosts[$i] \"cd diplom;csh $report_dir$report_name$report_details/$hosts[$i].job > $report_dir$report_name$report_details/job.$hosts[$i] &\" &\n";

  };
};

print "\n";


