#include "Histogram.h"
#include <fstream.h>
#include <iostream.h>
#include "Image.h"
#include "Image24.h"
#include <values.h>
#include <math.h>


void Histogram::adaptRect2ullr (int &x1, int &y1, int &x2, int &y2, \
                unsigned int maxWidth, unsigned int maxHeight) const {

  Point upperleft, lowerright;

  upperleft.x = (x1 < x2) ? x1 : x2;
  upperleft.y = (y1 < y2) ? y1 : y2;
  lowerright.x = (x1 > x2) ? x1 : x2;
  lowerright.y = (y1 > y2) ? y1 : y2;

  // check size of window
  if (x1 > (int) maxWidth)  { x1 = maxWidth;  };
  if (y1 > (int) maxHeight) { y1 = maxHeight; };
  if (x2 > (int) maxWidth)  { x2 = maxWidth;  };
  if (y2 > (int) maxHeight) { y2 = maxHeight; };

  if (upperleft.x < 0) {
    x1 = 0;
  } else {
    x1 = upperleft.x;
  };
  if (upperleft.y < 0) {
    y1 = 0;
  } else {
    y1 = upperleft.y;
  };

  if (lowerright.x < 0) {
    x2 = 0;
  } else {
    x2 = lowerright.x;
  };
  if (lowerright.y < 0) {
    y2 = 0;
  } else {
    y2 = lowerright.y;
  };

};


Histogram::Histogram () {
  #ifdef DEBUG
  cout << "Histogram::Histogram() called" << endl;
  cout.flush();
  #endif
  myPixels = 0;
  for (int i = 0; i < 256; i++) {
    myHistPercent[i] = 0.0;
    myHistCount[i] = 0;
  };
  myMaxValue = myMinValue = 0;
  myMaxCount = myMinCount = 0;
};


Histogram::Histogram(const Histogram& desc) {
  #ifdef DEBUG
  cout << "Histogram::Histogram(Histogram&) called" << endl;
  cout.flush();
  #endif
  myPixels = desc.myPixels;
  for (int i = 0; i < 256; i++) {
    myHistPercent[i] = desc.myHistPercent[i];
    myHistCount[i] = desc.myHistCount[i];
  };
  myMaxValue = desc.myMaxValue;
  myMinValue = desc.myMinValue;
  myMaxCount = desc.myMaxCount;
  myMinCount = desc.myMinCount;
};


Histogram::~Histogram () {
  #ifdef DEBUG
  cout << "Histogram::~Histogram() called ... nothing done in here" << endl;
  cout.flush();
  #endif
  // delete[] myHistPercent;
  // delete[] myHistCount;
};


Histogram& Histogram::operator=(const Histogram& rhs) {
  #ifdef DEBUG
  cout << "Histogram::operator=(const Histogram&) called" << endl;
  cout.flush();
  #endif
  if (this == &rhs) { 
    return *this;
  };
  myPixels = rhs.getPixelcount();
  // delete[] myHistPercent;
  // delete[] myHistCount;
  for (int i = 0; i < 256; i++) {
    myHistPercent[i] = rhs.getValuePercent(i);
    myHistCount[i] = rhs.getValueCount(i);
  };
  myMinValue = rhs.getMinValue();
  myMaxValue = rhs.getMaxValue();  
  myMinCount = rhs.getMinCount();
  myMaxCount = rhs.getMaxCount();  
  return *this;
};


void Histogram::print() const {

  cout << endl;
  cout << "Histogram" << endl;
  cout << "---------" << endl;
  cout << endl;
  cout << "Pixels: " << myPixels << endl;
  cout << "Greylevels from " << myMinValue << " to " << myMaxValue << endl;
  cout << "Counts from " << myMinCount << " to " << myMaxCount << endl;
  cout << endl;

  for (unsigned int i = 0; i < 256; i++) {
    cout << i << ": " << myHistCount[i] << " Pixels, " << myHistPercent[i] << " percent" << endl;
  };
  cout << endl;

};


void Histogram::generate(const Image *img) {
  
  myPixels = 0;
  for (unsigned int x = 0; x < img->getWidth(); x++) {
    for(unsigned int y = 0; y < img->getHeight(); y++) {
      myHistCount[img->getGreyscale(x,y)]++;
      myPixels++;
    };
  };

  myMinValue = (unsigned long) MAXLONG;
  myMaxValue = 0;
  myMinCount = (unsigned long) MAXLONG;
  myMaxCount = 0;

  //for (int i = 200; i < 256; i++) {
  //  myHistCount[i] = 0;
  //};

  for (unsigned int i = 0; i < 256; i++) {

    // Min und Max der x-Achse (Grauwert)
    if (myHistCount[i] > 0) {
      myMaxValue = i;
      myMinValue = (i < myMinValue) ? i : myMinValue;
    };
      
    // Min und Max der y-Achse (Anzahl)
    myMinCount = (myHistCount[i] < myMinCount) ? myHistCount[i] : myMinCount;
    myMaxCount = (myHistCount[i] > myMaxCount) ? myHistCount[i] : myMaxCount;

    myHistPercent[i] = (double) myHistCount[i] / (double) myPixels;
  };
};


void Histogram::generate(const Image *img, Rectangle window) {
  
  // normalize 2 points to upperleft and lowerright
  adaptRect2ullr(window.x1, window.y1, window.x2, window.y2, img->getWidth(), img->getHeight());

  myPixels = 0;
  for (int x = window.x1; x < window.x2; x++) {
    for (int y = window.y1; y < window.y2; y++) {
      myHistCount[img->getGreyscale(x,y)]++;
      myPixels++;
    };
  };

  myMinValue = (unsigned long) MAXLONG;
  myMaxValue = 0;
  myMinCount = (unsigned long) MAXLONG;
  myMaxCount = 0;

  //for (int i = 200; i < 256; i++) {
  //  myHistCount[i] = 0;
  //};

  for (unsigned int i = 0; i < 256; i++) {

    // Min und Max der x-Achse (Grauwert)
    if (myHistCount[i] > 0) {
      myMaxValue = i;
      myMinValue = (i < myMinValue) ? i : myMinValue;
    };
      
    // Min und Max der y-Achse (Anzahl)
    myMinCount = (myHistCount[i] < myMinCount) ? myHistCount[i] : myMinCount;
    myMaxCount = (myHistCount[i] > myMaxCount) ? myHistCount[i] : myMaxCount;

    myHistPercent[i] = (double) myHistCount[i] / (double) myPixels;
  };
};


Histogram::Histogram(const Image *img) {
  myPixels = 0;
  for (int i = 0; i < 256; i++) {
    myHistPercent[i] = 0.0;
    myHistCount[i] = 0;
  };
  generate(img); 
};


unsigned long* Histogram::getHistogramCount() {
  return myHistCount;
};


double* Histogram::getHistogramPercent() {
  return myHistPercent;
};


unsigned long Histogram::getValueCount(const byte pos) const {
  return myHistCount[pos];
};


double Histogram::getValuePercent(const byte pos) const {
  return myHistPercent[pos];
};


unsigned long Histogram::getPixelcount() const {
  return myPixels;
};


unsigned int Histogram::getMaxValue() const {
  return myMaxValue;
};


unsigned int Histogram::getMinValue() const {
  return myMinValue;
};


unsigned int Histogram::getDelta() const {
  return (myMaxValue-myMinValue);
};


unsigned long Histogram::getMaxCount() const {
  return myMaxCount;
};


unsigned long Histogram::getMinCount() const {
  return myMinCount;
};


int Histogram::otsu() const {

  float hist[256];
  for (int i=0; i<256; i++) {
    hist[i] = (float) getValuePercent(i);
  };
  int ngr = 256;
  int jthd;
  float rmean, vari, p0, p1, a0, a1, sep;

  ::thds2_(&hist[0], &ngr, &jthd, &rmean, &vari, &p0, &p1, &a0, &a1, &sep);

  // if ((jthd < 0) || (jthd >= ngr) || (isnan(vari) != 0) || (isinf(vari) != 0)) {
  if ((jthd < 0) || (jthd >= ngr) || (isnan(vari) != 0)) {
    jthd = -1;
  };

  cout << endl;
  if (jthd >= 0) {
    cout << "OTSU: Threshold:  " << jthd << endl;
    cout << "      Rmean,Vari: " << rmean << ", " << vari << endl;
    cout << "      P0, P1:     " << p0 << ", " << p1 << endl;
    cout << "      A0, A1:     " << a0 << ", " << a1 << endl;
    cout << "      Sep:        " << sep << endl;
  } else {
    cout << "OTSU: no valid result." << endl;
  };    
  cout << endl;

  return jthd;

};


int Histogram::otsudetails(float &rmean, float &vari, float &p0, float &p1, \
                           float &a0, float &a1, float &sep) const {

  float hist[256];
  for (int i=0; i<256; i++) {
    hist[i] = (float) getValuePercent(i);
  };
  int ngr = 256;
  int jthd;

  ::thds2_(&hist[0], &ngr, &jthd, &rmean, &vari, &p0, &p1, &a0, &a1, &sep);

  // if ((jthd < 0) || (jthd >= ngr) || (isnan(vari) != 0) || (isinf(vari) != 0)) {
  if ((jthd < 0) || (jthd >= ngr) || (isnan(vari) != 0)) {
    jthd = -1;
  };

  return jthd;

};


unsigned int Histogram::votsu() const {

	int    size = 256;

	int   i, threshold, bm, fm;
	long  s, r, num_pixels, sum_pixels;
	float m1, m2, q1, bgv, max_bgv;

	for (i = 0,num_pixels = sum_pixels = 0; i < size; i++) 
	{
		 num_pixels += myHistCount[i];
		 sum_pixels += myHistCount[i] * i;
	}

	s = myHistCount[0];
	r = 0;
	if (s) 
		 m1 = (float)r / s;
	else 
		 m1 = 0;

	if (num_pixels == s) 
	{
		 threshold = bm = fm = 0;

  #ifdef DEBUG
  cout << "VOTSU: " << (int) threshold << endl;
  cout.flush();
  #endif
		 return  threshold;

	}
	else 
		 m2 = (float)(sum_pixels - r) / (num_pixels - s);

	q1 = (float)s / num_pixels;
	threshold = bm = fm = 0;
	max_bgv = q1 * (1 - q1) * (m1 - m2) * (m1 - m2);
	for (i = 1; i < size; i++) 
	{
		 s += myHistCount[i];
		 r += myHistCount[i] * i;
		 q1 = (float)s / num_pixels;
		 if (s) 
		 {
			  m1 = (float)r / s;
			  if (s == num_pixels) m2 = 0;
			  else m2 = (float)(sum_pixels - r) / (num_pixels - s);
		 }
		 else m1 = 0;

		 bgv = q1 * (1 - q1) * (m1 - m2) * (m1 - m2);
		 if (bgv > max_bgv) 
		 {
			  threshold = i;
			  bm = (int)m1;
			  fm = (int)m2;
			  max_bgv = bgv;
		 }
	}


  #ifdef DEBUG
  cout << "VOTSU: " << (int) threshold << endl;

  cout << "m1: " << m1 << " m2: " << m2 << " q1: " << q1 << " bgv: " << bgv << " max_bgv: " << max_bgv << endl;

  cout.flush();
  #endif


	return threshold;

};


Image* Histogram::createImage() const {
 
  Image *result = new Image24(256,256);
  // result->setWhite();

  float scale = 256.0 / ( (float) (myMaxCount - myMinCount));  

  for (unsigned int x = 0; x < result->getWidth(); x++) {

    byte height;
    if ((myHistCount[x]*scale) > 255) {
      height = 255;
    } else {
      height = (byte) (myHistCount[x]*scale);
    };

    if ( (myHistCount[x] > 0) && (height == 0) ) {
      height = 1;
    };

    // black line
    for (unsigned int y = 0; y < height; y++) {
      result->setGreyscale(x,result->getHeight()-1-y,0);
    };

    // white space
    for (unsigned int y = height; y < result->getHeight(); y++) {
      result->setGreyscale(x,result->getHeight()-1-y,255);
    };
    
  };


  // OTSU rot darstellen
  float rmean, vari, p0, p1, a0, a1, sep;
  int threshold = otsudetails(rmean, vari, p0, p1, a0, a1, sep);
  Color red; red.r = 255; red.g = red.b = 0;
  if (threshold < 0)   { threshold = 0; };
  if (threshold > 255) { threshold = 255; };
  for (unsigned int y = 0; y < result->getHeight(); y++) {
    result->setColor(threshold,y,red);
  };

  return result;

};



unsigned int Histogram::darkPixels(unsigned int maxDarkPixels) const {

  unsigned int result = 0;
  unsigned int pixels = 0;
  
  while ((result < 256) && (pixels < maxDarkPixels)) {
    pixels += myHistCount[result];
    result++;
  };

  return result;

};


// This routine eliminates the count of the White
// Pixels with the grey level 255
void Histogram::eliminateWhitePixels() {

  cout << "[Histogram::eliminateWhitePixels] " << myHistCount[255] << " of " << myPixels << " eliminated." << endl;
  myPixels -= myHistCount[255];
  myHistCount[255] = 0;


  // Adapt Precent Representation and Statistics
  myMinValue = (unsigned long) MAXLONG;
  myMaxValue = 0;
  myMinCount = (unsigned long) MAXLONG;
  myMaxCount = 0;

  for (unsigned int i = 0; i < 256; i++) {

    // Min und Max der x-Achse (Grauwert)
    if (myHistCount[i] > 0) {
      myMaxValue = i;
      myMinValue = (i < myMinValue) ? i : myMinValue;
    };
      
    // Min und Max der y-Achse (Anzahl)
    myMinCount = (myHistCount[i] < myMinCount) ? myHistCount[i] : myMinCount;
    myMaxCount = (myHistCount[i] > myMaxCount) ? myHistCount[i] : myMaxCount;

    myHistPercent[i] = (double) myHistCount[i] / (double) myPixels;
  };

};







