#include "Image24.h"
#include <fstream.h>
#include <iostream.h>
#include "Image8.h"



Image24::Image24 () {
  #ifdef DEBUG
  cout << "Image24::Image24() called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image24", "Image24()", "+", (int) this);

  // smallest Image is 1 by 1 Pixel in black color
  myPlanes = 3;
  myWidth = 1;
  myHeight = 1;
  myImage = new byte[3];
  myImage[0] = myImage[1] = myImage[2] = 0;
};


Image24::~Image24() {
  #ifdef DEBUG
  cout << "Image24::~Image24() called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image24", "~Image24()", "-", id);

  delete[] myImage;
};


Image24::Image24 (unsigned int size_x, unsigned int size_y) {
  #ifdef DEBUG
  cout << "Image24::Image24(unsigned int, unsigned int) called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image24", "Image24(int,int)", "+", (int) this);

  myWidth = size_x,
  myHeight = size_y;
  myPlanes = 3;
  myImage = new byte[myPlanes*myWidth*myHeight];
  // setBlack();
};


Image24::Image24(const Image& desc) {
  #ifdef DEBUG
  cout << "Image24::Image24(Image&) called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image24", "Image24(Image&)", "+", (int) this);

  myWidth = desc.getWidth();
  myHeight = desc.getHeight();
  myPlanes = 3;
  myImage = desc.getImage24();
};


Image& Image24::operator=(const Image& rhs) {
  #ifdef DEBUG
  cout << "Image24::operator=(const Image&) called" << endl;
  cout.flush();
  #endif
  if (this == (Image*) &rhs) { 
    return *this;
  };
  myWidth = rhs.getWidth();
  myHeight = rhs.getHeight();
  myPlanes = 3;
  delete[] myImage;
  myImage = rhs.getImage24();  
  return *this;
};


Image* Image24::copy() const {
  #ifdef DEBUG
  cout << "Image24::copy() called" << endl;
  cout.flush();
  #endif

  // return new Image24(*this);
  Image* result = new Image24(myWidth, myHeight);
  result->adoptImage24(getImage24());

  return result;
};


inline byte Image24::getGreyscale(int x, int y) const {
  Color tmpColor = getColor(x,y);
  return (tmpColor.r + tmpColor.g + tmpColor.b) / 3;
};


inline void Image24::setGreyscale(unsigned int x, unsigned int y, byte grey) {
  if ((x < myWidth) && (y < myHeight)) {
    unsigned int pixelStart = 3*(x + myWidth*y);
    myImage[pixelStart+0] = grey;
    myImage[pixelStart+1] = grey;
    myImage[pixelStart+2] = grey;
  };
};


inline Color Image24::getColor(int x, int y) const {
  Color tmpColor;
  unsigned int pixelStart = 3*(checkX(x) + myWidth*checkY(y));
  tmpColor.r = myImage[pixelStart+0];
  tmpColor.g = myImage[pixelStart+1];
  tmpColor.b = myImage[pixelStart+2];
  return tmpColor;
};


inline void Image24::setColor(unsigned int x, unsigned int y, Color farbe) {
  if ((x < myWidth) && (y < myHeight)) {
    unsigned int pixelStart = 3*(x + myWidth*y);
    myImage[pixelStart+0] = farbe.r;
    myImage[pixelStart+1] = farbe.g;
    myImage[pixelStart+2] = farbe.b;
  };
};


Image24 Image24::operator-(const Image24& bild) const {

  // Image24* res = new Image24(bild.getWidth(), bild.getHeight());
  Image24 res(bild.getWidth(), bild.getHeight());
  Color f;
  int r, g, b;
  for (unsigned int y = 0; y < myHeight; y++) {
    for (unsigned int x = 0; x < myWidth; x++) {
      r = getColor(x,y).r - bild.getColor(x,y).r;
      g = getColor(x,y).g - bild.getColor(x,y).g;
      b = getColor(x,y).b - bild.getColor(x,y).b;
      if (r < 0) { r = 0; };
      if (g < 0) { g = 0; };
      if (b < 0) { b = 0; };
      f.r = r; f.g = g; f.b = b;
      res.setColor(x,y,f);
    };
  };
  return res;
};


Image24 Image24::operator+(const Image24& bild) const {

  // Image24* res = new Image24(bild.getWidth(), bild.getHeight());
  Image24 res(bild.getWidth(), bild.getHeight());
  Color f;
  int r, g, b;
  for (unsigned int y = 0; y < myHeight; y++) {
    for (unsigned int x = 0; x < myWidth; x++) {
      r = (255-getColor(x,y).r) + (255-bild.getColor(x,y).r);
      g = (255-getColor(x,y).g) + (255-bild.getColor(x,y).g);
      b = (255-getColor(x,y).b) + (255-bild.getColor(x,y).b);
      if (r > 255) { r = 255; };
      if (g > 255) { g = 255; };
      if (b > 255) { b = 255; };
      f.r = 255-r; f.g = 255-g; f.b = 255-b;
      res.setColor(x,y,f);
    };
  };
  return res;
};


void Image24::adoptImage8(byte *bild) {
  delete[] myImage;
  myImage = new byte[3*getWidth()*getHeight()];
  for (unsigned int i = 0; i < getWidth()*getHeight(); i++) {
    myImage[(3*i)]   = bild[i];
    myImage[(3*i)+1] = bild[i];
    myImage[(3*i)+2] = bild[i];
  };
  delete[] bild;
};


void Image24::adoptImage24(byte *bild) {
  delete[] myImage;
  myImage = bild;  
};


byte* Image24::getImage8() const {
  byte *result = new byte[getWidth()*getHeight()];
  for (unsigned int i = 0; i < getWidth()*getHeight(); i++) {
    result[i] = (myImage[(3*i)] + myImage[(3*i)+1] + myImage[(3*i)+2]) / 3;
  };
  return result; 
};


byte* Image24::getImage24() const {
  byte *result = new byte[3*getWidth()*getHeight()];
  memcpy(result, myImage, 3*getWidth()*getHeight());  
  return result; 
};


byte* Image24::getImage() const {
  return getImage24(); 
};

