#include "Image8.h"
#include <fstream.h>
#include <iostream.h>
#include "Image24.h"



Image8::Image8 () {
  #ifdef DEBUG
  cout << "Image8::Image8() called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image8", "Image8()", "+", (int) this);

  // smallest Image is 1 by 1 Pixel in black color
  myPlanes = 1;
  myWidth = 1;
  myHeight = 1;
  myImage = new byte[1];
  myImage[0] = 0;
};


Image8::~Image8() {
  #ifdef DEBUG
  cout << "Image8::~Image8() called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image8", "~Image8()", "-", id);

  delete[] myImage;
};


Image8::Image8 (unsigned int size_x, unsigned int size_y) {
  #ifdef DEBUG
  cout << "Image8::Image8(unsigned int, unsigned int) called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image8", "Image8(int,int)", "+", (int) this);

  myWidth = size_x,
  myHeight = size_y;
  myPlanes = 1;
  myImage = new byte[myWidth*myHeight];
  // setBlack();
};


Image8::Image8(const Image& desc) {
  #ifdef DEBUG
  cout << "Image8::Image8(Image&) called" << endl;
  cout.flush();
  #endif

  //writeDebugInformation("Image8", "Image8(Image&)", "+", (int) this);

  myWidth = desc.getWidth();
  myHeight = desc.getHeight();
  myPlanes = 1;
  myImage = desc.getImage8();  
};


Image& Image8::operator=(const Image& rhs) {
  #ifdef DEBUG
  cout << "Image8::operator=(const Image&) called" << endl;
  cout.flush();
  #endif
  if (this == &rhs) { 
    return *this;
  };
  myWidth = rhs.getWidth();
  myHeight = rhs.getHeight();
  myPlanes = 1;
  delete[] myImage;
  myImage = rhs.getImage8();  
  return *this;
};


Image* Image8::copy() const {
  #ifdef DEBUG
  cout << "Image8::copy() called" << endl;
  cout.flush();
  #endif

  // return new Image8(*this);
  Image* result = new Image8(myWidth, myHeight);
  result->adoptImage8(getImage8());

  return result;
};


inline byte Image8::getGreyscale(int x, int y) const {
  return myImage[checkX(x) + myWidth*checkY(y)];
};


inline void Image8::setGreyscale(unsigned int x, unsigned int y, byte grey) {
  if ((x < myWidth) && (y < myHeight)) {
    myImage[x + myWidth*y] = grey;
  };
};


inline Color Image8::getColor(int x, int y) const {
  Color res;
  res.r = res.g = res.b = myImage[checkX(x) + myWidth*checkY(y)];
  return res;
};


inline void Image8::setColor(unsigned int x, unsigned int y, Color farbe) {
  if ((x < myWidth) && (y < myHeight)) {
    byte grey = (farbe.r + farbe.g + farbe.b) / 3;
    myImage[x + myWidth*y] = grey;
  };
};


Image8 Image8::operator-(const Image8& bild) const {

  // Image8* res = new Image8(bild.getWidth(), bild.getHeight());
  Image8 res(bild.getWidth(), bild.getHeight());
  int grey;
  for (unsigned int y = 0; y < myHeight; y++) {
    for (unsigned int x = 0; x < myWidth; x++) {
      grey = getGreyscale(x,y) - bild.getGreyscale(x,y);
      if (grey < 0) { grey = 0; };
      res.setGreyscale(x,y,grey);
    };
  };
  return res;
};


Image8 Image8::operator+(const Image8& bild) const {

  // Image8* res = new Image8(bild.getWidth(), bild.getHeight());
  Image8 res(bild.getWidth(), bild.getHeight());
  int grey;
  for (unsigned int y = 0; y < myHeight; y++) {
    for (unsigned int x = 0; x < myWidth; x++) {
      grey = (255-getGreyscale(x,y)) + (255-bild.getGreyscale(x,y));
      if (grey > 255) { grey = 255; };
      res.setGreyscale(x,y,grey);
    };
  };
  return res;
};


void Image8::adoptImage8(byte *bild) {
  delete[] myImage;
  myImage = bild;  
};


void Image8::adoptImage24(byte *bild) {
  delete[] myImage;
  myImage = new byte[getWidth()*getHeight()];
  for (unsigned int i = 0; i < getWidth()*getHeight(); i++) {
    myImage[i] = (bild[(3*i)] + bild[(3*i)+1] + bild[(3*i)+2]) / 3;
  };
  delete[] bild;
};


byte* Image8::getImage8() const {
  byte *result = new byte[getWidth()*getHeight()];
  memcpy(result, myImage, getWidth()*getHeight());  
  return result; 
};


byte* Image8::getImage24() const {
  byte *result = new byte[3*getWidth()*getHeight()];
  for (unsigned int i = 0; i < getWidth()*getHeight(); i++) {
    result[(3*i)]   = myImage[i];
    result[(3*i)+1] = myImage[i];
    result[(3*i)+2] = myImage[i];
  };
  return result; 
};


byte* Image8::getImage() const {
  return getImage8(); 
};


