#include "ImageSourcePPM.h"
#include "PPMReader.h"
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <stdlib.h>
#include <ctype.h>


ImageSourcePPM::ImageSourcePPM() {

  actualImageNumber = 0;
  firstImageNumber = 1;
  lastImageNumber = 0;
  myWidth = myHeight = 0;

  strcpy(myBasename, "");
  strcpy(myFilename, "");
  strcpy(myDirectory, "");
  strcpy(myExtension, "");
  myLength = 0;

};


void ImageSourcePPM::setParameters(char* directory, char* basename, \
                                   unsigned int length, char* extension = ".ppm") {

  strcpy(myDirectory, directory);
  strcpy(myBasename, basename);
  myLength = length;
  strcpy(myExtension, extension);

  DIR *myDIR;
  dirent *myDirent;
  unsigned int maxIndex = 0;
  unsigned int minIndex = 0;
  unsigned int imageNumber = 0;

  myDIR = opendir(myDirectory);
  myDirent = readdir(myDIR);
 
  char *numberAscii = new char[myLength+1];// ASCII Representation of number
  unsigned int numberValue;
  char *firstPart = new char[strlen(myBasename)+1]; 

  while (myDirent != NULL) {

    strncpy(firstPart, myDirent->d_name, strlen(myBasename));
    firstPart[strlen(myBasename)] = '\0';
    if (strcmp(firstPart, myBasename) == 0) {

      char *numberStart = myDirent->d_name;
      numberStart += strlen(myBasename);

      strncpy(numberAscii, numberStart, myLength);
      numberAscii[myLength] = '\0';

      unsigned int isAlpha = 0;
      for (unsigned int i = 0; i < myLength; i++) {
        isAlpha += isalpha(numberAscii[i]);
      };
      if (isAlpha == 0) {

        numberValue = atoi(numberAscii);
/*
        #ifdef DEBUG
        cout << " ... " << numberValue << " ... " ;
        #endif  
*/
        if (imageNumber > 0) {
          imageNumber++;
          if (numberValue > maxIndex) {
            maxIndex = numberValue;
  	  };
          if (numberValue < minIndex) {
            minIndex = numberValue;
	  };
        } else {
          imageNumber = 1;
          maxIndex = minIndex = numberValue;
        };

      };

    };
/*
    #ifdef DEBUG
    cout << myDirent->d_ino << ": " << myDirent->d_name << endl;
    #endif
*/    
    myDirent = readdir(myDIR);

  };
  
  #ifdef DEBUG
  cout << "Number of Images: " << imageNumber << endl;
  cout << "Image Indexes from " << minIndex << " to " << maxIndex << endl;
  #endif

  if ((maxIndex-minIndex) == imageNumber - 1) {
 
    firstImageNumber = minIndex;
    lastImageNumber = maxIndex;

    Image* tmpImg;
    tmpImg = getImageAbsolute(getFirstIndex());
    myWidth = tmpImg->getWidth();
    myHeight = tmpImg->getHeight();
    delete tmpImg;

  } else {

    cout << "[ImageSourcePPM::setParameters] ERROR: Some of the frames are missing ... !" << endl;
    firstImageNumber = 1;
    lastImageNumber = 0;
    myWidth = 0;
    myHeight = 0;

  };

};


Image* ImageSourcePPM::getNextImage(int step = 1) {
  movePosition(step);
  return getImageRelative();
};


void ImageSourcePPM::setPosition(unsigned int pos) {
  if (pos < firstImageNumber) {
    actualImageNumber = firstImageNumber;
  } else {
    if (pos > lastImageNumber) {
      actualImageNumber = lastImageNumber;
    } else {
      actualImageNumber = pos;
    };
  };
};


unsigned int ImageSourcePPM::getPosition() {
  return actualImageNumber;
};


Image* ImageSourcePPM::getImageAbsolute(unsigned int pos) {

  unsigned int position;
  if (pos < firstImageNumber) {
    position = firstImageNumber;
  } else {
    if (pos > lastImageNumber) {
      position = lastImageNumber;
    } else {
      position = pos;
    };
  };

  PPMReader myPPMReader;

  // generate actual filename
  strcpy(myFilename, myDirectory);
  strcat(myFilename, myBasename);

  char number[myLength+1];               
  sprintf(number, "%.*d", (int) myLength, position);
  
  strcat(myFilename, number);
  strcat(myFilename, myExtension);

  cout << "Filename: " << myFilename << endl;

  return myPPMReader.apply(myFilename);

};


Image* ImageSourcePPM::getImageRelative(int index) {
  return getImageAbsolute(actualImageNumber + index);
};


void ImageSourcePPM::movePosition(int index = 1) {
  setPosition(actualImageNumber + index);
};

