#include "Interpolator.h"
#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <vector.h>
#include <stdlib.h>


Interpolator::Interpolator(const unsigned int maxInterpolate, \
                           const unsigned int maxDistance, \
                           const unsigned int maxGaps) {

  myMaxDistance = maxDistance;
  myMaxInterpolate = maxInterpolate;
  myMaxGaps = maxGaps;
  myVerboseMode = false;
  myMaskImage = NULL;

};

void Interpolator::setParameters(const unsigned int maxInterpolate, \
                                 const unsigned int maxDistance, \
                                 const unsigned int maxGaps) {

  myMaxDistance = maxDistance;
  myMaxInterpolate = maxInterpolate;
  myMaxGaps = maxGaps;
  myVerboseMode = false;

};

void Interpolator::setMaskImage(const Image* tmpMaskImage) {

  if (myMaskImage != NULL) {
    delete myMaskImage;
    myMaskImage = NULL;
  };

  myMaskImage = tmpMaskImage->copy();

};

void Interpolator::setVerboseMode(const int verboseMode) {
  myVerboseMode = verboseMode;
};

vector<Point> Interpolator::apply(const vector<Point> punkte) const {

  vector<Point> result(0);

  unsigned int pos1 = 0;
  unsigned int pos2 = 0;

  // 1. gueltigen Punkt suchen (vor Ende der Liste)
  while ((!punkte[pos1].valid()) && (pos1 < punkte.size())) {
    result.insert(result.end(), punkte[pos1]);
    pos1++;
    pos2++;
  };

  if (myVerboseMode != false) {
    cout << "1. Punkt bei " << pos1 << " (" << punkte[pos1].x << "/" << punkte[pos1].y << ")" << endl;
  };

  // Schleife solange pos2 <= punkte.size()
  while (pos2 < punkte.size()) {

    // pos1 fix lassen -> pos2 weiterschieben auf naechsten gueltigen Punkt
    pos2++;
    while ((!punkte[pos2].valid()) && (pos2 < punkte.size())) {
      pos2++;
    };


    if (myVerboseMode != false) {
      cout << "Verbindung von " << setw(3) << pos1 << " nach " << setw(3) << pos2 << "    (" << punkte[pos1].x << "/" << punkte[pos1].y << ") - (" << punkte[pos2].x << "/" << punkte[pos2].y << ")  Distance " << setw(8) << dist(punkte[pos1], punkte[pos2]);
    };
    
    // Verbindung von pos1 nach pos2 preufen
    if ((pos2-pos1) == 1) {
      // Direkte Verbindung ohne Interpolation
      if (dist(punkte[pos1],punkte[pos2]) <= myMaxDistance) {
        // Benachbarte Punkte verbinden -> so in Array belassen
        result.insert(result.end(), punkte[pos1]);

        if (myVerboseMode != false) {
          cout << "  connected" << endl;
	};

      } else {
        // keine Verbindung -> neuen ungueltigen Punkt einfuegen
        result.insert(result.end(), punkte[pos1]);
        result.insert(result.end(), Point());        

        if (myVerboseMode != false) {
	  cout << "  too far -> cut" << endl;
	};

      };
 
    } else if ((pos2-pos1) <= myMaxGaps) {
      // Gaps ev. Interpolieren
      if (dist(punkte[pos1],punkte[pos2]) <= myMaxInterpolate) {
        // Punkte interpolieren
        result.insert(result.end(), punkte[pos1]);
        double dx = double (punkte[pos2].x-punkte[pos1].x) / double(pos2-pos1);
        double dy = double (punkte[pos2].y-punkte[pos1].y) / double(pos2-pos1);
        double x = punkte[pos1].x;
        double y = punkte[pos1].y;
 
        #ifdef DEBUG
	cout << endl << "Interpolation von (" << punkte[pos1].x << "/" \
             << punkte[pos1].y << ") nach (" << punkte[pos2].x << "/" \
             << punkte[pos2].y << ") in " << (pos2-pos1-1) << " Schritten" << endl;
        cout << "dx = " << dx << ", dy = " << dy \
             << ", x = " << x << ", y = " << y << endl;
        #endif

        for (unsigned int j = 1; j < pos2-pos1; j++) {
          unsigned int xx = (unsigned int) (x+j*dx+0.5);  // add 0.5 to round
          unsigned int yy = (unsigned int) (y+j*dy+0.5);  // using trunc func.
          result.insert(result.end(), Point(xx,yy));
          #ifdef DEBUG
	  cout << j << ": " << result[result.size()-1].x \
               << "/" << result[result.size()-1].y << endl;
          #endif

	};

        if (myVerboseMode != false) {
	  cout << "  interpolated" << endl;
	};

      } else {
        // keine Verbindung -> Punkte 1:1 kopieren
        result.insert(result.end(), punkte[pos1]);
        for (unsigned int j = 1; j < pos2-pos1; j++) {
          result.insert(result.end(), Point());        
	};

        if (myVerboseMode != false) {
	  cout << "  too far to interpolate" << endl;
	};

      };

    } else {
      // zuviele Gaps -> Punkte belassen
      result.insert(result.end(), punkte[pos1]);
      for (unsigned int j = 1; j < pos2-pos1; j++) {
        result.insert(result.end(), Point());        
      };

      if (myVerboseMode != false) {
        cout << "  too many gaps" << endl;
      };

    };

  
    // neu ab pos2 pruefen
    pos1 = pos2;

  };
  result.insert(result.end(), Point());

  if (myVerboseMode != false) {    
    cout << "Input: " << punkte.size() << " points, Output: " << result.size() << " points." << endl;
  };

  return result;

};



inline double Interpolator::dist(const Point p1, const Point p2) const {

  return sqrt( (p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y) );

};











