#include "Point2D.h"
#include "Edge.h"
#include <math.h>


Point2D::Point2D(double _x, double _y) : x(_x), y(_y) { };

Point2D Point2D::operator+(const Point2D &p) {
  return Point2D(x+p.x, y+p.y);
};

Point2D Point2D::operator-(const Point2D &p) {
  return Point2D(x-p.x, y-p.y);
};

Point2D operator*(const double s, const Point2D &p) {
  return Point2D(s*p.x, s*p.y);
};

double Point2D::operator[](int i) {
  return (i == 0) ? x : y;
};

int Point2D::operator==(const Point2D &p) {
  return (x == p.x) && (y == p.y);
};

int Point2D::operator!=(const Point2D &p) {
  return !(*this == p);
};

int Point2D::operator<(const Point2D &p) {
  return ((x < p.x) || ((x == p.x) && (y < p.y)));
};

int Point2D::operator>(const Point2D &p) {
  return ((x > p.x) || ((x == p.x) && (y > p.y)));
};

int orientation(Point2D &p0, Point2D &p1, Point2D &p2) {
  Point2D a = p1 - p0;
  Point2D b = p2 - p0;
  double sa = a.x * b.y - b.x * a.y;
  if (sa > 0.0) return 1;
  if (sa < 0.0) return -1;
  return 0;
};

int Point2D::classify(Point2D &p0, Point2D &p1) {
  Point2D p2 = *this;
  Point2D a = p1 - p0;
  Point2D b = p2 - p0;
  double sa = a.x * b.y - b.x * a.y;
  if (sa > 0.0) return LEFT;
  if (sa < 0.0) return RIGHT;
  if ((a.x * b.x < 0.0) || (a.y * b.y < 0.0)) return BEHIND;
  if (a.length() < b.length()) return BEYOND;
  if (p0 == p2) return ORIGIN;
  if (p1 == p2) return DESTINATION;
  return BETWEEN;
};

int Point2D::classify(Edge &e) {
  return classify(e.org, e.dest);
};

double Point2D::polarAngle() {
  if ((x == 0) && (y == 0)) return -1.0;
  if (x == 0.0) return ((y > 0.0) ? 90 : 270);
  double theta = atan(y/x);          // in radians
  theta *= 360 / (2 * 3.1415926);    // converted to degrees
  if (x > 0.0)                       // quadrants 1 and 4
    return ((y >= 0.0) ? theta : 360 + theta);
  else                               // quadrants 2 and 3
    return (180 + theta);
};

double Point2D::length() {
  return sqrt(x*x + y*y);
};







