#include "Polygon.h"
#include <stdlib.h>



Polygon::Polygon() : _v(NULL), _size(0) { };

Polygon::Polygon(Polygon &p) {
  _size = p._size;
  if (_size == 0) 
    _v = NULL;
  else {
    _v = new Vertex(p.point());
    for (int i = 1; i < _size; i++) {
      p.advance(Vertex::CLOCKWISE);
      _v = _v->insert(new Vertex(p.point()));
    };
    p.advance(Vertex::CLOCKWISE);
    _v = _v->cw();
  };
};

Polygon::Polygon(Vertex *v) : _v(v) {
  resize();
};

void Polygon::resize() {
  if (_v == NULL) _size = 0;
  else {
    Vertex *v = _v->cw();
    for (_size = 1; v != _v; ++_size, v = v->cw())
      ;
  };
};

Polygon::~Polygon() {
  if (_v) {
    Vertex *w = _v->cw();
    while (_v != w) {
      delete w->remove();
      w = _v->cw();
    };
  delete _v;
  };
};

Vertex *Polygon::v() {
  return _v;
};

int Polygon::size() {
  return _size;
};

Point2D Polygon::point() {
  return _v->point();
};

Edge Polygon::edge() {
  return Edge(Point2D(), _v->cw()->point());
};

Vertex *Polygon::cw() {
  return _v->cw();
};

Vertex *Polygon::ccw() {
  return _v->ccw();
};

Vertex *Polygon::neighbor(int rotation) {
  return _v->neighbor(rotation);
};

Vertex *Polygon::advance(int rotation) {
  return _v = _v->neighbor(rotation);
};

Vertex *Polygon::setV(Vertex *v) {
  return _v = v;
};

Vertex *Polygon::insert(Point2D &p) {
  if (_size++ == 0) 
    _v = new Vertex(p);
  else 
    _v = _v->insert(new Vertex(p));
  return _v;
};

void Polygon::remove() {
  Vertex *v = _v;
  _v = (--_size == 0) ? NULL : _v->ccw();
  delete v->remove();
};

Polygon *Polygon::split(Vertex *b) {
  Vertex *bp = _v->split(b);
  resize();
  return new Polygon(bp);
};

bool Point2DInConvexPolygon(Point2D &s, Polygon &p) {
  if (p.size() == 1) return (s == p.point());
  if (p.size() == 2) {
    int c = s.classify(p.edge());
    return ((c==Point2D::BETWEEN) || (c==Point2D::ORIGIN) || (c==Point2D::DESTINATION));
  };
  Vertex *org = p.v();
  for (int i = 0; i < p.size(); i++, p.advance(Vertex::CLOCKWISE))
    if (s.classify(p.edge()) == Point2D::LEFT) {
      p.setV(org);
      return false;
    };
  return true;
};

