#ifndef _RANDOMIZEDNODE_H_
#define _RANDOMIZEDNODE_H_

#include "BraidedNode.h"
#include <stdlib.h>


template<class T> class RandomizedNode : public BraidedNode<T> {
  protected:
    RandomizedNode *_parent;
    double _priority;
    void rotateRight();
    void rotateLeft();
    void bubbleUp();
    void bubbleDown();
  public:
    RandomizedNode(T v, int seed = -1);
    RandomizedNode *lchild();
    RandomizedNode *rchild();
    RandomizedNode *next();
    RandomizedNode *prev();
    RandomizedNode *parent();
    double priority();
    friend class RandomizedSearchTree<T>;
};    

template<class T> RandomizedNode<T>::RandomizedNode(T v, int seed) : BraidedNode<T>(v) {
  if (seed != 1) srand(seed);
  _priority = (rand() % 32767) / 32767.0;
  _parent = NULL;
};

template<class T> void RandomizedNode<T>::rotateRight() {
  RandomizedNode<T> *y = this;
  RandomizedNode<T> *x = y->lchild();
  RandomizedNode<T> *p = y->parent();
  y->_lchild = x->rchild();
  if (y->lchild() != NULL) y->lchild()->_parent = y;
  if (p->rchild() == y) p->_rchild = x;
  else p->_lchild = x;
  x->_parent = p;
  x->_rchild = y;
  y->_parent = x;
};

template<class T> void RandomizedNode<T>::rotateLeft() {
  RandomizedNode<T> *x = this;
  RandomizedNode<T> *y = x->rchild();
  RandomizedNode<T> *p = x->parent();
  x->_rchild = y->lchild();
  if (x->rchild() != NULL) x->rchild()->_parent = x;
  if (p->lchild() == x) p->_lchild = y;
  else p->_rchild = y;
  y->_parent = p;
  y->_lchild = x;
  x->_parent = y;
};

template<class T> void RandomizedNode<T>::bubbleUp() {
  RandomizedNode<T> *p = parent();
  if (priority() < p->priority()) {
    if (p->lchild() == this) p->rotateRight();
    else p->rotateLeft();
    bubbleUp();
  };
};

template<class T> void RandomizedNode<T>::bubbleDown() {
  float lcPriority = lchild() ? lchild()->priority() : 2.0;
  float rcPriority = rchild() ? rchild()->priority() : 2.0;
  float minPriority = (lcPriority<rcPriority) ? lcPriority : rcPriority;
  if (priority() <= minPriority) return;
  if (lcPriority < rcPriority) rotateRight();
  else rotateLeft();
  bubbleDown();
};

template<class T> RandomizedNode<T> *RandomizedNode<T>::rchild() {
  return (RandomizedNode<T>*)_rchild;
};

template<class T> RandomizedNode<T> *RandomizedNode<T>::lchild() {
  return (RandomizedNode<T>*)_lchild;
};

template<class T> RandomizedNode<T> *RandomizedNode<T>::next() {
  return (RandomizedNode<T>*)_next;
};

template<class T> RandomizedNode<T> *RandomizedNode<T>::prev() {
  return (RandomizedNode<T>*)_prev;
};

template<class T> RandomizedNode<T> *RandomizedNode<T>::parent() {
  return (RandomizedNode<T>*)_parent;
};

template<class T> double RandomizedNode<T>::priority() {
  return _priority;
};


#endif // _RANDOMIZEDNODE_H_
