#include "Rectangle.h"


Rectangle::Rectangle() {
  x1 = y1 = x2 = y2 = -1;
};


Rectangle::Rectangle(unsigned int _x1, unsigned int _y1, 
                     unsigned int _x2, unsigned int _y2) {
  x1 = _x1;
  y1 = _y1;
  x2 = _x2;
  y2 = _y2;
  correctRectangle();
};


Rectangle::Rectangle(const Point p1, const Point p2) {
  x1 = p1.x;
  y1 = p1.y;
  x2 = p2.x;
  y2 = p2.y;
  correctRectangle();
};


Rectangle::Rectangle(const Point c, const unsigned int width, \
                     const unsigned int height) {
  x1 = c.x - width/2;
  y1 = c.y - height/2;
  x2 = c.x + width/2;
  y2 = c.y + height/2;
  correctRectangle();
};


Rectangle::Rectangle(const Rectangle& desc) {
  x1 = desc.x1;
  y1 = desc.y1;
  x2 = desc.x2;
  y2 = desc.y2;
  correctRectangle();
};


Rectangle& Rectangle::operator=(const Rectangle& rhs) {
  if (this == &rhs) { 
    return *this;
  };
  x1 = rhs.x1;
  y1 = rhs.y1;
  x2 = rhs.x2;
  y2 = rhs.y2;
  correctRectangle();
  return *this;
};


int Rectangle::operator==(const Rectangle& rhs) {
  return ((x1 == rhs.x1) && (y1 == rhs.y1) && (x2 == rhs.x2) && (y2 == rhs.y2));
};


int Rectangle::valid() {
  return ((x1 >= 0) && (y1 >= 0) && (x2 >= 0) && (y2 >= 0));
};


int Rectangle::getWidth() const {
  return (x2-x1+1);
};


int Rectangle::getHeight() const {
  return (y2-y1+1);
};


int Rectangle::getArea() const {
  return (getWidth() * getHeight());
};

void Rectangle::correctRectangle() {

  // adapt that (x1/x2) is upperleft
  // and that (x2/y2) is lowerleft

  Point upperleft, lowerright;

  upperleft.x = (x1 < x2) ? x1 : x2;
  upperleft.y = (y1 < y2) ? y1 : y2;
  lowerright.x = (x1 > x2) ? x1 : x2;
  lowerright.y = (y1 > y2) ? y1 : y2;

  if (upperleft.x < 0)  { x1 = 0; } 
                  else  { x1 = upperleft.x; };
  if (upperleft.y < 0)  { y1 = 0; } 
                  else  { y1 = upperleft.y; };
  if (lowerright.x < 0) { x2 = 0; } 
                   else { x2 = lowerright.x; };
  if (lowerright.y < 0) { y2 = 0; } 
                   else { y2 = lowerright.y; };

  // set new coordinates
  x1 = upperleft.x; 
  y1 = upperleft.y;
  x2 = lowerright.x;
  y2 = lowerright.y;

};







