/*
 * Report.C
 *
 * Copyright (C) 1994, Christoph Streit <streit@iam.unibe.ch>
 *                     Stephan Amann <amann@iam.unibe.ch>
 *                     University of Berne, Switzerland
 *
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 * Adapted from Craig Kolbs rayshade.
 *
 * ---------------------------------------------------------------------
 *  $Id: Report.C,v 1.7 1995/10/20 16:45:02 streit Exp $
 * ---------------------------------------------------------------------
 */

#include <stdlib.h>
#include <iostream.h>

#include "RCString.h"
#include "Report.h"
#include "Configuration.h"
#include "StreamOption.h"

//___________________________________________________________________ Report

ostream* Report::ourErrorStream = NULL;
int Report::ourVerboseFlag = 1;
int Report::ourDebugFlag   = 1;

void Report::debug(const RCString& msg)
{
  if (ourErrorStream == NULL)
    setErrorStream();

  if (ourDebugFlag)
    *ourErrorStream << "Debug message: " << msg << ".\n" << flush;
}

void Report::hint(const RCString& msg)
{
  if (ourErrorStream == NULL)
    setErrorStream();

  if (ourVerboseFlag) 
    *ourErrorStream << "Hint: " << msg << ".\n" << flush;
}

void Report::warning(const RCString& msg)
{
  if (ourErrorStream == NULL)
    setErrorStream();

  if (ourDebugFlag)  
    *ourErrorStream << "Warning: " << msg << ".\n" << flush;
}

void Report::recoverable(const RCString& msg)
{ 
  if (ourErrorStream == NULL)
    setErrorStream();

  *ourErrorStream << "Recoverable Error: " << msg << "!\n" << flush;
}

void Report::error(const RCString& msg)
{
  if (ourErrorStream == NULL)
    setErrorStream();

  *ourErrorStream << "Fatal Error: " << msg << "!\n" << flush;
  exit(1);
}

void Report::setVerbose(int verboseFlag)
{
  ourVerboseFlag = verboseFlag;
}

void Report::setDebug(int debugFlag)
{
  ourDebugFlag = debugFlag;
}

void Report::setErrorStream()
{
  getConfigurationOption(StreamOption, strOption, "Report.ErrorStream");

  if (strOption != NULL)
    ourErrorStream = (ostream *)strOption->getStream();

  if (ourErrorStream == NULL)
    ourErrorStream = &cerr;
}

