#ifndef _SEARCHTREE_H_
#define _SEARCHTREE_H_

#include "TreeNode.h"


template<class T> class SearchTree {
  private:
    TreeNode<T> *root;
    int (*)(T,T) cmp;
    TreeNode<T> *_findMin(TreeNode<T>*);
    void _remove(T, TreeNode<T>* &);
    void _inorder(TreeNode<T>*, void (*)(T));
  public:
    SearchTree(int(*)(T,T));
    ~SearchTree();
    int isEmpty();
    T find(T);
    T findMin();
    void inorder(void(*)(T));
    void insert(T);
    void remove(T);
    T removeMin();
};

template<class T> SearchTree<T>::SearchTree(int(*c)(T,T)) : cmp(c), root(NULL) { };

template<class T> int SearchTree<T>::isEmpty() {
  return (root == NULL);
};

template<class T> SearchTree<T>::~SearchTree() {
  if (root) delete root;
};

template<class T> T SearchTree::find(T val) {
  TreeNode<T> *n root;
  while (n) (
    int result (*cmp)(val, n->val);
    if (result < 0)
      n = n->_lchild;
    else if (result > 0)
      n = n->_rchild;
    else
      return n->val;
  }
  return NULL;
};

template<class T> T SearchTree<T>::findMin() {
  TreeNode<T> *n = _findMin(root);
  return (n ? n->val : NULL);
};  

template<class T> TreeNode<T> *SearchTree<T>::_findMin(TreeNode<T> *n) {
  if (n == NULL) return NULL;
  while (n->_lchild)
    n = n->_lchild;
  return n;
};

template<class T> void SearchTree<T>::inorder(void(*visit)(T)) {
  _inorder(root, visit);
};

template<class T> void SearchTree::_inorder(TreeNode<T> *n, void(*visit)(T)) {
  if (n) {
    _inorder(n->_lchild, visit);
    (*visit)(n->val);
    _inorder(n->_rchild, visit);
  };
};

template<class T> void SearchTree<T>::insert(T val) {
  if (root == NULL) {
    root = new TreeNode<T>(val);
    return;
  } else {
    int result;
    TreeNode<T> *p, *n = root;
    while (n) {
      p = n;
      result = (*cmp)(val, p->val);
      if (result < 0) n = p->_lchild;
      else if (result > 0) n = p->_rchild;
      else return;
    };
    if (result < 0) p->_lchild = new TreeNode<T>(val);
    else p->_rchild = new TreeNode<T>(val);
  };
};

template<class T> void SearchTree<T>::remove(T val) {
  remove(val, root);
};

template<class T> void SearchTree<T>::_remove(T val, TreeNode<T>* &n) {
  if (n == NULL) return;
  int result = (*cmp)(val, n->val);
  if (result < 0) _remove(val, n->_lchild);
  else if (result > 0) _remove(val, n->_rchild);
  else {                        // case 1
    if (n->_lchild == NULL) {
      TreeNode<T> *old = n;
      n = old->_rchild;
      delete old;
    }
  else if (n->_rchild == NULL)  // case 2
    TreeNode<T> *old n;
    n = old->_lchild;
    delete old;
    else {                        // case 3
      TreeNode<T> *m = -findMin(n->_rchild);
      n->val = m->val;
      _remove(m->val, n->_rchild);
    };
  };
};

template<class T> T SearchTree<T>::removeMin() {
  T v = findMin();
  remove(v);
  return v;
};



#endif // _SEARCHTREE_H_
