#include "SlidingWindow.h"
#include <iostream.h>
#include "Filter.h"

#include "Histogram.h"
#include <vector.h>
#include <math.h>
#include <values.h>



// DEBUG
#include "PPMWriter.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "FilterCutRectangle.h"
#include "FilterExpandGreyscale.h"
#include <fstream.h>
#include "FilterBinarize.h"


#include <time.h>
void writeDebugInformation(char *tmpClass, char *tmpMethod, char *tmpType, int tmpPointer) {

  ofstream deb;
  deb.open("memdebug.txt", ios::app);

  // create timestamp
  time_t seconds;
  seconds = time(&seconds);
  tm* currentTime;
  currentTime = localtime(&seconds);

  // deb << "19" << currentTime->tm_year << "-" << (currentTime->tm_mon + 1) << "-" << currentTime->tm_mday << " ";
  // deb << currentTime->tm_hour << ":" << currentTime->tm_min << ":" << currentTime->tm_sec << "|";
  deb << tmpClass << "|" << tmpMethod << "|" << tmpType << "|" << tmpPointer << endl;
  deb.close();

};




SlidingWindow::SlidingWindow() {

  for (int i = 0; i < MAXLOOKAHEAD+2; i++) {
    myImages[i] = NULL;
  };
  myDeltaImage = NULL;
  myImageSource = NULL;
  myDetector = NULL;
  myLookahead = -1;
  myPosition = -1;
  for (int i = 0; i < myDeltaMergeWaitCount; i++) {
    myDeltaMergeWait[i] = NULL;
  };
  myThresholdType = ThresholdNone;
  myThreshold = -1;
  myVarianceLimit = -1;
  strcpy(myReportingDirectory, "./");
  myReportImages = false;
  myReportDeltas = false;
  myReportDeltaMerge = false;
  myAreaLimit = 1.0;
  myActualVariance = -1.0;
};


SlidingWindow::SlidingWindow(unsigned int Lookahead) {

  if (Lookahead > MAXLOOKAHEAD) {
    myLookahead = MAXLOOKAHEAD;
  } else {
    myLookahead = Lookahead;
  };

  for (int i = 0; i < MAXLOOKAHEAD+2; i++) {
    myImages[i] = NULL;
  };
  myDeltaImage = NULL;
  myImageSource = NULL;
  myDetector = NULL;
  myPosition = -1;
  for (int i = 0; i < myDeltaMergeWaitCount; i++) {
    myDeltaMergeWait[i] = NULL;
  };
  myThresholdType = ThresholdNone;
  myThreshold = -1;
  myVarianceLimit = -1;
  strcpy(myReportingDirectory, "./");
  myReportImages = false;
  myReportDeltas = false;
  myReportDeltaMerge = false;
  myAreaLimit = 1.0;
  myActualVariance = -1.0;
};


SlidingWindow::~SlidingWindow() {
  for (int i = 0; i < MAXLOOKAHEAD+2; i++) {
    if (myImages[i] != NULL) {
      delete myImages[i];
    };
  };
  if (myDeltaImage != NULL) {
    delete myDeltaImage;
  };
  for (int i = 0; i < myDeltaMergeWaitCount; i++) {
    if (myDeltaMergeWait[i] != NULL) {
      delete myDeltaMergeWait[i];
    };
  };
};


void SlidingWindow::setBoundingBox(const Rectangle myBounds) {
  myBoundingBox = myBounds;
  if (myImageSource != NULL) {
    if (myBoundingBox.x2 >= (int) myImageSource->getImageWidth()) {
      myBoundingBox.x2 = myImageSource->getImageWidth()-1;
    };
    if (myBoundingBox.y2 >= (int) myImageSource->getImageHeight()) {
      myBoundingBox.y2 = myImageSource->getImageHeight()-1;
    };
  };
};


void SlidingWindow::setImageSource(ImageSource* tmpSource) {
  if (myImageSource != NULL) {
    delete myImageSource;
  };
  myImageSource = tmpSource;
  for (int i = 0; i < myDeltaMergeWaitCount; i++) {
    if (myDeltaMergeWait[i] != NULL) {
      delete myDeltaMergeWait[i];
    };
    myDeltaMergeWait[i] = new Image8(myImageSource->getImageWidth(), \
                                     myImageSource->getImageHeight());
    myDeltaMergeWait[i]->setWhite();
  };
  if (myBoundingBox.valid()) {
    if (myBoundingBox.x2 >= (int) myImageSource->getImageWidth()) {
      myBoundingBox.x2 = myImageSource->getImageWidth()-1;
    };
    if (myBoundingBox.y2 >= (int) myImageSource->getImageHeight()) {
      myBoundingBox.y2 = myImageSource->getImageHeight()-1;
    };
  } else {
    myBoundingBox.x1 = myBoundingBox.y1 = 0;
    myBoundingBox.x2 = myImageSource->getImageWidth()-1;
    myBoundingBox.y2 = myImageSource->getImageHeight()-1;
  };
};


void SlidingWindow::setDetector(Detector* tmpDetector) {
  if (myDetector != NULL) {
    delete myDetector;
  };
  myDetector = tmpDetector;
};

  
void SlidingWindow::addPreprocessFilter(FilterStrategy* tmpFilter) {
  myPreprocessing.push_back(tmpFilter);
};

  
void SlidingWindow::addPostprocessFilter(FilterStrategy* tmpFilter) {
  myPostprocessing.push_back(tmpFilter);
};
  

void SlidingWindow::setLookahead(unsigned int lookahead) {

  for (int i = 0; i < MAXLOOKAHEAD+2; i++) {
    if (myImages[i] != NULL) {
      delete myImages[i];
      myImages[i] = NULL;
    };
  };
  // myDeltaImage.setWhite();

  if (lookahead > MAXLOOKAHEAD) {
    myLookahead = MAXLOOKAHEAD;
  } else {
    myLookahead = lookahead;
  };

  generateImages();

};

  
int SlidingWindow::valid() {

  return ((myPosition >= 0) && (myLookahead >= 0) && \
          (myImageSource != NULL) && (myDetector != NULL));                 

};


unsigned int SlidingWindow::getPosition() {
  return (unsigned int) myPosition;
};


unsigned int SlidingWindow::getLookahead() {
  return (unsigned int) myLookahead;
};

 
void SlidingWindow::moveFirst() {
  myPosition = myImageSource->getFirstIndex()+1;
  generateImages();
};

  
void SlidingWindow::moveLast() {
  myPosition = myImageSource->getLastIndex()-myLookahead;
  generateImages();
};

  
void SlidingWindow::moveNext() {
  myPosition++;
  if (myPosition > (int) myImageSource->getLastIndex()-myLookahead) {
    myPosition = myImageSource->getLastIndex()-myLookahead;
    generateImages();
  } else {
    // optimized step foreward
    myImageSource->setPosition(myPosition);
    delete myImages[0];
    for (int i=0; i < myLookahead+1; i++) {
      myImages[i] = myImages[i+1];
    };
    myImages[myLookahead+1] = myImageSource->getImageRelative(myLookahead);
    applyPreprocessing(myLookahead+1);
    // myDeltaImage.setWhite();
  };
};

  
void SlidingWindow::movePrevious() {
  myPosition--;
  if (myPosition < (int) myImageSource->getFirstIndex()+1) {
    myPosition = myImageSource->getFirstIndex()+1;
    generateImages();
  } else {
    // optimized step back
    myImageSource->setPosition(myPosition);
    delete myImages[myLookahead+1];
    for (int i=myLookahead+1; i > 0; i--) {
      myImages[i] = myImages[i-1];
    };
    myImages[0] = myImageSource->getImageRelative(-1);
    applyPreprocessing(0);
    // myDeltaImage.setWhite();
  };
};

  
void SlidingWindow::move(int step) {

  myPosition += step;
  if (myPosition < (int) myImageSource->getFirstIndex()+1) {
    myPosition = myImageSource->getFirstIndex()+1;
  };
  if (myPosition > (int) myImageSource->getLastIndex()-myLookahead) {
    myPosition = myImageSource->getLastIndex()-myLookahead;
  };
  generateImages();

};



// routines for convex hull with grahamscan
void swap(Point a[], int i, int j) {
  Point t = a[i];
  a[i] = a[j];
  a[j] = t;
};
float theta(Point p1, Point p2) {
  int dx, dy, ax, ay;
  float t;
  dx = p2.x - p1.x; ax = abs(dx);
  dy = p2.y - p1.y; ay = abs(dy);
  t = (ax+ay == 0) ? 0 : (float) dy/(ax+ay);
  if (dx < 0) t = 2-t; else if (dy < 0) t = 4+t;
  return t*90.0;
};
void shellsort(Point a[], int N) {
  int i, j, h; Point v;
  for (h = 1; h <= N/9; h = 3*h+1) ;
  for ( ; h > 0; h /= 3)
    for (i = h+1; i <= N; i += 1) {
      v = a[i]; j = i;
      while (j>h && theta(a[1],a[j-h])>theta(a[1],v)) {
        a[j] = a[j-h];
        j -= h;
      };
      a[j] = v;
    };
};
int ccw(Point p0, Point p1, Point p2) {
  int dx1, dx2, dy1, dy2;
  dx1 = p1.x - p0.x; dy1 = p1.y - p0.y;
  dx2 = p2.x - p0.x; dy2 = p2.y - p0.y;
  if (dx1*dy2 > dy1*dx2) return +1;
  if (dx1*dy2 < dy1*dx2) return -1;
  if ((dx1*dx2 < 0) || (dy1*dy2 < 0)) return -1;
  if ((dx1*dx1+dy1*dy1) < (dx2*dx2+dy2*dy2)) return +1;
  return 0;
};
int grahamscan(Point p[], int N) {
  int i, min, M;
  for (min = 1, i = 2; i <= N; i++) 
    if (p[i].y < p[min].y) min = i;
  for (i = 1; i <= N; i++)
    if (p[i].y == p[min].y)
      if (p[i].x > p[min].x) min = i;
  swap(p, 1, min);
  shellsort(p, N);
  p[0] = p[N];
  for (M = 3, i = 4; i <= N; i++) {
    while (ccw(p[M], p[M-1], p[i]) >= 0) M--;
    M++; swap(p, i, M);
  };
  return M;
};
double areaOfHull(const Point s[], const int num) {
 
  // preprocess point array
  Point p[num+2];       // 2 spare points
  p[0] = Point();
  cout << "Hull input points: ";
  for (int i = 1; i <= num; i++) {
    p[i] = Point(s[i-1]);
    cout << "(" << p[i].x << "/" << p[i].y << ") ";
  };
  p[num+1] = Point();
  cout << endl;

  // genrate hull with grahamscan
  int m = grahamscan(p, num);
  cout << "Grahamscan Convex Hull with " << m << " points: ";
  for (int i = 1; i <= m; i++) {
    cout << "(" << p[i].x << "/" << p[i].y << ") ";
  };

  // calculate area of hull
  p[0] = p[m];
  double a = 0.0;
  if (m >= 3 && num >= 3) {
    // calc 4-edged hull with 2 triangles
    if (m == 4) {
      // Triangle #1:  p[0], p[1], p[3]
      Point u(p[1].x-p[0].x, p[1].y-p[0].y);
      Point v(p[3].x-p[0].x, p[3].y-p[0].y);
      double a1 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
      // Triangle #2:  p[1], p[2], p[3]
      u = Point(p[2].x-p[1].x, p[2].y-p[1].y);
      v = Point(p[3].x-p[1].x, p[3].y-p[1].y);
      double a2 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
      a = a1 + a2;
    } else {       
      for (int i = 0; i < m; i++) {
        a += ((p[i+1].y - p[i].y)*(p[i+1].x + p[i].x));
      };
      a /= 2.0;
    };
  };
  cout << " -> " << fabs(a) << endl;

  return fabs(a);
};


bool checkOtsu(int variLimit, int otsu, float vari) {

    if (variLimit > 0) {
      if ((vari >= (float) variLimit) && (otsu >= 0)) {
        return true;
      } else {
        return false;
      };
    } else {
      if (otsu >= 0) {
        return true;
      } else {
        return false;
      } 
    };

    return false;

};


Point SlidingWindow::apply(const Rectangle tmpWindow) {

  Point result;
  PPMWriter myWriter;
  char filename[255];


  // 1. Preprocessing already done reading the images


  // 2. Generate Delta images
  Filter myFilter;
  if (myDeltaImage != NULL) {
    delete myDeltaImage;
  };
  myDeltaImage = myFilter.applyDeltaImage(myImages[0], myImages[1], myBoundingBox);
  for (int i = 1; i < myLookahead+1; i++) {
    myFilter.applyDeltaImageCorrection(myDeltaImage, myImages[i], myImages[i+1], \
                                       myBoundingBox, 0 /*limit*/);
  };


  // 3. Remove already detected pixels
  FilterExpandGreyscale myFilterExpandGreyscale(myBoundingBox);
  Image* tempDeltaMerge = myFilterExpandGreyscale.apply(myDeltaMergeWait[0]);
  myFilter.applySubtractImage(myDeltaImage, tempDeltaMerge, myBoundingBox);
  FilterCutRectangle myFilterCutRectangle(myBoundingBox);
  if (myReportDeltaMerge != false) {
    strcpy(filename, myReportingDirectory);
    strcat(filename, "deltamerge.ppm");
    myWriter.setFilename(filename);
    Image* outputImage = myFilterCutRectangle.apply(tempDeltaMerge);
    myWriter.apply(outputImage);
    delete outputImage;
  };
  delete tempDeltaMerge;


  // 4. Postprocessing
  applyPostprocessing();
  char tempFilename[255];
  if (myReportDeltas != false) {
    strcpy(filename, myReportingDirectory);
    sprintf(tempFilename, "deltadebug%.*d.ppm", 3, myPosition);
    strcat(filename, tempFilename);  
    myWriter.setFilename(filename);
    Image* outputImage = myFilterCutRectangle.apply(myDeltaImage);
    myWriter.apply(outputImage);
    delete outputImage;
  };


  // 5. Detection (select used threshold)
  int detectionUsed = applyDetection(tmpWindow); 
  result = myDetector->apply(myDeltaImage, tmpWindow);


  // 6. Save detected pixels (only applyed if really used ...)
  Image *tmpImage = (myDeltaMergeWait[myDeltaMergeWaitCount-1])->copy();
  if (detectionUsed == true) {
    // if detection was applyed -> merge points
    // otherwise just keep the existing image
    myFilter.applyMergeImages(myDeltaImage, tmpImage, tmpWindow); 
  };
  delete myDeltaMergeWait[0];
  for(int i = 0; i < myDeltaMergeWaitCount-1; i++) {
    myDeltaMergeWait[i] = myDeltaMergeWait[i+1];
  };
  myDeltaMergeWait[myDeltaMergeWaitCount-1] = tmpImage;


  // 7. Calculate area spanned by detected points
  double SpannedArea = 0.0;
  if (result.valid()) {
    if (applyCheckSpannedArea(tmpWindow, SpannedArea) == false) {
      // Area is too large -> no Point detected
      result = Point();
    };
  };


  // 8. Debug output
  if (myReportImages != false) {
    strcpy(filename, myReportingDirectory);
    sprintf(tempFilename, "imagedebug%.*d.ppm", 3, myPosition);
    strcat(filename, tempFilename);  
    myWriter.setFilename(filename);
    Image24* colImage = new Image24(myImages[0]->getWidth(), \
                                    myImages[0]->getHeight());
    colImage->adoptImage24(myImages[0]->getImage24());
    Image* outputImage = myFilterCutRectangle.apply(colImage);
    delete colImage;
    if (result.valid() == true) {
      outputImage->setColor(result.x-myBoundingBox.x1,\
                            result.y-myBoundingBox.y1,Color(0,255,0));
    };
    myWriter.apply(outputImage);
    delete outputImage;
  };
  if (myReportHTML == true) {
    HTMLreportEntry(SpannedArea, tmpWindow, result);
  };  


  // 9. Return resulting Point
  return result;

};


void SlidingWindow::generateImages() {

  if (valid()) {

    myImageSource->setPosition(myPosition);

    for (int i = 0; i < MAXLOOKAHEAD+2; i++) {
      if (myImages[i] != NULL) {
        delete myImages[i];
      };
      if (i < myLookahead+2) {
        myImages[i] = myImageSource->getImageRelative(i-1);
        applyPreprocessing(i);
      } else {
        myImages[i] = NULL;
      };     
    };
  };

};


inline void SlidingWindow::applyPreprocessing(unsigned int index) {
  
  // index points to image in myImages[]
  if (((int) index < myLookahead+2) && (myImages[index] == NULL)) {
    return;
  };
  Image* result = (myImages[index])->copy();
  if (valid()) {

    list<FilterStrategy*>::iterator I = myPreprocessing.begin();
    Image* newImage;

    while (I != myPreprocessing.end()) {
      newImage = (*I)->apply(result);
      delete result;
      result = newImage;
      I++;
    };

  };

  delete myImages[index];
  myImages[index] = result;

};


inline void SlidingWindow::applyPostprocessing() {

  if (myPostprocessing.size() > 0) {
    Image *result = myDeltaImage->copy();

    if (valid()) {
      list<FilterStrategy*>::iterator I = myPostprocessing.begin();
      Image* newImage;
      while (I != myPostprocessing.end()) {
        newImage = (*I)->apply(result);
        delete result;
        result = newImage;
        I++;
      };
    };

    delete myDeltaImage;
    myDeltaImage = result;
  };

};


int SlidingWindow::applyDetection(Rectangle tmpWindow) {

  Histogram myHistogram;
  float vari = -1;
  switch (myThresholdType) {
  case ThresholdFixed: {
    myDetector->setThreshold(myThreshold);
    cout << "[SlidingWindow::apply] Using fixed Threshold of " << myThreshold << endl;
    myActualVariance = -1.0;
    return true;
    break; }

  case ThresholdOtsu: {
    myHistogram.generate(myDeltaImage, tmpWindow);
    myHistogram.eliminateWhitePixels();
    float rmean, /* vari, */ p0, p1, a0, a1, sep;
    int otsu = myHistogram.otsudetails(rmean, vari, p0, p1, a0, a1, sep);
    int bwLimit;

    // Check of Histogram Width and Variance
    int OtsuOK = checkOtsu(myVarianceLimit, otsu, vari);
    if (OtsuOK) {
      bwLimit = otsu;
    } else {
      bwLimit = 0;
    };

    cout << "[SlidingWindow::apply] Otsu: " << otsu << ", Variance: " << vari \
         << ", using Otsu-Threshold of " << bwLimit << endl;
    myDetector->setThreshold(bwLimit);
    myActualVariance = vari;
    return OtsuOK;
    break; }

  default: {
    cout << "[SlidingWindow::apply] Using no Threshold" << endl;
    myDetector->setThreshold(0);
    return true; }

  };

};


int SlidingWindow::applyCheckSpannedArea(Rectangle tmpWindow, double &SpannedArea) {

  int result = true;
  SpannedArea = 0.0;

  // insert black pixels into list 
  int num = 0;
  int xmin, ymin, xmax, ymax;
  xmin = ymin = MAXINT;
  xmax = ymax = 0;
  vector<Point> points(0);
  for (int x = tmpWindow.x1; x < tmpWindow.x2; x++) {
    if ((x >= myBoundingBox.x1) && (x <= myBoundingBox.x2)) {
      for (int y = tmpWindow.y1; y < tmpWindow.y2; y++) {
        if ((y >= myBoundingBox.y1) && (y <= myBoundingBox.y2)) {
          if (myDeltaImage->getGreyscale(x,y) <= myDetector->getThreshold()) {
            points.push_back(Point(x,y));
            num++;
            if (x > xmax) xmax = x;
            if (y > ymax) ymax = y;
            if (x < xmin) xmin = x;
            if (y < ymin) ymin = y;
          };
        };
      }; 
    };
  };

  if (num < 3) {        // only 0, 1 or 2 pixels -> no area
    SpannedArea = 0.0;
  } else {              // at least 3 pixels
    // First check -> Size of BoundingBox around pixels
    //cout << "Bounding Box: " << xmax-xmin+1 << " * " << ymax-ymin+1;
    SpannedArea = (double) ((xmax-xmin+1)*(ymax-ymin+1));
    //cout << " = " << SpannedArea << endl;
    double RectArea = (double) (tmpWindow.getWidth() * tmpWindow.getHeight());
    double SAR = SpannedArea / RectArea;
    if (SAR <= myAreaLimit) {
      // BoundingBox is already smaller (Convex Hull is smaller for sure!!!)
      //
      // -> Result is OK
      cout << "Size of Bounding Box (" << SpannedArea << ") is small enough!!!" << endl;
    } else {

      Point p[num];
      for (int i = 0; i < num; i++) p[i] = points[i];
      SpannedArea = areaOfHull(p, num);

      // Vergleich mit Flaeche des Fensters
      SAR = SpannedArea / RectArea;
      if (SAR > myAreaLimit) {
        // invalid result
        result = false;
        cout << "Size of Convex Hull is too large!!!" << endl;
      };

      cout << "SPA  : " << SpannedArea << endl;
      cout << "Rect : " << RectArea << endl;
      cout << "SAR  : " << SAR << " <= " << myAreaLimit << " ?" << endl;

    };
  };

  return result;

};


void SlidingWindow::HTMLreportEntry(double SpannedArea, Rectangle tmpWindow, Point result) {

  char filename[255];
  PPMWriter myWriter;
  char tempFilename[255];
  Filter myFilter;
  FilterCutRectangle myFilterCutRectangle(myBoundingBox);


  strcpy(filename, myReportingDirectory);
  strcat(filename, "report.htm");
  ofstream datei(filename, ios::app);
  char url[255];
  char shell[255];
  datei << "<TR><TD>";
  datei << "POS: " << myPosition << "<BR>";
  datei << "HOT: " << myDetector->getThreshold() << "<BR>";
  datei << "HOV: " << myActualVariance << "<BR>";
  datei << "Area: " << SpannedArea << " vs " << tmpWindow.getWidth()*tmpWindow.getHeight()  << "<BR>";
  datei << "SAR: " << ((int)(SpannedArea/tmpWindow.getArea()*100.0*100.0))/100  << " %<BR>";
  if (result.valid()) {
    datei << "Result valid";
  } else { 
    datei << "Result invalid";
  };
  datei << "</TD>";

  // scaled width
  int width = (myBoundingBox.x2-myBoundingBox.x1+1)*4;
  int height = (myBoundingBox.y2-myBoundingBox.y1+1)*4;

  // actual Image
  char number[255];
  sprintf(number, "%.*d", 3, myPosition);
  strcpy(shell, "ppmtogif -interlace ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "imagedebug");
  strcat(shell, number);
  strcat(shell, ".ppm > ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "imagedebug");
  strcat(shell, number);
  strcat(shell, ".gif &");
  system(shell);   
  sprintf(url, "<IMG SRC=\"imagedebug%.*d.gif\" width=%d height=%d>", 3, myPosition, width, height);
  datei << "<TD>" << url << "</TD>";

  // Delta Image
  strcpy(shell, "ppmtogif -interlace ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltadebug");
  strcat(shell, number);
  strcat(shell, ".ppm > ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltadebug");
  strcat(shell, number);
  strcat(shell, ".gif &");
  system(shell);
  sprintf(url, "<IMG SRC=\"deltadebug%.*d.gif\" width=%d height=%d>", 3, myPosition, width, height);
  datei << "<TD>" << url << "</TD>";

  // scharze Punkte markieren 
  Image* bwImage;
  Image* greyImage = myFilterCutRectangle.apply(myDeltaImage);
  if (result.valid() == true) {
    FilterBinarize myFilterBinarize(myDetector->getThreshold());
    bwImage = myFilterBinarize.apply(greyImage);
  } else {
    bwImage = new Image8(greyImage->getWidth(), greyImage->getHeight());
    bwImage->setWhite();
  };
  delete greyImage;
  Image* posImage = new Image24(bwImage->getWidth(), bwImage->getHeight());
  posImage->adoptImage24(bwImage->getImage24());
  delete bwImage;
  // draw bounding box into image
  for(int i = tmpWindow.x1; i <= tmpWindow.x2; i++) {
    posImage->setColor(i-myBoundingBox.x1,tmpWindow.y1-myBoundingBox.y1,Color(0,0,255));
    posImage->setColor(i-myBoundingBox.x1,tmpWindow.y2-myBoundingBox.y1,Color(0,0,255));
  };
  for(int i = tmpWindow.y1; i <= tmpWindow.y2; i++) {
    posImage->setColor(tmpWindow.x1-myBoundingBox.x1,i-myBoundingBox.y1,Color(0,0,255));
    posImage->setColor(tmpWindow.x2-myBoundingBox.x1,i-myBoundingBox.y1,Color(0,0,255));
  };
  // draw detected pixel center
  if (result.valid() == true) {
    posImage->setColor(result.x-myBoundingBox.x1,result.y-myBoundingBox.y1,Color(0,255,0));
  };
  strcpy(filename, myReportingDirectory);
  sprintf(tempFilename, "deltapos%.*d.ppm", 3, myPosition);
  strcat(filename, tempFilename);  
  myWriter.setFilename(filename);
  myWriter.apply(posImage);
  delete posImage;
  strcpy(shell, "ppmtogif -interlace ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltapos");
  strcat(shell, number);
  strcat(shell, ".ppm > ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltapos");
  strcat(shell, number);
  strcat(shell, ".gif &");
  system(shell);
  sprintf(url, "<IMG SRC=\"deltapos%.*d.gif\" width=%d height=%d>", 3, myPosition, width, height);
  datei << "<TD>" << url << "</TD>";

  Image* tempDeltaMerge = new Image8(myDeltaImage->getWidth(), \
                                     myDeltaImage->getHeight());
  myFilter.applyExpandGreyscale(myDeltaMergeWait[0], tempDeltaMerge, \
                                myBoundingBox);
  strcpy(filename, myReportingDirectory);
  sprintf(tempFilename, "deltamerge%.*d.ppm", 3, myPosition);
  strcat(filename, tempFilename);  
  myWriter.setFilename(filename);
  Image* outputImage = myFilterCutRectangle.apply(tempDeltaMerge);
  delete tempDeltaMerge;
  myWriter.apply(outputImage);
  delete outputImage;
  strcpy(shell, "ppmtogif -interlace ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltamerge");
  strcat(shell, number);
  strcat(shell, ".ppm > ");
  strcat(shell, myReportingDirectory);
  strcat(shell, "deltamerge");
  strcat(shell, number);
  strcat(shell, ".gif &");
  system(shell);
  sprintf(url, "<IMG SRC=\"deltamerge%.*d.gif\" width=%d height=%d>", 3, myPosition, width, height);
  datei << "<TD>" << url << "</TD>";


  datei << "</TR>" << endl;
  datei.close();

};

 
void SlidingWindow::setThresholdType(typeThreshold tmpType) {
  myThresholdType = tmpType;
  switch (myThresholdType) {
    case ThresholdFixed:
      myThreshold = 205;
      myVarianceLimit = -1;
      break;
    case ThresholdOtsu:
      myThreshold = -1;
      myVarianceLimit = 100;
      break;
    default:
      myThreshold = -1;
      myVarianceLimit = -1;
      myThresholdType = ThresholdNone;
      break;
  };
};
  

void SlidingWindow::setThresholdFixed(unsigned int threshold) {
  myThreshold = (int) threshold;
  myVarianceLimit = -1;
  myThresholdType = ThresholdFixed;
};
  

void SlidingWindow::setThresholdOtsu(unsigned int variance) {
  myThreshold = -1;
  myVarianceLimit = (int) variance;
  myThresholdType = ThresholdOtsu;
};


void SlidingWindow::setReportingDirectory(char* directory) {
  strcpy(myReportingDirectory, directory);
};


void SlidingWindow::setAreaLimit(double areaLimit) {
  if (areaLimit < 0.0) {
    myAreaLimit = 0.0;
  } else if (areaLimit > 1.0) {
    myAreaLimit = 1.0;
  } else {
    myAreaLimit = areaLimit;
  };
};


void SlidingWindow::setReporting(int ReportImages, int ReportDeltas, \
                                 int ReportDeltaMerge, int ReportHTML) {
  myReportImages = ReportImages;
  myReportDeltas = ReportDeltas;
  if (!myReportHTML && ReportHTML) {
    char filename[255];
    strcpy(filename, myReportingDirectory);
    strcat(filename, "report.htm");
    ofstream datei(filename);
    datei << "<HTML>" << endl;
    datei << "<HEAD><H1>Check of Images and Deltas</H1></HEAD>" << endl;
    datei << "<BODY>" << endl;
    datei << "<HR>" << endl;
    datei << "<TABLE BORDER=2>" << endl;
    datei << "<TR><TH>Nr</TH><TH>Image</TH><TH>Delta</TH><TH>Detection</TH><TH>Merged Deltas</TH></TR>" << endl;
    datei.close();
  } else {
    if (myReportHTML && !ReportHTML) {
      char filename[255];
      strcpy(filename, myReportingDirectory);
      strcat(filename, "report.htm");
      ofstream datei(filename, ios::app);
      datei << "</TABLE>" << endl;
      datei << "</BODY>" << endl;
      datei << "</HTML>" << endl;
      datei.close();
    };
  };
  myReportHTML = ReportHTML;

  // Delta merge sofort ausgeben wenn eingeschaltet
  Filter myFilter;
  if ((myReportDeltaMerge == false) && (ReportDeltaMerge != false)) {
    Image* tempDeltaMerge = new Image8(myDeltaImage->getWidth(), \
                                       myDeltaImage->getHeight());
    myFilter.applyExpandGreyscale(myDeltaMergeWait[0], tempDeltaMerge, \
                                  myBoundingBox);
    myFilter.applySubtractImage(myDeltaImage, tempDeltaMerge, myBoundingBox);

    char filename[255];
    strcpy(filename, myReportingDirectory);
    strcat(filename, "deltamerge.ppm");
    PPMWriter myWriter;
    myWriter.setFilename(filename);
    FilterCutRectangle myFilterCutRectangle(myBoundingBox);
    Image* outputDeltaMerge = myFilterCutRectangle.apply(tempDeltaMerge);
    myWriter.apply(outputDeltaMerge);
    delete outputDeltaMerge;
    delete tempDeltaMerge;
  };

  myReportDeltaMerge = ReportDeltaMerge;  

};









