#ifndef _SLIDINGWINDOW_H_
#define _SLIDINGWINDOW_H_

#include "Detector.h"
#include "FilterStrategy.h"
#include "ImageSource.h"
#include <list.h>
#include "Image.h"
#include "Image8.h"
#include "Rectangle.h"



#define MAXLOOKAHEAD 10



class SlidingWindow {

public:
 
  SlidingWindow();
  ~SlidingWindow();
  SlidingWindow(const unsigned int Lookahead);

  // set Parameters needed by SlidingWindow
  void setImageSource(ImageSource*);
  void setDetector(Detector*);
  void addPreprocessFilter(FilterStrategy*);
  void addPostprocessFilter(FilterStrategy*);
  void setLookahead(const unsigned int);
  unsigned int getPosition();
  unsigned int getLookahead();
  int valid();
  void setReportingDirectory(char* directory);
  void setReporting(int ReportImages, int ReportDeltas, \
                    int ReportDeltaMerge, int ReportHTML);
  void setBoundingBox(const Rectangle);

  // Ways to select threshold (fixed or otsu)
  enum typeThreshold { ThresholdNone, ThresholdFixed, ThresholdOtsu };
  void setThresholdType(typeThreshold);
  void setThresholdFixed(unsigned int);
  void setAreaLimit(double);
  void setThresholdOtsu(unsigned int varianceLimit); 

  // use the Sliding Window
  void moveFirst();
  void moveLast();
  void moveNext();
  void movePrevious();
  void move(int step);

  // get Result out of Sliding Window
  Point apply(const Rectangle);


private:
  
  Image* myImages[MAXLOOKAHEAD+2];
  Image* myDeltaImage;

  ImageSource* myImageSource;

  int myLookahead;
  int myPosition;

  // Threshold Parameters
  typeThreshold myThresholdType;
  int myThreshold;
  int myVarianceLimit;
  double myAreaLimit;
  double myActualVariance;

  list<FilterStrategy*> myPreprocessing;
  list<FilterStrategy*> myPostprocessing;
  Detector* myDetector;

  inline void applyPreprocessing(unsigned int index);    // on Images
  inline void applyPostprocessing();                     // on Delta Image
  int applyDetection(Rectangle);                  // Detect position
  int applyCheckSpannedArea(Rectangle, double&);  // Check Spanned Area
  void HTMLreportEntry(double, Rectangle, Point);                         // Single Row of report
  void generateImages();

  // Image8* myDeltaMerge;
  const int myDeltaMergeWaitCount = 3;
  Image* myDeltaMergeWait[myDeltaMergeWaitCount];

  char myReportingDirectory[255];

  int myReportImages;
  int myReportDeltas;
  int myReportDeltaMerge;
  int myReportHTML;

  Rectangle myBoundingBox;

};

#endif  // _SLIDINGWINDOW_H_
 







