// Header Files for Motif
#include <X11/Intrinsic.h>     // The Intrinsics                   
#include <Xm/Xm.h>             // Motif symbols                    
#include <Xm/MainW.h>          // XmMainWindow widget              
#include <Xm/ScrolledW.h>      // XmScrolledWindow widget          
#include <Xm/RowColumn.h>      // XmRowColumn widget               
#include <Xm/Form.h>           // XmForm widget                    
#include <Xm/Text.h>           // XmText widget                    
#include <Xm/CascadeB.h>       // XmCascadeButton widget           
#include <Xm/PushB.h>          // XmPushButton widget              
#include <Xm/DrawingA.h>       // XmDrawingArea widget             
#include <Xm/TextF.h>          // XmTextField widget               
#include <Xm/Scale.h>          // XmScale widget                   
#include <Xm/Frame.h>          // XmFrame widget                   
#include <Xm/Label.h>          // XmLabel widget                   
#include <Xm/FileSB.h>         // XmFileSelectionBox widget        
#include <Xm/ToggleB.h>        // XmToggleButton widget            

// own Header Files
#include <iostream.h>
#include <strstream.h>
#include "image/ImageSourcePPM.h"
#include "image/Image.h"
#include "image/Image8.h"
#include <stdlib.h>
#include "image/Analysis.h"

#include "geometry/SpannedAreaOfPoints.h"
#include "geometry/Point2D.h"
#include "geometry/ConvexHull.h"
#include "geometry/Polygon.h"
#include <vector.h>
#include <math.h>

// Globals for Motif
XtAppContext  context;
Widget     toplevel, mainManager, commandPanel;
Widget     CMDexit, CMDdoit, CMDiterations, CMDclear;
Widget     graphics;
Widget     menuBar, menuButtonFile, menuFile, menuFileExit;
Widget     statusLine, statusText, statusPosition;
Widget     CMDhull;



// Globals for X11 and libimage
Image8* myImage8;
Pixmap myPixmap;


// Global constants
const PixelSize = 11;
const ImageSize = 25;



// Callback Routines
void CMDexitCallback(Widget, XtPointer, XtPointer);
void CMDclearCallback(Widget, XtPointer, XtPointer);
void CMDdoitCallback(Widget, XtPointer, XtPointer);
void CMDhullCallback(Widget, XtPointer, XtPointer);
void graphicsInputCallback(Widget, XtPointer, XtPointer);
void graphicsRepaintCallback(Widget, XtPointer, XtPointer);
void graphicsInputEvent(Widget, XtPointer, XEvent*, Boolean*);
void paint();
void repaint();
Pixel GetRGBColor(Widget, byte, byte, byte);
void DrawCross(const unsigned int x, const unsigned int y, const unsigned int size, Color);
void DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, Color);




main(int argc, char **argv) {


  // Init. Toolkit, App. Context, Application Shell
  toplevel = XtAppInitialize( &context, "Analysetest",
      NULL, 0, &argc, argv, NULL, NULL, 0); 


  // Create widget tree, bind callbacks
  mainManager = XtCreateManagedWidget( "mainManager",
      xmMainWindowWidgetClass, toplevel, NULL, 0);


  graphics = XtCreateManagedWidget( "graphics",
      xmDrawingAreaWidgetClass, mainManager, NULL, 0);
  XtAddCallback(graphics, XmNinputCallback, graphicsInputCallback, NULL);
  XtAddCallback(graphics, XmNexposeCallback, graphicsRepaintCallback, NULL);
  XtAddEventHandler(graphics, PointerMotionMask, False, graphicsInputEvent, NULL);

 
  commandPanel = XtCreateManagedWidget( "commandPanel",
      xmRowColumnWidgetClass, mainManager, NULL, 0);

  CMDiterations = XtCreateManagedWidget( "CMDiterations",
      xmScaleWidgetClass, commandPanel, NULL, 0);
  CMDexit = XtCreateManagedWidget( "CMDexit",
      xmPushButtonWidgetClass, commandPanel, NULL, 0);
  XtAddCallback(CMDexit, XmNactivateCallback, CMDexitCallback, NULL);
  CMDdoit = XtCreateManagedWidget( "CMDdoit",
      xmPushButtonWidgetClass, commandPanel, NULL, 0);
  XtAddCallback(CMDdoit, XmNactivateCallback, CMDdoitCallback, NULL);
  CMDclear = XtCreateManagedWidget( "CMDclear",
      xmPushButtonWidgetClass, commandPanel, NULL, 0);
  XtAddCallback(CMDclear, XmNactivateCallback, CMDclearCallback, NULL);
  CMDhull = XtCreateManagedWidget( "CMDhull",
      xmPushButtonWidgetClass, commandPanel, NULL, 0);
  XtAddCallback(CMDhull, XmNactivateCallback, CMDhullCallback, NULL);


  menuBar = XmCreateMenuBar (mainManager, "menuBar", NULL, 0); 
  XtManageChild (menuBar);
  menuFile = XmCreatePulldownMenu (menuBar, "menuFile", NULL, 0);
  menuFileExit = XtCreateManagedWidget( "menuFileExit",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileExit, XmNactivateCallback, CMDexitCallback, NULL);
  Arg arglist[1];
  XtSetArg(arglist[0], XmNsubMenuId, menuFile);
  menuButtonFile = XtCreateManagedWidget( "menuButtonFile",
      xmCascadeButtonWidgetClass, menuBar, arglist, 1);


  statusLine =  XtCreateManagedWidget( "statusLine",
      xmFormWidgetClass, mainManager, NULL, 0);
  statusText =  XtCreateManagedWidget( "statusText",
      xmTextFieldWidgetClass, statusLine, NULL, 0);
  statusPosition =  XtCreateManagedWidget( "statusPosition",
      xmTextFieldWidgetClass, statusLine, NULL, 0);

  XtVaSetValues(statusText, 
      XmNeditable, false, 
      XmNvalue, "Hello world",
      NULL);
  XtVaSetValues(statusPosition, 
      XmNeditable, false, 
      XmNvalue, " 0 / 0",
      NULL);



  // Tell XmMainWindow about children
  XtVaSetValues(mainManager, 
      XmNmenuBar, menuBar, 
      XmNworkWindow, graphics,
      XmNcommandWindow, commandPanel, 
      XmNmessageWindow, statusLine,
      NULL );
  XtVaSetValues(commandPanel, 
      XmNrightAttachment, XmATTACH_WIDGET, 
      XmNrightWidget, mainManager, 
      NULL);

  // Attach widgets in XmForm
  XtVaSetValues(graphics, 
      XmNheight, PixelSize*ImageSize, 
      XmNwidth,  PixelSize*ImageSize, 
      NULL);
  XtVaSetValues(graphics, 
      XmNtopAttachment, XmATTACH_WIDGET, 
      XmNtopWidget, menuBar, 
      NULL);
  XtVaSetValues(graphics, 
      XmNbottomAttachment, XmATTACH_WIDGET, 
      XmNbottomWidget, commandPanel, 
      NULL);



  // Init Images and Pixmaps
  myImage8 = new Image8(ImageSize, ImageSize);
  myImage8->setWhite();
  paint();
  XtRealizeWidget(toplevel);           // Realize all              
  XtAppMainLoop(context);              // Can't get past here !!!  

};










// Callback routines implementation


void repaint() {

  XClearWindow(XtDisplay(graphics), XtWindow(graphics));

  XCopyArea(XtDisplay(graphics), myPixmap, XtWindow(graphics), \
            XDefaultGCOfScreen(XtScreen(graphics)), 0, 0, \
            ImageSize*PixelSize, ImageSize*PixelSize, 0, 0);

  XFlush(XtDisplay(graphics));

};


void paint() {

  if (myPixmap != 0) {
    XFreePixmap(XtDisplay(graphics), myPixmap);
    myPixmap = 0;
  };

  int depth;
  XtVaGetValues (graphics, XtNdepth, &depth, NULL);

  myPixmap = XCreatePixmap(XtDisplay(graphics), \
                           XRootWindowOfScreen(XtScreen(graphics)), \
                           ImageSize*PixelSize, ImageSize*PixelSize, \
                           depth);
  for(int y = 0; y < PixelSize*ImageSize; y++) {
    for(int x = 0; x < PixelSize*ImageSize; x++) {
      if (myImage8->getGreyscale(x,y) == 0) {
        XSetForeground(XtDisplay(graphics), XDefaultGCOfScreen(XtScreen(graphics)), \
                       XBlackPixelOfScreen(XtScreen(graphics)));
      } else {
        XSetForeground(XtDisplay(graphics), XDefaultGCOfScreen(XtScreen(graphics)), \
                       XWhitePixelOfScreen(XtScreen(graphics)));
      };
      XFillRectangle(XtDisplay(graphics), myPixmap, \
                     XDefaultGCOfScreen(XtScreen(graphics)), \
                     x*PixelSize, y*PixelSize, PixelSize, PixelSize);
    };
  }; 

};



void CMDexitCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  exit(0);
};


void CMDclearCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  delete myImage8;
  myImage8 = new Image8(ImageSize, ImageSize);
  myImage8->setWhite();
  paint();
  repaint();
};
  

void CMDdoitCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  Analysis myAnalyser;

  if (myAnalyser.getNumberOfBlackPixels(myImage8) > 0) {

    Rectangle win;
    win.x1 = win.y1 = 0;
    win.x2 = win.y2 = ImageSize-1;
    int iterations;
    XtVaGetValues(CMDiterations, \
      XmNvalue, &iterations,
      NULL);

    Point pos;
    if (iterations > 0) {
      pos = myAnalyser.getCenterOfBlackPixelsIterative(myImage8, win, iterations);
    } else {
      pos = myAnalyser.getCenterOfBlackPixelsMedian(myImage8, win);
    };

    repaint();

    Color Red;
    Red.r = 255; Red.g = Red.b = 0;
    unsigned int x = (pos.x * PixelSize) + (PixelSize/2);
    unsigned int y = (pos.y * PixelSize) + (PixelSize/2);
    DrawCross(x, y, PixelSize, Red);

  };

};


void swap(Point a[], int i, int j) {
  Point t(a[i]);
  a[i] = a[j];
  a[j] = t;
};


float theta(Point p1, Point p2) {
  int dx, dy, ax, ay;
  float t;
  dx = p2.x - p1.x; ax = abs(dx);
  dy = p2.y - p1.y; ay = abs(dy);
  t = (ax+ay == 0) ? 0 : (float) dy/(ax+ay);
  if (dx < 0) t = 2-t; else if (dy < 0) t = 4+t;
  return t*90.0;
};


void shellsort(Point a[], int N) {
  int i, j, h; Point v;
  for (h = 1; h <= N/9; h = 3*h+1) ;
  for ( ; h > 0; h /= 3)
    for (i = h+1; i <= N; i += 1) {
      v = a[i]; j = i;
      while (j>h && theta(a[1],a[j-h])>theta(a[1],v)) {
        a[j] = a[j-h];
        j -= h;
      };
      a[j] = v;
    };
};


int ccw(Point p0, Point p1, Point p2) {
  int dx1, dx2, dy1, dy2;
  dx1 = p1.x - p0.x; dy1 = p1.y - p0.y;
  dx2 = p2.x - p0.x; dy2 = p2.y - p0.y;
  if (dx1*dy2 > dy1*dx2) return +1;
  if (dx1*dy2 < dy1*dx2) return -1;
  if ((dx1*dx2 < 0) || (dy1*dy2 < 0)) return -1;
  if ((dx1*dx1+dy1*dy1) < (dx2*dx2+dy2*dy2)) return +1;
  return 0;
};


int grahamscan(Point p[], int N) {
  int i, min, M;
  for (min = 1, i = 2; i <= N; i++) 
    if (p[i].y < p[min].y) min = i;
  for (i = 1; i <= N; i++)
    if (p[i].y == p[min].y)
      if (p[i].x > p[min].x) min = i;
  swap(p, 1, min);
  shellsort(p, N);
  p[0] = p[N];
  for (M = 3, i = 4; i <= N; i++) {
    while (ccw(p[M], p[M-1], p[i]) >= 0) M--;
    M++; swap(p, i, M);
  };
  return M;
};

int wrap(Point p[], int N) {
  int i, min, M;
  double th, v;
  for (min = 0, i = 0; i < N; i++)
    if (p[i].y < p[min].y) min = i;
  p[0] = p[min]; th = 0.0;
  for (M = 0; M < N; M++) {
    swap(p, M, min);
    min = N; v = th; th = 360.0;
    for (i = M+1; i <= N; i++)
      if (theta(p[M], p[i]) > v)
        if (theta(p[M], p[i]) < th) {
          min = i;
          th = theta(p[M], p[min]);
        };
    if (min == N) return M;
  };
};

double areaOfHull(const Point s[], const int num) {
 
  // preprocess point array
  Point p[num+2];       // 2 spare points
  p[0] = Point();
  cout << "Hull input points: ";
  for (int i = 1; i <= num; i++) {
    p[i] = Point(s[i-1]);
    cout << "(" << p[i].x << "/" << p[i].y << ") ";
  };
  p[num+1] = Point();
  cout << endl;

  // genrate hull with grahamscan
  int m = grahamscan(p, num);
  cout << "Grahamscan Convex Hull with " << m << " points: ";
  for (int i = 1; i <= m; i++) {
    cout << "(" << p[i].x << "/" << p[i].y << ") ";
  };

  // calculate area of hull
  p[0] = p[m];
  double a = 0.0;
  if (m >= 3 && num >= 3) {
    // calc 4-edged hull with 2 triangles
    if (m == 4) {
      // Triangle #1:  p[0], p[1], p[3]
      Point u(p[1].x-p[0].x, p[1].y-p[0].y);
      Point v(p[3].x-p[0].x, p[3].y-p[0].y);
      double a1 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
      // Triangle #2:  p[1], p[2], p[3]
      u = Point(p[2].x-p[1].x, p[2].y-p[1].y);
      v = Point(p[3].x-p[1].x, p[3].y-p[1].y);
      double a2 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
      a = a1 + a2;
    } else {       
      for (int i = 0; i < m; i++) {
        a += ((p[i+1].y - p[i].y)*(p[i+1].x + p[i].x));
      };
      a /= 2.0;
    };
  };
  cout << " -> " << fabs(a) << endl;

  return fabs(a);
};

void CMDhullCallback(Widget w, XtPointer client_data, XtPointer call_data) {


    // Schwarze Punkte in Liste eintragen
    int num = 0;
    vector<Point> points(0);
    for (int x = 0; x < myImage8->getWidth(); x++) {
      for (int y = 0; y < myImage8->getHeight(); y++) {
        if (myImage8->getGreyscale(x,y) <= 128) {
          points.push_back(Point(x,y));
          num++;
        }; 
      };
    };
    Point p[num+2];       // 2 spare points
    p[0] = Point();
    cout << "Input points: ";
    for (int i = 1; i <= num; i++) {
      p[i] = points[i-1];
      cout << "(" << p[i].x << "/" << p[i].y << ") ";
    };
    p[num+1] = Point();
    cout << endl;


    // Konvexe Huelle bestimmen
    int m = grahamscan(p, num);
    cout << "Grahamscan Convex Hull with " << m << " points: ";
    for (int i = 1; i <= m; i++) {
      cout << "(" << p[i].x << "/" << p[i].y << ") ";
    };


    // Flaeche der Konvexen Huelle berechnen
    p[0] = p[m];
    double a = 0.0;
    if (m >= 3 && num >= 3) {
      // 4-eck als 2 Dreiecke berechnen
      if (m == 4) {
        // Dreick #1:  p[0], p[1], p[3]
        Point u(p[1].x-p[0].x, p[1].y-p[0].y);
        Point v(p[3].x-p[0].x, p[3].y-p[0].y);
        double a1 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
        // Dreick #2:  p[1], p[2], p[3]
        u = Point(p[2].x-p[1].x, p[2].y-p[1].y);
        v = Point(p[3].x-p[1].x, p[3].y-p[1].y);
        double a2 = (0.5 * fabs(u.x*v.y - u.y*v.x)); 
        a = a1 + a2;
      } else {       
        for (int i = 0; i < m; i++) {
          a += ((p[i+1].y - p[i].y)*(p[i+1].x + p[i].x));
        };
        a /= 2.0;
      };
    };
    cout << " -> " << fabs(a) << endl;


    // Konvexe Huelle Zeichnen
    if (a != 0.0) {
      for (int i = 0; i < m; i++) {
        int size = (int) (PixelSize/2.0 + 0.5); // cut of digits after '.' results
	                                        // with add of 0.5 in correct rounding
        DrawLine((int) p[i].x  *PixelSize+size, (int) p[i].y  *PixelSize+size, \
                 (int) p[i+1].x*PixelSize+size, (int) p[i+1].y*PixelSize+size, \
                 Color(0,0,255));
      };
    };

};


void graphicsInputCallback(Widget w, XtPointer client_data, XtPointer call_data) {

 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

 if (cbs->event->type == ButtonPress) {
  if (cbs->event->xbutton.button == Button1) {

    int x = cbs->event->xbutton.x / PixelSize;
    int y = cbs->event->xbutton.y / PixelSize; 

    if (myImage8->getGreyscale(x,y) == 0) {
      myImage8->setGreyscale(x,y,255);
    } else {
      myImage8->setGreyscale(x,y,0);
    };

    paint();
    repaint();

  };
 };


};


void graphicsRepaintCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  repaint();
};


void graphicsInputEvent(Widget w, XtPointer client_data, XEvent* event, Boolean* dis) {

  if (event->type == MotionNotify) {

    int x = event->xbutton.x / PixelSize;
    int y = event->xbutton.y / PixelSize; 

    char pos[255];
    if ( (x < 0) || (y < 0) ) {
      strcpy(pos, " No Position !");
    } else {
      sprintf(pos, " Position: %d / %d V: %d", \
            x, y, (int) (myImage8->getGreyscale(x, y)));
    };

    XtVaSetValues(statusPosition, 
      XmNvalue, pos,
      NULL);

  };

};


void DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, Color col) {

  Pixel fg;
  fg = GetRGBColor(graphics, col.r, col.g, col.b);
  XSetForeground(XtDisplay(graphics), XDefaultGCOfScreen(XtScreen(graphics)), fg);

  XDrawLine(XtDisplay(graphics), XtWindow(graphics), XDefaultGCOfScreen(XtScreen(graphics)), \
            x1, y1, x2, y2);

};


void DrawCross(const unsigned int x, const unsigned int y, const unsigned int size, Color col) {

  Pixel fg;
  fg = GetRGBColor(graphics, col.r, col.g, col.b);
  XSetForeground(XtDisplay(graphics), XDefaultGCOfScreen(XtScreen(graphics)), fg);

  XDrawLine(XtDisplay(graphics), XtWindow(graphics), XDefaultGCOfScreen(XtScreen(graphics)), \
            x-size, y, x+size, y);
  XDrawLine(XtDisplay(graphics), XtWindow(graphics), XDefaultGCOfScreen(XtScreen(graphics)), \
            x, y - size, x, y+size);

};


// Set a RGB Color inside GC
Pixel GetRGBColor(Widget w, byte r, byte g, byte b) {

 Display   *display;
 Colormap  cmap;
 XColor    alloc_color;

 display = XtDisplay(w);                    /* Get display pointer   */

 XtVaGetValues(w, XtNcolormap, &cmap, NULL );

 alloc_color.red   = ((r+1) * 256) - 1;        /* Red value, 0 - 65535  */
 alloc_color.green = ((g+1) * 256) - 1;        /* Green value, 0 - 65535*/
 alloc_color.blue  = ((b+1) * 256) - 1;        /* Blue value, 0 - 65535 */
 alloc_color.flags =  DoRed|DoGreen|DoBlue;      /* Set all */

 if (XAllocColor(display, cmap, &alloc_color)){  /* Success */
    return alloc_color.pixel;
 }
 else {                /* Failed  */
    printf("Can't allocate color!\n");     /* Print warning */
    return (Pixel)NULL;                    /* Return NULL   */
 }

};

