
#include <strstream.h>
#include <iomanip.h>
#include "image/Analysis.h"
#include "image/Histogram.h"
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <values.h>

#include "image/FilterGreyscale.h"
#include "image/FilterCutMask.h"
#include "image/FilterRotate180.h"
#include "image/FilterExpandBlack.h"
#include "image/DetectorNeighbour.h"



#define ZWISCHENRAUM 20

Point punkte[1000];
unsigned int punkte_index = 0;


// the routine paint creates the up to 5 Pixmaps in the X-Server
// that represent the up to 5 XImages in the X-Client Application
void paint();

// the routine repaint refreshes the screen of the
// X-Server out of the Pixmaps saved in the X-Server
void repaint();

Pixel GetRGBColor(Widget, byte, byte, byte);
void DrawCross(const unsigned int x, const unsigned int y, const unsigned int size, Color);
void DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, Color);
void DrawRectangle(unsigned int offset_x, unsigned int offset_y, Rectangle, Color);
void DrawPoints(Boolean interpolate, Boolean stepByStep);

double dist(Point, Point);
void HistogramOtsu(Histogram*);

void HTMLheader(ofstream*, unsigned int, unsigned int, float, unsigned int);
void HTMLentry(ofstream *datei, unsigned int line, Image* DeltaImage, Point lastPos, Point newPos, float&, float&);
void HTMLfooter(ofstream*, float, float);


// calcultates the poition inside an image and gives a pointer to this image
getActualImageAndPosition (unsigned int x, unsigned int y, \
                           Image* &img, int &img_x, int &img_y) {

  img_x = -1;
  img_y = -1;
  img = NULL;
 
    // erstes Deltabild
    if ( (y >= theImageHeight + ZWISCHENRAUM) &&
         (x <  theImageWidth) ) {
      img_x = x;
      img_y = y - theImageHeight - ZWISCHENRAUM;
      img = deltas[0];
    };

    // zweites Deltabild
    if ( (y >= theImageHeight + ZWISCHENRAUM) &&
         (x >= theImageWidth + ZWISCHENRAUM) &&
         (x < 2*theImageWidth + ZWISCHENRAUM) ) {
      img_x = x - theImageWidth - ZWISCHENRAUM;
      img_y = y - theImageHeight - ZWISCHENRAUM;
      img = deltas[1];
    };

    // Delta Merge Bild
    if ( (y >= theImageHeight + ZWISCHENRAUM) &&
         (x >= 2*(theImageWidth + ZWISCHENRAUM) ) ) {
      img_x = x - 2*(theImageWidth + ZWISCHENRAUM);
      img_y = y - theImageHeight - ZWISCHENRAUM;
      img = delta_merge;
    };

    // Linkes Bild
    if ( (y < theImageHeight) &&
         (x <  theImageWidth) ) {
      img_x = x;
      img_y = y;
      img = images[0];
    };

    // Mittleres Bild
    if ( (y < theImageHeight) &&
         (x >= theImageWidth + ZWISCHENRAUM) &&
         (x < 2*theImageWidth + ZWISCHENRAUM) ) {
      img_x = x - theImageWidth - ZWISCHENRAUM;
      img_y = y;
      img = images[1];
    };

    // Rechtes Bild
    if ( (y < theImageHeight) &&
         (x >= 2*(theImageWidth + ZWISCHENRAUM) ) ) {
      img_x = x - 2*(theImageWidth + ZWISCHENRAUM);
      img_y = y;
      img = images[2];
    };

}; 


void NextImageCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  ChangeScrollPosition(CMDscroll, actualImage+1);

};


void Next5ImageCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  ChangeScrollPosition(CMDscroll, actualImage+5);

};


void PrevImageCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  ChangeScrollPosition(CMDscroll, actualImage-1);

};


void Prev5ImageCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  ChangeScrollPosition(CMDscroll, actualImage-5);

};


void CMDscrollCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  int value;
  XmScaleGetValue(CMDscroll, &value);
  ChangeScrollPosition(CMDscroll, value);

};


void GraphicsRepaintCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  if (XtIsRealized(w)) {

    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

    if (cbs->event->xexpose.count == 0) { 
      // cout << "repaint entered ... "; cout.flush();
      repaint();
      // cout << "finished" << endl; cout.flush();
    } else {
      // cout << "skip ";
    };

  } else {

    cout << "Widget graphics not yet realized\n";

  };

};


void HistogramRepaintCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  if (XtIsRealized(w)) {

    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

    //if (cbs->event->xexpose.count == 0) { 
      XCopyArea(theDisp, HistogramPixmap, XtWindow(HistogramGraphics), theGC, \
                0, 0, HistogramXImage->width, HistogramXImage->height, 8, 8);
      //};

    XFlush(theDisp);

  } else {

    cout << "Widget 'HistogramGraphics' not yet realized\n";

  };

};


void ZOOMRepaintCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  if (XtIsRealized(w)) {

    Pixmap theSource = 0;
    Window theDestWin;

    if (w == ZOOM1display) {
      theSource = zoom1Pixmap;
      theDestWin = XtWindow(ZOOM1display);
    };
    if (w == ZOOM2display) {
      theSource = zoom2Pixmap;
      theDestWin = XtWindow(ZOOM2display);
    };

    if (theSource != 0) {
      XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

      if (cbs->event->xexpose.count == 0) { 
        XCopyArea(theDisp, theSource, theDestWin, theGC, \
                  0, 0, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, 0, 0);
      };
      XFlush(theDisp);

    };

  } else {

    cout << "[ZOOMRepaintCallback] WARNING: Destination Widget not yet realized\n";

  };

};


void ZOOMInputCallback(Widget w, XtPointer client_data, XtPointer call_data) {

 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

 if (cbs->event->type == ButtonPress) {
  if (cbs->event->xbutton.button == Button1) {

    unsigned int x,y;
    x = cbs->event->xbutton.x / FIXZOOM_PIXELSIZE;
    y = cbs->event->xbutton.y / FIXZOOM_PIXELSIZE;

    if (zoom1Pixmap != 0) {
      XCopyArea(theDisp, zoom1Pixmap, XtWindow(ZOOM1display), theGC, \
                0, 0, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, 0, 0);
      XFlush(theDisp);
      Pixel fg1 = GetRGBColor(ZOOM1display, 0, 0, 255);  // Blue
      XSetForeground(XtDisplay(ZOOM1display), theGC, fg1);
      XDrawRectangle(XtDisplay(ZOOM1display), XtWindow(ZOOM1display), theGC, \
                     x*FIXZOOM_PIXELSIZE, y*FIXZOOM_PIXELSIZE, \
                     FIXZOOM_PIXELSIZE, FIXZOOM_PIXELSIZE);
    };


    if (zoom2Pixmap != 0) {
      XCopyArea(theDisp, zoom2Pixmap, XtWindow(ZOOM2display), theGC, \
                0, 0, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, 0, 0);
      XFlush(theDisp);
      Pixel fg2 = GetRGBColor(ZOOM2display, 0, 0, 255);  // Blue
      XSetForeground(XtDisplay(ZOOM2display), theGC, fg2);
      XDrawRectangle(XtDisplay(ZOOM2display), XtWindow(ZOOM2display), theGC, \
                     x*FIXZOOM_PIXELSIZE, y*FIXZOOM_PIXELSIZE, \
                     FIXZOOM_PIXELSIZE, FIXZOOM_PIXELSIZE);
    };

  };
 };

};



void GraphicsInputCallback(Widget w, XtPointer client_data, XtPointer call_data) {

 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*) call_data;

 if (cbs->event->type == ButtonPress) {
  if (cbs->event->xbutton.button == Button1) {

  

    // Histogram initialisieren
    repaint();
    Image *tmpImg;        
    int HistoX, HistoY;
    getActualImageAndPosition (cbs->event->xbutton.x, cbs->event->xbutton.y, \
                               tmpImg, HistoX, HistoY);
    Histogram myHistogram;
    Rectangle myWindow;
    unsigned int WindowSize;
    XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
    myWindow.x1 = HistoX - WindowSize/2;
    myWindow.y1 = HistoY - WindowSize/2;
    myWindow.x2 = HistoX + WindowSize/2;
    myWindow.y2 = HistoY + WindowSize/2;
    myHistogram.generate(tmpImg, myWindow);
    if (XmToggleButtonGetState(ANALYSISeliminateHistogram)) {     
      myHistogram.eliminateWhitePixels();
    };
    Image *histogram = myHistogram.createImage();
    HistogramXImage = Pic24ToXImage(histogram->getImage24(), \
                                    histogram->getWidth(), histogram->getHeight());  
    if (HistogramPixmap != 0) {
      XFreePixmap(theDisp, HistogramPixmap);
      HistogramPixmap = 0;
    };
    int depth;
    XtVaGetValues (toplevel, XtNdepth, &depth, NULL);
    HistogramPixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(toplevel)), \
                                    HistogramXImage->width, HistogramXImage->height, depth);
    XPutImage(theDisp, HistogramPixmap, theGC, HistogramXImage, 0, 0, 0, 0, \
              HistogramXImage->width, HistogramXImage->height);
    XFlush(theDisp);

    // create histogram window if not yet available
    if (HistogramShell == NULL) {
      Arg arglist[5];
      int i = 0;
      XtSetArg(arglist[i], XmNdialogStyle, XmDIALOG_MODELESS); i++;
      XtSetArg(arglist[i], XmNautoUnmanage, True); i++;
      XtSetArg(arglist[i], XmNallowShellResize, False); i++;
      XtSetArg(arglist[i], XmNtitle, "Histogram"); i++;
      HistogramShell = XmCreateFormDialog(toplevel, "HistogramShell", arglist, i);
 
      HistogramGraphics = XtCreateManagedWidget( "HistogramGraphics",
        xmDrawingAreaWidgetClass, HistogramShell, arglist, i);

      HistogramClose = XtCreateManagedWidget( "HistogramClose",
        xmPushButtonWidgetClass, HistogramShell, NULL, 0);

      XtVaSetValues(HistogramGraphics, 
        XmNheight, 272,
	XmNwidth,  272,
        XmNbottomAttachment, XmATTACH_WIDGET, 
        XmNbottomWidget, HistogramClose, 
        NULL);

      XtAddCallback(HistogramClose, XmNactivateCallback, HistogramCloseCallback, NULL);
      XtAddCallback(HistogramGraphics, XmNexposeCallback, HistogramRepaintCallback, NULL);
      XtManageChild(HistogramShell);
    } else {

      HistogramRepaintCallback(HistogramGraphics, NULL, NULL);

    };

    // Bereich wo Pixels gesucht wurden markieren
    unsigned int offset_x, offset_y;
    offset_x = cbs->event->xbutton.x - WindowSize/2;
    offset_y = cbs->event->xbutton.y - WindowSize/2;
    Rectangle rect;
    rect.x1 = rect.y1 = 0;
    rect.x2 = rect.y2 = WindowSize;
    DrawRectangle(offset_x, offset_y, rect, blue);

  

  } else if (cbs->event->xbutton.button == Button2) {

    if (ZOOM1_ENABLED == False) {
      ZOOM1_ENABLED = True;
      ZOOM2_ENABLED = False;
      // cout << "ZOOM 1 on" << endl;
      XtVaSetValues(ZOOM1display, 
        XmNheight, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
        XmNwidth,  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
        NULL);

      if (zoom1Pixmap != 0) {
        XFreePixmap(theDisp, zoom1Pixmap);
        zoom1Pixmap = 0;
      };
      int depth;
      XtVaGetValues (ZOOM1display, XtNdepth, &depth, NULL);
      zoom1Pixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(ZOOM1display)), \
                                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, depth);

    } else {
      ZOOM1_ENABLED = False;
      // cout << "ZOOM 1 off" << endl;

    };

  } else if  (cbs->event->xbutton.button == Button3) {

    if (ZOOM2_ENABLED == False) {
      ZOOM2_ENABLED = True;
      ZOOM1_ENABLED = False;
      // cout << "ZOOM 2 on" << endl;
      XtVaSetValues(ZOOM2display, 
        XmNheight, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
        XmNwidth,  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
        NULL);

      if (zoom2Pixmap != 0) {
        XFreePixmap(theDisp, zoom2Pixmap);
        zoom2Pixmap = 0;
      };
      int depth;
      XtVaGetValues (ZOOM2display, XtNdepth, &depth, NULL);
      zoom2Pixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(ZOOM2display)), \
                                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, depth);

    } else {
      ZOOM2_ENABLED = False;
      // cout << "ZOOM 2 off" << endl;
    };

  };
 };

};



// Writes the actual Position inside the image to the Widget statusPosition
void updatePosition (unsigned int x, unsigned int y, Widget w) {

  int newX;
  int newY;
  Image *tmpImg;

  getActualImageAndPosition (x, y, tmpImg, newX, newY);

  char pos[255];
  if ( (newX < 0) || (newY < 0) ) {
    strcpy(pos, " No Position !");
  } else {
    sprintf(pos, " Position: %d / %d V: %d", \
            newX, newY, (int) (tmpImg->getGreyscale(newX, newY)));
  };

  XtVaSetValues(w, XmNvalue, pos, NULL);

};



void GraphicsInputEvent(Widget w, XtPointer client_data, XEvent *event, Boolean *dis) {

  if (event->type == MotionNotify) {

    updatePosition(event->xbutton.x, event->xbutton.y, statusPosition);

    if ((ZOOM1_ENABLED == True) || (ZOOM2_ENABLED == True)) {

      Drawable theDest;
      Window theDestWin;
      if (ZOOM1_ENABLED == True) {
        theDest = zoom1Pixmap;
        theDestWin = XtWindow(ZOOM1display);
        updatePosition(event->xbutton.x, event->xbutton.y, ZOOM1pos);
      } else {
        theDest = zoom2Pixmap;
        theDestWin = XtWindow(ZOOM2display);
        updatePosition(event->xbutton.x, event->xbutton.y, ZOOM2pos);
      };

      XImage *XZoomSource;
      int sx = event->xbutton.x - (FIXZOOM_PIXELCOUNT/2);
      int sy = event->xbutton.y - (FIXZOOM_PIXELCOUNT/2);
      unsigned int startx = (sx < 0) ? 0 : (unsigned int) sx;
      unsigned int starty = (sy < 0) ? 0 : (unsigned int) sy;
      unsigned int width, height;
      XtVaGetValues(graphics, 
        XtNheight, &height,
        XtNwidth,  &width,
        NULL);
      width = width - FIXZOOM_PIXELCOUNT;
      height = height - FIXZOOM_PIXELCOUNT;
      startx = (startx > width) ? width : startx;
      starty = (starty > height) ? height : starty;      

      XZoomSource = XGetImage(theDisp, XtWindow(graphics), startx, starty, 
                              FIXZOOM_PIXELCOUNT, FIXZOOM_PIXELCOUNT, MAXLONG, ZPixmap);

      for(int y = 0; y < FIXZOOM_PIXELCOUNT; y++) {
        for(int x = 0; x < FIXZOOM_PIXELCOUNT; x++) {
          XSetForeground(theDisp, theGC, XGetPixel(XZoomSource, x, y));
          XFillRectangle(theDisp, theDest, theGC, x*FIXZOOM_PIXELSIZE,
                         y*FIXZOOM_PIXELSIZE, FIXZOOM_PIXELSIZE, FIXZOOM_PIXELSIZE);
          };
        }; 

      XDestroyImage(XZoomSource);

      if (theDest != 0) {
        XCopyArea(theDisp, theDest, theDestWin, theGC, \
                  0, 0, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, \
                  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT, 0, 0);
      };
      XFlush(theDisp);
    };


  };
};


void ExitCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  if (delta_merge_count > 0) {

    cout << delta_merge_count << " Images merged together" << endl;
    PPMWriter myWriter("delta_merge.ppm");
    myWriter.apply(delta_merge);

  };

  exit(0);

};


void FileSetDirOKCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  XtVaGetValues(DirectorySelection, XmNdirectory, &sourceDirectory, NULL);

  XtUnmanageChild(DirectorySelection);

  char *tmpText;
  Boolean status;  
  status = XmStringGetLtoR (sourceDirectory, XmFONTLIST_DEFAULT_TAG, &tmpText);

  cout << "Dir returned: " << tmpText << endl;

};


void FileSetDirCancelCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  XtUnmanageChild(DirectorySelection);
  sourceDirectory = XmStringCreateLocalized(""); 

  cout << "No directory returned." << endl;

};


void FileSetDirCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  DirectorySelection = XmCreateFileSelectionDialog( mainManager, "DirectorySelection", NULL, 0);
  XtManageChild(DirectorySelection);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_OK_BUTTON),
                 XmNactivateCallback, FileSetDirOKCallback, NULL);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_CANCEL_BUTTON),
                 XmNactivateCallback, FileSetDirCancelCallback, NULL);

};


void HistogramCloseCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  XtUnmanageChild(HistogramShell);

  HistogramShell = NULL;

};


void DELTAdoit1Callback(Widget w, XtPointer client_data, XtPointer call_data) {

    // unkorrigierte Delta Bilder

    Filter myFilter;
    Image *bild_empty = new Image8(theImageWidth, theImageHeight);

    for (int i = 0; i < lookAheadSize-1; i++) {
      delete deltas[i];
      // deltas[i] = bild_empty->copy();
      deltas[i] = myFilter.applyDeltaImage(images[i], images[i+1], theRectangle);
    };

    int bwLimit;
    XtVaGetValues(DELTAbwLimit, XmNvalue, &bwLimit, NULL);

    if (XmToggleButtonGetState(DELTAswitchDESPECKLE)) {     
      for (int i = 0; i < lookAheadSize-1; i++) {
        myFilter.applyDespeckle(deltas[i], deltas[i]);   // default size 3*3, Limit 255
      };
    };

    XDestroyImage(firstDelta);
    firstDelta = Pic24ToXImage(deltas[0]->getImage24(), deltas[0]->getWidth(), deltas[0]->getHeight());
    XDestroyImage(secondDelta);
    secondDelta = Pic24ToXImage(deltas[1]->getImage24(), deltas[1]->getWidth(), deltas[1]->getHeight());

    paint();    // update Pixmaps in X-Server
    repaint();  // display Pixmaps on Screen

};



void DELTAdoit2Callback(Widget w, XtPointer client_data, XtPointer call_data) {

    // korrigierte Delta Bilder

    Filter myFilter;
    Image *bild_empty = new Image8(theImageWidth, theImageHeight);
 
    // 1. unkorrigierte Delta Bilder
    for (int i = 0; i < lookAheadSize-1; i++) {
      delete deltas[i];
      // deltas[i] = bild_empty->copy();
      deltas[i] = myFilter.applyDeltaImage(images[i], images[i+1], theRectangle);
    };

    // 2. jedes weitere Delta-Bild korrigierend einfliessen lassen
    int limit;
    XtVaGetValues(DELTAlimit,  XmNvalue, &limit,  NULL);
    for (int i = 1; i < lookAheadSize-1; i++) {
      myFilter.applyDeltaImageCorrection(deltas[0], images[i], images[i+1], theRectangle, limit);
    };

    // 3. Postprocessing & Pixmaps regenerieren
    int bwLimit;
    XtVaGetValues(DELTAbwLimit, XmNvalue, &bwLimit, NULL);

    // schon mal schwarze Pixels werden im linken Deltabild nicht beruecksichtigt
    // delta_merge temporaer expandieren
    Image* tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
    myFilter.applyExpandGreyscale(delta_merge, tempDeltaMerge, theRectangle);
    myFilter.applySubtractImage(deltas[0], tempDeltaMerge, theRectangle);
    //myFilter.applySubtractImage(deltas[0], delta_merge); 
    delete tempDeltaMerge;

    if (XmToggleButtonGetState(DELTAswitchBW)) { 
      myFilter.applyBlackAndWhite(deltas[0], deltas[0], bwLimit);
    };
  
    if (XmToggleButtonGetState(DELTAswitchDESPECKLE)) {     
      myFilter.applyDespeckle(deltas[0], deltas[0]);   // default size 3*3, Limit 255
    };

    XDestroyImage(firstDelta);
    firstDelta = Pic24ToXImage(deltas[0]->getImage24(), deltas[0]->getWidth(), deltas[0]->getHeight());
    XDestroyImage(secondDelta);
    secondDelta = Pic24ToXImage(deltas[1]->getImage24(), deltas[1]->getWidth(), deltas[1]->getHeight());

    paint();    // update Pixmaps in X-Server
    repaint();  // display Pixmaps on Screen

};



Rectangle generateDELTA() {

  // Delta Bilder ausfuehren
  Filter myFilter;
  Image *bild_empty = new Image8(images[0]->getWidth(), images[0]->getHeight());
  int limit, bwLimit;
  XtVaGetValues(DELTAlimit,  XmNvalue, &limit,  NULL);
  XtVaGetValues(DELTAbwLimit, XmNvalue, &bwLimit, NULL);

  // 1. unkorrigierte Delta Bilder
  for (int i = 0; i < lookAheadSize-1; i++) {
    delete deltas[i];
    //deltas[i] = bild_empty->copy();
    deltas[i] = myFilter.applyDeltaImage(images[i], images[i+1], theRectangle);
  };

  // 2. jedes weitere Delta-Bild korrigierend einfliessen lassen
  for (int i = 1; i < lookAheadSize-1; i++) {
    myFilter.applyDeltaImageCorrection(deltas[0], images[i], images[i+1], theRectangle, limit);
  };

  // schon mal schwarze Pixels werden im linken Deltabild nicht beruecksichtigt
  // delta_merge temporaer expandieren
  Image* tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
  myFilter.applyExpandGreyscale(delta_merge, tempDeltaMerge, theRectangle);
  myFilter.applySubtractImage(deltas[0], tempDeltaMerge, theRectangle);
  //myFilter.applySubtractImageMasked(deltas[0], delta_merge, mask_source);
  delete tempDeltaMerge;

  // Use OTSU to detect Black&White Threshold
  int UseDeltaMerge = false;   // True if either Fixed threshold 
                               // or Otsu showed reasonable results
  Rectangle rectotsu;
  if (XmToggleButtonGetState(DELTAswitchOTSU)) { 
    if (lastPosition.valid()) {
      unsigned int WindowSize;
      XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
      rectotsu.x1 = lastPosition.x - WindowSize/2;
      rectotsu.y1 = lastPosition.y - WindowSize/2; 
      rectotsu.x2 = lastPosition.x + WindowSize/2;
      rectotsu.y2 = lastPosition.y + WindowSize/2;
    } else {
      rectotsu.x1 = rectotsu.y1 = 0;
      rectotsu.x2 = deltas[0]->getWidth();
      rectotsu.y2 = deltas[0]->getHeight();
    };
    Histogram myHistogram;
    myHistogram.generate(deltas[0], rectotsu);
    if (XmToggleButtonGetState(ANALYSISeliminateHistogram)) {     
      myHistogram.eliminateWhitePixels();
    };

    float rmean, vari, p0, p1, a0, a1, sep;
    int otsu = myHistogram.otsudetails(rmean, vari, p0, p1, a0, a1, sep);
    // int otsu = myHistogram.otsu();

    unsigned int histWidth = myHistogram.getDelta();

    // check if the OTSU is reasonable
    unsigned int myVarianceLimit, myThresholdScale, myWidthLimit;
    XtVaGetValues (DELTAvarianceLimit, XtNvalue, &myVarianceLimit, NULL);
    XtVaGetValues (DELTAwidthLimit, XtNvalue, &myWidthLimit, NULL);
    XtVaGetValues (DELTAthresholdScale, XtNvalue, &myThresholdScale, NULL);

    // only Variance Limit
    int OtsuOK = false; // will be used later
    if ((myVarianceLimit > 0) && (myWidthLimit <= 0)) {
      if ((vari >= (float) myVarianceLimit) && (otsu >= 0)) {
        bwLimit = (int) ((float) otsu * myThresholdScale);
        OtsuOK = true;
      } else {
        bwLimit = 0;
        OtsuOK = false;
      };
    } else {

    // only Width Limit
    if ((myVarianceLimit <= 0) && (myWidthLimit > 0)) {
      if ((histWidth >= myWidthLimit) && (otsu >= 0)) {
        bwLimit = (int) ((float) otsu * ( (float) myThresholdScale / 100.0));
        OtsuOK = true;
      } else {
        bwLimit = 0;
        OtsuOK = false;
      };
    } else {

    // both Limits
    if ((myVarianceLimit > 0) && (myWidthLimit > 0)) {
      if ((vari >= (float) myVarianceLimit) && (histWidth >= myWidthLimit) \
         && (otsu >= 0)) {
        bwLimit = (int) ((float) otsu * ( (float) myThresholdScale / 100.0));
        OtsuOK = true;
      } else {
        bwLimit = 0;
        OtsuOK = false;
      };

    } else {

    // none of them -> take all
    if (otsu >= 0) {
      bwLimit = (int) ((float) otsu * ( (float) myThresholdScale / 100.0));
      OtsuOK = true;
    } else {
      bwLimit = 0;
      OtsuOK = false;
    } 
    } } };

    UseDeltaMerge = OtsuOK;
    XtVaSetValues(DELTAbwLimit, XmNvalue, bwLimit, NULL);

    cout << "[DELTAdoit3Callback] threshold = " << bwLimit << "  (otsu = " << otsu << ", vari = " << vari << ", delta = " << histWidth << ")" << endl;


  } else {
    // Fixed Threshold
    UseDeltaMerge = true;
  };


  if (XmToggleButtonGetState(DELTAswitchBW)) { 
    myFilter.applyBlackAndWhite(deltas[0], deltas[0], bwLimit);
  };


  if (XmToggleButtonGetState(DELTAswitchDESPECKLE)) {     
    myFilter.applyDespeckle(deltas[0], deltas[0]);   // default size 3*3, Limit 255
  };


  // Graustufen zu delta_merge addieren
  // nur anwenden wenn Otsu sinnvoll war oder fixer threshold
  if (UseDeltaMerge == true) {
    myFilter.applyMergeImages(delta_merge_wait[delta_merge_wait_count-1], delta_merge, rectotsu); 
    delete delta_merge_wait[delta_merge_wait_count-1];
    for(int i = delta_merge_wait_count-1; i>0; i--) {
     delta_merge_wait[i] = delta_merge_wait[i-1];
    };
    delta_merge_wait[0] = deltas[0]->copy();
    delta_merge_count++;
  };
  
  delete bild_empty;

  return rectotsu;

};


Rectangle generateANALYSIS() {

  // Analyse des linken Deltabildes
  int bwLimit;
  XtVaGetValues(DELTAbwLimit, XmNvalue, &bwLimit, NULL);
  Analysis myAnalyser;

    Rectangle rect;

    // Draw cross on left Delta image
    if (lastPosition.valid()) {
      unsigned int WindowSize;
      XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
      rect.x1 = lastPosition.x - WindowSize/2;
      rect.y1 = lastPosition.y - WindowSize/2;
      rect.x2 = lastPosition.x + WindowSize/2;
      rect.y2 = lastPosition.y + WindowSize/2;
    } else {
      rect.x1 = rect.y1 = 0;
      rect.x2 = deltas[0]->getWidth();
      rect.y2 = deltas[0]->getHeight();
    };
    // unsigned int points = myAnalyser.getNumberOfBlackPixels(deltas[0], rect);

    unsigned int points = myAnalyser.getNumberOfDarkPixels(deltas[0], rect, bwLimit);
    cout << "Anzahl dunkle Punkte links: " << points << endl;
    if (points > 0) {

      // Variante der Analyse feststellen
      Point middle;
      if (XmToggleButtonGetState(ANALYSISaverageBW)) {
        middle = myAnalyser.getCenterOfBlackPixels(deltas[0], rect, bwLimit);
      };
      if (XmToggleButtonGetState(ANALYSISmedianBW)) {
        middle = myAnalyser.getCenterOfBlackPixelsMedian(deltas[0], rect, bwLimit);
      };
      if (XmToggleButtonGetState(ANALYSISgrey)) {
        middle = myAnalyser.getCenterOfBlackPixelsGreyscale(deltas[0], rect, bwLimit);
      };
      if (XmToggleButtonGetState(ANALYSISiterativeBW)) {
        middle = myAnalyser.getCenterOfBlackPixelsIterative(deltas[0], rect, 3, bwLimit);
      };
      if (XmToggleButtonGetState(ANALYSISgreyIterative)) {
	cout << "combined" << endl;
        middle = myAnalyser.getCenterOfBlackPixelsGreyscaleIterative(deltas[0], rect, 3, bwLimit);
      };
      if (XmToggleButtonGetState(ANALYSISneighbour)) {
        DetectorNeighbour myDetector(theRectangle, bwLimit);
        middle = myDetector.apply(deltas[0], rect);
      };
     
      cout << "Mittelpunkt links: (" << middle.x << "/" << middle.y << ")";
      cout << " in Bereich (" << rect.x1 << " / " << rect.y1 << ") - (";
      cout << rect.x2 << " / " << rect.y2 << ")" << endl;
      
      // aktuelle Position wird lastPosition fuer naechstes Bild
      if ((middle.x > 0) && (middle.y > 0)) {
        lastPosition.x = middle.x;
        lastPosition.y = middle.y;
      };

      punkte[punkte_index++] = middle;

    } else {

      Point noPosition;
      noPosition.x = noPosition.y = -1;     // invalid Point
      punkte[punkte_index++] = noPosition;

    };

  return rect;

};


void DELTAdoit3Callback(Widget w, XtPointer client_data, XtPointer call_data) {


  Rectangle rectotsu = generateDELTA();


  // Delta-Bilder anzeigen
  if ((END == START) || (XmToggleButtonGetState(CMDdisplayFlag))) {
    Filter myFilter;
    Image* tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
    tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
    myFilter.applyExpandGreyscale(delta_merge, tempDeltaMerge, theRectangle);
    XDestroyImage(mergeDelta);
    mergeDelta = Pic24ToXImage(tempDeltaMerge->getImage24(), tempDeltaMerge->getWidth(), tempDeltaMerge->getHeight());
    XDestroyImage(firstDelta);
    firstDelta = Pic24ToXImage(deltas[0]->getImage24(), deltas[0]->getWidth(), deltas[0]->getHeight());
    XDestroyImage(secondDelta);
    secondDelta = Pic24ToXImage(deltas[1]->getImage24(), deltas[1]->getWidth(), deltas[1]->getHeight());
    paint();
    repaint();
  };


  Rectangle rect = generateANALYSIS();


  // Analyse darstellen
  if ((END == START) || (XmToggleButtonGetState(CMDdisplayFlag))) {
    unsigned int start_x, start_y;
    start_x = 0;
    start_y = theImageHeight + ZWISCHENRAUM; 
    DrawCross(start_x + punkte[punkte_index-1].x, start_y + punkte[punkte_index-1].y, 5, red);

    start_x = theImageWidth + ZWISCHENRAUM;
    start_y = 0;
    DrawCross(start_x + punkte[punkte_index-1].x, start_y + punkte[punkte_index-1].y, 5, red);

    // Bereich wo Pixels gesucht wurden markieren
    unsigned int offset_x, offset_y;
    offset_x = 0;
    offset_y = theImageHeight + ZWISCHENRAUM;

    DrawRectangle(offset_x, offset_y, rect, green);
    
    DrawRectangle(0, theImageHeight + ZWISCHENRAUM, rectotsu, blue);
    
    DrawPoints(XmToggleButtonGetState(ANALYSISswitchInterpolate), False);
  };

};


void DELTAdoit4Callback(Widget w, XtPointer client_data, XtPointer call_data) {

    Rectangle rect;
    if ((lastPosition.x > 0) && (lastPosition.y > 0)) {
      unsigned int WindowSize;
      XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
      rect.x1 = lastPosition.x - WindowSize/2;
      rect.y1 = lastPosition.y - WindowSize/2;
      rect.x2 = lastPosition.x + WindowSize/2;
      rect.y2 = lastPosition.y + WindowSize/2;
    } else {
      rect.x1 = rect.y1 = 0;
      rect.x2 = deltas[0]->getWidth();
      rect.y2 = deltas[0]->getHeight();
    };

    Histogram myHistogram;
    myHistogram.generate(deltas[0], rect);
    int otsu = myHistogram.otsu();
    cout << otsu << endl;
    XtVaSetValues (DELTAbwLimit, XtNvalue, otsu, NULL);
    // Bereich wo OTSU gesucht wurde markieren
    unsigned int offset_x, offset_y;
    offset_x = 0;
    offset_y = theImageHeight + ZWISCHENRAUM;
    DrawRectangle(offset_x, offset_y, rect, blue);

};


void DELTAtestCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  // deltas[0] nach Punktenen dunkler als DeltaBWlimit durchsuchen

  // Bild leeren
  repaint();

  Rectangle rect;
  if (lastPosition.valid()) {
    unsigned int WindowSize;
    XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
    WindowSize /= 2;
    rect.x1 = lastPosition.x - WindowSize;
    rect.y1 = lastPosition.y - WindowSize;
    rect.x2 = lastPosition.x + WindowSize;
    rect.y2 = lastPosition.y + WindowSize;
  } else {
    rect.x1 = rect.y1 = 0;
    rect.x2 = deltas[0]->getWidth();
    rect.y2 = deltas[0]->getHeight();
  };

  unsigned int offset_x, offset_y;
  offset_x = 0;
  offset_y = theImageHeight + ZWISCHENRAUM;

  DrawRectangle(offset_x, offset_y, rect, green);

  Pixel fg;
  fg = GetRGBColor(graphics, 0, 0, 255);
  XSetForeground(XtDisplay(graphics), theGC, fg);

  int bwLimit;
  XtVaGetValues(DELTAbwLimit, XmNvalue, &bwLimit, NULL);
  for (int x = rect.x1; x <= rect.x2; x++) {
    for (int y = rect.y1; y <= rect.y2; y++) {
      // wennn dunkler -> blauer Punkt
      if ((deltas[0])->getGreyscale(x,y) <= bwLimit) {

        cout << "Punkt: " << x << " / " << y << endl;

        XDrawPoint(XtDisplay(graphics), XtWindow(graphics), theGC, \
                   offset_x+x, offset_y+y);

      };
    };
  };


  cout << endl;

  XFlush(theDisp);

};


void ReloadRightCallback(Widget w, XtPointer client_data, XtPointer call_data) {

    delete images[2];
    images[2] = bild_ori->copy();
    XDestroyImage(rightImage);
    rightImage = Pic24ToXImage(images[2]->getImage24(), images[2]->getWidth(), images[2]->getHeight());

  paint();    // update Pixmaps in X-Server
  repaint();  // display Pixmaps on Screen

};



void ResetCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  START = END = 0;

  theRectangle.x1 = theRectangle.y1 = 0;
  theRectangle.x2 = (unsigned int) (theImageWidth - 1);
  theRectangle.y2 = (unsigned int) (theImageHeight - 1);
  lastPosition.x = lastPosition.y = -1;
  
  // empty delta_merge
  delete delta_merge;
  delta_merge = new Image8(theImageWidth, theImageHeight);
  delta_merge->setWhite();
  delta_merge_count = 0;
  for (int i = 0; i < delta_merge_wait_count; i++) {
    delete delta_merge_wait[i];
    delta_merge_wait[i] = delta_merge->copy();
  };
  XDestroyImage(mergeDelta);
  mergeDelta = Pic24ToXImage(delta_merge->getImage24(), delta_merge->getWidth(), delta_merge->getHeight());

  for (unsigned int i = 0; i <= punkte_index; i++) {
    punkte[i] = Point();
  };
  punkte_index = 0;

  paint();    // update Pixmaps in X-Server
  repaint();  // display Pixmaps on Screen

};




// fill the Pixmap's in the X-Server 
// out of the Ximage's in the X-Client
// and resize the drawable appropriate
void paint() {

  // cout << "paint started ... ";
  // cout.flush();
  // XSync(theDisp, False);  

  // clear the Pixmaps if already allocated
  if (leftPixmap != 0) {
    XFreePixmap(theDisp, leftPixmap);
    leftPixmap = 0;
  };
  if (middlePixmap != 0) {
    XFreePixmap(theDisp, middlePixmap);
    middlePixmap = 0;
  };
  if (rightPixmap != 0) {
    XFreePixmap(theDisp, rightPixmap);
    rightPixmap = 0;
  };
  if (firstPixmap != 0) {
    XFreePixmap(theDisp, firstPixmap);
    firstPixmap = 0;
  };
  if (secondPixmap != 0) {
    XFreePixmap(theDisp, secondPixmap);
    secondPixmap = 0;
  };
  if (mergePixmap != 0) {
    XFreePixmap(theDisp, mergePixmap);
    mergePixmap = 0;
  };

  int depth;
  XtVaGetValues (graphics, XtNdepth, &depth, NULL);

    // generate the Pixmaps and fill them depending on the display mode
    firstPixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                                firstDelta->width, firstDelta->height, depth);
    XPutImage(theDisp, firstPixmap, theGC, firstDelta, 0, 0, 0, 0, \
              firstDelta->width, firstDelta->height);
    secondPixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                                 secondDelta->width, secondDelta->height, depth);
    XPutImage(theDisp, secondPixmap, theGC, secondDelta, 0, 0, 0, 0, \
              secondDelta->width, secondDelta->height);
    mergePixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                                 mergeDelta->width, mergeDelta->height, depth);
    XPutImage(theDisp, mergePixmap, theGC, mergeDelta, 0, 0, 0, 0, \
              mergeDelta->width, mergeDelta->height);
   
    rightPixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                                rightImage->width, rightImage->height, depth);
    XPutImage(theDisp, rightPixmap, theGC, rightImage, 0, 0, 0, 0, \
              rightImage->width, rightImage->height);
    middlePixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                                 middleImage->width, middleImage->height, depth);
    XPutImage(theDisp, middlePixmap, theGC, middleImage, 0, 0, 0, 0, \
              middleImage->width, middleImage->height);
    leftPixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                               leftImage->width, leftImage->width, depth);
    XPutImage(theDisp, leftPixmap, theGC, leftImage, 0, 0, 0, 0, \
              leftImage->width, leftImage->height);



  // calculate size of drawable
  int width, upperHeight, lowerHeight, height;
  width = height = upperHeight = lowerHeight = 0;
    if (firstDelta->height > secondDelta->height) {
      lowerHeight = firstDelta->height;
    } else {
      lowerHeight = secondDelta->height;
    };

    width += ZWISCHENRAUM;
    width += rightImage->width;
    if (rightImage->height > upperHeight) {
      upperHeight = rightImage->height;
    };

    width += ZWISCHENRAUM;
    width += middleImage->width;
    if (middleImage->height > upperHeight) {
      upperHeight = middleImage->height;
    };

    width += leftImage->width;
    if (leftImage->height > upperHeight) {
      upperHeight = leftImage->height;
    };

    if (lowerHeight > 0) {
      height = upperHeight + ZWISCHENRAUM + lowerHeight;
    } else {
      height = upperHeight;
    };

  XtVaSetValues(graphics, 
      XmNheight, height, 
      XmNwidth, width,
      NULL);

  // cout << "paint ended." << endl;
  // cout.flush();

};



// redisplay the Pixmap's on the screen
void repaint() {

  // cout << "repaint started ... ";
  // cout.flush();
  // XSync(theDisp, False);

  XClearWindow(theDisp, XtWindow(graphics));

    int upperHeight = theImageHeight + ZWISCHENRAUM;
   
    XCopyArea(theDisp, firstPixmap, XtWindow(graphics), theGC, 0, 0, \
              firstDelta->width, firstDelta->height, \
              0, upperHeight);
    XCopyArea(theDisp, secondPixmap, XtWindow(graphics), theGC, 0, 0, \
              secondDelta->width, secondDelta->height, \
              theImageWidth + ZWISCHENRAUM, upperHeight);
    XCopyArea(theDisp, mergePixmap, XtWindow(graphics), theGC, 0, 0, \
              mergeDelta->width, mergeDelta->height, \
              2*(theImageWidth + ZWISCHENRAUM), upperHeight);

    XCopyArea(theDisp, rightPixmap, XtWindow(graphics), theGC, 0, 0, rightImage->width, rightImage->height,
              2*(theImageWidth + ZWISCHENRAUM), 0);
    XCopyArea(theDisp, middlePixmap, XtWindow(graphics), theGC, 0, 0, middleImage->width, middleImage->height,
              theImageWidth + ZWISCHENRAUM, 0);
    XCopyArea(theDisp, leftPixmap, XtWindow(graphics), theGC, 0, 0, leftImage->width, leftImage->height, 0, 0);


  XFlush(theDisp);

  // cout << "repaint ended." <<endl;
  // cout.flush();

};



void DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, Color col) {

  Pixel fg;
  fg = GetRGBColor(graphics, col.r, col.g, col.b);
  XSetForeground(XtDisplay(graphics), theGC, fg);

  XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
            x1, y1, x2, y2);

};


void DrawRectangle(unsigned int start_x, unsigned int start_y, Rectangle rectangle, Color col) {

  if (!rectangle.valid()) {
    return;
  };

  Rectangle rect;
  rect.x1 = rectangle.x1 + start_x;
  rect.x2 = rectangle.x2 + start_x;
  rect.y1 = rectangle.y1 + start_y;
  rect.y2 = rectangle.y2 + start_y;

  DrawLine(rect.x1, rect.y1, rect.x2, rect.y1, col);
  DrawLine(rect.x1, rect.y2, rect.x2, rect.y2, col);
  DrawLine(rect.x1, rect.y1, rect.x1, rect.y2, col);
  DrawLine(rect.x2, rect.y1, rect.x2, rect.y2, col);

};


void DrawCross(const unsigned int x, const unsigned int y, const unsigned int size, Color col) {

  Pixel fg;
  fg = GetRGBColor(graphics, col.r, col.g, col.b);
  XSetForeground(XtDisplay(graphics), theGC, fg);

  XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
            x-size, y, x+size, y);
  XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
            x, y - size, x, y+size);

};


// Set a RGB Color inside GC
Pixel GetRGBColor(Widget w, byte r, byte g, byte b) {

 Display   *display;
 Colormap  cmap;
 XColor    alloc_color;

 display = XtDisplay(w);                    /* Get display pointer   */

 XtVaGetValues(w, XtNcolormap, &cmap, NULL );

 alloc_color.red   = ((r+1) * 256) - 1;        /* Red value, 0 - 65535  */
 alloc_color.green = ((g+1) * 256) - 1;        /* Green value, 0 - 65535*/
 alloc_color.blue  = ((b+1) * 256) - 1;        /* Blue value, 0 - 65535 */
 alloc_color.flags =  DoRed|DoGreen|DoBlue;      /* Set all */

 if (XAllocColor(display, cmap, &alloc_color)){  /* Success */
    return alloc_color.pixel;
 }
 else {                /* Failed  */
    printf("Can't allocate color!\n");     /* Print warning */
    return (Pixel)NULL;                    /* Return NULL   */
 }

};



void GAMMAdoitCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  Filter myFilter;
  double teiler;

  Matrix<int> box(5,5);
  box[0][0] = 1; box[1][0] =  4; box[2][0] =  6; box[3][0] =  4; box[4][0] =  1;
  box[0][1] = 4; box[1][1] = 16; box[2][1] = 24; box[3][1] = 16; box[4][1] =  4;
  box[0][2] = 6; box[1][2] = 24; box[2][2] = 36; box[3][2] = 24; box[4][2] =  6;
  box[0][3] = 4; box[1][3] = 16; box[2][3] = 24; box[3][3] = 16; box[4][3] =  4;
  box[0][4] = 1; box[1][4] =  4; box[2][4] =  6; box[3][4] =  4; box[4][4] =  1;
  teiler = 1.0 / 256.0;

  /*
  Matrix<int> box(7,7);
  box[0][0] =  1; box[1][0] =   6; box[2][0] =  15; box[3][0] =  20; box[4][0] =  15; box[5][0] =   6; box[6][0] =  1;
  box[0][1] =  6; box[1][1] =  36; box[2][1] =  90; box[3][1] = 120; box[4][1] =  90; box[5][1] =  36; box[6][1] =  6;
  box[0][2] = 15; box[1][2] =  90; box[2][2] = 225; box[3][2] = 300; box[4][2] = 225; box[5][2] =  90; box[6][2] = 15;
  box[0][3] = 20; box[1][3] = 120; box[2][3] = 300; box[3][3] = 400; box[4][3] = 300; box[5][3] = 120; box[6][3] = 20;
  box[0][4] = 15; box[1][4] =  90; box[2][4] = 225; box[3][4] = 300; box[4][4] = 225; box[5][4] =  90; box[6][4] = 15;
  box[0][5] =  6; box[1][5] =  36; box[2][5] =  90; box[3][5] = 120; box[4][5] =  90; box[5][5] =  36; box[6][5] =  6;
  box[0][6] =  1; box[1][6] =   6; box[2][6] =  15; box[3][6] =  20; box[4][6] =  15; box[5][6] =   6; box[6][6] =  1;
  teiler = 1.0 / 4096.0;  
  */

  // boxbild = filter.applyMatrixCut(quelle, box, teiler, 0);

    myFilter.applyMatrixCut(bild_ori, images[2], box, teiler, 0);
    XDestroyImage(rightImage);
    rightImage = Pic24ToXImage(images[2]->getImage24(), images[2]->getWidth(), images[2]->getHeight());

  paint();    // update Pixmaps in X-Server
  repaint();  // display Pixmaps on Screen

};










// flexible box in image to select region out of text 

unsigned int start_x, start_y;
unsigned int last_x, last_y;
Boolean rubberbanding = False;



void XorRectangle(Widget w, unsigned int x1, unsigned int y1, \
                            unsigned int x2, unsigned int y2) {


  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, x1, y1, x2, y1 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, x2, y1, x2, y2 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, x2, y2, x1, y2 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, x1, y2, x1, y1 );

}


void StoreObject() {       

  int sx, sy, lx, ly;
  Image* img;
  getActualImageAndPosition (start_x, start_y, img, sx, sy);
  getActualImageAndPosition (last_x, last_y, img, lx, ly);

  if ((lx == sx) || (ly == sy)) {
    cout << "No rectangle selected !!!" << endl;
    return;
  };

  cout << "Resulting box: (" << sx << "/" << sy << ") - (" << lx << "/" << ly << ")" << endl;
  if (sx < lx) {
    theRectangle.x1 = sx;
    theRectangle.x2 = lx;
  } else {
    theRectangle.x1 = lx;
    theRectangle.x2 = sx;
  };
  if (sy < ly) {
    theRectangle.y1 = sy;
    theRectangle.y2 = ly;
  } else {
    theRectangle.y1 = ly;
    theRectangle.y2 = sy;
  };

  
  unsigned int WindowSize;
  XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
  lastPosition.x = theRectangle.x1 + WindowSize/2;
  lastPosition.y = (theRectangle.y1 + theRectangle.y2)/2;


  // Maske generieren
  Filter myFilter;
  Image* ori = myImageSource.getImageRelative();
  delete mask_source;
  mask_source = new Image8(ori->getWidth(), ori->getHeight());
  myFilter.applyRotate180(ori, mask_source);
  PPMWriter myPPMWriter1("mask_source.ppm");
  myPPMWriter1.apply(mask_source);
  delete maske;
  delete ori;

  FilterExpandBlack myFilterExpandBlack(expandCount, theRectangle);
  maske = myFilterExpandBlack.apply(mask_source);

  PPMWriter myPPMWriter("mask.ppm");;
  myPPMWriter.apply(maske);

  FilterCutMask myFilterCutMask(maske);
  Image* lastImage = myFilterCutMask.apply(mask_source);
  PPMWriter myPPMWriter2("lastImage.ppm");;
  myPPMWriter2.apply(lastImage);
  delete lastImage;

  ChangeScrollPosition(CMDscroll, actualImage);
 
};


void StartRubber(Widget w, XtPointer client_data, XtPointer call_data) {
 /* Check call_data for button press event */
 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*)call_data;
 if ((cbs->event->type == ButtonPress) 
         && (cbs->event->xbutton.button == Button1) ) {
   rubberbanding = True;
   start_x = cbs->event->xbutton.x;
   start_y = cbs->event->xbutton.y;
   last_x = start_x;
   last_y = start_y; 

   XGCValues gcvals;
   unsigned long gcvals_mask;
   Pixel fg, bg;
   XtVaGetValues (graphics, \
      XtNforeground, &fg, \
      XtNbackground, &bg, \
      NULL);
   gcvals.foreground = bg ^ fg;
   gcvals.background = 0L;
   gcvals.function = GXxor;
   gcvals_mask = GCForeground | GCBackground | GCFunction;
   xorGC = XCreateGC(XtDisplay(graphics), XtWindow(graphics), gcvals_mask, &gcvals);

   // XSetFunction(XtDisplay(graphics), theGC, GXxor);
   // XSetForeground(XtDisplay(graphics), theGC, fore ^ back);
   // XSetBackground(XtDisplay(graphics), theGC, 0L);

   XorRectangle(w, start_x, start_y, last_x, last_y);
 }
};


void Rubberband(Widget w, XtPointer client_data, XEvent *event, Boolean *dis) {
 if ((event->type == MotionNotify) && (rubberbanding)) {
   XorRectangle(w, start_x, start_y, last_x, last_y);
   last_x = event->xmotion.x;           /* Get latest pointer coords */
   last_y = event->xmotion.y;
   XorRectangle(w, start_x, start_y, last_x, last_y);
 } 
}


void EndRubber(Widget w, XtPointer client_data, XtPointer call_data) {
 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*)call_data;
 if ((cbs->event->type == ButtonRelease)  /* Finish this object      */ 
     && (cbs->event->xbutton.button == 1) ) { 
   rubberbanding = False;               /* Stop rubberbanding        */
   XorRectangle(w, start_x, start_y, last_x, last_y);
   last_x = cbs->event->xbutton.x;      /* Get final pointer         */
   last_y = cbs->event->xbutton.y;      /*  coordinates              */
   StoreObject();                       /* Put in display list       */
 }
}

void setStart(Widget w, XtPointer client_data, XtPointer call_data) {
  START = actualImage;
};

void setEnd(Widget w, XtPointer client_data, XtPointer call_data) {

  END = actualImage;

  cout << "Calculating Ink Positions from " << START << " to " << END << endl;

  ChangeScrollPosition(CMDscroll, START-1);

  ofstream *datei;
  if (XmToggleButtonGetState(DELTAswitchHTMLextended)) {
    system("rm reporting/*.ppm; rm reporting/*.gif; rm reporting/*.html");
    datei = new ofstream("reporting/HistogramCheck.html");
    unsigned int WindowSize, varianceLimit, thresholdScale, widthLimit;
    XtVaGetValues (DELTAvarianceLimit, XtNvalue, &varianceLimit, NULL);
    XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
    XtVaGetValues (DELTAthresholdScale, XtNvalue, &thresholdScale, NULL);
    XtVaGetValues (DELTAwidthLimit, XtNvalue, &widthLimit, NULL);
    HTMLheader(datei, varianceLimit, WindowSize, float(thresholdScale) / 100.0, widthLimit);
  };

  float minVari, maxVari;
  minVari = MAXFLOAT;
  maxVari = MINFLOAT; 

  while (actualImage <= END) {
    NextImageCallback(CMDnext, NULL, NULL);
    DELTAdoit3Callback(DELTAdoit3, NULL, NULL);
    if (XmToggleButtonGetState(DELTAswitchHTMLextended)) {
      // HTMLentry(ofstream *datei, unsigned int line, Image* DeltaImage, Point lastPos, Point newPos);
      HTMLentry(datei, actualImage, deltas[0], lastPosition, punkte[punkte_index-1], minVari, maxVari);
    };
  };

  if (XmToggleButtonGetState(DELTAswitchHTMLextended)) {
    HTMLfooter(datei, minVari, maxVari);
  };

  ChangeScrollPosition(CMDscroll, actualImage);
  Filter myFilter;
  Image* tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
  tempDeltaMerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
  myFilter.applyExpandGreyscale(delta_merge, tempDeltaMerge, theRectangle);
  XDestroyImage(mergeDelta);
  mergeDelta = Pic24ToXImage(tempDeltaMerge->getImage24(), tempDeltaMerge->getWidth(), tempDeltaMerge->getHeight());
  paint();
  repaint();

  DrawPoints(XmToggleButtonGetState(ANALYSISswitchInterpolate), False);

  START = END = 0;

};


Image* DoPreprocessing(Image* source) {

  // Image *result1 = new Image8(source->getWidth(), source->getHeight());
  Image *result2 = new Image8(source->getWidth(), source->getHeight());
  Image *result3 = new Image8(source->getWidth(), source->getHeight());

  int dummy;

  // FilterGreyscale myFilterGreyscale;
  // result1 = myFilterGreyscale.apply(source);

  FilterRotate180 myFilterRotate180;
  result2 = myFilterRotate180.apply(source);

  FilterCutMask myFilterCutMask(maske);
  result3 = myFilterCutMask.apply(result2);

  delete source;
  //delete result1;
  delete result2;
  return result3;

};


void ChangeScrollPosition(Widget w, int pos) {

  int value, min, max, position;
  XtVaGetValues(w, \
      XmNminimum, &min,
      XmNmaximum, &max,
      XmNvalue, &value, 
      NULL);
  position = pos;
  if (pos < min) position = min;
  if (pos > max) position = max;
  XmScaleSetValue(w, position);

  Image* bild_empty = new Image8(theImageWidth, theImageHeight);

  // Verschiebung feststellen
  actualImage = position;
  myImageSource.setPosition(actualImage);
  switch (position-value) {
  case 1:  // 1 nach rechts
    delete images[0];
    for(int i=0; i < lookAheadSize; i++) {
      images[i] = images[i+1];
    };
    images[lookAheadSize-1] = DoPreprocessing(myImageSource.getImageRelative(lookAheadSize-2));
    XDestroyImage(leftImage);
    leftImage = middleImage;
    middleImage = rightImage;
    rightImage = Pic24ToXImage(images[2]->getImage24(), images[2]->getWidth(), images[2]->getHeight());
    delete bild_ori;
    bild_ori = (images[2])->copy();
    delete deltas[0];
    for(int i=0; i < lookAheadSize - 1; i++) {
      deltas[i] = deltas[i+1];
    };
    deltas[lookAheadSize-2] = bild_empty->copy();
    XDestroyImage(firstDelta);
    firstDelta = secondDelta;
    secondDelta = Pic24ToXImage(deltas[1]->getImage24(), deltas[1]->getWidth(), deltas[1]->getHeight());
    break;

  case -1: // 1 nach links
    delete images[lookAheadSize-1];
    for(int i = lookAheadSize-1; i > 0; i--) {
      images[i] = images[i-1];
    };
    images[0] = DoPreprocessing(myImageSource.getImageRelative(-1));
    XDestroyImage(rightImage);
    rightImage = middleImage;
    middleImage = leftImage;
    leftImage = Pic24ToXImage(images[0]->getImage24(), images[0]->getWidth(), images[0]->getHeight());
    delete bild_ori;
    bild_ori = (images[2])->copy();
    delete deltas[lookAheadSize-2];
    for(int i = lookAheadSize-2; i > 0; i--) {
      deltas[i] = deltas[i-1];
    };
    deltas[0] = bild_empty->copy();
    XDestroyImage(secondDelta);
    secondDelta = firstDelta;
    firstDelta = Pic24ToXImage(deltas[0]->getImage24(), deltas[0]->getWidth(), deltas[0]->getHeight());
    break;

  default: // sonst
    // Arrays  images & deltas  initialisieren
    for (int i = 0; i < lookAheadSize; i++) {
      images[i] = DoPreprocessing(myImageSource.getImageRelative(i-1));
    };
    delete bild_ori;
    bild_ori = (images[2])->copy();
    for (int i = 0; i < lookAheadSize - 1; i++) {
      deltas[i] = new Image8(theImageWidth, theImageHeight);
    };
    XDestroyImage(leftImage);
    XDestroyImage(middleImage);
    XDestroyImage(rightImage);
    leftImage = Pic24ToXImage(images[0]->getImage24(), images[0]->getWidth(), images[0]->getHeight());
    middleImage = Pic24ToXImage(images[1]->getImage24(), images[1]->getWidth(), images[1]->getHeight());
    rightImage = Pic24ToXImage(images[2]->getImage24(), images[2]->getWidth(), images[2]->getHeight());
    XDestroyImage(firstDelta);
    XDestroyImage(secondDelta);
    firstDelta = Pic24ToXImage(deltas[0]->getImage24(), deltas[0]->getWidth(), deltas[0]->getHeight());
    secondDelta = Pic24ToXImage(deltas[1]->getImage24(), deltas[1]->getWidth(), deltas[1]->getHeight());

  };

  delete bild_empty;

  if ((END == START) || (END == 0) || (XmToggleButtonGetState(CMDdisplayFlag))) {
    paint();    // update Pixmaps in X-Server
    repaint();  // display Pixmaps on Screen
  };

};




void writePoints2File(char* file, Point* points, unsigned int n, char* comments, 
                      unsigned int width, unsigned int height, unsigned int start, 
                      unsigned int end, unsigned int step) {


  ofstream ofs = file;
  ofs << "# " << comments << endl;

  ofs << "Step:   " << step << endl;
  ofs << "Start:  " << start << endl;
  ofs << "Stop:   " << end << endl;
  ofs << "Width:  " << width << endl;
  ofs << "Height: " << height << endl;
  ofs << endl;

  unsigned int i = 0;
  for (unsigned int pos = start+2; pos <= end; pos+=step) {
    ofs << pos << " - " << i << ": " << points[i].x << " / " << points[i].y << endl;
    i++;
  };

  cout << endl;
  cout << "# the end." << endl;

};




double dist(Point p1, Point p2) {

  return sqrt( (p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y) );

};



void DrawPoints(Boolean interpolate, Boolean stepByStep) {

  ofstream datei("points.txt");

  cout << "# diplom" << endl;
  datei << "Step:   1" << endl;
  datei << "Start:  " << START << endl;
  datei << "Stop:   " << END << endl;
  datei << "Width:  " << theImageWidth << endl;
  datei << "Height: " << theImageHeight << endl;
  datei << "Rectangle: " << theRectangle.x1 << " " << theRectangle.y1 << " " ;
  datei << theRectangle.x2 << " " << theRectangle.y2 << endl;
  datei << "#" << endl;

  Point lastPoint;
  int dummy;

  int minInterpolate, maxInterpolate;
  XtVaGetValues(ANALYSISinterpolateMin, \
      XmNvalue, &minInterpolate,
      NULL); 
  XtVaGetValues(ANALYSISinterpolateMax, \
      XmNvalue, &maxInterpolate,
      NULL); 

  for (unsigned int i = 0; i < punkte_index; i++) {
  
    if (stepByStep == True) {
      cout << "Press ENTER to continue ... ";
      dummy = cin.get();
      cout << endl;
    };

    cout << i << ": ";
    datei << i << ": ";

    if (punkte[i].valid()) {

      if (lastPoint.valid()) {
        if (dist(lastPoint,punkte[i]) <= (double) maxInterpolate) {
          DrawLine(lastPoint.x, lastPoint.y, punkte[i].x, punkte[i].y, blue);
          cout << "down (" << punkte[i].x << "/" << punkte[i].y << ")" << endl;
          datei << punkte[i].x << " / " << punkte[i].y << endl;
	} else {
          cout << "down (" << punkte[i].x << "/" << punkte[i].y << ") suppressed dist " << dist(lastPoint,punkte[i]) << endl;
          datei << punkte[i].x << " / " << punkte[i].y << endl;
	}
      } else {
        cout << "down (" << punkte[i].x << "/" << punkte[i].y << ")" << endl;
        datei << punkte[i].x << " / " << punkte[i].y << endl;
      };
      lastPoint = punkte[i];

    } else {

      if (interpolate && lastPoint.valid()) {
        if (punkte[i+1].valid() && (dist(lastPoint,punkte[i+1]) <= (double) maxInterpolate)) {
          Point newPoint1;
          newPoint1.x = (punkte[i+1].x + lastPoint.x) / 2;
          newPoint1.y = (punkte[i+1].y + lastPoint.y) / 2;
          cout << "interpolate (" << newPoint1.x << "/" << newPoint1.y;
          cout << ") dist: " << dist(punkte[i+1],lastPoint) << endl;
          datei << newPoint1.x << " / " << newPoint1.y << endl;
          DrawLine(lastPoint.x, lastPoint.y, newPoint1.x, newPoint1.y, blue);        
          lastPoint = newPoint1;
	} else {
          if (punkte[i+2].valid() && (dist(lastPoint,punkte[i+2]) <= (double) minInterpolate)) {
            Point newPoint1, newPoint2;
            newPoint1.x = (punkte[i+2].x + 2*lastPoint.x) / 3;
            newPoint1.y = (punkte[i+2].y + 2*lastPoint.y) / 3;
            newPoint2.x = (2*punkte[i+2].x + lastPoint.x) / 3;
            newPoint2.y = (2*punkte[i+2].y + lastPoint.y) / 3;
            cout << "interpolate (" << newPoint1.x << "/" << newPoint1.y << ")";
            cout << " dist: " << dist(punkte[i+2],lastPoint) << endl;
            datei << newPoint1.x << " / " << newPoint1.y <<  endl;
            i++;
            cout << i << ": ";
            datei << i << ": ";
            cout << "interpolate (" << newPoint2.x << "/" << newPoint2.y << ")" << endl;
            datei << newPoint2.x << " / " << newPoint2.y <<  endl;
            DrawLine(lastPoint.x, lastPoint.y, newPoint1.x, newPoint1.y, blue);        
            DrawLine(newPoint1.x, newPoint1.y, newPoint2.x, newPoint2.y, blue);        
            lastPoint = newPoint2;
          } else {
            datei << " ";
            cout << "up (don't interpolate) ";
            if (punkte[i+2].valid() && dist(lastPoint,punkte[i+2]) > (double) minInterpolate) {
              datei << "";
              cout << dist(lastPoint,punkte[i+2]) << " > " << minInterpolate;
	    };
            cout << endl;
            datei << endl;
            lastPoint = punkte[i];
	  };
	};
      } else { 
        datei << "" << endl;
        cout << "up" << endl;
        lastPoint = punkte[i];
      };

    };    
    
  };
    
  cout << endl;
  datei << endl;
  datei.close();

};





void RedrawLinesCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  repaint();
  DrawPoints(XmToggleButtonGetState(ANALYSISswitchInterpolate), False);

};


void RedrawLinesStepCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  repaint();
  DrawPoints(XmToggleButtonGetState(ANALYSISswitchInterpolate), True);

};



void HTMLheader(ofstream *datei, unsigned int VariLimit, unsigned int WindowSize, float ThresholdScale, unsigned int WidthLimit) {
  (*datei) << "<HTML>" << endl;
  (*datei) << "<HEAD><H1>Check of Delta Image, Otsu, Histogram and resulting ink position</H1></HEAD>" << endl;
  (*datei) << "<BODY>" << endl;
  (*datei) << "<HR>" << endl;
  (*datei) << "<P>" << endl;
  (*datei) << "Variance Limit: " << VariLimit << "<BR>" << endl;
  (*datei) << "Width Limit: " << WidthLimit << "<BR>" << endl;
  (*datei) << "Window Size: " << WindowSize << "<BR>" << endl;
  (*datei) << "Threshold Scale: " << ThresholdScale << "<BR>" << endl;
  (*datei) << "<P>" << endl;
  (*datei) << "<TABLE border = 2>" << endl;
  (*datei) << "<TR><TD>Nr</TD><TD>Histogram Data</TD><TD>Greyscale with Center</TD><TD>B&W with Median</TD><TD>B&White after Variance Criterium</TD><TD>Histogram whitout 255er data</TD><TD>Source Image Greyscale</TD><TD>Delta Merge</TD></TR>" << endl;
};


void HTMLfooter(ofstream *datei, float minVari, float maxVari) {
  (*datei) << "</TABLE>" << endl;
  (*datei) << "<P>" << endl;
  (*datei) << "Largest rejected Variance: " << maxVari << "<BR>" << endl;
  (*datei) << "Smallest accepted Variance: " << minVari << "<BR>" << endl;
  (*datei) << "<P>" << endl;
  (*datei) << "<HR>" << endl;
  (*datei) << "created by <A HREF=\"mailto:Thomas@von.Siebenthal.ch\">Thomas von Siebenthal</A>" << endl;
  (*datei) << "</BODY>" << endl;
  (*datei) << "</HTML>" << endl;
};


void HTMLentry(ofstream *datei, unsigned int line, Image* DeltaImage, Point lastPos, Point newPos, float &minVari, float &maxVari) {

  if (lastPos.valid() == False) {
    return;
  };


  (*datei) << "<TR>" << endl;
  // rows consits of Numer, Text and Image
  (*datei) << "  <TD>" << line << "</TD>";

  Rectangle rect;
  unsigned int WindowSize;
  XtVaGetValues (ANALYSISwinSize, XtNvalue, &WindowSize, NULL);
  rect.x1 = lastPos.x - WindowSize/2;
  rect.y1 = lastPos.y - WindowSize/2; 
  rect.x2 = lastPos.x + WindowSize/2;
  rect.y2 = lastPos.y + WindowSize/2;
  Histogram myHistogram;
  myHistogram.generate(deltas[0], rect);
  if (XmToggleButtonGetState(ANALYSISeliminateHistogram)) {     
    myHistogram.eliminateWhitePixels();
  };
  unsigned int delta = myHistogram.getDelta();
  (*datei) << "<TD>";
  float rmean, vari, p0, p1, a0, a1, sep;
  int otsu = myHistogram.otsudetails(rmean, vari, p0, p1, a0, a1, sep);
  (*datei) << "Threshold: " << otsu << "->";
  unsigned int thresholdScale;
  XtVaGetValues (DELTAthresholdScale, XtNvalue, &thresholdScale, NULL);
  otsu = (int) ((float) otsu * ((float) thresholdScale / 100.0) );
  (*datei) << otsu << "<BR>";
  (*datei) << "Rmean: " << setprecision(3) << rmean << "<BR>";
  (*datei) << "Vari: " << setprecision(3) << vari << "<BR>";
  (*datei) << "Diff: " << setprecision(3) << fabs(otsu - rmean) << "<BR>";
  (*datei) << "last Pos: (" << lastPos.x << " / " << lastPos.y << " ) <BR>";
  (*datei) << "<BR>";
  (*datei) << "P0: " << setprecision(3) << p0 << "<BR>";
  (*datei) << "P1: " << setprecision(3) << p1 << "<BR>";
  (*datei) << "A0: " << setprecision(3) << a0 << "<BR>";
  (*datei) << "A1: " << setprecision(3) << a1 << "<BR>";
  (*datei) << "Sep: " << setprecision(3) << sep << "<BR>";
  (*datei) << "<BR>"; 
  (*datei) << "Delta: " << delta << "<BR>";
  (*datei) << "</TD>";
  Image *histogram = myHistogram.createImage();
  Color Green(0,255,0);
  for (int y=0; y<=255; y++) {
    histogram->setColor(otsu,y,Green);
  };
  char filename[255];
  sprintf(filename, "reporting/h%.*d.ppm", 2, line);
  PPMWriter myWriterHisto(filename);
  myWriterHisto.apply(histogram);
  delete histogram;
  char shell[255];
  sprintf(shell, "ppmtogif reporting/h%.*d.ppm > reporting/h%.*d.gif &", 2, line, 2, line);
  system(shell);




  unsigned int xx, yy;
  xx = yy = 0;
  Color Red(255,0,0);
  Image *deltaimage = new Image24(WindowSize+1, WindowSize+1);
  for (int x = rect.x1; x <= rect.x2; x++) {
    yy = 0;
    for (int y = rect.y1; y <= rect.y2; y++) {
      if ((newPos.x == x) && (newPos.y == y)) {
        deltaimage->setColor(xx,yy,Red);
      } else {
        deltaimage->setGreyscale(xx,yy,deltas[0]->getGreyscale(x,y));
      };
      yy++;
    };
    xx++;
  };
  sprintf(filename, "reporting/d%.*d.ppm", 2, line);
  PPMWriter myWriterDelta(filename);
  myWriterDelta.apply(deltaimage);
  delete deltaimage;
  sprintf(shell, "ppmtogif -interlace reporting/d%.*d.ppm > reporting/d%.*d.gif &", 2,  line, 2, line);
  system(shell);


  xx = yy = 0;
  Image *bwimage = new Image24(WindowSize+1, WindowSize+1);
  if (otsu < 0) {
    bwimage->setWhite();
  } else {
    for (int x = rect.x1; x <= rect.x2; x++) {
      yy = 0;
      for (int y = rect.y1; y <= rect.y2; y++) {
        if (deltas[0]->getGreyscale(x,y) <= otsu) {
          bwimage->setGreyscale(xx,yy,0); 
        } else {
          bwimage->setGreyscale(xx,yy,deltas[0]->getGreyscale(x,y));
        };
        yy++;
      };
      xx++;
    };
  };
  Analysis myAnalyser;
  Point bwPos;
  if (myAnalyser.getNumberOfBlackPixels(bwimage) > 0) { 
    Rectangle tmpRect;
    tmpRect.x1 = tmpRect.y1 = 0;
    tmpRect.x2 = tmpRect.y2 = WindowSize;
    bwPos = myAnalyser.getCenterOfBlackPixelsIterative(bwimage, tmpRect, 3, otsu);
    bwimage->setColor(bwPos.x, bwPos.y, Green);
  };
  sprintf(filename, "reporting/b%.*d.ppm", 2, line);
  PPMWriter myWriterBW(filename);
  myWriterBW.apply(bwimage);
  sprintf(shell, "ppmtogif -interlace reporting/b%.*d.ppm > reporting/b%.*d.gif &", 2,  line, 2, line);
  system(shell);

  
  Image *corrimage;
  unsigned int varianceLimit;
  XtVaGetValues (DELTAvarianceLimit, XtNvalue, &varianceLimit, NULL);

  /*
  if ((vari >= (float) varianceLimit) && (otsu >= 0)) {
    if (vari < minVari) { minVari = vari; };
    corrimage = bwimage->copy();
  } else {
    if (vari > maxVari) { maxVari = vari; };
    corrimage = new Image8(WindowSize+1, WindowSize+1);
    corrimage->setWhite();
  };
  */

  unsigned int myWidthLimit;
  XtVaGetValues (DELTAwidthLimit, XtNvalue, &myWidthLimit, NULL);
  if ((delta >= myWidthLimit) && (otsu >= 0)) {
    if (delta < minVari) { minVari = delta; };
    corrimage = bwimage->copy();
  } else {
    if (delta > maxVari) { maxVari = delta; };
    corrimage = new Image8(WindowSize+1, WindowSize+1);
    corrimage->setWhite();
  };

  sprintf(filename, "reporting/c%.*d.ppm", 2, line);
  PPMWriter myWriterCorr(filename);
  myWriterCorr.apply(corrimage);
  delete bwimage;
  delete corrimage;
  sprintf(shell, "ppmtogif -interlace reporting/c%.*d.ppm > reporting/c%.*d.gif &", 2,  line, 2, line);
  system(shell);
  

  xx = yy = 0;
  Image *srcimage = new Image24(WindowSize+1, WindowSize+1);
  for (int x = rect.x1; x <= rect.x2; x++) {
    yy = 0;
    for (int y = rect.y1; y <= rect.y2; y++) {
      //if ((newPos.x == x) && (newPos.y == y)) {
      //  srcimage->setColor(xx,yy,Green);
      //} else {
        if ((lastPos.x == x) && (lastPos.y == y)) {
          srcimage->setColor(xx,yy,Red);
	} else {
          srcimage->setGreyscale(xx,yy,images[1]->getGreyscale(x,y));
	};
      //};
      yy++;
    };
    xx++;
  };
  if (bwPos.valid()) {
    srcimage->setColor(bwPos.x,bwPos.y,Green);
  };
  sprintf(filename, "reporting/s%.*d.ppm", 2, line);
  PPMWriter myWriterSource(filename);
  myWriterSource.apply(srcimage);
  delete srcimage;
  sprintf(shell, "ppmtogif -interlace reporting/s%.*d.ppm > reporting/s%.*d.gif &", 2,  line, 2, line);
  system(shell);


  xx = yy = 0;
  Image* tmpmerge = new Image8(delta_merge->getWidth(), delta_merge->getHeight());
  Filter myFilter;
  myFilter.applyExpandGreyscale(delta_merge, tmpmerge, rect);
  Image *mergeimg = new Image24(WindowSize+1, WindowSize+1);
  for (int x = rect.x1; x <= rect.x2; x++) {
    yy = 0;
    for (int y = rect.y1; y <= rect.y2; y++) {
      //if ((newPos.x == x) && (newPos.y == y)) {
      //  mergeimg->setColor(xx,yy,Green);
      //} else {
        if ((lastPos.x == x) && (lastPos.y == y)) {
          mergeimg->setColor(xx,yy,Red);
	} else {
          mergeimg->setGreyscale(xx,yy,tmpmerge->getGreyscale(x,y));
	};
      //};
      yy++;
    };
    xx++;
  };
  if (bwPos.valid()) {
    mergeimg->setColor(bwPos.x,bwPos.y,Green);
  };
  sprintf(filename, "reporting/m%.*d.ppm", 2, line);
  PPMWriter myWriterMerge(filename);
  myWriterMerge.apply(mergeimg);
  delete mergeimg;
  delete tmpmerge;
  sprintf(shell, "ppmtogif -interlace reporting/m%.*d.ppm > reporting/m%.*d.gif &", 2,  line, 2, line);
  system(shell);


  char url[255];
  sprintf(url, "<IMG SRC=\"d%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  sprintf(url, "<IMG SRC=\"b%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  sprintf(url, "<IMG SRC=\"c%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  sprintf(url, "<IMG SRC=\"h%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  sprintf(url, "<IMG SRC=\"s%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  sprintf(url, "<IMG SRC=\"m%.*d.gif\" width=256 height=256>", 2, line);
  (*datei) << "<TD>" << url << "</TD>";

  (*datei) << "</TR>" << endl;

};














