
// Header Files for Motif
#include <X11/Intrinsic.h>     // The Intrinsics                   
#include <Xm/Xm.h>             // Motif symbols                    
#include <Xm/MainW.h>          // XmMainWindow widget              
#include <Xm/ScrolledW.h>      // XmScrolledWindow widget          
#include <Xm/RowColumn.h>      // XmRowColumn widget               
#include <Xm/Form.h>           // XmForm widget                    
#include <Xm/Text.h>           // XmText widget                    
#include <Xm/CascadeB.h>       // XmCascadeButton widget           
#include <Xm/PushB.h>          // XmPushButton widget              
#include <Xm/DrawingA.h>       // XmDrawingArea widget             
#include <Xm/TextF.h>          // XmTextField widget               
#include <Xm/Scale.h>          // XmScale widget                   
#include <Xm/Frame.h>          // XmFrame widget                   
#include <Xm/Label.h>          // XmLabel widget                   
#include <Xm/FileSB.h>         // XmFileSelectionBox widget        
#include <Xm/ToggleB.h>        // XmToggleButton widget            

// own Header Files
#include "image/Image.h"
#include "image/Filter.h"
#include "image/Histogram.h"
#include "image/PPMReader.h"
#include "image/PPMWriter.h"
#include <iostream.h>
#include "xv/xv.c"
#include "image/ImageSourcePPM.h"
#include "callback.h"
#include <fstream.h>


// Globals for Motif
XtAppContext  context;
Widget     toplevel, mainManager;
Widget     menuBar;
Widget     menuButtonFile, menuFile, menuFileSetDir, menuFileExit;
Widget     menuButtonEdit;
Widget     menuButtonMode, menuMode, menuMode1, menuMode2, menuMode3, menuMode3d;
Widget     menuButtonSpare;
Widget     menuButtonHelp;
Widget     commandPanel;
Widget     DELTAframe, DELTAframeTitel, DELTAmanager, DELTAdoit1, \
           DELTAdoit2, DELTAbwLimit, DELTAlimit, DELTAdoit3, DELTAswitchOTSU, \
           DELTAswitchBW, DELTAswitchDESPECKLE, DELTAdoit4, DELTAwinSize, DELTAtest, \
           DELTAswitchCORRECT, DELTAbuttonrow1, DELTAbuttonrow2, DELTAswitchrow1, \
           DELTAswitchrow2, DELTAswitchHTMLextended, DELTAswitchHTMLnormal, \
           DELTAvarianceLimit, DELTAthresholdScale, DELTAwidthLimit;
Widget     ANALYSISframe, ANALYSISframeTitel, ANALYSISmanager, \
           ANALYSISswitchInterpolate, ANALYSISinterpolateMin, \
           ANALYSISinterpolateMax, ANALYSISwinSize, ANALYSISeliminateHistogram, \
           ANALYSISaverageBW, ANALYSISmedianBW, ANALYSISgrey, ANALYSISneighbour, \
           ANALYSISiterativeBW, ANALYSISgreyIterative, ANALYSISswitchrow;
Widget     CMDframe, CMDframeTitel, CMDmanager, CMDgamma, CMDbuttonrow1, \
           CMDnext, CMDnext5, CMDreloadRight, CMDbuttonrow2, CMDredrawLines, CMDdisplayFlag, \
           CMDscroll, CMDprev, CMDprev5, CMDsetStart, CMDsetEnd, CMDreset, CMDredrawLinesStep;
Widget     ZOOM1frame, ZOOM1frameTitel, ZOOM1manager, ZOOM1display, ZOOM1pos;
Widget     ZOOM2frame, ZOOM2frameTitel, ZOOM2manager, ZOOM2display, ZOOM2pos;
Widget     graphics, graphicsScroll;
Widget     statusLine, statusText, statusPosition;
Widget     DirectorySelection;
Widget     HistogramShell, HistogramGraphics, HistogramClose;


// Globals for X11 and libimage

const expandCount = 3;           // Number of mask Expansions

const lookAheadSize = 6;         // 5 images -> 4 deltas -> 3 correction levels
Image* images[lookAheadSize];    // left, middle and right source image
Image* deltas[lookAheadSize-1];  // first and second delta image

unsigned int theImageWidth, theImageHeight;

Image *delta_merge;
int delta_merge_count = -1;
const delta_merge_wait_count = 1;
Image* delta_merge_wait[delta_merge_wait_count];

Image *bild_temp;
Image *bild_ori;
PPMReader myPPMReader;
unsigned int actualImage = 0;
ImageSourcePPM myImageSource;
XmString sourceDirectory;
Image *maske;
Image *mask_source;


extern void XSetup ();

//enum tModeStatus { modeOneFrame, modeTwoFrames, modeThreeFrames, modeDeltaFrames };
//tModeStatus actualMode = modeDeltaFrames;

int i;

Boolean ZOOM1_ENABLED = False;
Boolean ZOOM2_ENABLED = False;
unsigned int ZOOM_PIXELSIZE = 10;
unsigned int ZOOM_PIXELCOUNT = 21;
const byte FIXZOOM_PIXELSIZE = 8;
const byte FIXZOOM_PIXELCOUNT = 25;

Rectangle theRectangle;
unsigned int START = 0;
unsigned int END = 0;


// Sliding Window
#include "image/SlidingWindow.h"
SlidingWindow mySlidingWindow(4);
#include "image/DetectorMedian.h"
#include "image/FilterRotate180.h"





main(int argc, char **argv) {


  ofstream deb("memdebug.txt");

  /*
  deb << "Debugging Report for Memory Leaks in Image classes" << endl;
  deb << "--------------------------------------------------" << endl;
  deb << endl;
  deb << "Timestamp;Class;Method;Type;Pointer" << endl;
  deb << endl;
  */

  deb.close();  


  // Init. Toolkit, App. Context, Application Shell
  toplevel = XtAppInitialize( &context, "Diplom",
      NULL, 0, &argc, argv, NULL, NULL, 0); 


  // Create widget tree, bind callbacks
  mainManager = XtCreateManagedWidget( "mainManager",
      xmMainWindowWidgetClass, toplevel, NULL, 0);

  menuBar = XmCreateMenuBar (mainManager, "menuBar", NULL, 0); 
  XtManageChild (menuBar);
  menuFile = XmCreatePulldownMenu (menuBar, "menuFile", NULL, 0);
  menuFileSetDir = XtCreateManagedWidget( "menuFileSetDir",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileSetDir, XmNactivateCallback, FileSetDirCallback, NULL);
  menuFileExit = XtCreateManagedWidget( "menuFileExit",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileExit, XmNactivateCallback, ExitCallback, NULL);
  Arg arglist[1];
  XtSetArg(arglist[0], XmNsubMenuId, menuFile);
  menuButtonFile = XtCreateManagedWidget( "menuButtonFile",
      xmCascadeButtonWidgetClass, menuBar, arglist, 1);

  menuButtonEdit = XtCreateManagedWidget( "menuButtonEdit",
      xmCascadeButtonWidgetClass, menuBar, NULL, 0);

  menuMode = XmCreatePulldownMenu (menuBar, "menuMode", NULL, 0);
  menuMode1 = XtCreateManagedWidget( "menuMode1",
      xmPushButtonWidgetClass, menuMode, NULL, 0);
  //XtAddCallback(menuMode1, XmNactivateCallback, Mode1Callback, NULL);
  menuMode2 = XtCreateManagedWidget( "menuMode2",
      xmPushButtonWidgetClass, menuMode, NULL, 0);
  //XtAddCallback(menuMode2, XmNactivateCallback, Mode2Callback, NULL);
  menuMode3 = XtCreateManagedWidget( "menuMode3",
      xmPushButtonWidgetClass, menuMode, NULL, 0);
  //XtAddCallback(menuMode3, XmNactivateCallback, Mode3Callback, NULL);
  menuMode3d= XtCreateManagedWidget( "menuMode3d",
      xmPushButtonWidgetClass, menuMode, NULL, 0);
  //XtAddCallback(menuMode3d, XmNactivateCallback, Mode3dCallback, NULL);
  // Arg arglist[1];
  XtSetArg(arglist[0], XmNsubMenuId, menuMode);
  menuButtonMode = XtCreateManagedWidget( "menuButtonMode",
      xmCascadeButtonWidgetClass, menuBar, arglist, 1);

  menuButtonSpare = XtCreateManagedWidget( "menuButtonSpare",
      xmCascadeButtonWidgetClass, menuBar, NULL, 0);
  menuButtonHelp = XtCreateManagedWidget( "menuButtonHelp",
      xmCascadeButtonWidgetClass, menuBar, NULL, 0);
  XtVaSetValues(menuBar, 
      XmNmenuHelpWidget, menuButtonHelp,
      NULL);
  

  graphicsScroll = XtCreateManagedWidget( "graphicsScroll",
      xmScrolledWindowWidgetClass, mainManager, NULL, 0);
  graphics = XtCreateManagedWidget( "graphics",
      xmDrawingAreaWidgetClass, graphicsScroll, NULL, 0);
  XtAddCallback(graphics, XmNinputCallback, GraphicsInputCallback, NULL);
  XtAddCallback(graphics, XmNexposeCallback, GraphicsRepaintCallback, NULL);
  XtAddEventHandler(graphics, PointerMotionMask, False, GraphicsInputEvent, NULL);


 /* Trap pointer button press and release   */
 XtAddCallback(graphics, XmNinputCallback, StartRubber, NULL );
 XtAddCallback(graphics, XmNinputCallback, EndRubber, NULL );

 /* Trap motion events to track cursor */
 XtAddEventHandler(graphics, Button1MotionMask, False, Rubberband, NULL);

 
  commandPanel = XtCreateManagedWidget( "commandPanel",
      xmRowColumnWidgetClass, mainManager, NULL, 0);


  statusLine =  XtCreateManagedWidget( "statusLine",
      xmFormWidgetClass, mainManager, NULL, 0);
  statusText =  XtCreateManagedWidget( "statusText",
      xmTextFieldWidgetClass, statusLine, NULL, 0);
  statusPosition =  XtCreateManagedWidget( "statusPosition",
      xmTextFieldWidgetClass, statusLine, NULL, 0);

  XtVaSetValues(statusText, 
      XmNeditable, false, 
      XmNvalue, "Hello world",
      NULL);
  XtVaSetValues(statusPosition, 
      XmNeditable, false, 
      XmNvalue, " 0 / 0",
      NULL);


  DELTAframe = XtCreateManagedWidget( "DELTAframe",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  DELTAframeTitel = XtCreateManagedWidget( "DELTAframeTitel",
      xmLabelWidgetClass, DELTAframe, NULL, 0);
  XtVaSetValues(DELTAframeTitel, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  DELTAmanager = XtCreateManagedWidget( "DELTAmanager",
      xmRowColumnWidgetClass, DELTAframe, NULL, 0);
  DELTAbwLimit = XtCreateManagedWidget( "DELTAbwLimit",
      xmScaleWidgetClass, DELTAmanager, NULL, 0);
  DELTAlimit = XtCreateManagedWidget( "DELTAlimit",
      xmScaleWidgetClass, DELTAmanager, NULL, 0);
  DELTAbuttonrow1 = XtCreateManagedWidget("DELTAbuttonrow1",
      xmRowColumnWidgetClass, DELTAmanager, NULL, 0);
  DELTAdoit1 = XtCreateManagedWidget( "DELTAdoit1",
      xmPushButtonWidgetClass, DELTAbuttonrow1, NULL, 0);
  XtAddCallback(DELTAdoit1, XmNactivateCallback, DELTAdoit1Callback, NULL);
  DELTAdoit2 = XtCreateManagedWidget( "DELTAdoit2",
      xmPushButtonWidgetClass, DELTAbuttonrow1, NULL, 0);
  XtAddCallback(DELTAdoit2, XmNactivateCallback, DELTAdoit2Callback, NULL);
  DELTAbuttonrow2 = XtCreateManagedWidget("DELTAbuttonrow2",
      xmRowColumnWidgetClass, DELTAmanager, NULL, 0);
  DELTAdoit3 = XtCreateManagedWidget( "DELTAdoit3",
      xmPushButtonWidgetClass, DELTAbuttonrow2, NULL, 0);
  XtAddCallback(DELTAdoit3, XmNactivateCallback, DELTAdoit3Callback, NULL);
  DELTAdoit4 = XtCreateManagedWidget( "DELTAdoit4",
      xmPushButtonWidgetClass, DELTAbuttonrow2, NULL, 0);
  XtAddCallback(DELTAdoit4, XmNactivateCallback, DELTAdoit4Callback, NULL);
  DELTAtest = XtCreateManagedWidget( "DELTAtest",
      xmPushButtonWidgetClass, DELTAbuttonrow2, NULL, 0);
  XtAddCallback(DELTAtest, XmNactivateCallback, DELTAtestCallback, NULL);
  DELTAswitchrow1 = XtCreateManagedWidget("DELTAswitchrow1",
      xmRowColumnWidgetClass, DELTAmanager, NULL, 0);
  DELTAswitchBW = XtCreateManagedWidget( "DELTAswitchBW",
      xmToggleButtonWidgetClass, DELTAswitchrow1, NULL, 0);
  DELTAswitchDESPECKLE = XtCreateManagedWidget( "DELTAswitchDESPECKLE",
      xmToggleButtonWidgetClass, DELTAswitchrow1, NULL, 0);
  DELTAswitchOTSU = XtCreateManagedWidget( "DELTAswitchOTSU",
      xmToggleButtonWidgetClass, DELTAswitchrow1, NULL, 0);
  DELTAswitchCORRECT = XtCreateManagedWidget( "DELTAswitchCORRECT",
      xmToggleButtonWidgetClass, DELTAswitchrow1, NULL, 0);
  DELTAswitchrow2 = XtCreateManagedWidget("DELTAswitchrow2",
      xmRowColumnWidgetClass, DELTAmanager, NULL, 0);
  DELTAswitchHTMLextended = XtCreateManagedWidget( "DELTAswitchHTMLextended",
      xmToggleButtonWidgetClass, DELTAswitchrow2, NULL, 0);
  DELTAswitchHTMLnormal = XtCreateManagedWidget( "DELTAswitchHTMLnormal",
      xmToggleButtonWidgetClass, DELTAswitchrow2, NULL, 0);
  DELTAvarianceLimit = XtCreateManagedWidget( "DELTAvarianceLimit",
      xmScaleWidgetClass, DELTAmanager, NULL, 0);
  DELTAwidthLimit = XtCreateManagedWidget( "DELTAwidthLimit",
      xmScaleWidgetClass, DELTAmanager, NULL, 0);
  DELTAthresholdScale = XtCreateManagedWidget( "DELTAthresholdScale",
      xmScaleWidgetClass, DELTAmanager, NULL, 0);


  ANALYSISframe = XtCreateManagedWidget( "ANALYSISframe",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  ANALYSISframeTitel = XtCreateManagedWidget( "ANALYSISframeTitel",
      xmLabelWidgetClass, ANALYSISframe, NULL, 0);
  XtVaSetValues(ANALYSISframeTitel, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  ANALYSISmanager = XtCreateManagedWidget( "ANALYSISmanager",
      xmRowColumnWidgetClass, ANALYSISframe, NULL, 0);
  ANALYSISswitchrow = XtCreateManagedWidget("ANALYSISswitchrow",
      xmRowColumnWidgetClass, ANALYSISmanager, NULL, 0);
  XtVaSetValues(ANALYSISswitchrow, 
      XmNradioBehavior, True, 
      NULL);
  ANALYSISaverageBW = XtCreateManagedWidget( "ANALYSISaverageBW",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISmedianBW = XtCreateManagedWidget( "ANALYSISmedianBW",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISgrey = XtCreateManagedWidget( "ANALYSISgrey",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISiterativeBW = XtCreateManagedWidget( "ANALYSISiterativeBW",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISgreyIterative = XtCreateManagedWidget( "ANALYSISgreyIterative",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISneighbour = XtCreateManagedWidget( "ANALYSISneighbour",
      xmToggleButtonWidgetClass, ANALYSISswitchrow, NULL, 0);
  ANALYSISswitchInterpolate = XtCreateManagedWidget( "ANALYSISswitchInterpolate",
      xmToggleButtonWidgetClass, ANALYSISmanager, NULL, 0);
  ANALYSISinterpolateMin = XtCreateManagedWidget( "ANALYSISinterpolateMin",
      xmScaleWidgetClass, ANALYSISmanager, NULL, 0);
  ANALYSISinterpolateMax = XtCreateManagedWidget( "ANALYSISinterpolateMax",
      xmScaleWidgetClass, ANALYSISmanager, NULL, 0);
  ANALYSISwinSize = XtCreateManagedWidget( "ANALYSISwinSize",
      xmScaleWidgetClass, ANALYSISmanager, NULL, 0);
  ANALYSISeliminateHistogram = XtCreateManagedWidget( "ANALYSISeliminateHistogram",
      xmToggleButtonWidgetClass, ANALYSISmanager, NULL, 0);


  CMDframe = XtCreateManagedWidget( "CMDframe",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  CMDframeTitel = XtCreateManagedWidget( "CMDframeTitel",
      xmLabelWidgetClass, CMDframe, NULL, 0);
  XtVaSetValues(CMDframeTitel, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  CMDmanager = XtCreateManagedWidget( "CMDmanager",
      xmRowColumnWidgetClass, CMDframe, NULL, 0);
  CMDgamma = XtCreateManagedWidget( "CMDgamma",
      xmPushButtonWidgetClass, CMDmanager, NULL, 0);
  XtAddCallback(CMDgamma, XmNactivateCallback, GAMMAdoitCallback, NULL);

  CMDbuttonrow1 = XtCreateManagedWidget("CMDbuttonrow1",
      xmRowColumnWidgetClass, CMDmanager, NULL, 0);

  CMDprev = XtCreateManagedWidget( "CMDprev",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDprev, XmNactivateCallback, PrevImageCallback, NULL);
  CMDprev5 = XtCreateManagedWidget( "CMDprev5",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDprev5, XmNactivateCallback, Prev5ImageCallback, NULL);
  CMDnext = XtCreateManagedWidget( "CMDnext",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDnext, XmNactivateCallback, NextImageCallback, NULL);
  CMDnext5 = XtCreateManagedWidget( "CMDnext5",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDnext5, XmNactivateCallback, Next5ImageCallback, NULL);
  CMDscroll = XtCreateManagedWidget( "CMDscroll",
      xmScaleWidgetClass, CMDmanager, NULL, 0);
  XtVaSetValues(CMDscroll, 
      XmNminimum, 0,
      XmNmaximum, 1,
      XmNvalue, 0, 
      NULL);
  XtAddCallback(CMDscroll, XmNvalueChangedCallback, CMDscrollCallback, NULL);
  CMDreloadRight = XtCreateManagedWidget( "CMDreloadRight",
      xmPushButtonWidgetClass, CMDmanager, NULL, 0);
  XtAddCallback(CMDreloadRight, XmNactivateCallback, ReloadRightCallback, NULL);
  CMDbuttonrow2 = XtCreateManagedWidget("CMDbuttonrow2",
      xmRowColumnWidgetClass, CMDmanager, NULL, 0);
  CMDsetStart = XtCreateManagedWidget( "CMDsetStart",
      xmPushButtonWidgetClass, CMDbuttonrow2, NULL, 0);
  XtAddCallback(CMDsetStart, XmNactivateCallback, setStart, NULL);
  CMDsetEnd = XtCreateManagedWidget( "CMDsetEnd",
      xmPushButtonWidgetClass, CMDbuttonrow2, NULL, 0);
  XtAddCallback(CMDsetEnd, XmNactivateCallback, setEnd, NULL);
  CMDredrawLines = XtCreateManagedWidget( "CMDredrawLines",
      xmPushButtonWidgetClass, CMDmanager, NULL, 0);
  XtAddCallback(CMDredrawLines, XmNactivateCallback, RedrawLinesCallback, NULL);
  CMDredrawLinesStep = XtCreateManagedWidget( "CMDredrawLinesStep",
      xmPushButtonWidgetClass, CMDmanager, NULL, 0);
  XtAddCallback(CMDredrawLinesStep, XmNactivateCallback, RedrawLinesStepCallback, NULL);
  CMDreset = XtCreateManagedWidget( "CMDreset",
      xmPushButtonWidgetClass, CMDmanager, NULL, 0);
  XtAddCallback(CMDreset, XmNactivateCallback, ResetCallback, NULL);
  CMDdisplayFlag = XtCreateManagedWidget( "CMDdisplayFlag",
      xmToggleButtonWidgetClass, CMDmanager, NULL, 0);


  ZOOM1frame = XtCreateManagedWidget( "ZOOM1frame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  ZOOM1frameTitel = XtCreateManagedWidget( "ZOOM1frameTitel",
      xmLabelWidgetClass, ZOOM1frame, NULL, 0);
  XtVaSetValues(ZOOM1frameTitel, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  ZOOM1manager = XtCreateManagedWidget( "ZOOM1manager",
      xmFormWidgetClass, ZOOM1frame, NULL, 0);
  ZOOM1display = XtCreateManagedWidget( "ZOOM1display",
      xmDrawingAreaWidgetClass, ZOOM1manager, NULL, 0);
  XtVaSetValues(ZOOM1display, 
      XmNheight, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
      XmNwidth,  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
      NULL);
  ZOOM1pos =  XtCreateManagedWidget( "ZOOM1pos",
      xmTextFieldWidgetClass, ZOOM1manager, NULL, 0);
  XtVaSetValues(ZOOM1pos, 
      XmNeditable, false, 
      XmNvalue, " Position: none",
      XmNbottomAttachment, XmATTACH_WIDGET, 
      XmNbottomWidget, ZOOM1manager, 
      XmNleftAttachment, XmATTACH_WIDGET, 
      XmNleftWidget, ZOOM1manager, 
      XmNrightAttachment, XmATTACH_WIDGET, 
      XmNrightWidget, ZOOM1manager, 
      NULL);
  XtAddCallback(ZOOM1display, XmNexposeCallback, ZOOMRepaintCallback, NULL);
  XtAddCallback(ZOOM1display, XmNinputCallback, ZOOMInputCallback, NULL);


  
  ZOOM2frame = XtCreateManagedWidget( "ZOOM2frame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  ZOOM2frameTitel = XtCreateManagedWidget( "ZOOM2frameTitel",
      xmLabelWidgetClass, ZOOM2frame, NULL, 0);
  XtVaSetValues(ZOOM2frameTitel, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  ZOOM2manager = XtCreateManagedWidget( "ZOOM2manager",
      xmFormWidgetClass, ZOOM2frame, NULL, 0);
  ZOOM2display = XtCreateManagedWidget( "ZOOM2display",
      xmDrawingAreaWidgetClass, ZOOM2manager, NULL, 0);
  XtVaSetValues(ZOOM2display, 
      XmNheight, FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
      XmNwidth,  FIXZOOM_PIXELSIZE*FIXZOOM_PIXELCOUNT,
      NULL);
  ZOOM2pos =  XtCreateManagedWidget( "ZOOM2pos",
      xmTextFieldWidgetClass, ZOOM2manager, NULL, 0);
  XtVaSetValues(ZOOM2pos, 
      XmNeditable, false, 
      XmNvalue, " Position: none",
      XmNbottomAttachment, XmATTACH_WIDGET, 
      XmNbottomWidget, ZOOM2manager, 
      XmNleftAttachment, XmATTACH_WIDGET, 
      XmNleftWidget, ZOOM2manager, 
      XmNrightAttachment, XmATTACH_WIDGET, 
      XmNrightWidget, ZOOM2manager, 
      NULL);
  XtAddCallback(ZOOM2display, XmNexposeCallback, ZOOMRepaintCallback, NULL);
  XtAddCallback(ZOOM2display, XmNinputCallback, ZOOMInputCallback, NULL);




  // Tell XmMainWindow about children
  XtVaSetValues(mainManager, 
      XmNmenuBar, menuBar, 
      XmNworkWindow, graphicsScroll,
      XmNcommandWindow, commandPanel, 
      XmNmessageWindow, statusLine,
      NULL );
  XtVaSetValues(commandPanel, 
      XmNrightAttachment, XmATTACH_WIDGET, 
      XmNrightWidget, mainManager, 
      NULL);


  // Attach widgets in XmForm
  XtVaSetValues(graphicsScroll, 
      XmNtopAttachment, XmATTACH_WIDGET, 
      XmNtopWidget, menuBar, 
      NULL);

  XtVaSetValues(graphicsScroll, 
      XmNbottomAttachment, XmATTACH_WIDGET, 
      XmNbottomWidget, commandPanel, 
      NULL);

  XtVaSetValues(statusText, 
      XmNrightAttachment, XmATTACH_WIDGET, 
      XmNrightWidget, statusPosition, 
      NULL);




  //  Initialize Image Library and X11
  XSetup();


  char filename[255];
  if (argc > 1) {
    strcpy(filename, "/home/siebenth/diplom/mpeg/");
    strcat(filename, argv[1]);
    strcat(filename, "/");
  } else {
    strcpy(filename, "/home/siebenth/diplom/mpeg/5/");
  };

  //myImageSource.setParameters("/home/siebenth/diplom/test_img4/", "test", 4);
  myImageSource.setParameters(filename, "test", 4);
  //myImageSource.setBaseName("/home/siebenth/diplom/xxx/test");


  // read in the first three images out of the ImageSource
  if (myImageSource.getLastIndex()-myImageSource.getFirstIndex() < 2) {
    cout << "[diplom::main] ERROR: This ImageSource has less than 3 images." << endl;
    exit(0);
  };
  actualImage = myImageSource.getFirstIndex() + 1;
  XtVaSetValues(CMDscroll, 
      XmNminimum, myImageSource.getFirstIndex()+1,
      XmNmaximum, myImageSource.getLastIndex()-1,
      XmNvalue, actualImage, 
      NULL);
  myImageSource.setPosition(actualImage);
  theImageWidth = myImageSource.getImageWidth();
  theImageHeight = myImageSource.getImageHeight();
  theRectangle = Rectangle(0,0,theImageWidth-1, theImageHeight-1);


  // Maske erstellen
  maske = new Image8(theImageWidth, theImageHeight);
  maske->setBlack();
  mask_source = new Image8(theImageWidth, theImageHeight);
  mask_source->setBlack();


  // Arrays  images & deltas  initialisieren
  for (int i = 0; i < lookAheadSize; i++) {
    images[i] = DoPreprocessing(myImageSource.getImageRelative(i-1));
  };
  bild_ori = (images[2])->copy();
  for (int i = 0; i < lookAheadSize - 1; i++) {
    deltas[i] = new Image8(theImageWidth, theImageHeight);
  };


  // delta_merge
  delta_merge = new Image8(theImageWidth, theImageHeight);
  delta_merge->setWhite();
  delta_merge_count = 0;
  for (int i = 0; i < delta_merge_wait_count; i++) {
    delta_merge_wait[i] = delta_merge->copy();
  };



  // generate XImages out of own images
  leftImage = Pic24ToXImage(images[0]->getImage24(), \
                            images[0]->getWidth(), images[0]->getHeight());
  middleImage = Pic24ToXImage(images[1]->getImage24(), \
                              images[1]->getWidth(), images[1]->getHeight());
  rightImage = Pic24ToXImage(images[2]->getImage24(), \
                             images[2]->getWidth(), images[2]->getHeight());
  firstDelta = Pic24ToXImage(deltas[0]->getImage24(), \
                             deltas[0]->getWidth(), deltas[0]->getHeight());
  secondDelta = Pic24ToXImage(deltas[1]->getImage24(), \
                              deltas[1]->getWidth(), deltas[1]->getHeight());
  mergeDelta = Pic24ToXImage(delta_merge->getImage24(), \
                             delta_merge->getWidth(), delta_merge->getHeight());


  XtRealizeWidget(toplevel);           // Realize all              

  paint();    // convert X-Client XImage's to X-Server Pixmap's

  // repaint();  // display the pixmaps
  // XSync(theDisp, False);
  // cout << "main loop starts ..." << endl;
  // cout.flush();

  lastPosition.x = lastPosition.y = -1;

  XtAppMainLoop(context);              // Can't get past here !!!  

};



#include "callback.C"









