// Direct conversion from MPEG/PPM sequences to point positions


#include "image/Image.h"
#include "image/Filter.h"
#include "image/Histogram.h"
#include "image/PPMReader.h"
#include "image/PPMWriter.h"
#include <iostream.h>
#include "image/ImageSourcePPM.h"
#include "image/Analysis.h"
#include "image/Point.h"
#include "image/Rectangle.h"
#include <time.h>
#include <string.h>
#include "image/Interpolator.h"
#include <stdio.h>
#include "image/Drawing.h"



// Sliding Window
#include "image/SlidingWindow.h"
#include "image/DetectorMedian.h"
#include "image/DetectorGreyscale.h"
#include "image/DetectorGreyscaleIterative.h"
#include "image/DetectorIterative.h"
#include "image/DetectorSimple.h"
#include "image/DetectorNeighbour.h"
#include "image/FilterRotate180.h"
#include "image/FilterExpandBlack.h"
#include "image/FilterCutMask.h"
#include "image/FilterCutRectangle.h"
#include "image/PPMWriter.h"




// Paramater aus Command Line
char* theMpegDirectory = "/home/siebenth/diplom/mpeg/5/";
char* theMpegPrefix = "test";
unsigned int theMpegDigits = 4;
unsigned int theMaskSize = 3;
unsigned int theWindowSize = 40;
int theThreshold = -1;              // don't use it -> use OTSU
int theOtsuVariance = 75;           // if negative -> use fixed Threshold 
Rectangle theWindow(16,67,249,116);
unsigned int theStart = 28;
unsigned int theEnd = 368;
unsigned int theLookahead = 2;
enum tDetectionMethod { detectMedian, detectSimple, detectGreyscale, detectNeighbour,\
                        detectIterative, detectGreyscaleIterative, detectNone };
tDetectionMethod theDetectionMethod = detectMedian;
char* theResultDir = "results/test/";
char* theWord;
int theReportImages = false;
int theReportDeltas = false;
unsigned int theMinInterpolate = 0;
unsigned int theMaxInterpolate = 0;
unsigned int theGapInterpolate = 0;
int theVerbose = false;
double theAreaLimit = 0.5;  // the detected point may span area 
                            // larger than 0.05 of the detection-window



#include <fstream.h>
void writePoints2File(char* filename, vector<Point>, char* comments, unsigned int width, \
                      unsigned int height, unsigned int start, \
                      unsigned int end, unsigned int step, Rectangle, char*);
void parseCmdLine(int argc, char **argv);
void writeHTML(char *filename);





main(int argc, char **argv) {

  // Command line parameters
  // 
  // theMpegDirectory    -mpeg dir pre n         /home/siebenth/diplom/mpeg/5/ test 4
  // theMpegPrefix
  // theMpegDigits
  // theWindow           -win x1 y1 x2 y2          -win 16 67 249 116
  // theStart            -interval start end       -interval 28 368
  // theEnd              
  // theMaskSize         -mask xyz                 -mask 3
  // theWindowSize       -winsize xyz              -winsize 40
  // theLookahead        -lookahead xyz            -lookahead 2
  // theDetectionMethod  -method string            -method median|greyscale|iterative|...
  // theOtsuVariancew    -otsu variance            -otsu 75
  // theTreshold         -threshold xyz            -threshold 205
  // theResultDir        -result string            -result ./results/test/
  // theReportImages     -reportImages
  // theReportDeltas     -reportDeltas
  // theXYZInterpolate   -interpolate min max gap  -interpolate 7 10 5
  // theVerbose          -verbose                  Turn verbose output on
  // theAreaLimit        -area ratio (0.0 .. 1.0)  -area 0.1
  // theWord             -word TrueWord            -word Please


  theWord = new char[255];
  strcpy(theWord, "");
  parseCmdLine(argc, argv);

  
  time_t dummy;
  time_t start = time(&dummy);
 


  // Hilfsvariablen
  char shell[255];
  char filename[255];
  PPMWriter myWriter;
  // Point punkte[2*(theEnd-theStart)];
  vector<Point> punkte(0);
  unsigned int pos = 0;



  // Filter initialisieren
  FilterRotate180 myFilterRotate180;
  FilterCutRectangle myFilterCutRectangle(theWindow);


  // ImageSourcePPM initialisieren und Maske erstellen
  ImageSourcePPM myImageSource; 
  myImageSource.setParameters(theMpegDirectory, theMpegPrefix, theMpegDigits);
  if (myImageSource.getFirstIndex() > theStart) {
    theStart = myImageSource.getFirstIndex();
  };
  if (myImageSource.getLastIndex() < theEnd) {
    theEnd = myImageSource.getLastIndex();
  };

  FilterExpandBlack myFilterExpandBlack(theMaskSize, theWindow);
  // Image* source = myImageSource.getImageAbsolute(myImageSource.getLastIndex());
  Image* source = myImageSource.getImageAbsolute(theEnd);
  source = myFilterRotate180.apply(source);
  Image* mask = myFilterExpandBlack.apply(source);
  FilterCutMask myFilterCutMask(mask);
  strcpy(shell, "mkdir ");
  strcat(shell, theResultDir);
  system(shell);
  Image* lastImage1 = myFilterCutMask.apply(source);
  Image* lastImage3 = myFilterCutRectangle.apply(lastImage1);
  strcpy(filename, theResultDir);
  strcat(filename, "lastImage.ppm");
  myWriter.setFilename(filename);
  myWriter.apply(lastImage3);
  //delete lastImage1;
  delete lastImage3;
  strcpy(filename, theResultDir);
  strcat(filename, "masksource.ppm");
  myWriter.setFilename(filename);
  Image* outputSource = myFilterCutRectangle.apply(source);
  myWriter.apply(outputSource);
  delete outputSource;
  delete source;
  strcpy(filename, theResultDir);
  strcat(filename, "mask.ppm");
  myWriter.setFilename(filename);
  Image* outputMask = myFilterCutRectangle.apply(mask);
  myWriter.apply(outputMask);
  delete outputMask;
 

  // Sliding Window initialisieren
  SlidingWindow mySlidingWindow(theLookahead);
  mySlidingWindow.setImageSource(&myImageSource);
  mySlidingWindow.setBoundingBox(theWindow);
  mySlidingWindow.setAreaLimit(theAreaLimit);
  mySlidingWindow.setReportingDirectory(theResultDir);
  mySlidingWindow.setReporting(theReportImages, theReportDeltas, false, \
                               (theReportImages || theReportDeltas));
  Detector *myDetector = NULL;
  switch (theDetectionMethod) {
  case detectMedian:
    myDetector = new DetectorMedian(theWindow);
    break;
  case detectSimple:
    myDetector = new DetectorSimple(theWindow);
    break;
  case detectGreyscale:
    myDetector = new DetectorGreyscale(theWindow);
    break;
  case detectIterative:
    myDetector = new DetectorIterative(theWindow);
    myDetector->setIterations(3);
    break;
  case detectGreyscaleIterative:
    myDetector = new DetectorGreyscaleIterative(theWindow);
    myDetector->setIterations(3);
    break;
  case detectNeighbour:
    myDetector = new DetectorNeighbour(theWindow);
    break;
  default:
    exit(1);
  };
  if (myDetector == NULL) {
    exit(1);
  };
  mySlidingWindow.setDetector(myDetector);
  if (theOtsuVariance > 0) {
    mySlidingWindow.setThresholdOtsu(theOtsuVariance);
  } else {
    mySlidingWindow.setThresholdFixed(theThreshold);
  };
  // Source images are already greyscale to save space and time
  //mySlidingWindow.addPreprocessFilter(&myFilterGreyscale);
  mySlidingWindow.addPreprocessFilter(&myFilterRotate180);
  mySlidingWindow.addPreprocessFilter(&myFilterCutMask);



  // SlidingWindow nutzen
  mySlidingWindow.moveFirst();
  mySlidingWindow.move(theStart);
  Point lastValidPosition;
  Rectangle theRect;
  for (int j = theStart; j <= theEnd; j++) {

    if (lastValidPosition.valid()) {
      theRect = Rectangle(lastValidPosition, theWindowSize, theWindowSize);
      if (theRect.x1 < theWindow.x1) theRect.x1 = theWindow.x1;
      if (theRect.y1 < theWindow.y1) theRect.y1 = theWindow.y1;
      if (theRect.x2 > theWindow.x2) theRect.x2 = theWindow.x2;
      if (theRect.y2 > theWindow.y2) theRect.y2 = theWindow.y2;
    } else {
      // theRect = Rectangle(0,0,myImageSource.getImageWidth(),myImageSource.getImageHeight());
      theRect.x1 = theWindow.x1;
      theRect.y1 = theWindow.y1;
      theRect.x2 = theWindow.x1 + theWindowSize;
      theRect.y2 = theWindow.y2;
    };

    Point result = mySlidingWindow.apply(theRect);
    punkte.insert(punkte.end(), result);
    pos++;
    // punkte[pos++] = result;

    if (result.valid()) {
      cout << "POINT: (" << result.x << "/" << result.y << ")" << endl;
      lastValidPosition = result;
    } else {
      cout << "NO POINT" << endl;
    };

    mySlidingWindow.moveNext();
    
  };


  // Reporting DeltaMerge einschalten -> Output des letzten DeltaMerge Bildes
  mySlidingWindow.setReporting(theReportImages, theReportDeltas, true, false);


  // Ausgangspunkte ausgeben
  if (theVerbose != false) {
    cout << endl << endl;
    for (int j = 0; j < pos; j++) {
      if (punkte[j].valid()) {
        cout << j << ": " << punkte[j].x << "/" << punkte[j].y << endl;
      } else {
        cout << j << ": up" << endl;
      };
    };
    cout << endl;
  };


  // time_t dummy;
  time_t ende = time(&dummy);  
  cout << endl;
  cout << "mpeg2points was running for " << (ende-start) << " seconds" << endl;
  cout << "the average throughput was " << ((double)(theEnd-theStart))/((double)(ende-start));
  cout << " frames / second" << endl;
  cout << endl;


  // Output without Interpolation
  cout << "writing points.txt ..." << endl;
  strcpy(filename, theResultDir);
  strcat(filename, "points.txt");
  writePoints2File(filename, punkte, "mpeg2points", myImageSource.getImageWidth(), \
                   myImageSource.getImageHeight(), theStart, theEnd, 1, theWindow, theWord);

  cout << "writing imageLine.ppm ..." << endl;
  // Image* imageLine = new Image24(*lastImage1);
  Image* imageLine = new Image8(lastImage1->getWidth(), lastImage1->getHeight());
  imageLine->setWhite();
  Drawing myDraw;
  for (int j = 1; j < pos; j++) {
    if (punkte[j].valid() && punkte[j-1].valid()) {
      myDraw.line(imageLine, punkte[j-1], punkte[j], Color(0,0,0));
    };
  }; 
  strcpy(filename, theResultDir);
  strcat(filename, "imageLine.ppm");
  myWriter.setFilename(filename);
  Image* ImageReporting = myFilterCutRectangle.apply(imageLine);
  delete imageLine;
  myWriter.apply(ImageReporting);
  delete ImageReporting;  
  ImageReporting = NULL;


  cout << "writing imagePoint.ppm ..." << endl;
  Image* imagePoint = new Image24(*lastImage1);
  for (int j = 0; j < pos; j++) {
    if (punkte[j].valid()) {
      myDraw.cross(imagePoint, punkte[j], Color(0,0,255), 1);    // Kreuz aus 5 Pixel
    };
  }; 
  strcpy(filename, theResultDir);
  strcat(filename, "imagePoint.ppm");
  myWriter.setFilename(filename);
  ImageReporting = myFilterCutRectangle.apply(imagePoint);
  delete imagePoint;
  myWriter.apply(ImageReporting);
  delete ImageReporting;  
  ImageReporting = NULL;


  
  // Output with Interpolation
  if ((theMinInterpolate > 0) && (theMaxInterpolate > 0)) {
    // Interpolator myInterpolator(theMinInterpolate,theMaxInterpolate);
    // Point *res = myInterpolator.apply(punkte, theEnd-theStart);

    Interpolator myInterpolator(theMinInterpolate,theMaxInterpolate,theGapInterpolate);
    myInterpolator.setVerboseMode(theVerbose);

    myInterpolator.setMaskImage(mask);

    vector<Point> res = myInterpolator.apply(punkte);

    cout << "writing imageLineI.ppm ..." << endl;
    // Image* imageLineI = new Image24(*lastImage1);
    Image* imageLineI = new Image8(lastImage1->getWidth()*4, lastImage1->getHeight()*4);
    // Image* imageLineI = new Image8(lastImage1->getWidth(), lastImage1->getHeight());
    imageLineI->setWhite();
    Drawing myDraw;
    for (int j = 1; j < res.size(); j++) {
      if (res[j].valid() && res[j-1].valid()) {
        Point start(4*res[j-1].x, 4*res[j-1].y);
        Point stop(4*res[j].x, 4*res[j].y);
        myDraw.line(imageLineI, start, stop, Color(0,0,0));
        // myDraw.line(imageLineI, res[j-1], res[j], Color(0,0,0));
      };
    }; 
    strcpy(filename, theResultDir);
    strcat(filename, "imageLineI.ppm");
    myWriter.setFilename(filename);
    Rectangle tempWin(theWindow.x1*4, theWindow.y1*4, theWindow.x2*4, theWindow.y2*4); 
    FilterCutRectangle tempFilterCutRectangle(tempWin);
    // ImageReporting = myFilterCutRectangle.apply(imageLineI);
    ImageReporting = tempFilterCutRectangle.apply(imageLineI);
    delete imageLineI;
    myWriter.apply(ImageReporting);
    delete ImageReporting;  
    ImageReporting = NULL;

    // Interpolierte Punkte ausgeben
    if (theVerbose != false) {
      cout << endl << endl;
      for (int j = 0; j < res.size(); j++) {
        if (res[j].valid()) {
          cout << j << ": " << res[j].x << "/" << res[j].y << endl;
        } else {
          cout << j << ": up" << endl;
        };
      };
      cout << endl;
    };

  };
  delete lastImage1;



  strcpy(filename, theResultDir);
  strcat(filename, "index.html");
  writeHTML(filename);

};









void writePoints2File(char* file, vector<Point> points, char* comments, 
                      unsigned int width, unsigned int height, unsigned int start, 
                      unsigned int end, unsigned int step, Rectangle win, char* word) {


  ofstream ofs = file;
  ofs << "# " << comments << endl;
  ofs << "Word:   " << word << endl;
  ofs << "Step:   " << step << endl;
  ofs << "Start:  " << start << endl;
  ofs << "Stop:   " << end << endl;
  ofs << "Width:  " << width << endl;
  ofs << "Height: " << height << endl;
  ofs << "Rectangle: " << win.x1 << " " << win.y1 << " " ;
  ofs << win.x2 << " " << win.y2 << endl;
  ofs << "#" << endl;

  for (unsigned int i = 0; i < points.size(); i++) {
    ofs << i << ": ";
    if (points[i].valid()) {
      ofs << points[i].x << " / " << points[i].y;
    }; 
    ofs << endl;
  };

  ofs << "#" << endl;
  ofs << "# the end." << endl;

};






void parseCmdLine(int argc, char **argv) {
  // Command Line parsen
  cout << endl;
  cout << "Reading Command Line ..." << endl << endl;
  int i = 1;
  while (i < argc) {

    if (strcmp(argv[i], "-mpeg") == 0) {
      theMpegDirectory = argv[i+1];
      theMpegPrefix = argv[i+2];
      theMpegDigits = atoi(argv[i+3]);
      i += 4;
      cout << "   MPEG Source: " << theMpegDirectory << ", " << theMpegPrefix << ", " \
                                 << theMpegDigits << endl;
    } else {
      if (strcmp(argv[i], "-win") == 0) {
        theWindow.x1 = atoi(argv[i+1]);
        theWindow.y1 = atoi(argv[i+2]);
        theWindow.x2 = atoi(argv[i+3]);
        theWindow.y2 = atoi(argv[i+4]);
        i += 5;
        cout << "   Window: (" << theWindow.x1 << "/" << theWindow.y1 << ") - (" \
                               << theWindow.x2 << "/" << theWindow.y2 << ")" << endl;
      } else {
        if (strcmp(argv[i], "-interval") == 0) {
          theStart = atoi(argv[i+1]);
          theEnd = atoi(argv[i+2]);
          i += 3;
          if (theStart > theEnd) {
            unsigned int temp = theStart;
            theStart = theEnd;
            theEnd = temp;
	  };
          cout << "   Interval: " << theStart << " - " << theEnd << endl;
	} else {
          if (strcmp(argv[i], "-mask") == 0) {
            theMaskSize = atoi(argv[i+1]);
            i += 2;
            cout << "   Mask Expand: " << theMaskSize << endl;
	  } else {
            if (strcmp(argv[i], "-winsize") == 0) {
              theWindowSize = atoi(argv[i+1]);
              i += 2;
              cout << "   Window Size: " << theWindowSize << endl;
	    } else {
              if (strcmp(argv[i], "-lookahead") == 0) {
                theLookahead = atoi(argv[i+1]);
                i += 2;
                cout << "   Lookahead: " << theLookahead << endl;
	      } else {
               if (strcmp(argv[i], "-area") == 0) {
                 theAreaLimit = atof(argv[i+1]);
                 i += 2;
                 cout << "   Area Ratio Limit: " << theAreaLimit*100.0 << " %" << endl;
	       } else {
                if (strcmp(argv[i], "-otsu") == 0) {
                  theOtsuVariance = atoi(argv[i+1]);
                  theThreshold = -1;
                  i += 2;
                  cout << "   Otsu Variance: "  << theOtsuVariance << endl;
		} else {
                 if (strcmp(argv[i], "-threshold") == 0) {
                   theThreshold = atoi(argv[i+1]);
                   i += 2;
                   theOtsuVariance = -1;
                   cout << "   Threshold: " << theThreshold << endl;
		 } else {
                   if (strcmp(argv[i], "-word") == 0) {
                     strcpy(theWord, argv[i+1]);
                     i += 2;
                     cout << "   Word: " << theWord << endl;
                  } else {
                   if (strcmp(argv[i], "-result") == 0) {
                     theResultDir = argv[i+1];
                     i += 2;
                     cout << "   Result Directory: " << theResultDir << endl;
		   } else {
                     if (strcmp(argv[i], "-reportImages") == 0) {
                       theReportImages = true;
                       i++;
                       cout << "   Report Images" << endl;
		     } else {
                       if (strcmp(argv[i], "-reportDeltas") == 0) {
                         theReportDeltas = true;
                         i++;
                         cout << "   Report Deltas" << endl;
	    	       } else {
                         if (strcmp(argv[i], "-interpolate") == 0) {
                           i++;
                           theMinInterpolate = atoi(argv[i]);
                           i++;
                           theMaxInterpolate = atoi(argv[i]);
                           i++;
                           theGapInterpolate = atoi(argv[i]);
                           i++;
                           cout << "   Interpolate Points, " << theMinInterpolate << ", " << theMaxInterpolate << ", " << theGapInterpolate << endl;
			 } else {
                           if (strcmp(argv[i], "-verbose") == 0) {
                             i++;
                             theVerbose = true;
                             cout << "   Verbose output turned on" << endl;
			   } else {
                             if (strcmp(argv[i], "-method") == 0) {
                               theDetectionMethod = detectNone;
                               if (strcmp(argv[i+1],"median") == 0) {
                                 theDetectionMethod = detectMedian;
    	      	               };
                               if (strcmp(argv[i+1],"simple") == 0) {
                                 theDetectionMethod = detectSimple;
		               };
                               if (strcmp(argv[i+1],"grey") == 0) {
                                 theDetectionMethod = detectGreyscale;
		               };
                               if (strcmp(argv[i+1],"iterative") == 0) {
                                 theDetectionMethod = detectIterative;
  		               };
                               if (strcmp(argv[i+1],"greyiterative") == 0) {
                                 theDetectionMethod = detectGreyscaleIterative;
		               };
                               if (strcmp(argv[i+1],"neighbour") == 0) {
                                 theDetectionMethod = detectNeighbour;
		               };
                               i += 2;
                               cout << "   Detection Medod: ";
                               switch (theDetectionMethod) {
                               case detectMedian:
                                 cout << "Median";
                                 break;
                               case detectSimple:
                                 cout << "Simple Center";
                                 break;
                               case detectGreyscale:
                                 cout << "Greyscale";
                                 break;
                               case detectIterative:
                                 cout << "Iterative Center";
                                 break;
                               case detectGreyscaleIterative:
                                 cout << "Greyscale Iterative";
                                 break;
                               case detectNeighbour:
                                 cout << "Neighbour";
                                 break;
                               default:
                                 cout << "NONE";
		               };
                               cout << endl;
  	  	             } else {
                               if (strcmp(argv[i], "-help") == 0) {
                                 cout << "Usage: " << endl;
                                 cout << "-mpeg basedirectory filenameprefix digitcount     -mpeg /home/mpeg/ test 4" << endl;
                                 cout << "-win x1 y1 x2 y2                                  -win 16 67 249 116" << endl;
                                 cout << "-interval start end                               -interval 28 368" << endl;
                                 cout << "-mask integer                                     -mask 4" << endl;
                                 cout << "-winsize integer                                  -winsize 40" << endl;
                                 cout << "-lookahead integer                                -lookahead 2" << endl;
                                 cout << "-threshold integer                                -threshold 205" << endl;
			         cout << "-otsu variance                                    -otsu" << endl;
			         cout << " only -threshold or -otsu allowed" << endl;
                                 cout << "-result directory                                 -directory results/dummy/" << endl;
                                 cout << "-method grey|simple|median|greyiterative|iterative|neighbour" << endl;
                                 cout << "-reportImages" << endl;
                                 cout << "-reportDeltas" << endl;
                                 cout << "-interpolate min max                              -interpolate 5 10" << endl;
                                 cout << "-verbose" << endl;
                                 cout << "-word TrueWord                                    -word Please" << endl;
                                 cout << endl << endl; 
                                 i++;
                                 exit(0);
		               } else {
                                 cout << "   Ignored: " << argv[i] << endl;
                                 i++;
			       };
			     };
		           };
		         };
                        };
		       };
		      };
		     };
		   };
		 };
		};
	      };
	    };
	  };
	};
      };
    };




  }; 
  cout << endl << endl;

};




void writeHTML(char *filename) {

  ofstream datei(filename);

  datei << "<HTML>" << endl;
  datei << "<HEAD><H1>mpeg2points - ";
  switch (theDetectionMethod) {
  case detectMedian:
    datei << "median";
    break;
  case detectSimple:
    datei << "simple";
    break;
  case detectGreyscale:
    datei << "greyscale";
    break;
  case detectIterative:
    datei << "iterative";
    break;
  case detectGreyscaleIterative:
    datei << "greyscale iterative";
    break;
  case detectNeighbour:
    datei << "neighbour";
    break;
  default:
    exit(1);
  };
  datei << "</H1></HEAD>" << endl;
  datei << "<BODY>" << endl;
  datei << "<HR>" << endl;
  datei << "<P>" << endl;

  datei << "Word: " << theWord << "<BR>" << endl;
  datei << "MPEG Source: " << theMpegDirectory << ", " << theMpegPrefix << ", " << theMpegDigits << "<BR>" << endl;
  datei << "Window Size: " << theWindowSize << "<BR>" << endl;
  datei << "Mask Thickness: " << theMaskSize << "<BR>" << endl;
  datei << "Threshold: " << theThreshold << "<BR>" << endl;
  datei << "Otsu Variance Limit: " << theOtsuVariance << "<BR>" << endl;
  datei << "Window: (" << theWindow.x1 << "/" << theWindow.y1 << ") - (";
  datei << theWindow.x2 << "/" << theWindow.y2 << ")<BR>" << endl;
  datei << "Intervall: " << theStart << " - " << theEnd << "<BR>" << endl;
  datei << "Lookahead: " << theLookahead << "<BR>" << endl;
  datei << "<P>" << endl;

  datei << "<TABLE border = 2>" << endl;

  char shell[255];
  strcpy(shell, "ppmtogif -interlace ");
  strcat(shell, theResultDir);
  strcat(shell, "lastImage.ppm > ");
  strcat(shell, theResultDir);
  strcat(shell, "lastImage.gif");
  system(shell);
  datei << "<TR><TD>Source Image:</TD>";
  datei << "<TD><IMG SRC=\"lastImage.gif\"></TD>" << endl;
  datei << "</TR>" << endl;

  strcpy(shell, "pnmscale 2 ");
  strcat(shell, theResultDir);
  strcat(shell, "imagePoint.ppm ");
  strcat(shell, "| ppmtogif -interlace > ");
  strcat(shell, theResultDir);
  strcat(shell, "imagePoint.gif");
  system(shell);
  datei << "<TR><TD>Positions:</TD>";
  datei << "<TD><IMG SRC=\"imagePoint.gif\"></TD>" << endl;
  datei << "</TR>" << endl;

  strcpy(shell, "pnmscale 2 ");
  strcat(shell, theResultDir);
  strcat(shell, "imageLine.ppm ");
  strcat(shell, "| ppmtogif -interlace > ");
  strcat(shell, theResultDir);
  strcat(shell, "imageLine.gif");
  system(shell);
  datei << "<TR><TD>Lines Raw:</TD>";
  datei << "<TD><IMG SRC=\"imageLine.gif\"></TD>" << endl;
  datei << "</TR>" << endl;

  // strcpy(shell, "pnmscale 2 ");
  strcpy(shell, "pnmscale 0.5 ");
  strcat(shell, theResultDir);
  strcat(shell, "imageLineI.ppm ");
  strcat(shell, "| ppmtogif -interlace > ");
  strcat(shell, theResultDir);
  strcat(shell, "imageLineI.gif");
  system(shell);
  datei << "<TR><TD>Lines Interpolated:</TD>";
  datei << "<TD><IMG SRC=\"imageLineI.gif\"></TD>" << endl;
  datei << "</TR>" << endl;

  datei << "</TABLE>" << endl;
  datei << "<HR>" << endl;
  datei << "created by <A HREF=\"mailto:Thomas@von.Siebenthal.ch\">Thomas von Siebenthal</
A>" << endl;
  datei << "</BODY>" << endl;
  datei << "</HTML>" << endl;
};

















