
      SUBROUTINE THDS2(RHST,NGR,JTHD,RMEAN,VARI,P0,P1,A0,A1,SEP)        00000100
C                                                                       00000200
C     Copyright (c) 1983 by AIST MITI(JAPAN)                            00000300
C                                                                       00000400
CS    CALL THDS2(RHST,NGR,JTHD,RMEAN,VARI,P0,P1,A0,A1,SEP)              00000500
C                                                                       00000600
CP     Threshold selection for binarization by the Otsu's discriminate  00000700
CP     analysis method.                                                 00000800
C                                                                       00000900
CK       THRESHOLD SELECTION, BINARIZATION, DISCRIMINANT ANALYSIS       00001000
C                                                                       00001100
CA     RHST(NGR): Gray-level histogram                               (I)00001200
CA     NGR      : Number of gray levels                              (I)00001300
CA     JTHD     : Threshold value selected                           (O)00001400
CA     RMEAN    : Global mean value                                  (O)00001500
CA     VARI     : Global variance                                    (O)00001600
CA     P0,P1    : Probability of occurrence in each class            (O)00001700
CA     A0,A1    : Mean value of each class                           (O)00001800
CA     SEP      : Class separability                                 (O)00001900
C                                                                       00002000
CN      References                                                      00002100
CN [1]  N. Otsu, "A threshold selection method from gray-level          00002200
CN      histogram," IEEE Trans., vol. SMC-9, pp. 62-66, Jan. 1979.      00002300
CN [2]  N. Otsu, "An automatic threshold selection method based on      00002400
CN      discriminant and least squares criteria," IECE Trans., vol.     00002500
CN      J63-D, pp. 349-356, April 1980 (in Japanese).                   00002600
C                                                                       00002700
CN           NOTE:  INPUT HISTOGRAM MUST BE NORMALIZED SO THAT          00002800
CN                 ITS TOTAL AMOUNT IS 1.0.                             00002900
CN                                                                      00003000
C                                                                       00003100
CD          FEB  20, 1979, PROGRAMMED BY M.TEZUKA                       00003200
CD          JUNE  3, 1980, MODIFIED   BY H.TAMURA                       00003300
CD          JULY  7, 1980, REFORMED   BY E.UENO                         00003400
C                                                                       00003500
C                                                                       00003600
C                                                                       00003700
      DIMENSION  RHST(NGR)                                              00003800
      DATA EPS /1.E-10/                                                 00003900
C                                                                       00004000
C                                                                       00004100
      ITHD = 0                                                          00004200
      NGR1 = NGR-1                                                      00004300
C                                                                       00004400
CF ** COMPUTATION OF GLOBAL MEAN AND VARIANCE                           00004500
      RM = 0.0                                                          00004600
      VA = 0.0                                                          00004700
      DO 10 I=1,NGR                                                     00004800
        RI = FLOAT(I-1)                                                 00004900
        RH = RI*RHST(I)                                                 00005000
        RM = RM+RH                                                      00005100
        VA = VA+RI*RH                                                   00005200
   10 CONTINUE                                                          00005300
      RMEAN = RM                                                        00005400
      VARI = VA-RM*RM                                                   00005500
C                                                                       00005600
CF ** DETERMINATION OF OPTIMAL THRESHOLD VALUE                          00005700
      SMAX = 0.0                                                        00005800
      P = 0.0                                                           00005900
      A = 0.0                                                           00006000
      DO 20 I=1,NGR1                                                    00006100
        RH = RHST(I)                                                    00006200
        P = P+RH                                                        00006300
        A = A+FLOAT(I-1)*RH                                             00006400
        S = RM*P-A                                                      00006500
        D = P*(1.0-P)                                                   00006600
        IF (D .LT. EPS) GO TO 20                                        00006700
        S = S*S/D                                                       00006800
        IF (S .LT. SMAX) GO TO 20                                       00006900
        SMAX = S                                                        00007000
        AMAX = A                                                        00007100
        PMAX = P                                                        00007200
        ITHD = I                                                        00007300
   20 CONTINUE                                                          00007400
C                                                                       00007500
CF ** RETURN WITH SEVERAL OUTPUT PARAMETER VALUES                       00007600
      JTHD = ITHD-1                                                     00007700
      P0 = PMAX                                                         00007800
      P1 = 1.0-P0                                                       00007900
      A0 = AMAX/PMAX                                                    00008000
      A1 = (RM-AMAX)/P1                                                 00008100
      SEP = SMAX/VARI                                                   00008200
C                                                                       00008300
C                                                                       00008400
      RETURN                                                            00008500
      END                                                               00008600
