//
// small programm to convert a point sequence
// containing a single word inthe the ETHZ format
//
// Usage: points2ethz [-interpolate max_inter max_dist gaps] 
//                    [-runaway factor] -word w -dir directory
// 
//        points2ethz -interpolate 5 10 3 -runaway 2.0 -word Please -dir results/c_Please/
//        points2ethz -word Wall -dir results/t_Wall/neighbour/
// 
// reads per default points.txt and writes points.ethz
//


// own Header Files
#include <iostream.h>
#include <fstream.h>
#include "image/Point.h"
#include "image/Rectangle.h"
#include "image/Interpolator.h"
#include <vector.h>
#include <string.h>


// global variables
int interpolate = false;
int eliminaterunaway = false;
int maxInterpolate = 0;
int maxDistance = 0;
int gaps = 0;
float runawayFactor = 0.0;
char word[255];
char directory[255];
unsigned int timeETHZ = 10;   // delay in ms between two pen positions
int currentdir = false; 



main(int argc, char **argv) {

  // parse command line
  cout << endl;
  cout << "parse command line ... " ;
  strcpy(directory, "./");
  int i = 1;
  while (i < argc) {
    if (strcmp(argv[i], "-interpolate") == 0) {
      i++;
      interpolate = true;
      maxInterpolate = atoi(argv[i++]);
      maxDistance = atoi(argv[i++]);
      gaps = atoi(argv[i++]);
      cout << "Interpolate: " << maxInterpolate << ", " << maxDistance << ", " << gaps << "  ";
    } else {
      if (strcmp(argv[i], "-runaway") == 0) {
        i++;
        eliminaterunaway = true;
        runawayFactor = atof(argv[i++]);
        cout << "Eliminate Runaways " << runawayFactor << " times longer" <<  " ";
      } else {
        if (strcmp(argv[i], "-word") == 0) {
          i++;
          strcpy(word, argv[i++]); 
          cout << "  Word: " << word;
	} else {
          if (strcmp(argv[i], "-dir") == 0) {
            i++;
            strcpy(directory, argv[i++]);
            cout << endl << "Directory: " << directory << endl;
          } else {
            if (strcmp(argv[i], "-currentdir") == 0) {
              currentdir = true;
              i++;
              cout << "output is written to current directory" << endl;              
  	    } else {
              cout << "ignored parameter: " << argv[i++] << endl;
	    };
	  };
	};
      };
    };
  };

 

  
  // import points from file
  cout << "read points from file ... ";
  vector<Point> in_points(0);
  char filename[255];
  strcpy(filename, directory);
  strcat(filename, "/points.txt");
  ifstream datei(filename);
  if (!datei) {
    cout << "File " << filename << " not found ... exiting!" << endl;
    exit(0);
  };
  char dummy[255];
  unsigned char test;
  int x, y;
  while (!datei.eof()) {
    test = datei.peek();
    switch (test) {
    case '#':
    case 'S':
    case 'R':
    case 'H':
      // ignore
      while (test != 10)
        datei.get(test);
      break;

    case 'W':
      datei.get(test);
      test = datei.peek();
      if (test == 'o') {
        while (test != ' ') {
          datei.get(test);
          test = datei.peek();
        };  
        // read the true word
        datei >> word;
        while ((datei.peek() == 10) || (datei.peek() == 13))
          datei.get(test);

	cout << "IMPORTED WORD: [" << word << "]  " ;

      } else {
        // ignore
        while (test != 10)
          datei.get(test);
      };
      break;

    default:
      datei >> i >> dummy;
      test = datei.peek();
      while (test == ' ') {
        datei.get(test);
        test = datei.peek();
      }; 
      if ((test != 10) && (test != 13)) {
        datei >> x >> dummy >> y;
      } else {
        x = y = -1;
      };
      while ((datei.peek() == 10) || (datei.peek() == 13))
        datei.get(test);
      in_points.push_back(Point(x,y));
      // cout << in_points.size() << ". point: " << in_points[in_points.size()-1] << endl;
    };
  };  
  cout << in_points.size() << " points" << endl;




  // Convert the Points to interpolated version
  vector<Point> out_points(0);
  if (interpolate) {
    // interpolate it
    cout << "interpolating points ..." ;
    Interpolator aInterpolator(maxInterpolate, maxDistance, gaps);
    out_points = aInterpolator.apply(in_points);
  } else {
    // just copy it
    cout << "copying points ..." << endl;
    for (int j = 0; j < in_points.size(); j++) {
      out_points.push_back(in_points[j]);
    };
  };



  
  // eliminate runaways
  if (eliminaterunaway) {
    cout << " eliminate runaways ..." ;
    for (int j = 1; j < out_points.size(); j++) {
      // compare with previous and next position
      if (out_points[j-1].valid() && out_points[j].valid() && out_points[j+1].valid()) {
        double directDist = out_points[j-1].dist(out_points[j+1]);
        double normalDist = out_points[j-1].dist(out_points[j]) + \
                            out_points[j].dist(out_points[j+1]);
        if (normalDist <= runawayFactor * directDist) {
          // distance is OK
        } else {
          // actual Point is a runaway
          // -> interpolate between prev and next point
          int x = (unsigned int) ((out_points[j-1].x+out_points[j+1].x)/2.0 + 0.5);
          int y = (unsigned int) ((out_points[j-1].y+out_points[j+1].y)/2.0 + 0.5);
          out_points[j] = Point(x,y);
        };
      };
    };
  };




  // write it to the ETHZ file
  cout << " write points in ETHZ format to disk ..." ;
  if (currentdir == true) {
    strcpy(filename, word);
    strcat(filename, ".ethz");
  } else {
    strcpy(filename, directory);
    strcat(filename, "/points.ethz");
  };
  ofstream ETHZ(filename);
  // ETHZ << 1 << endl;  // 1 word
  ETHZ << '"' << word << '"' << endl;
  ETHZ << "[" << endl;
  unsigned int time = 0;
  for (int j = 0; j < out_points.size(); j++) {
    time += timeETHZ;
    if (out_points[j].valid()) {
      ETHZ << "[ " << out_points[j].x << "\t";
      ETHZ << out_points[j].y << "\t" << time << " ]" << endl;
    };
  };
  ETHZ << "]" << endl;




  // that's it
  cout << " done." << endl << endl;

};
