
/*---------------------------------------------------------------------------
|     filename: spider.h
|       author: Peter Steiner <psteiner@iam.unibe.ch>
|                             <100326.2444@compuserve.com>
|               Marcel Zumb"uhl <zumbuehl@iam.unibe.ch>
|
|     expanded: Jan Holler <holler@iamexwi.unibe.ch>
|                          <holler@holli.link-ch1.ch>
|
|  expanded by: mmntX_,   rgedX_,   rmrgX_,   stptX_   (X stands for a number)
|       tested: no no no  no no no  no no no  no no  (change to yes if tested)
|
|      history: spider.h 1.3  1994/09/08 14:05:26 psteiner Exp $
|          $Id: spider.h 1.31 1995/01/10 21:05:26 psteiner/jholler Exp $
+----------------------------------------------------------------------------
|   prototypes for SPIDER library functions
+----------------------------------------------------------------------------*/

#ifndef __SPIDER_H
#define __SPIDER_H

#ifdef __cplusplus
extern "C" {
#endif

#define INTP    const int*
#define CINTP   const int* const
#define FLTP    const float*
#define CFLTP   const float* const

/* affine transform of an image using linear interpolation */ 
void afin1_(
    CINTP ip,                   /* => input grayscale image */
    INTP  jp,                   /* <= output grayscale image */
    CINTP isx1, CINTP isy1,     /* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP isx2, CINTP isy2,     /* => dimensions of jp: jp[ isx2, isy2 ] */
    CFLTP t,                    /* => transformation matrix t[2,3] */
    CINTP isw,                  /* => parameter specifiing direction of matrix */
    INTP  kerr                  /* <=> error code */
);

/* affine transform of an image using quadratic interpolation */ 
void afin2_(
    CINTP ip,                   /* => input grayscale image */
    INTP  jp,                   /* <= output grayscale image */
    CINTP isx1, CINTP isy1,     /* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP isx2, CINTP isy2,     /* => dimensions of jp: jp[ isx2, isy2 ] */
    CFLTP t,                    /* => transformation matrix t[2,3] */
    CINTP isw,                  /* => parameter specifiing direction of matrix */
    INTP  kerr                  /* <=> error code */
);

/* measures the size of a specified region in a labeled image */
void area1_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => Label of specified region (0..NR) */
    INTP  jf,			/* <= array with area of each region */
    CINTP nr			/* => number of regions: jf[ nr ] */
);

/* measures the size of a specified region in a labeled image */
void area2_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => Label of specified region (0..NR) */
    INTP  jf,			/* <= array with area of each region */
    CINTP nr,			/* => number of regions: jf[ nr ] */
    CINTP isp			/* => starting points of each region (isp[2*nr]) */
);

/* measures the size of a specified region in a labeled image */
void area3_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => Label of specified region (0..NR) */
    INTP  jf,			/* <= array with area of each region */
    CINTP nr,			/* => number of regions: jf[ nr ] */
    CINTP iql			/* => circumscribing rect. of each region (iql[4*nr]) */
);

/* local selective averaging for edge preserving smoothing */
void asmt_(
    CINTP ip,                   /* => input image */
    INTP  jp,                   /* <= output image */
    CINTP isx, CINTP isy,	/* => dimensions of ip [isx, isy] */
    INTP  iwork1, INTP iwork2,  /*    work space */
    CINTP itime,                /* => number of iterations */
    CFLTP round                 /* => variable for specifying the 
				  rounding method*/
);

/* traces one boundary line in input image and returns a pair of list of their coords */
void bdfl1_( 
    CINTP ip,                   /* => input binary image */
    INTP  jp,                   /* <= output trinary image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    INTP  listx, INTP listy,    /* <= arrays with coordinates of boundary lines */
    CINTP leng,                 /* => size of listx, listy */
    INTP  leng1,                /* <=> length of listx/y actually used */
    CINTP ix0, CINTP iy0,	/* => starting point of trace */
    CINTP is,                   /* => direction in which trace starts (0 or 1) */
    CINTP nc,                   /* => neighbor connectedness (4 or 8) */
    INTP  kerr                  /* <=> error code: -1 if leng too small, 0 else */
);

/* traces all boundary lines in input image and returns a pair of list of their coords */
void bdfl2_( 
    CINTP ip,                   /* => input binary image */
    INTP  jp,                   /* <= output trinary image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    INTP  listx, INTP listy,    /* <= arrays with coordinates of boundary lines */
    CINTP leng,                 /* => size of listx, listy */
    INTP  leng1,                /* <=> length of listx/y actually used */
    INTP  nbd,                  /* <= number of boundary lines */
    CINTP nc,                   /* => neighbor connectedness (4 or 8) */
    INTP  kerr                  /* <=> error code: -1 if leng too small, 0 else */
);

/* gets the center of gravity of a specified (or every) region in a labeled image */
void cgrv1_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with center of gravity of each region */
    CINTP nr			/* => number of regions: jf[ 2*nr ] */
);

/* gets the center of gravity of a specified (or every) region in a labeled image */
void cgrv2_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with center of gravity of each region */
    CINTP nr,			/* => number of regions: jf[ 2*nr ] */
    CINTP isp			/* => starting points of each region (isp[2*nr]) */
);

/* gets the center of gravity of a specified (or every) region in a labeled image */
void cgrv3_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with center of gravity of each region */
    CINTP nr,			/* => number of regions: jf[ 2*nr ] */
    CINTP iql			/* => circumscribing rect. of each region (iql[4*nr]) */
);

/* gets the circumscribing rectangle of a specified (or every) region in a labeled image */
void cqlt1_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with circumscribing rect. of each region */
    CINTP nr			/* => number of regions: jf[ 4*nr ] */
);

/* gets the circumscribing rectangle of a specified (or every) region in a labeled image */
void cqlt1_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with circumscribing rect. of each region */
    CINTP nr			/* => number of regions: jf[ 4*nr ] */
);

/* gets the circumscribing rectangle of a specified (or every) region in a labeled image */
void cqlt2_(
    CINTP ip,			/* => input labeled image */
    CINTP isx, CINTP isy,	/* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,			/* => label of specified region (0..NR) */
    INTP  jf,			/* <= array with circumscribing rect. of each region */
    CINTP nr,			/* => number of regions: jf[ 4*nr ] */
    CINTP isp			/* => starting points of each region (isp[2*nr]) */
);

/* edge detection using the Kirsch operator. Outputs edge magnitude only. */
void egks1_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy        /* => dimensions of ip and jp: ip[ isx, jsy ] */
);

/* edge detection using the Kirsch operator. Outputs edge magnitude and direction of gradient. */
void egks2_(
    CINTP ip,                   /* => input image array */
    INTP  jp1,                  /* <= edge magnitude array */
    INTP  jp2,                  /* <= array to store gradient directions */
    CINTP isx, CINTP isy        /* => dimensions of ip and jpX: ip[ isx, jsy ] */
);

/* edge detection using a 3 x 3 Laplacian operator */
void eglp_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    CINTP itype                 /* => switch for selecting Laplacian operators */
);


/* edge preserving smoothing */
void egpr_(
    CINTP ip,                   /* => input image */
    INTP  jp,                   /* <= output image */
    CINTP isx,                  /* => input image size x */
    CINTP isy                   /* => input image size y */ 
);


/* edge detection using a using the Prewitt operator (differential type). Outputs edge magnitude only. */
void egpw1_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    CINTP itype                 /* => type of computation equation: 1..2 */
);

/* edge detection using the Prewitt operator (differential type). Outputs edge magnitude and direction of gradient. */
void egpw2_(
    CINTP ip,                   /* => input image array */
    INTP  jp1,                  /* <= edge magnitude array */
    INTP  jp2,                  /* <= array to store gradient directions */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jpX: ip[ isx, jsy ] */
    CINTP itype                 /* => type of computation equation: 1..2 */
);

/* edge detection using the Prewitt operator (template type). Outputs edge magnitude only. */
void egpw3_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy        /* => dimensions of ip and jp: ip[ isx, jsy ] */
);

/* edge detection using the Prewitt operator (template type). Outputs edge magnitude and direction of gradient. */
void egpw4_(
    CINTP ip,                   /* => input image array */
    INTP  jp1,                  /* <= edge magnitude array */
    INTP  jp2,                  /* <= array to store gradient directions */
    CINTP isx, CINTP isy        /* => dimensions of ip and jpX: ip[ isx, jsy ] */
);

/* edge detection using a using the Roberts operator */
void egrb_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    CINTP itype                 /* => type of computation in edge magnitude: 1..2 */
);

/* edge detection using a using the Sobel operator. Outputs edge magnitude only. */
void egsb1_(
    CINTP ip,                   /* => input image array */
    INTP  jp,                   /* <= edge magnitude array */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    CINTP itype                 /* => type of computation equation: 1..2 */
);

/* edge detection using a using the Sobel operator. Outputs edge magnitude and direction of gradient. */
void egsb2_(
    CINTP ip,                   /* => input image array */
    INTP  jp1,                  /* <= edge magnitude array */
    INTP  jp2,                  /* <= array to store gradient directions */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jpX: ip[ isx, jsy ] */
    CINTP itype                 /* => type of computation equation: 1..2 */
);

/* erase regions with vertical length less than a given threshold */
void ersr1_(
    CINTP ip,                   /* => input labeled image */
    INTP  jp,                   /* <= output image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    INTP  iq,			/*    work area (iq[nr]) */
    INTP  ir,			/*    work area (ir[nr]) */
    CINTP nr,			/* => must be greater than maximum label valuew */
    CINTP itate,		/* => vertical length threshold */
    INTP  jnr,			/* <= maximum label value after region relabeling */
    INTP  jerr			/* <= error code (-1 if nr too small, 0 else) */
);
    
/* erase regions with horizontal length less than a given threshold */
void ersr2_(
    CINTP ip,                   /* => input labeled image */
    INTP  jp,                   /* <= output image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    INTP  iq,			/*    work area (iq[nr]) */
    INTP  ir,			/*    work area (ir[nr]) */
    CINTP nr,			/* => must be greater than maximum label value */
    CINTP iyoko,		/* => horizontal length threshold */
    INTP  jnr,			/* <= maximum label value after region relabeling */
    INTP  jerr			/* <= error code (-1 if nr too small, 0 else) */
);

/* erase regions with area less than a given threshold */
void ersr3_(
    CINTP ip,                   /* => input labeled image */
    INTP  jp,                   /* <= output image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, jsy ] */
    INTP  iq,			/*    work area (iq[nr]) */
    INTP  ir,			/*    work area (ir[nr]) */
    CINTP nr,			/* => must be greater than maximum label value */
    CINTP iarea,		/* => region size threshold */
    INTP  jnr,			/* <= maximum label value after region relabeling */
    INTP  jerr			/* <= error code (-1 if nr too small, 0 else) */
);

/* hough transform using the Duda and Hart method */
void etdh1_(
    CINTP ip,                   /* => Edge magnitude array */
    INTP  jrt,                  /* <= two-dimensional array of theta and rho*/
    FLTP  sc,                   /*    work area (sines and cosines): sc[ 2*ith ] */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    CINTP ith, CINTP iro        /* => theta,rho: dimensions of jrt: jrt[ ith, iro ] */
);

/* 1-dim. fast Fourier transform */
void ffta1_(
    CFLTP ar,                   /* => Real part of one-dim. input data, [is] */
    CFLTP ai,                   /* => Imaginary part ... */
    FLTP  br,                   /* <= Real part of results of the DFT/IDFT [isf], */
    FLTP  bi,                   /* <= Imaginary part ... */
    FLTP  wr,                   /*    Work space for phase terms; real part [isf], */
    FLTP  wi,                   /*          "          "          imaginary   *    */
    CINTP is,                   /* => Data length (arbitrary) */
    CINTP isf,                  /* => Minimum number, which is  expressed in terms
                                  of powers of 2 and is greater than or equal
                                  to 2 * is-1. */
    FLTP  wst,                  /*    Work space to store sine table */
    FLTP  wct,                  /*    Work space to store cosine table */
    INTP ibr,                   /*    bit-reversed order integer sequence */ 
    CINTP inv                   /* => Parameter to specify the forward or inverse
                                  transform.
                                  inv= 1 ... forward transform
                                  inv=-1 ... inverse transform */
);

/* 2-dim. fast fourier transform */

void ffta2_(
    CFLTP ar,                   /* => Real part of an input image [isx,isy]*/
    CFLTP ai,                   /* => Imaginary part ...              "    */
    FLTP  br,                   /* <= Real part of computation results of the
                                  two-dimensional DFT/IDFT [isx,isy] */
    FLTP  bi,                   /* <= Imaginary part ...                 */
    FLTP  wr,                   /*    Work space for phase terms; real part, 
                                  [isfx,isfy] */
    FLTP  wi,                   /*         "              "     ; imaginary part */
    CINTP isx,                  /* => Image size (X-axis) */
    CINTP isy,                  /* =>      "     (Y-axis) */ 
    CINTP isxf, CINTP isyf,     /* => Minimum numbers, which are expressed in
                                  terms of powers of 2 and satisfy the 
                                  following inequalities:
                                  ISXF >= 2 * ISX-1,
                                  ISYF >= 2 * ISY-1. */ 
    FLTP  wst, FLTP  wct,       /*    Work space to store sine & cosine table */
    CINTP ibr,                  /*    Work space to store the table of the 
                                  bit-reversed order integer sequence. */
    CINTP ismf,                 /* => ismf=isxf+isyf (ismf > isxf + isyf is also 
                                  permitted.) */
    CINTP inv,                  /* => Parameter to specify the forward or inverse
                                  transform.
                                  inv= 1 ... forward transform
                                  inv=-1 ... inverse transform */
    INTP jerr                   /*    Error code */
);

/* expanding gray-scale by multiplying a constant value */
void gmlt_(
    CINTP ip,			/* => input image */
    INTP  jp,			/* <= transformed image */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    CFLTP c,			/* => constant for multiplying */
    CINTP ngro,			/* => number of gray levels of ip */
    CINTP ngrn,			/* => number of gray levels of jp (new range) */
    INTP  jtbl			/*    work area: jtbl[ ngro ] */
);

/* executes a gray scale transformation */
void gtrn1_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= output image [isx,isy] */
    CINTP isx,                  /* => x dimension */
    CINTP isy,                  /* => y dimension */
    CINTP itbl,                 /* => gray scale transf. table [ngr] */
    CINTP ngr                   /* => nr of gray scales in input im. */
);

/* executes a gray scale transformation (the same gray levels are subdivided) */
void gtrn2_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= output image [isx,isy] */
    CINTP isx,                  /* => x dimension */
    CINTP isy,                  /* => y dimension */
    CFLTP hste,                 /* => target histogram [ngrn] */
    CFLTP hstn,                 /* => histogram [ngrn] of output image */
    CINTP ngro,                 /* => nr of gray levels in input im. */
    CINTP ngrn,                 /* => nr of gray levels in output im. */
    CINTP jtbl,                 /* => gray scale transf. table [2,ngro] */
    CFLTP wtbl,                 /* => work table [4,ngro] */
    CFLTP rt                    /* => reciprocal of total number of pixels */
);

/* obtains a histogram of gray levels in image */
void hist1_(
    CINTP ip,                   /* => input image */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    FLTP  rhst,                 /* <= output histogram */
    CINTP ngr                   /* => size of rhst */
);

/* obtains a histogram of gray levels of a masked region in image */
void hist1m_(
    CINTP ip, CINTP mp,         /* => input image (ip), => mask plane (mp) */
    CINTP isx, CINTP isy,       /* => dimensions of ip (mp): ip(mp) [ isx, jsy ] */
    FLTP  hsto,                 /* <= output histogram */
    CINTP ngro,                 /* => size of rhst (= nr. of gray levels) */
    CINTP imode                 /* Mask plane mode: */
                                /* When IMODE=0, points (I,J) for which */
                                /* MP(I,J)=0 are counted in the histogram. */
                                /* When IMODE .NE. 0, points (I,J) for which */
                                /* MP(I,J) .NE. 0 are counted in the histogram. */
);

/* generates a gray scale transformation table */
void htbl1_(
    CFLTP hsto,                 /* => histogram of input image [ngro]*/
    CFLTP hste,                 /* => target histogram [ngrn]*/
    CINTP ngro,                 /* => nr of gray levels of input im. */
    CINTP ngrn,                 /* => nr of gray levels of output im. */
    INTP  jtbl                  /* <= transformation table */
);

/* generates a gray scale transformation table (with tie-breaking) */
void htbl2_(
    CFLTP hsto,                 /* => histogram of input image [ngro]*/
    CFLTP hste,                 /* => target histogram [ngrn]*/
    CINTP ngro,                 /* => nr of gray levels of input im. */
    CINTP ngrn,                 /* => nr of gray levels of output im. */
    INTP  jtbl,                 /* <= transformation table [2,ngro] */
    FLTP  btbl                  /* <= work table [4,ngro] */
);

/* histogram equalization without subdivision of the same gray 
   level */
void hgeq1_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= transformed image [isx,isy] */
    CINTP isx, CINTP isy,       /* => image width and height */
    FLTP  hsto,                 /* <= histogram of input image [ngro] */
    FLTP  hstn,                 /* <= histogram of transformed image 
                                  [ngrn] */
    CINTP ngro,                 /* => number of gray levels of inp. im.*/
    CINTP ngrn,                 /* =>     "      "      "      outp. " */
    INTP  jtbl                  /*    histogram table [ngro] */
);

/* histogram equalization with subdivision of the same gray level */
void hgeq2_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= transformed image [isx,isy] */
    CINTP isx, CINTP isy,       /* => image width and height */
    FLTP  hsto,                 /* <= histogram of input image [ngro] */
    FLTP  hste,                 /*    work space [ngrn] */
    FLTP  hstn,                 /* <= histogram of transformed image 
                                      [ngrn] */
    CINTP ngro,                 /* => number of gray levels of inp. im.*/
    CINTP ngrn,                 /* =>     "      "      "      outp. " */
    INTP  jtbl,                 /*    histogram table [2,ngro]*/
    FLTP  wtbl                  /*   work table [4,ngro] */
);

/* histogram hyperbolization */
void hghy1_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= transformed image [isx,isy] */
    CINTP isx, CINTP isy,       /* => image width and height */
    CFLTP c,                    /* => parameter to determine hyperbolic
			              distribution ( > 0 ) */
    FLTP  hsto,                 /* <= histogram of input image [ngro] */
    FLTP  hstn,                 /* <= histogram of output image [ngrn] */
    CINTP ngro,                 /* => number of gray levels of inp. im.*/
    CINTP ngrn,                 /* =>     "      "      "      outp. " */
    INTP  jtbl                  /*    histogram table [ngro]*/
);

void hgtr2_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= output image [isx,isy] */
    CINTP isx,                  /* => x dimension */
    CINTP isy,                  /* => y dimension */
    FLTP hsto,                  /* <= histogram of input image [ngro] */
    CFLTP hste,                 /* => target histogram [ngrn] */
    FLTP hstn,                  /* <= histogram [ngrn] of output image */
    CINTP ngro,                 /* => nr of gray levels in input im. */
    CINTP ngrn,                 /* => nr of gray levels in output im. */
    INTP jtbl,                  /* <= gray scale transf. table [2,ngro] */
    FLTP wtbl                   /* <= work table [4,ngro] */
);

/* standard hysteresis smoothing */
void hysm1_(
    CINTP ip,                   /* => input image [isx,isy] */
    INTP  jp,                   /* <= transformed image [isx,isy] */
    CINTP isx, CINTP isy,       /* => dimensions of kp: kp[ isx, jsy ] */
    CINTP ncurs                 /* => cursor length */
);

/* measures a moment of a specified region or every region in a labeled
image */
void mmnt1_(
    CINTP ip,                   /* => input image [isx,isy] */
    CINTP isx1, CINTP isy1,     /* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,                  /* => label of specified region (0..NR) */
    FLTP  bt,                   /* <= moment of each region bt[NR] */
    CINTP nr,                   /* => number of moments, if 0 every moment */
    CINTP ii,                   /* => order of x component of moment */
    CINTP jj,                   /* => order of y component of moment */
    CINTP nlz                   /* => specification of normalization
                                      nlz = 0 moment around center of gravity
                                      nlz = 1 moment based on principal axis */
);

/* measures a moment of a specified region or every region in a labeled
image */
void mmnt2_(
    CINTP ip,                   /* => input image [isx,isy] */
    CINTP isx1, CINTP isy1,     /* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,                  /* => label of specified region (0..NR) */
    FLTP  bt,                   /* <= moment of each region bt[NR] */
    CINTP nr,                   /* => number of moments, if 0 every moment */
    CINTP ii,                   /* => order of x component of moment */
    CINTP jj,                   /* => order of y component of moment */
    CINTP nlz,                  /* => specification of normalization
                                      nlz = 0 moment around center of gravity
                                      nlz = 1 moment based on principal axis */
    CINTP isp			/* => starting points of each region (isp[2*nr]) */
);

/* measures a moment of a specified region or every region in a labeled
image */
void mmnt3_(
    CINTP ip,                   /* => input image [isx,isy] */
    CINTP isx1, CINTP isy1,     /* => dimensions of ip: ip[ isx1, isy1 ] */
    CINTP irn,                  /* => label of specified region (0..NR) */
    FLTP  bt,                   /* <= moment of each region bt[NR] */
    CINTP nr,                   /* => number of moments, if 0 every moment */
    CINTP ii,                   /* => order of x component of moment */
    CINTP jj,                   /* => order of y component of moment */
    CINTP nlz,                  /* => specification of normalization
                                      nlz = 0 moment around center of gravity
                                      nlz = 1 moment based on principal axis */
    CINTP iql			/* => circumscribing rect. of each region (iql[4*nr]) */
);

/* generates a line passing through two points */
void line1_(
    INTP  kp,                   /* <=> image in which the line is drawn */
    CINTP isx, CINTP isy,       /* => dimensions of kp: kp[ isx, jsy ] */
    CINTP ix1, CINTP iy1,       /* => first point */
    CINTP ix2, CINTP iy2        /* => second point */
);

/* generates a line segment between two points */
void line2_(
    INTP  kp,                   /* <=> image in which the line is drawn */
    CINTP isx, CINTP isy,       /* => dimensions of kp: kp[ isx, jsy ] */
    CINTP ix1, CINTP iy1,       /* => first point */
    CINTP ix2, CINTP iy2        /* => second point */
);

/* normalizes a histogram so that the total of the values of its elements is 1 */
void noht1r_(
    CFLTP histi,                /* => input histogram */
    FLTP  histo,                /* <= output histogram */
    CINTP nsize                 /* => size of histi and histo */
);

/* normalizes the grayscale to av(erage) and s(tandard) d(eviat.) */
void norm1_(
    CINTP ip,                   /* => input image */
    INTP  jp,                   /* <= output image */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    CINTP ngro,                 /* => number gray levels of input im.*/
    CINTP ngrn,                 /* => number gray levels of outp. im.*/
    CFLTP av,                   /* => average of output gray scale */
    CFLTP sd,                   /* => stand. dev. of output gray scale */
    INTP  jtbl,                 /*    work space */
    CINTP isw                   /* => mode: 0: output image values are 
                                           constrained to ngrn gray
                                           levels.
                                     <> 0 : no constraints */
);

/* projects an image in the x- or y-direction */
void proj1_(
    CINTP ip,                   /* => input image */
    INTP  jpr,			/* <= projection array (jpr[isx] or jpr[isy]) */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    CINTP iort			/* => direction (0 : y-, 1 : x-direction) */
);

/* projects part of an image in the x- or y-direction */
void proj1w_(
    CINTP ip,                   /* => input image */
    INTP  jpr,			/* <= projection array (jpr[isx] or jpr[isy]) */
    CINTP isx, CINTP isy,       /* => dimensions of ip: ip[ isx, jsy ] */
    CINTP iw,			/* => window in ip (iw[4]) */
    CINTP iort			/* => direction (0 : y-, 1 : x-direction) */
);

/* extracts or deletes regios in a labeled image the value of which property
   is in a specified range */
void rged1_(
    CINTP ip,                    /* => input image */
    INTP  jp,                    /* <= labeled image */
    CINTP isx, CINTP isy,        /* => dimensions of ip: ip[ isx, isy ] */
    CINTP mode,                  /* => MODE = 0   delete
                                       MODE = 1   extract */
    CINTP nr,                    /* => number of regions */
    CINTP ithd,                  /* => threshold and specification of range
                                            ithd[3]
                                       ithd[1]  smaller threshold
                                       ithd[2]  larger threshold
                                       ithd[3]=0  within treshold range
                                       ithd[3]=1  outside treshold range */
    CINTP lbl                    /* => label assigned to extracted region */
);

/* extracts or deletes regios in a labeled image the value of which property
   is in a specified range */
void rged2_(
    CINTP ip,                    /* => input image */
    INTP  jp,                    /* <= labeled image */
    CINTP isx, CINTP isy,        /* => dimensions of ip: ip[ isx, isy ] */
    CINTP mode,                  /* => MODE = 0   delete
                                       MODE = 1   extract */
    CINTP nr,                    /* => number of regions */
    CINTP ithd,                  /* => threshold and specification of range
                                            ithd[3]
                                       ithd[1]  smaller threshold
                                       ithd[2]  larger threshold
                                       ithd[3]=0  within treshold range
                                       ithd[3]=1  outside treshold range */
    CINTP lbl,                    /* => label assigned to extracted region */
    CINTP isp			/* => starting points of each region (isp[2*nr]) */
);

/* extracts or deletes regios in a labeled image the value of which property
   is in a specified range */
void rged3_(
    CINTP ip,                    /* => input image */
    INTP  jp,                    /* <= labeled image */
    CINTP isx, CINTP isy,        /* => dimensions of ip: ip[ isx, isy ] */
    CINTP mode,                  /* => MODE = 0   delete
                                       MODE = 1   extract */
    CINTP nr,                    /* => number of regions */
    CINTP ithd,                  /* => threshold and specification of range
                                            ithd[3]
                                       ithd[1]  smaller threshold
                                       ithd[2]  larger threshold
                                       ithd[3]=0  within treshold range
                                       ithd[3]=1  outside treshold range */
    CINTP lbl,                    /* => label assigned to extracted region */
    CINTP iql			/* => circumscribing rect. of each region (iql[4*nr]) */
);



/* segments an image into regios by the Kolmogorov-Smirnov test method
   proposed by Muerle an Allen, and labels each region */
void rmrg1_(
    CINTP ip,                    /* => input image */
    INTP  jp,                    /* <= labeled image (result of reg. segm.) */
    CINTP isx1, CINTP isy1,      /* => dimensions of ip: ip[ isx1, isy1 ] */
    INTP  isx2, CINTP isy2,      /* <= dimensions of jp: jp[ isx2, isy2 ] */
    CINTP nx, CINTP ny,          /* => cell size */
    CINTP nt,                    /* => number of thresholds */
    CINTP thd,                   /* => thresholds thd[nt] */
    CINTP ia,                    /* => maximum area of region in which
                                       treshold is effective ia[nt] */
    CINTP lvl,                   /* => number of gray levels */
    INTP  nr,                    /* <= number of regions */
    FLTP  ihst                   /* <=> work area for hist.comp. ihst[lvl,2] */
);

/* segments an image into regios by the Kolmogorov-Smirnov test method
   proposed by Muerle an Allen, and labels each region */
void rmrg2_(
    CINTP ip,                    /* => input image */
    INTP  jp,                    /* <= labeled image (result of reg. segm.) */
    CINTP isx, CINTP isy,        /* => dimensions of ip: ip[ isx, isy ] */
    CINTP thd,                   /* => thresholds thd[3] */
    INTP  nr,                    /* <= number of regions */
    FLTP  iwork                  /* <=> work area   iwork[isx,isy] */
);

/* segments an image into regios by the (iterative) merging method, and
   labels each region */
void rmrg3_(
    CINTP ip,                    /* => input image */
    INTP  jp1,                   /* <= labeled image */
    INTP  jp2,                   /* <= averaged image array */
    CINTP isx, CINTP isy,        /* => dimensions of ip: ip[ isx, isy ] */
    CINTP nthd,                  /* => threshold */
    CINTP kavr,                  /* => KAVR = 0   input value
                                       KAVR = 1   averaged value */
    INTP nr,                     /* <= number of regions */
    CINTP nc                     /* => connectedness (4 or 8) */
);

/* assigns a constant to a two-dimensional (image) array */
void rpla1i_(
    INTP  jp,                   /* <=> binary image */
    CINTP isx, CINTP isy,       /* => dimensions of jp: ip[ isx, jsy ] */
    CINTP n			/* => constant to be assigned */
);

/* assigns a constant to the interior or exterior of a window of an image */
void rpla1w_(
    INTP  jp,                   /* <=> binary image */
    CINTP isx, CINTP isy,       /* => dimensions of jp: jp[ isx, isy ] */
    CINTP iw,			/* => window in jp (iw[4]) */
    CINTP n,			/* => constant to be assigned */
    CINTP isw			/* => where to assign (0:, 1:) */
);

/* binarizes a gray scale image by changing all non-zero gray levels to 1's */
void sclp_(
    CINTP ip,                   /* => input grayscale image */
    INTP  jp,                   /* <= output binary image */
    CINTP isx, CINTP isy        /* => dimensions of ip and jp: ip[ isx, isy ] */
);

/* binarizes a gray scale image using a single threshold */
void slth1_(
    CINTP ip,                   /* => input grayscale image */
    INTP  jp,                   /* <= output binary image */
    CINTP isx, CINTP isy,       /* => dimensions of ip and jp: ip[ isx, isy ] */
    CINTP ithd,                 /* => threshold for binarization */
    CINTP isw                   /* => binarization mode: 1..4 (>,>=,<,<= threshold) */
);

/* obtains elementary statistics from a given set of data */
void stas1_(
    CINTP ip,			/* => input data */
    CINTP isxy,                 /* => dimensions of ip: ip[ isxy ] */
    FLTP  hist,			/* <= one-dimensional array for histogram */
    CINTP n,			/* => size of histogram: hist[n] */
    CFLTP xlb,			/* => lower bound for computing histogram */
    CFLTP d,			/* => width of intervals in histogram */
    CFLTP w,			/* => parameter setting method for computing histogram */
    FLTP  xmax, FLTP xmin,	/* <= maximum and minimum values */
    FLTP  e,			/* <= mean */
    FLTP  v,			/* <= variance */
    FLTP  sigma,                /* <= standard deviation */
    FLTP  range			/* <= range (xmax-xmin) */
);

/* calculates the maximum, minimum, mean and standard deviation from a given histogram */
void stas5_(
    CFLTP rhst,                 /* => normalized histogram */
    CINTP ngr,                  /* => size of rhst */
    FLTP  rmean,                /* <= mean */
    FLTP  stdev,                /* <= standard deviation */
    INTP  maxv, INTP minv       /* <= maximum and minimum */
);

/* mesures the coordinates of the starting point of a specified region or every
   region in a labeled image */
void stpt1_(
    CINTP ip,                   /* => labeled image array */
    CINTP irn,			           /* => Label of specified region (0..NR) */
    INTP  jf,                   /* <= array with area of each region */
    CINTP nr                    /* => number of regions */
);

/* mesures the coordinates of the starting point of a specified region or every
   region in a labeled image */
void stpt2_(
    CINTP ip,                   /* => labeled image array */
    CINTP irn,			           /* => Label of specified region (0..NR) */
    INTP  jf,                   /* <= array with area of each region */
    CINTP nr,                   /* => number of regions */
    CINTP iql			/* => circumscribing rect. of each region (iql[4*nr]) */
);

/* threshold selection for binarization by the p-tile method */
void thds1_(
    CFLTP rhst,                 /* => input histogram */
    CINTP ngr,                  /* => nr of gray levels in input hist. */
    CFLTP ptile,                /* => percentile */
    INTP jthd                   /* <= selected threshold */
);

/* threshold selection for binarization by Otsu's method */
void thds2_(
    CFLTP rhst,                 /* => input histogram */
    CINTP ngr,                  /* => size of rhst */
    INTP  jthd,                 /* <= selected threshold value */
    FLTP  rmean,                /* <= global mean */
    FLTP  vari,                 /* <= global variance */
    FLTP  p0, FLTP p1,          /* <= probability of classes below/above threshold */
    FLTP  a0, FLTP a1,          /* <= mean of classes below/above */
    FLTP  sep                   /* <= class separability */
);

/* computes a transform matrix when a rotation angle and the center of the rotation are given */
void trmt_(
    FLTP  t,                    /* <=> transformation matrix t[2,3] */ 
    CFLTP x0, CFLTP y0,         /* => coordinates of the center of the rotation */
    CFLTP rad                   /* => rotation angle (radian) */
);

void txav_(
    CINTP ip1,                  /* => input image array [isx,isy] */
    INTP  jp2,                  /* <= output image array [isx,isy] */
    CINTP isx,                  /* => image size x */
    CINTP isy,                  /* => image size y */
    CINTP k                     /* => size of window (2**k) */
);

void txdf1_(
    CINTP ip2,                  /* => input image array [isx,isy] */
    INTP  jp3,                  /* <= output image array [isx,isy] */
    CINTP isx,                  /* => image size x */
    CINTP isy,                  /* => image size y */
    CINTP k,                    /* => size of window (2**k) */
    CINTP id                    /* => edge orientation (0=horiz.,
				  1=vert.) */
);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __SPIDER_H */
