#!/usr/local/bin/perl

# absolute mode: constants to generate trainsets
@mins = (5,6,7,8);        # max: 5 to 8
@maxs = (6,7,8,9,10);     # max: 5 to 10
@gaps = (3,4,5);          # max: 3 to 5

# relative mode: testdata around optimal data
# e.g. intermin = optimal_intermin +/- dmin
$dmin = 2;                # +/- 2 -> 5 values
$dmax = 2;                # +/- 2 -> 5 values
$dgap = 1;                # +/- 1 -> 3 values


# "optimal" detection method
$meth = "neighbour";            
$program = "src/points2ethz";


# variables
@optimal = ();     # best result for this one word
@result = ();      # actual interpolated list for word
@trainset = ();    # actual trainset-list for word
$trainsets = 0;    # number of trainsets for actual word
$equalsets = 0;    # number of rejected train sets

if ($#ARGV < 2) {
    print "\n";
    print "Usage: testdata.pl -relative|-absolute destination source1 [src2 src3 ...]\n";
    print "\n";
    exit;
};
$relative = 1;
$absolute = 2;
$mode = 0;
if ($ARGV[0] eq "-relative") {
    $mode = $relative;
};
if ($ARGV[0] eq "-absolute") {
    $mode = $absolute;
};
if ($mode == 0) {
    print "\n";
    print "Usage: testdata.pl -relative|-absolute destination source1 [src2 src3 ...]\n";
    print "\n";
    exit;
};
$destination = $ARGV[1];
for($i = 2; $i <= $#ARGV; $i++) {

  # read config file
  $source = $ARGV[$i];
  open(CONFIG, $source) || die("config file ($source) not found.\n");
  while (<CONFIG>) {
    if ((substr($_,0,1) ne '#') && (substr($_,0,1) ne ' ') && ($_ ne "\n")) {
      ($variable, $values) = split("=", $_);
      if (substr($values, length($values)-1,1) !~ "\d") {
        # remove trailing newline
        $values = substr($values, 0, length($value)-1);
      };
      @values = split(",",$values);
      if ($variable eq 'methods') {
        # methods has to be list but single value is allowed !!!
        if ($#values > 0) {
          # list of values
          eval "\@$variable = \@values";
          #print "\@$variable = @values\n";
        } else {
          # single value
          eval "\@$variable = ($values[0])";
          #print "\@$variable = ($values[0])\n";
        };
      } elsif ($#values > 0) {
        # list of values
        eval "\@$variable = \@values";
        #print "\@$variable = @values\n";
      } else {
        # single value
        eval "\$$variable = '$values[0]'";
        #print "\$$variable = '$values[0]'\n";
      };
    };
  };
  close CONFIG;


#
# patch to apply neighbour-method only
@methods = ("neighbour");
# end of patch
#


  # first generate optimal list of points
  $directory = $report_dir . $report_name . $report_details . "/"; 
  $directory .= "$meth" . "_";
  $directory .= "look" . $lookahead . "_"; 
  $directory .= "otsu" . $otsuvariance . "_";
  $directory .= "ipol" . $intermin . "-" . $intermax . "-" . $intergap . "_";
  $directory .= "win" . $winsize . "_";
  $directory .= "mask" . $masksize . "_";
  $directory .= "area" . $area . "/";  
  system "$program -interpolate $intermin $intermax $intergap -word $word -dir $directory";
  print "\n";
  @optimal = ();
  open (OPT, "$directory/points.ethz");
  @optimal = <OPT>;
  close OPT;


  # start reporting into TXT file
  $resname = "$destination/$report_name.txt";
  $resname =~ s/\/\//\//g;
  open (REPORT, ">$resname");
  print REPORT "ETHZ Trainset for $source\n";
  print REPORT "\n";
  print REPORT "Methods:  ";
  foreach $method (@methods) { print REPORT "$method "; }; print REPORT "\n";
  if ($mode eq $relative) {
    # fill the arrays @mins, @maxs, @gaps relative to optimal parameters
    @mins = ();
    for ($min = $intermin - $dmin; $min <= $intermin + $dmin; $min++) {
      push @mins, $min;  
    };
    @maxs = ();
    for ($max = $intermax - $dmax; $max <= $intermax + $dmax; $max++) {
      push @maxs, $max;  
    };
    @gaps = ();
    for ($gap = $intergap - $dgap; $gap <= $intergap + $dgap; $gap++) {
      push @gaps, $gap;  
    };
    print REPORT "Interpolation of testdata is relative to optimal parameters\n";
  };
  print REPORT "Intermin: @mins \n";
  print REPORT "Intermax: @maxs \n";
  print REPORT "Intergap: @gaps \n";
  print REPORT "\n";
  print REPORT "Optimal values: $intermin, $intermax, $intergap, $meth\n";
  print REPORT "\n";


  # process every detection routine
  $trainsets = 0;
  $equalsets = 0;
  @trainset = ();
  $optimal_set_in_range = 0;
  foreach $method (@methods) {
    
    # generate the full pathname
    $directory = $report_dir . $report_name . $report_details . "/"; 
    $directory .= "$method" . "_";
    $directory .= "look" . $lookahead . "_"; 
    $directory .= "otsu" . $otsuvariance . "_";
    $directory .= "ipol" . $intermin . "-" . $intermax . "-" . $intergap . "_";
    $directory .= "win" . $winsize . "_";
    $directory .= "mask" . $masksize . "_";
    $directory .= "area" . $area . "/";
    print "processing $directory\n";

    # collect all the max/min/gap combinations into 1 file
    foreach $min (@mins) {
      foreach $max (@maxs) {
	foreach $gap (@gaps) {

          # calculate interpoaltion
          system "$program -interpolate $min $max $gap -word $word -dir $directory";
          @result = ();
          open (RES, "$directory/points.ethz");
          @result = <RES>;
          close RES;

          # compare result with optimal result
          if (($min == $intermin) && ($max == $intermax) && ($gap == $intergap)) {
	    $optimal_set_in_range++;
          };
          $equal = 1;
          if ($#result != $#optimal) {
            $equal = 0;
  	  } else {
	    for ($j = 0; $j <= $#result; $j ++) {
	      if ($result[$j] ne $optimal[$j]) {
                $equal = 0;
              };  
            };
          };
          if ($equal != 1) {
            # different -> add result to trainset
            push @trainset, @result;
	    $trainsets++;
	  } else {
	    $equalsets++;
            print REPORT "Interpolation $min $max $gap is equal to optimal interpolation\n";  
	  };

        };
      };
    };

  };
  if ($optimal_set_in_range >= 1) {
    print REPORT "optimal set in range of interpolation\n";
  } else {
    print REPORT "optimal set NOT in range of interpolation\n";
  };
  print REPORT "$trainsets trainsets accepted, $equalsets sets rejected\n";
  close REPORT;


  # finsish this config-file
  $resname = "$destination/$report_name";
  $resname =~ s/\/\//\//g;
  open (OPTIMAL, ">$resname.test");
  print OPTIMAL @optimal;
  close OPTIMAL;
  open (TRAIN, ">$resname.train");
  print TRAIN "$trainsets\n";
  print TRAIN @trainset;
  close TRAIN;

  print "all done.\n";

};

# every source file done !










