
// Header Files for Motif
#include <X11/Intrinsic.h>     // The Intrinsics                   
#include <Xm/Xm.h>             // Motif symbols                    
#include <Xm/MainW.h>          // XmMainWindow widget              
#include <Xm/ScrolledW.h>      // XmScrolledWindow widget          
#include <Xm/RowColumn.h>      // XmRowColumn widget               
#include <Xm/Form.h>           // XmForm widget                    
#include <Xm/Text.h>           // XmText widget                    
#include <Xm/CascadeB.h>       // XmCascadeButton widget           
#include <Xm/PushB.h>          // XmPushButton widget              
#include <Xm/DrawingA.h>       // XmDrawingArea widget             
#include <Xm/TextF.h>          // XmTextField widget               
#include <Xm/Scale.h>          // XmScale widget                   
#include <Xm/Frame.h>          // XmFrame widget                   
#include <Xm/Label.h>          // XmLabel widget                   
#include <Xm/FileSB.h>         // XmFileSelectionBox widget        
#include <Xm/ToggleB.h>        // XmToggleButton widget            
#include <Xm/List.h>           // XmList widget            



// own Header Files
#include "image/Image.h"
#include "image/Image8.h"
#include "image/ImageSourcePPM.h"
#include "xv/xv.c"
#include <iostream.h>
#include "image/FilterRotate180.h"
#include "image/FilterExpandBlack.h"
#include "image/FilterCutMask.h"
#include "image/FilterCutRectangle.h"
#include "image/Rectangle.h"
#include <fstream.h>




// Globals for Motif
XtAppContext  context;
Widget     toplevel, mainManager, commandPanel;
Widget     graphicsManager, graphics, graphicsScroll;
Widget     controlFrame, controlFrameTitle, controlManager, controlClearMask;
Widget     controlScroll, controlNext, controlNext5, controlPrev, controlPrev5;
Widget     controlSetStart, controlSetEnd, controlGotoStart, controlGotoEnd;
Widget     controlRow1, controlRow2, controlRow3;
Widget     menuBar, menuButtonHelp, menuFileSetReportDir;
Widget     menuButtonFile, menuFile, menuFileSetSourceDir, menuFileExit;
Widget     DirectorySelection;
Widget     sourceFrame, sourceFrameTitle, sourceManager, sourceDigits;
Widget     sourceManagerDir, sourceDir, sourceDirLabel, sourceReload;
Widget     sourceManagerPrefix, sourcePrefix, sourcePrefixLabel, sourceManagerScroll;
Widget     reportFrame, reportFrameTitle, reportManager;
Widget     reportManagerDir, reportDir, reportDirLabel;
Widget     reportManagerName, reportName, reportNameLabel;
Widget     interFrame, interFrameTitle, interManager;
Widget     interMin, interMax, interGap;
Widget     imageFrame, imageFrameTitle, imageManager;
Widget     imageBox, imageInterval;
Widget     paramsFrame, paramsFrameTitle, paramsManager;
Widget     paramsMasksize, paramsLookahead, paramsOtsuVariance;
Widget     methodFrame, methodFrameTitle, methodManager;
Widget     methodNeighbour, methodMedian, methodIterative, methodGreyscale;
Widget     methodSimple, methodGreyIterative;
Widget     exportFrame, exportFrameTitle, exportManager;
Widget     exportLabel, exportFilename, exportDoit;
Widget     exportWordLabel, exportWord;
Widget     windowFrame, windowFrameTitle, windowManager, windowSize;
Widget     winexpManager;



// Globals for X11, libimage and program
ImageSourcePPM theSource;
char theSourceDirectory[255];
char theSourcePrefix[255];
int theSourceDigits = 4;
Image* theImage;
XImage* theXImage = NULL;
Pixmap thePixmap;
unsigned int theWidth, theHeight;
unsigned int theStart, theEnd;
Rectangle theBoundingBox;
Image* theMask;
GC xorGC;




// Callbacks
void RepaintCallback(Widget, XtPointer, XtPointer);
void ExitCallback(Widget, XtPointer, XtPointer);
void ScrollCallback(Widget, XtPointer, XtPointer);
void NextCallback(Widget, XtPointer, XtPointer);
void Next5Callback(Widget, XtPointer, XtPointer);
void PrevCallback(Widget, XtPointer, XtPointer);
void Prev5Callback(Widget, XtPointer, XtPointer);
void SetStartCallback(Widget, XtPointer, XtPointer);
void SetEndCallback(Widget, XtPointer, XtPointer);
void GotoStartCallback(Widget, XtPointer, XtPointer);
void GotoEndCallback(Widget, XtPointer, XtPointer);
void ExportCallback(Widget, XtPointer, XtPointer);
void ClearMaskCallback(Widget, XtPointer, XtPointer);
void ReportNameCallback(Widget, XtPointer, XtPointer);
void FileSetSourceDirCallback(Widget, XtPointer, XtPointer);
void FileSetReportDirCallback(Widget, XtPointer, XtPointer);
void ReloadCallback(Widget, XtPointer, XtPointer);
void Image2Pixmap();
void makeNewMask();
void StartRubber(Widget, XtPointer, XtPointer);
void Rubberband(Widget, XtPointer, XEvent*, Boolean*);
void EndRubber(Widget, XtPointer, XtPointer);


main(int argc, char **argv) {

  // Init. Toolkit, App. Context, Application Shell
  toplevel = XtAppInitialize( &context, "ViewImages",
      NULL, 0, &argc, argv, NULL, NULL, 0); 


  // Create widget tree, bind callbacks
  mainManager = XtCreateManagedWidget( "mainManager",
      xmMainWindowWidgetClass, toplevel, NULL, 0);


  menuBar = XmCreateMenuBar (mainManager, "menuBar", NULL, 0); 
  XtManageChild (menuBar);
  menuFile = XmCreatePulldownMenu (menuBar, "menuFile", NULL, 0);
  menuFileSetSourceDir = XtCreateManagedWidget( "menuFileSetSourceDir",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileSetSourceDir, XmNactivateCallback, FileSetSourceDirCallback, NULL);
  menuFileSetReportDir = XtCreateManagedWidget( "menuFileSetReportDir",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileSetReportDir, XmNactivateCallback, FileSetReportDirCallback, NULL);
  menuFileExit = XtCreateManagedWidget( "menuFileExit",
      xmPushButtonWidgetClass, menuFile, NULL, 0);
  XtAddCallback(menuFileExit, XmNactivateCallback, ExitCallback, NULL);
  Arg arglist[1];
  XtSetArg(arglist[0], XmNsubMenuId, menuFile);
  menuButtonFile = XtCreateManagedWidget( "menuButtonFile",
      xmCascadeButtonWidgetClass, menuBar, arglist, 1);
  menuButtonHelp = XtCreateManagedWidget( "menuButtonHelp",
      xmCascadeButtonWidgetClass, menuBar, NULL, 0);
  XtVaSetValues(menuBar, 
      XmNmenuHelpWidget, menuButtonHelp,
      NULL);


  graphicsManager = XtCreateManagedWidget( "graphicsManager",
      xmFormWidgetClass, mainManager, NULL, 0);
  graphicsScroll = XtCreateManagedWidget( "graphicsScroll",
      xmScrolledWindowWidgetClass, graphicsManager, NULL, 0);
  graphics = XtCreateManagedWidget( "graphics",
      xmDrawingAreaWidgetClass, graphicsScroll, NULL, 0);
  XtAddCallback(graphics, XmNexposeCallback, RepaintCallback, NULL);
  XtAddCallback(graphics, XmNinputCallback, StartRubber, NULL );
  XtAddCallback(graphics, XmNinputCallback, EndRubber, NULL );
  XtAddEventHandler(graphics, Button1MotionMask, False, Rubberband, NULL);
  controlFrame = XtCreateManagedWidget( "controlFrame",
      xmFrameWidgetClass, graphicsManager, NULL, 0);
  controlFrameTitle = XtCreateManagedWidget( "controlFrameTitle",
      xmLabelWidgetClass, controlFrame, NULL, 0);
  XtVaSetValues(controlFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  controlManager = XtCreateManagedWidget( "controlManager",
      xmRowColumnWidgetClass, controlFrame, NULL, 0);  
  controlScroll = XtCreateManagedWidget( "controlScroll",
      xmScaleWidgetClass, controlManager, NULL, 0);
  XtVaSetValues(controlScroll, 
      XmNminimum, 0,
      XmNmaximum, 1,
      XmNvalue, 0, 
      NULL);
  XtAddCallback(controlScroll, XmNvalueChangedCallback, ScrollCallback, NULL);
  XtAddCallback(controlScroll, XmNdragCallback, ScrollCallback, NULL);
  controlRow1 = XtCreateManagedWidget("controlRow1",
      xmRowColumnWidgetClass, controlManager, NULL, 0);
  controlPrev5 = XtCreateManagedWidget( "controlPrev5",
      xmPushButtonWidgetClass, controlRow1, NULL, 0);
  XtAddCallback(controlPrev5, XmNactivateCallback, Prev5Callback, NULL);
  controlPrev = XtCreateManagedWidget( "controlPrev",
      xmPushButtonWidgetClass, controlRow1, NULL, 0);
  XtAddCallback(controlPrev, XmNactivateCallback, PrevCallback, NULL);
  controlNext = XtCreateManagedWidget( "controlNext",
      xmPushButtonWidgetClass, controlRow1, NULL, 0);
  XtAddCallback(controlNext, XmNactivateCallback, NextCallback, NULL);
  controlNext5 = XtCreateManagedWidget( "controlNext5",
      xmPushButtonWidgetClass, controlRow1, NULL, 0);
  XtAddCallback(controlNext5, XmNactivateCallback, Next5Callback, NULL);
  controlRow2 = XtCreateManagedWidget("controlRow2",
      xmRowColumnWidgetClass, controlManager, NULL, 0);
  controlSetStart = XtCreateManagedWidget( "controlSetStart",
      xmPushButtonWidgetClass, controlRow2, NULL, 0);
  XtAddCallback(controlSetStart, XmNactivateCallback, SetStartCallback, NULL);
  controlSetEnd = XtCreateManagedWidget( "controlSetEnd",
      xmPushButtonWidgetClass, controlRow2, NULL, 0);
  XtAddCallback(controlSetEnd, XmNactivateCallback, SetEndCallback, NULL);
  controlRow3 = XtCreateManagedWidget("controlRow3",
      xmRowColumnWidgetClass, controlManager, NULL, 0);
  controlGotoStart = XtCreateManagedWidget( "controlGotoStart",
      xmPushButtonWidgetClass, controlRow3, NULL, 0);
  XtAddCallback(controlGotoStart, XmNactivateCallback, GotoStartCallback, NULL);
  controlGotoEnd = XtCreateManagedWidget( "controlGotoEnd",
      xmPushButtonWidgetClass, controlRow3, NULL, 0);
  XtAddCallback(controlGotoEnd, XmNactivateCallback, GotoEndCallback, NULL);
  controlClearMask = XtCreateManagedWidget( "controlClearMask",
      xmPushButtonWidgetClass, controlManager, NULL, 0);
  XtAddCallback(controlClearMask, XmNactivateCallback, ClearMaskCallback, NULL);
  imageFrame = XtCreateManagedWidget( "imageFrame",
      xmFrameWidgetClass, graphicsManager, NULL, 0);
  imageFrameTitle = XtCreateManagedWidget( "imageFrameTitle",
      xmLabelWidgetClass, imageFrame, NULL, 0);
  XtVaSetValues(imageFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  imageManager = XtCreateManagedWidget( "imageManager",
      xmRowColumnWidgetClass, imageFrame, NULL, 0);  
  imageBox =  XtCreateManagedWidget( "imageBox",
      xmLabelWidgetClass, imageManager, NULL, 0);
  imageInterval =  XtCreateManagedWidget( "imageInterval",
      xmLabelWidgetClass, imageManager, NULL, 0);



  commandPanel = XtCreateManagedWidget( "commandPanel",
      xmRowColumnWidgetClass, mainManager, NULL, 0);

  sourceFrame = XtCreateManagedWidget( "sourceFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  sourceFrameTitle = XtCreateManagedWidget( "sourceFrameTitle",
      xmLabelWidgetClass, sourceFrame, NULL, 0);
  XtVaSetValues(sourceFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  sourceManager = XtCreateManagedWidget( "sourceManager",
      xmRowColumnWidgetClass, sourceFrame, NULL, 0);  
  sourceManagerDir = XtCreateManagedWidget( "sourceManagerDir",
      xmRowColumnWidgetClass, sourceManager, NULL, 0);
  sourceDirLabel =  XtCreateManagedWidget( "sourceDirLabel",
      xmLabelWidgetClass, sourceManagerDir, NULL, 0);
  sourceDir =  XtCreateManagedWidget( "sourceDir",
      xmTextFieldWidgetClass, sourceManagerDir, NULL, 0);
  sourceManagerPrefix = XtCreateManagedWidget( "sourceManagerPrefix",
      xmRowColumnWidgetClass, sourceManager, NULL, 0);
  sourcePrefixLabel =  XtCreateManagedWidget( "sourcePrefixLabel",
      xmLabelWidgetClass, sourceManagerPrefix, NULL, 0);
  sourcePrefix =  XtCreateManagedWidget( "sourcePrefix",
      xmTextFieldWidgetClass, sourceManagerPrefix, NULL, 0);
  sourceManagerScroll = XtCreateManagedWidget( "sourceManagerScroll",
      xmRowColumnWidgetClass, sourceManager, NULL, 0);
  sourceDigits = XtCreateManagedWidget( "sourceDigits",
      xmScaleWidgetClass, sourceManagerScroll, NULL, 0);
  sourceReload = XtCreateManagedWidget( "sourceReload",
      xmPushButtonWidgetClass, sourceManagerScroll, NULL, 0);
  XtAddCallback(sourceReload, XmNactivateCallback, ReloadCallback, NULL);
  reportManagerDir = XtCreateManagedWidget( "reportManagerDir",
      xmRowColumnWidgetClass, sourceManager, NULL, 0);
  reportDirLabel =  XtCreateManagedWidget( "reportDirLabel",
      xmLabelWidgetClass, reportManagerDir, NULL, 0);
  reportDir =  XtCreateManagedWidget( "reportDir",
      xmTextFieldWidgetClass, reportManagerDir, NULL, 0);
  reportManagerName = XtCreateManagedWidget( "reportManagerName",
      xmRowColumnWidgetClass, sourceManager, NULL, 0);
  reportNameLabel =  XtCreateManagedWidget( "reportNameLabel",
      xmLabelWidgetClass, reportManagerName, NULL, 0);
  reportName =  XtCreateManagedWidget( "reportName",
      xmTextFieldWidgetClass, reportManagerName, NULL, 0);
  XtAddCallback(reportName, XmNvalueChangedCallback, ReportNameCallback, NULL);


  paramsFrame = XtCreateManagedWidget( "paramsFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  paramsFrameTitle = XtCreateManagedWidget( "paramsFrameTitle",
      xmLabelWidgetClass, paramsFrame, NULL, 0);
  XtVaSetValues(paramsFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  paramsManager = XtCreateManagedWidget( "paramsManager",
      xmRowColumnWidgetClass, paramsFrame, NULL, 0);  
  paramsMasksize = XtCreateManagedWidget( "paramsMasksize",
      xmScaleWidgetClass, paramsManager, NULL, 0);
  paramsLookahead = XtCreateManagedWidget( "paramsLookahead",
      xmScaleWidgetClass, paramsManager, NULL, 0);
  paramsOtsuVariance = XtCreateManagedWidget( "paramsOtsuVariance",
      xmScaleWidgetClass, paramsManager, NULL, 0);

  interFrame = XtCreateManagedWidget( "interFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  interFrameTitle = XtCreateManagedWidget( "interFrameTitle",
      xmLabelWidgetClass, interFrame, NULL, 0);
  XtVaSetValues(interFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  interManager = XtCreateManagedWidget( "interManager",
      xmRowColumnWidgetClass, interFrame, NULL, 0);  
  interMin = XtCreateManagedWidget( "interMin",
      xmScaleWidgetClass, interManager, NULL, 0);
  interMax = XtCreateManagedWidget( "interMax",
      xmScaleWidgetClass, interManager, NULL, 0);
  interGap = XtCreateManagedWidget( "interGap",
      xmScaleWidgetClass, interManager, NULL, 0);


  methodFrame = XtCreateManagedWidget( "methodFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  methodFrameTitle = XtCreateManagedWidget( "methodFrameTitle",
      xmLabelWidgetClass, methodFrame, NULL, 0);
  XtVaSetValues(methodFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  methodManager = XtCreateManagedWidget( "methodManager",
      xmRowColumnWidgetClass, methodFrame, NULL, 0);  
  methodNeighbour = XtCreateManagedWidget( "methodNeighbour",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);
  methodMedian = XtCreateManagedWidget( "methodMedian",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);
  methodIterative = XtCreateManagedWidget( "methodIterative",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);
  methodGreyscale = XtCreateManagedWidget( "methodGreyscale",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);
  methodSimple = XtCreateManagedWidget( "methodSimple",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);
  methodGreyIterative = XtCreateManagedWidget( "methodGreyIterative",
      xmToggleButtonWidgetClass, methodManager, NULL, 0);


  winexpManager = XtCreateManagedWidget( "winexpManager",
      xmRowColumnWidgetClass, commandPanel, NULL, 0);  
  windowFrame = XtCreateManagedWidget( "windowFrame",
      xmFrameWidgetClass, winexpManager, NULL, 0);
  windowFrameTitle = XtCreateManagedWidget( "windowFrameTitle",
      xmLabelWidgetClass, windowFrame, NULL, 0);
  XtVaSetValues(windowFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  windowManager = XtCreateManagedWidget( "windowManager",
      xmRowColumnWidgetClass, windowFrame, NULL, 0);  
  windowSize = XtCreateManagedWidget( "windowSize",
      xmScaleWidgetClass, windowManager, NULL, 0);

  exportFrame = XtCreateManagedWidget( "exportFrame",
      xmFrameWidgetClass, winexpManager, NULL, 0);
  exportFrameTitle = XtCreateManagedWidget( "exportFrameTitle",
      xmLabelWidgetClass, exportFrame, NULL, 0);
  XtVaSetValues(exportFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  exportManager = XtCreateManagedWidget( "exportManager",
      xmRowColumnWidgetClass, exportFrame, NULL, 0);  
  exportLabel =  XtCreateManagedWidget( "exportLabel",
      xmLabelWidgetClass, exportManager, NULL, 0);
  exportFilename =  XtCreateManagedWidget( "exportFilename",
      xmTextFieldWidgetClass, exportManager, NULL, 0);
  exportWordLabel =  XtCreateManagedWidget( "exportWordLabel",
      xmLabelWidgetClass, exportManager, NULL, 0);
  exportWord =  XtCreateManagedWidget( "exportWord",
      xmTextFieldWidgetClass, exportManager, NULL, 0);
  exportDoit = XtCreateManagedWidget( "exportDoit",
      xmPushButtonWidgetClass, exportManager, NULL, 0);
  XtAddCallback(exportDoit, XmNactivateCallback, ExportCallback, NULL);






  // Tell XmMainWindow about children
  XtVaSetValues(mainManager, 
      XmNmenuBar, menuBar, 
      XmNworkWindow, graphicsManager,
      XmNcommandWindow, commandPanel,
      NULL );

  XtVaSetValues(graphicsScroll, 
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, controlFrame,
      NULL);

  XtVaSetValues(controlFrame, 
      XmNbottomAttachment, XmATTACH_WIDGET,
      XmNbottomWidget, imageFrame,
      NULL);




  //  Initialize Image Library and X11
  XSetup();



  // parse command line (dir, prefix, digits is only passed)
  strcpy(theSourceDirectory, "./");
  strcpy(theSourcePrefix, "test");
  theSourceDigits = 4;
  switch (argc) {
  case 4:
    theSourceDigits = atoi(argv[3]);
    if (theSourceDigits <= 0) {
      theSourceDigits = 4;
    };
  case 3:
    strcpy(theSourcePrefix, argv[2]);
  case 2:
    strcpy(theSourceDirectory, argv[1]);
    char last[255];
    strcpy(last, &(theSourceDirectory[strlen(theSourceDirectory)-1]));
    if (strcmp(last, "/") != 0) {
      strcat(theSourceDirectory, "/");
    };
  case 1:
    break;
  default:
    // error message about usage
    cout << "Usage: viewimages [directory [prefix [digits]]] \n\n";
    exit(0);
  };
  cout << "ImageSourcePPM: " << theSourceDirectory << ", " << theSourcePrefix << ", " << theSourceDigits << endl; 
  XtVaSetValues(sourceDir, XmNvalue, theSourceDirectory, NULL);
  XtVaSetValues(sourcePrefix, XmNvalue, theSourcePrefix, NULL);
  XtVaSetValues(sourceDigits, XmNvalue, theSourceDigits, NULL);
 
  

  // create ImageSourcePPM
  ReloadCallback(graphics, NULL, NULL);



  XtRealizeWidget(toplevel);           // Realize all              
  XtVaSetValues(graphics, 
      XmNwidth, theWidth, 
      XmNheight, theHeight, 
      NULL);

  XtAppMainLoop(context);              // Can't get past here !!!  

};





void ExitCallback(Widget, XtPointer client_data, XtPointer call_data) {
  exit(0);
};


void RepaintCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  if (XtIsRealized(w)) {
    XClearWindow(theDisp, XtWindow(graphics));
    XCopyArea(theDisp, thePixmap, XtWindow(graphics), theGC, 0, 0, \
              theXImage->width, theXImage->height, 0, 0);
  };
  XFlush(theDisp);

};



void writeDebugInformation(char *tmpClass, char *tmpMethod, char *tmpType, int tmpPointer) {

  ofstream deb;
  deb.open("memdebug.txt", ios::app);
  deb << tmpClass << "|" << tmpMethod << "|" << tmpType << "|" << tmpPointer << endl;
  deb.close();

};


void NextCallback(Widget w, XtPointer, XtPointer) {
  int pos, max;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  XtVaGetValues(controlScroll, XmNmaximum, &max, NULL);
  if (pos < max) pos++;
  XtVaSetValues(controlScroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void PrevCallback(Widget w, XtPointer, XtPointer) {
  int pos, min;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  XtVaGetValues(controlScroll, XmNminimum, &min, NULL);
  if (pos > min) pos--;
  XtVaSetValues(controlScroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void Next5Callback(Widget w, XtPointer, XtPointer) {
  int pos, max;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  XtVaGetValues(controlScroll, XmNmaximum, &max, NULL);
  if (pos+5 < max) pos+=5;
  XtVaSetValues(controlScroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void Prev5Callback(Widget w, XtPointer, XtPointer) {
  int pos, min;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  XtVaGetValues(controlScroll, XmNminimum, &min, NULL);
  if (pos-1 > min) pos-=5;
  XtVaSetValues(controlScroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void ScrollCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XFlush(theDisp);
  int pos;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  theSource.setPosition(pos);
  delete theImage;
  theImage = theSource.getImageRelative(0);

  //writeDebugInformation("ScrollCallback", "theImage", "+", theImage->id);

  Image2Pixmap();
  RepaintCallback(w, NULL, NULL);
};

void SetStartCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  int pos;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  if (pos <= theEnd) {
    theStart = pos;
  } else {
    theStart = theEnd;
    theEnd = pos;
  };
  char interval[255];
  sprintf(interval, "Interval: %d - %d", theStart, theEnd);
  XmString label = XmStringCreateSimple(interval);
  XtVaSetValues(imageInterval, XmNlabelString, label, NULL);
};

void SetEndCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  int pos;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  if (pos >= theStart) {
    theEnd = pos;
  } else {
    theEnd = theStart;
    theStart = pos;
  };
  char interval[255];
  sprintf(interval, "Interval: %d - %d", theStart, theEnd);
  XmString label = XmStringCreateSimple(interval);
  XtVaSetValues(imageInterval, XmNlabelString, label, NULL);
  delete theMask;
  theMask = new Image8(theWidth, theHeight);
  theMask->setBlack();
  ScrollCallback(w, NULL, NULL);   
};

void GotoStartCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XtVaSetValues(controlScroll, XmNvalue, theStart, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void GotoEndCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XtVaSetValues(controlScroll, XmNvalue, theEnd, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void ClearMaskCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  theBoundingBox = Rectangle(0,0,theWidth-1,theHeight-1);
  delete theMask;
  theMask = new Image8(theWidth, theHeight);
  theMask->setBlack();
  ScrollCallback(w, NULL, NULL);   
};


void FileSetSourceDirOKCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XmString tmpDirectory;
  XtVaGetValues(DirectorySelection, XmNdirectory, &tmpDirectory, NULL);
  XtUnmanageChild(DirectorySelection);
  Boolean status; 
  char *tmpString;
  status = XmStringGetLtoR(tmpDirectory, XmFONTLIST_DEFAULT_TAG, &tmpString);
  strcpy(theSourceDirectory, tmpString);
  XtVaSetValues(sourceDir, XmNvalue, theSourceDirectory, NULL);
  cout << "New Source Directory: " << theSourceDirectory << endl;
};

void FileSetReportDirOKCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XmString tmpDirectory;
  XtVaGetValues(DirectorySelection, XmNdirectory, &tmpDirectory, NULL);
  XtUnmanageChild(DirectorySelection);
  Boolean status; 
  char *tmpString;
  status = XmStringGetLtoR(tmpDirectory, XmFONTLIST_DEFAULT_TAG, &tmpString);
  XtVaSetValues(reportDir, XmNvalue, tmpString, NULL);
  cout << "New Report Directory: " << tmpString << endl;
};

void FileSetDirCancelCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  XtUnmanageChild(DirectorySelection);
  cout << "No directory returned." << endl;
};

void FileSetSourceDirCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  DirectorySelection = XmCreateFileSelectionDialog(mainManager, "DirectorySelection", NULL, 0);
  XtManageChild(DirectorySelection);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_OK_BUTTON),
                 XmNactivateCallback, FileSetSourceDirOKCallback, NULL);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_CANCEL_BUTTON),
                 XmNactivateCallback, FileSetDirCancelCallback, NULL);
};

void FileSetReportDirCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  DirectorySelection = XmCreateFileSelectionDialog(mainManager, "DirectorySelection", NULL, 0);
  XtManageChild(DirectorySelection);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_OK_BUTTON),
                 XmNactivateCallback, FileSetReportDirOKCallback, NULL);
  XtAddCallback( XmFileSelectionBoxGetChild(DirectorySelection, XmDIALOG_CANCEL_BUTTON),
                 XmNactivateCallback, FileSetDirCancelCallback, NULL);
};

void Image2Pixmap() {

  // apply preprocessing (Rotate 180, Cut Mask, Cut Rectangle)
  FilterRotate180 theFilterRotate180;
  FilterCutMask theFilterCutMask(theMask);
  Image* temp = theFilterRotate180.apply(theImage);

  //writeDebugInformation("Image2Pixmap", "temp", "+", temp->id);

  delete theImage;
  theImage = theFilterCutMask.apply(temp);

  //writeDebugInformation("Image2Pixmap", "theImage", "+", theImage->id);

  delete temp;


  // convert Image to XImage
  if (theXImage != NULL) {
    XDestroyImage(theXImage);
  };
  theXImage = Pic24ToXImage(theImage->getImage24(), \
                            theImage->getWidth(), \
                            theImage->getHeight());

  // convert XImage to Pixmap
  if (thePixmap != 0) {
    XFreePixmap(theDisp, thePixmap);
    thePixmap = 0;
  };
  int depth; XtVaGetValues (graphics, XtNdepth, &depth, NULL);
  thePixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), 
                            theXImage->width, theXImage->height, depth);
  XPutImage(theDisp, thePixmap, theGC, theXImage, 0, 0, 0, 0, \
            theXImage->width, theXImage->height);

};


void makeNewMask() {

  unsigned int theMaskSize;
  //XtVaGetValues(masksize, XmNvalue, &theMaskSize, NULL);
  theMaskSize = 3;
  FilterExpandBlack myFilterExpandBlack(theMaskSize, theBoundingBox);
  delete theMask;
  delete theImage;
  int pos;
  XtVaGetValues(controlScroll, XtNvalue, &pos, NULL);
  theImage = theSource.getImageAbsolute(pos);
  FilterRotate180 theFilterRotate180;
  Image* temp = theFilterRotate180.apply(theImage);
  theMask = myFilterExpandBlack.apply(temp);
  delete temp;

  // generate the Images new
  delete theImage;
  theImage = theSource.getImageRelative();


  Image2Pixmap();

  // paint it all on the Screen
  ScrollCallback(graphics, NULL, NULL);   

};


// Sepcial routines for rubberbanding BoundingBox
unsigned int start_x, start_y;
unsigned int last_x, last_y;
Boolean rubberbanding = False;

void XorRectangle(Widget w, Rectangle r) {
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, r.x1, r.y1, r.x2, r.y1 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, r.x2, r.y1, r.x2, r.y2 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, r.x2, r.y2, r.x1, r.y2 );
  XDrawLine(XtDisplay(w), XtWindow(w),
      xorGC, r.x1, r.y2, r.x1, r.y1 );
}

void GenerateWindowAndMask() {       

  // generate Bounding Box
  if ((last_x == start_x) || (last_y == start_y)) {
    cout << "No rectangle selected !!!" << endl;
    return;
  };
  if (start_x < last_x) {
    theBoundingBox.x1 = start_x;
    theBoundingBox.x2 = last_x;
  } else {
    theBoundingBox.x1 = last_x;
    theBoundingBox.x2 = start_x;
  };
  if (start_y < last_y) {
    theBoundingBox.y1 = start_y;
    theBoundingBox.y2 = last_y;
  } else {
    theBoundingBox.y1 = last_y;
    theBoundingBox.y2 = start_y;
  };
  cout << "Resulting box: (" << theBoundingBox.x1 << "/" << theBoundingBox.y1 << ") - (" \
                             << theBoundingBox.x2 << "/" << theBoundingBox.y2 << ")" << endl;
  char box[255];
  sprintf(box, "Box: (%d/%d) - (%d/%d)", theBoundingBox.x1, theBoundingBox.y1, \
                                    theBoundingBox.x2, theBoundingBox.y2);
  XmString label = XmStringCreateSimple(box);
  XtVaSetValues(imageBox, XmNlabelString, label, NULL);


  // set End to this image
  int pos;
  XtVaGetValues(controlScroll, XmNvalue, &pos, NULL);
  if (pos >= theStart) {
    theEnd = pos;
  } else {
    theEnd = theStart;
    theStart = pos;
  };
  char interval[255];
  sprintf(interval, "Interval: %d - %d", theStart, theEnd);
  label = XmStringCreateSimple(interval);
  XtVaSetValues(imageInterval, XmNlabelString, label, NULL);

  
  // generate Mask
  makeNewMask();  // repaints all

};


void StartRubber(Widget w, XtPointer client_data, XtPointer call_data) {
 /* Check call_data for button press event */
 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*)call_data;
 if ((cbs->event->type == ButtonPress) 
         && (cbs->event->xbutton.button == Button1) ) {
   rubberbanding = True;
   start_x = cbs->event->xbutton.x;
   start_y = cbs->event->xbutton.y;
   last_x = start_x;
   last_y = start_y; 
   XGCValues gcvals;
   unsigned long gcvals_mask;
   Pixel fg, bg;
   XtVaGetValues (graphics, \
      XtNforeground, &fg, \
      XtNbackground, &bg, \
      NULL);
   gcvals.foreground = bg ^ fg;
   gcvals.background = 0L;
   gcvals.function = GXxor;
   gcvals_mask = GCForeground | GCBackground | GCFunction;
   xorGC = XCreateGC(XtDisplay(graphics), XtWindow(graphics), gcvals_mask, &gcvals);
   XorRectangle(w, Rectangle(start_x,start_y, last_x,last_y));
 }
};


void Rubberband(Widget w, XtPointer client_data, XEvent *event, Boolean *dis) {
 if ((event->type == MotionNotify) && (rubberbanding)) {
   XorRectangle(w, Rectangle(start_x,start_y, last_x,last_y));
   XFlush(theDisp);
   last_x = event->xmotion.x;           /* Get latest pointer coords */
   last_y = event->xmotion.y;
   XorRectangle(w, Rectangle(start_x,start_y, last_x,last_y));
 } 
}


void EndRubber(Widget w, XtPointer client_data, XtPointer call_data) {
 XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct*)call_data;
 if ((cbs->event->type == ButtonRelease)  /* Finish this object      */ 
     && (cbs->event->xbutton.button == 1) ) { 
   rubberbanding = False;               /* Stop rubberbanding        */
   XorRectangle(w, Rectangle(start_x,start_y, last_x,last_y));
   last_x = cbs->event->xbutton.x;      /* Get final pointer         */
   last_y = cbs->event->xbutton.y;      /*  coordinates              */
   GenerateWindowAndMask();             /* Put in display list       */
 }
}



void ReloadCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  // read parameters
  char* string = XmTextFieldGetString(sourceDir);
  strcpy(theSourceDirectory, string);
  char temp[255];
  strcpy(temp, &(theSourceDirectory[strlen(theSourceDirectory)-1]));
  if (strcmp(temp, "/") != 0) {
    strcat(theSourceDirectory, "/");
    XtVaSetValues(sourceDir, XmNvalue, theSourceDirectory, NULL);
  };

  // recognize the last part of the directory 
  // -> let it be the default name for savings
  strncpy(temp, theSourceDirectory, strlen(theSourceDirectory)-1);
  temp[strlen(theSourceDirectory)-1] = '\0';
  char* name = strrchr(temp, '/');
  name++;
  XtVaSetValues(reportName, XmNvalue, name, NULL);
  strcat(temp, ".cfg");
  XtVaSetValues(exportFilename, XmNvalue, name, NULL);
  

  string = XmTextFieldGetString(sourcePrefix);
  strcpy(theSourcePrefix, string);
  XtVaGetValues(sourceDigits, XmNvalue, &theSourceDigits, NULL);


  // adjust ImageSourcePPM
  theSource.setParameters(theSourceDirectory, theSourcePrefix, theSourceDigits);
  if (theSource.getLastIndex()-theSource.getFirstIndex() < 2) {
    cout << "This ImageSourcePPM has less than 3 images." << endl;
    exit(0);
  };
  XtVaSetValues(controlScroll, 
      XmNminimum, theSource.getFirstIndex(),
      XmNmaximum, theSource.getLastIndex(),
      XmNvalue, theSource.getFirstIndex(), 
      NULL);
  theStart = theSource.getFirstIndex();
  theEnd = theSource.getLastIndex();
  char interval[255];
  sprintf(interval, "Interval: %d - %d", theStart, theEnd);
  XmString label = XmStringCreateSimple(interval);
  XtVaSetValues(imageInterval, XmNlabelString, label, NULL);
  theWidth = theSource.getImageWidth();
  theHeight = theSource.getImageHeight();
  theBoundingBox = Rectangle(0,0,theWidth-1,theHeight-1);
  char box[255];
  sprintf(box, "Box: (%d/%d) - (%d/%d)", theBoundingBox.x1, theBoundingBox.y1, \
                                    theBoundingBox.x2, theBoundingBox.y2);
  label = XmStringCreateSimple(box);
  XtVaSetValues(imageBox, XmNlabelString, label, NULL);
  theSource.setPosition(theSource.getFirstIndex());
  theImage = theSource.getImageRelative(0);
  theMask = new Image8(theWidth, theHeight);
  theMask->setBlack();
  Image2Pixmap();

  XtVaSetValues(graphics, 
      XmNwidth, theWidth, 
      XmNheight, theHeight, 
      NULL);

  ScrollCallback(w, NULL, NULL);

};


int XmScrollbarGetValue(Widget w) {
  int result;
  XtVaGetValues(w, XmNvalue, &result, NULL);
  return result;
};


void ExportCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  // check if there's a filename in the box
  char* theCFGfile = XmTextFieldGetString(exportFilename);
  if (strlen(theCFGfile) <= 0) {
    cout << "invalid CFG file ...\n";
    return;
  };
  ofstream CFG(theCFGfile);
  if (!CFG) {
    cout << "invalid CFG file ...\n";
    return;
  };

  // collect all the parameters
  //
  // theSourceDirectory
  // theSourcePrefix
  // theSourceDigits
  // theStart
  // theEnd
  // theBoundingBox
  char* theReportDirectory = XmTextFieldGetString(reportDir);
  char* theReportName = XmTextFieldGetString(reportName);
  char* theExportWord = XmTextFieldGetString(exportWord);
  // char theReportDetails = '\0';
  unsigned int theMasksize = XmScrollbarGetValue(paramsMasksize);
  unsigned int theLookahead = XmScrollbarGetValue(paramsLookahead);
  unsigned int theOtsuVariance = XmScrollbarGetValue(paramsOtsuVariance);
  unsigned int theInterMin = XmScrollbarGetValue(interMin);
  unsigned int theInterMax = XmScrollbarGetValue(interMax);
  unsigned int theInterGap = XmScrollbarGetValue(interGap);
  unsigned int theWinsize = XmScrollbarGetValue(windowSize);
  float theArea = 0.1;


  // write the CFG file
  CFG << "# Config File to generate an HTML report" << endl;
  CFG << "#" << endl;
  if (strlen(theExportWord) > 0) {
    CFG << "word=" << theExportWord << endl;
  };
  CFG << endl;
  CFG << "# Destination config" << endl;
  CFG << "report_dir=" << theReportDirectory << endl;
  CFG << "report_name=" << theReportName << endl;
  CFG << "report_details=" << endl;
  CFG << endl;
  CFG << "# Source config" << endl;
  CFG << "source_dir=" << theSourceDirectory << endl;
  CFG << "source_name=" << theSourcePrefix << endl;
  CFG << "source_digits=" << theSourceDigits << endl;
  CFG << endl;
  CFG << "# Select window and interval of source" << endl;
  CFG << "window=" << theBoundingBox.x1 << " " << theBoundingBox.y1 << " " \
                    << theBoundingBox.x2 << " " << theBoundingBox.y2 << endl;
  CFG << "start=" << theStart << endl;
  CFG << "end=" << theEnd << endl;
  CFG << endl;
  CFG << "# Defaults for variable parameters " << endl;
  CFG << "# those can be used for variable rows" << endl;
  CFG << "winsize=" << theWinsize << endl;
  CFG << "lookahead=" << theLookahead << endl;
  CFG << "otsuvariance=" << theOtsuVariance << endl;
  CFG << "intermin=" << theInterMin << endl;
  CFG << "intermax=" << theInterMax << endl;
  CFG << "intergap=" << theInterGap << endl;
  CFG << "masksize=" << theMasksize << endl;
  CFG << "area=" << theArea << endl;
  CFG << endl;
  CFG << "# List of detection methods, used for columns" << endl;
  CFG << "methods=";
  int num = 0;
  if (XmToggleButtonGetState(methodNeighbour)) {
    if (num++ != 0) CFG << ",";
    CFG << "neighbour";
  };
  if (XmToggleButtonGetState(methodMedian)) {
    if (num++ != 0) CFG << ",";
    CFG << "median";
  };
  if (XmToggleButtonGetState(methodIterative)) {
    if (num++ != 0) CFG << ",";
    CFG << "iterative";
  };
  if (XmToggleButtonGetState(methodGreyscale)) {
    if (num++ != 0) CFG << ",";
    CFG << "greyscale";
  };
  if (XmToggleButtonGetState(methodGreyIterative)) {
    if (num++ != 0) CFG << ",";
    CFG << "greyiterative";
  };
  if (XmToggleButtonGetState(methodSimple)) {
    if (num++ != 0) CFG << ",";
    CFG << "simple";
  };
  CFG << endl;
  CFG.close();

  cout << "CFG file " << theCFGfile << " sucessfully written ..." << endl;

};



void ReportNameCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  char* string = XmTextFieldGetString(reportName);
  char cfg[255];
  strcpy(cfg, string);
  strcat(cfg, ".cfg");  
  XtVaSetValues(exportFilename, XmNvalue, cfg,NULL);
};
