
// Header Files for Motif
#include <X11/Intrinsic.h>     // The Intrinsics                   
#include <Xm/Xm.h>             // Motif symbols                    
#include <Xm/MainW.h>          // XmMainWindow widget              
#include <Xm/ScrolledW.h>      // XmScrolledWindow widget          
#include <Xm/RowColumn.h>      // XmRowColumn widget               
#include <Xm/Form.h>           // XmForm widget                    
#include <Xm/Text.h>           // XmText widget                    
#include <Xm/CascadeB.h>       // XmCascadeButton widget           
#include <Xm/PushB.h>          // XmPushButton widget              
#include <Xm/DrawingA.h>       // XmDrawingArea widget             
#include <Xm/TextF.h>          // XmTextField widget               
#include <Xm/Scale.h>          // XmScale widget                   
#include <Xm/Frame.h>          // XmFrame widget                   
#include <Xm/Label.h>          // XmLabel widget                   
#include <Xm/FileSB.h>         // XmFileSelectionBox widget        
#include <Xm/ToggleB.h>        // XmToggleButton widget            
#include <Xm/List.h>           // XmList widget            


// own Header Files
#include "image/Image.h"
#include "image/PPMReader.h"
#include "image/PPMWriter.h"
#include "image/FilterScale.h"
#include "image/FilterCutRectangle.h"
#include <iostream.h>
#include "xv/xv.c"
#include <fstream.h>
#include "image/Point.h"
#include <math.h>
#include "image/Interpolator.h"
#include <vector.h>
#include "image/Rectangle.h"
#include "image/Point.h"
#include "image/Drawing.h"


// Globals for Motif
XtAppContext  context;
Widget     toplevel, mainManager, commandPanel;
Widget     CMDdraw, CMDexit, CMDimage, CMDscroll, CMDbuttonrow1, CMDbuttonrow2;
Widget     CMDclear, CMDnext, CMDprev, CMDnext5, CMDprev5, CMDgenerate, CMDethz;
Widget     graphics, graphicsscroll;
Widget     EthzManager, ButtonManager, InterpolationManager;
Widget     EthzManagerLeft, EthzManagerRight, EthzWordList, EthzClearWords;
Widget     EthzFrame, ButtonFrame, InterpolationFrame;
Widget     EthzFrameTitle, ButtonFrameTitle, InterpolationFrameTitle;
Widget     EthzPos, EthzAddWord, EthzWord, EthzManagerWord, EthzManagerPos;
Widget     InterpolateFlag, InterpolateMin, InterpolateMax;
Widget     InterpolateGaps, RunawayFlag, RunawayMulti, GeneratePoints;
Widget     EthzPendown, EthzPenup, EthzPosLabel, EthzWordLabel;
Widget     EthzReplaceWord, EthzDeleteWord;


// Globals for X11 and libimageand Program
Image* theImageSmall;
Image* theImageLarge;
Image* theImageBlank;
Image* theMaskSmall;
Image* theMaskLarge;
unsigned int theScale = 4;
XImage *theXImage;
Pixmap thePixmap = 0;
vector<Point> punkte(0);
Rectangle theRect;
char *thePath;
const theImageBorder = 20;
int theImageHeight;
const Color lineColor(0,0,255);    // blue
const Color pointColor(0,255,0);   // green
const Color Black(0,0,0);
char word[255];


// Callbacks
void DrawCallback(Widget, XtPointer, XtPointer);
void ClearCallback(Widget, XtPointer, XtPointer);
void ImageCallback(Widget, XtPointer, XtPointer);
void ExitCallback(Widget, XtPointer, XtPointer);
void RepaintCallback(Widget, XtPointer, XtPointer);
void EthzAddWordCallback(Widget, XtPointer, XtPointer);
void NextCallback(Widget, XtPointer, XtPointer);
void PrevCallback(Widget, XtPointer, XtPointer);
void Next5Callback(Widget, XtPointer, XtPointer);
void Prev5Callback(Widget, XtPointer, XtPointer);
void GenerateCallback(Widget, XtPointer, XtPointer);
void ScrollCallback(Widget, XtPointer, XtPointer);
void EthzClearWordsCallback(Widget, XtPointer, XtPointer);
void EthzWordListCallback(Widget, XtPointer, XtPointer);
void EthzDeleteWordCallback(Widget, XtPointer, XtPointer);
void EthzReplaceWordCallback(Widget, XtPointer, XtPointer);
void EthzCallback(Widget, XtPointer, XtPointer);



// Display list to redraw Lines, Points, etc.
struct tSegment {
  Point startPoint;  // required to be valid
  Point endPoint;    // if valid line is drawn
  Color lineColor;
  Color pointColor;
};
vector<tSegment> segments(0);
vector<Point> res(0);





main(int argc, char **argv) {

  // Init. Toolkit, App. Context, Application Shell
  toplevel = XtAppInitialize( &context, "ViewPoints",
      NULL, 0, &argc, argv, NULL, NULL, 0); 


  // Create widget tree, bind callbacks
  mainManager = XtCreateManagedWidget( "mainManager",
      xmMainWindowWidgetClass, toplevel, NULL, 0);

  graphicsscroll = XtCreateManagedWidget( "graphicsscroll",
      xmScrolledWindowWidgetClass, mainManager, NULL, 0);
  graphics = XtCreateManagedWidget( "graphics",
      xmDrawingAreaWidgetClass, graphicsscroll, NULL, 0);
  XtAddCallback(graphics, XmNexposeCallback, RepaintCallback, NULL);

  commandPanel = XtCreateManagedWidget( "commandPanel",
      xmRowColumnWidgetClass, mainManager, NULL, 0);

  InterpolationFrame = XtCreateManagedWidget( "InterpolationFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  InterpolationFrameTitle = XtCreateManagedWidget( "InterpolationFrameTitle",
      xmLabelWidgetClass, InterpolationFrame, NULL, 0);
  XtVaSetValues(InterpolationFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  InterpolationManager = XtCreateManagedWidget( "InterpolationManager",
      xmRowColumnWidgetClass, InterpolationFrame, NULL, 0);  
  InterpolateFlag = XtCreateManagedWidget( "InterpolateFlag",
      xmToggleButtonWidgetClass, InterpolationManager, NULL, 0);
  RunawayFlag = XtCreateManagedWidget( "RunawayFlag",
      xmToggleButtonWidgetClass, InterpolationManager, NULL, 0);
  InterpolateMin = XtCreateManagedWidget( "InterpolateMin",
      xmScaleWidgetClass, InterpolationManager, NULL, 0);
  InterpolateMax = XtCreateManagedWidget( "InterpolateMax",
      xmScaleWidgetClass, InterpolationManager, NULL, 0);
  InterpolateGaps = XtCreateManagedWidget( "InterpolateGaps",
      xmScaleWidgetClass, InterpolationManager, NULL, 0);
  RunawayMulti = XtCreateManagedWidget( "RunawayMulti",
      xmScaleWidgetClass, InterpolationManager, NULL, 0);


  EthzFrame = XtCreateManagedWidget( "EthzFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  EthzFrameTitle = XtCreateManagedWidget( "EthzFrameTitle",
      xmLabelWidgetClass, EthzFrame, NULL, 0);
  XtVaSetValues(EthzFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  EthzManager = XtCreateManagedWidget( "EthzManager",
      xmRowColumnWidgetClass, EthzFrame, NULL, 0);  
  EthzManagerLeft = XtCreateManagedWidget( "EthzManagerLeft",
      xmRowColumnWidgetClass, EthzManager, NULL, 0);  
  EthzPenup = XtCreateManagedWidget( "EthzPenup",
      xmScaleWidgetClass, EthzManagerLeft, NULL, 0);
  EthzPendown = XtCreateManagedWidget( "EthzPendown",
      xmScaleWidgetClass, EthzManagerLeft, NULL, 0);
  EthzManagerPos = XtCreateManagedWidget( "EthzManagerPos",
      xmRowColumnWidgetClass, EthzManagerLeft, NULL, 0);  
  EthzPosLabel =  XtCreateManagedWidget( "EthzPosLabel",
      xmLabelWidgetClass, EthzManagerPos, NULL, 0);
  EthzPos =  XtCreateManagedWidget( "EthzPos",
      xmTextFieldWidgetClass, EthzManagerPos, NULL, 0);
  EthzManagerWord = XtCreateManagedWidget( "EthzManagerWord",
      xmRowColumnWidgetClass, EthzManagerLeft, NULL, 0);  
  EthzWordLabel =  XtCreateManagedWidget( "EthzWordLabel",
      xmLabelWidgetClass, EthzManagerWord, NULL, 0);
  EthzWord =  XtCreateManagedWidget( "EthzWord",
      xmTextFieldWidgetClass, EthzManagerWord, NULL, 0);
  EthzAddWord = XtCreateManagedWidget( "EthzAddWord",
      xmPushButtonWidgetClass, EthzManagerLeft, NULL, 0);
  XtAddCallback(EthzAddWord, XmNactivateCallback, EthzAddWordCallback, NULL);
  EthzReplaceWord = XtCreateManagedWidget( "EthzReplaceWord",
      xmPushButtonWidgetClass, EthzManagerLeft, NULL, 0);
  XtAddCallback(EthzReplaceWord, XmNactivateCallback, EthzReplaceWordCallback, NULL);
  EthzManagerRight = XtCreateManagedWidget( "EthzManagerRight",
      xmRowColumnWidgetClass, EthzManager, NULL, 0);  
  EthzWordList = XtCreateManagedWidget( "EthzWordList",
      xmListWidgetClass, EthzManagerRight, NULL, 0);
  XtAddCallback(EthzWordList, XmNdefaultActionCallback, EthzWordListCallback, NULL);
  EthzClearWords = XtCreateManagedWidget( "EthzClearWords",
      xmPushButtonWidgetClass, EthzManagerRight, NULL, 0);
  XtAddCallback(EthzClearWords, XmNactivateCallback, EthzClearWordsCallback, NULL);
  EthzDeleteWord = XtCreateManagedWidget( "EthzDeleteWord",
      xmPushButtonWidgetClass, EthzManagerRight, NULL, 0);
  XtAddCallback(EthzDeleteWord, XmNactivateCallback, EthzDeleteWordCallback, NULL);


  ButtonFrame = XtCreateManagedWidget( "ButtonFrame",
      xmFrameWidgetClass, commandPanel, NULL, 0);
  ButtonFrameTitle = XtCreateManagedWidget( "ButtonFrameTitle",
      xmLabelWidgetClass, ButtonFrame, NULL, 0);
  XtVaSetValues(ButtonFrameTitle, 
      XmNchildType, XmFRAME_TITLE_CHILD, 
      NULL);
  ButtonManager = XtCreateManagedWidget( "ButtonManager",
      xmRowColumnWidgetClass, ButtonFrame, NULL, 0);  
  CMDbuttonrow2 = XtCreateManagedWidget("CMDbuttonrow2",
      xmRowColumnWidgetClass, ButtonManager, NULL, 0);
  CMDdraw = XtCreateManagedWidget( "CMDdraw",
      xmPushButtonWidgetClass, CMDbuttonrow2, NULL, 0);
  XtAddCallback(CMDdraw, XmNactivateCallback, DrawCallback, NULL);
  CMDclear = XtCreateManagedWidget( "CMDclear",
      xmPushButtonWidgetClass, CMDbuttonrow2, NULL, 0);
  XtAddCallback(CMDclear, XmNactivateCallback, ClearCallback, NULL);
  CMDbuttonrow1 = XtCreateManagedWidget("CMDbuttonrow1",
      xmRowColumnWidgetClass, ButtonManager, NULL, 0);
  CMDprev5 = XtCreateManagedWidget( "CMDprev5",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDprev5, XmNactivateCallback, Prev5Callback, NULL);
  CMDprev = XtCreateManagedWidget( "CMDprev",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDprev, XmNactivateCallback, PrevCallback, NULL);
  CMDnext = XtCreateManagedWidget( "CMDnext",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDnext, XmNactivateCallback, NextCallback, NULL);
  CMDnext5 = XtCreateManagedWidget( "CMDnext5",
      xmPushButtonWidgetClass, CMDbuttonrow1, NULL, 0);
  XtAddCallback(CMDnext5, XmNactivateCallback, Next5Callback, NULL);
  CMDscroll = XtCreateManagedWidget( "CMDscroll",
      xmScaleWidgetClass, ButtonManager, NULL, 0);
  XtVaSetValues(CMDscroll, 
      XmNminimum, 0,
      XmNmaximum, 1,
      XmNvalue, 0, 
      NULL);
  XtAddCallback(CMDscroll, XmNvalueChangedCallback, ScrollCallback, NULL);
  XtAddCallback(CMDscroll, XmNdragCallback, ScrollCallback, NULL);
  CMDgenerate = XtCreateManagedWidget( "CMDgenerate",
      xmPushButtonWidgetClass, ButtonManager, NULL, 0);
  XtAddCallback(CMDgenerate, XmNactivateCallback, GenerateCallback, NULL);
  CMDimage = XtCreateManagedWidget( "CMDimage",
      xmPushButtonWidgetClass, ButtonManager, NULL, 0);
  XtAddCallback(CMDimage, XmNactivateCallback, ImageCallback, NULL);
  CMDethz = XtCreateManagedWidget( "CMDethz",
      xmPushButtonWidgetClass, ButtonManager, NULL, 0);
  XtAddCallback(CMDethz, XmNactivateCallback, EthzCallback, NULL);
  CMDexit = XtCreateManagedWidget( "CMDexit",
      xmPushButtonWidgetClass, ButtonManager, NULL, 0);
  XtAddCallback(CMDexit, XmNactivateCallback, ExitCallback, NULL);




  // Tell XmMainWindow about children
  XtVaSetValues(mainManager, 
      XmNworkWindow, graphicsscroll,
      XmNcommandWindow, commandPanel,
      NULL );


  XtVaSetValues(EthzPos, 
      XmNvalue, 0,
      XmNleftAttachment, XmATTACH_WIDGET,
      XmNleftWidget, EthzPosLabel,
      NULL);
  XtVaSetValues(EthzWord,
      XmNleftAttachment, XmATTACH_WIDGET,
      XmNleftWidget, EthzWordLabel,
      NULL);



  //  Initialize Image Library and X11
  XSetup();


  
  // Punkte einlesen
  char filename[255];
  thePath = new char[255];
  if (argc > 1) {
    strcpy(thePath, argv[1]);
    strcpy(filename, argv[1]);
    strcat(filename, "points.txt");
  } else {
    strcpy(filename, "points.txt"); 
  };
  ifstream datei(filename);
  if (!datei) {
    cout << "File " << filename << " not found ... exiting!" << endl;
    exit(0);
  };
  char dummy[255];
  unsigned char test;
  int i, x, y;
  while (!datei.eof()) {

    test = datei.peek();

    switch (test) {
    case '#':
      while (test != 10) {
        datei.get(test);
      };
      break;
    
    case 'S':
    case 'H':
      while (test != 10) {
        datei.get(test);
      };
      break;

    case 'W':
      datei.get(test);
      test = datei.peek();
      if (test == 'o') {
        while (test != ' ') {
          datei.get(test);
          test = datei.peek();
        };  
        // read the true word
        datei >> word;
        while ((datei.peek() == 10) || (datei.peek() == 13))
          datei.get(test);

        cout << "IMPORTED WORD: [" << word << "]" << endl;

      } else {
        // ignore
        strcpy(word, "");
        while (test != 10)
          datei.get(test);
      };
      break;

    case 'R':
      while (test != ':') {
        datei.get(test);
      };
      datei.get(test);
      datei >> theRect.x1 >> theRect.y1 >> theRect.x2 >> theRect.y2;
      datei.get(test);
      break;

    default:
      datei >> i >> dummy;
      test = datei.peek();
      while (test == ' ') {
        datei.get(test);
        test = datei.peek();
      }; 
      if ((test != 10) && (test != 13)) {
        datei >> x >> dummy >> y;
      } else {
        x = y = -1;
      };
      while ((datei.peek() == 10) || (datei.peek() == 13)) {
        datei.get(test);
      };

      punkte.insert(punkte.end(), Point(x,y));
      cout << punkte.size() << ". point: " << punkte[punkte.size()-1] << endl;
    };      

  };  


  // Bilder einlesen
  PPMReader myReader;
  if (argc > 1) {
    strcpy(filename, argv[1]);
    strcat(filename, "lastImage.ppm");
  } else {
    strcpy(filename, "lastImage.ppm"); 
  };
  theImageSmall = myReader.apply(filename);
  FilterScale myFilterScale(theScale);
  FilterCutRectangle myFilterCutRectangle(theRect);
  if ((theImageSmall->getWidth() > (theRect.x2-theRect.x1+1)) && \
      (theImageSmall->getHeight() > (theRect.y2-theRect.y1+1))) {
    Image* temp1 = myFilterCutRectangle.apply(theImageSmall);
    Image* temp2 = myFilterScale.apply(temp1);
    theImageLarge = new Image24(temp2->getWidth(), temp2->getHeight());
    theImageLarge->adoptImage24(temp2->getImage24());
    delete temp1;
    delete temp2;
    cout << "LastImage cut" << endl;
  } else {
    Image* temp = myFilterScale.apply(theImageSmall);
    theImageLarge = new Image24(temp->getWidth(), temp->getHeight());
    theImageLarge->adoptImage24(temp->getImage24());
    delete temp;
  }; 
  theXImage = Pic24ToXImage(theImageLarge->getImage24(), \
                            theImageLarge->getWidth(), \
                            theImageLarge->getHeight());
  theImageHeight = theImageLarge->getHeight();


  // Maske einlesen
  if (argc > 1) {
    strcpy(filename, argv[1]);
    strcat(filename, "mask.ppm");
  } else {
    strcpy(filename, "mask.ppm"); 
  };
  theMaskSmall = myReader.apply(filename);
  if ((theMaskSmall->getWidth() > (theRect.x2-theRect.x1+1)) && \
      (theMaskSmall->getHeight() > (theRect.y2-theRect.y1+1))) {
    Image* theMaskTemp = myFilterCutRectangle.apply(theMaskSmall);
    theMaskLarge = myFilterScale.apply(theMaskTemp);
    delete theMaskTemp;

    cout << "Mask cut" << endl;

  } else {
    theMaskLarge = myFilterScale.apply(theMaskSmall);
  }; 



  XtRealizeWidget(toplevel);           // Realize all              
  XtVaSetValues(graphics, 
      XmNwidth, theImageLarge->getWidth(), 
      XmNheight, theImageHeight*2 + theImageBorder, 
      NULL);


  if (thePixmap != 0) {
    XFreePixmap(theDisp, thePixmap);
    thePixmap = 0;
  };
  int depth;
  XtVaGetValues (graphics, XtNdepth, &depth, NULL);
  thePixmap = XCreatePixmap(theDisp, XRootWindowOfScreen(XtScreen(graphics)), \
                            theXImage->width, theXImage->height, depth);
  XPutImage(theDisp, thePixmap, theGC, theXImage, 0, 0, 0, 0, \
            theXImage->width, theXImage->height);

  // word darstellen
  if (strlen(word) > 0) {
    XtVaSetValues(EthzWord, XmNvalue, word, NULL);
  } else {
    XtVaSetValues(EthzWord, XmNvalue, "type word here", NULL);
  };


  XtAppMainLoop(context);              // Can't get past here !!!  

};




void ExitCallback(Widget, XtPointer client_data, XtPointer call_data) {
  exit(0);
};


// Set a RGB Color inside GC
Pixel GetRGBColor(Widget w, byte r, byte g, byte b) {

 Display   *display;
 Colormap  cmap;
 XColor    alloc_color;

 display = XtDisplay(w);                    /* Get display pointer   */

 XtVaGetValues(w, XtNcolormap, &cmap, NULL );

 alloc_color.red   = ((r+1) * 256) - 1;        /* Red value, 0 - 65535  */
 alloc_color.green = ((g+1) * 256) - 1;        /* Green value, 0 - 65535*/
 alloc_color.blue  = ((b+1) * 256) - 1;        /* Blue value, 0 - 65535 */
 alloc_color.flags =  DoRed|DoGreen|DoBlue;      /* Set all */

 if (XAllocColor(display, cmap, &alloc_color)){  /* Success */
    return alloc_color.pixel;
 }
 else {                /* Failed  */
    printf("Can't allocate color!\n");     /* Print warning */
    return (Pixel)NULL;                    /* Return NULL   */
 }

};



void DrawLineImage(Image* image, Point start, Point end, Color c) {
  if (start.valid() && end.valid()) {
    Drawing aDraw;
    aDraw.line(image, theScale*start, theScale*end, c, theMaskLarge);
  };
};

void DrawLineScreen(Point start, Point end, Color c) {
  if (start.valid() && end.valid()) {
    Pixel fg;
    fg = GetRGBColor(graphics, c.r, c.g, c.b);
    XSetForeground(XtDisplay(graphics), theGC, fg);

    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*start.x), (int)(theScale*start.y), \
              (int)(theScale*end.x), (int)(theScale*end.y));
    unsigned int yd = theImageHeight + theImageBorder;
    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*start.x), (int)(theScale*start.y)+yd, \
              (int)(theScale*end.x), (int)(theScale*end.y)+yd);
  };
};


void DrawPointScreen(Point p, Color c) {
  if (p.valid()) {
    Pixel fg;
    fg = GetRGBColor(graphics, c.r, c.g, c.b);
    XSetForeground(XtDisplay(graphics), theGC, fg);

    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*p.x)-1, (int)(theScale*p.y), \
              (int)(theScale*p.x)+1, (int)(theScale*p.y));
    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*p.x), (int)(theScale*p.y)-1, \
              (int)(theScale*p.x), (int)(theScale*p.y)+1);

    unsigned int yd = theImageHeight + theImageBorder;
    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*p.x)-1, (int)(theScale*p.y)+yd, \
              (int)(theScale*p.x)+1, (int)(theScale*p.y)+yd);
    XDrawLine(XtDisplay(graphics), XtWindow(graphics), theGC, \
              (int)(theScale*p.x), (int)(theScale*p.y)-1+yd, \
              (int)(theScale*p.x), (int)(theScale*p.y)+1+yd);
  };
};


void DrawPointImage(Image* image, Point p, Color c) {
  if (p.valid()) {
    Drawing aDraw;
    aDraw.line(image, Point((int)(theScale*p.x)-1, (int)(theScale*p.y)), \
                      Point((int)(theScale*p.x)+1, (int)(theScale*p.y)), \
                      c, theMaskLarge); 
    aDraw.line(image, Point((int)(theScale*p.x), (int)(theScale*p.y)-1), \
                      Point((int)(theScale*p.x), (int)(theScale*p.y)+1), \
                      c, theMaskLarge);
  };
};


void RepaintCallback(Widget, XtPointer client_data, XtPointer call_data) {

  XClearWindow(theDisp, XtWindow(graphics));
  XCopyArea(theDisp, thePixmap, XtWindow(graphics), theGC, 0, 0, \
            theXImage->width, theXImage->height, 0, 0);
  XFlush(theDisp);

  int min, pos;
  XtVaGetValues(CMDscroll, XmNminimum, &min, NULL);
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  if ((pos-min) > 0) {
    // cout << "redrawing " << pos-min << " segments" << endl;
    for(int i=min; i <= pos; i++) {
      DrawPointScreen(segments[i].startPoint, segments[i].pointColor);
      DrawLineScreen(segments[i].startPoint, segments[i].endPoint, segments[i].lineColor);
      // cout << i << ": " << segments[i].startPoint << " - " << segments[i].endPoint << endl;
    };
  };

  XFlush(theDisp);
};


double dist(Point p1, Point p2) {
  return sqrt( (p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y) );
};






// Draw Lines into PPM image and write it to disk
void ImageCallback(Widget w, XtPointer client_data, XtPointer call_data) {

  cout << "Write PPM files to disk ... ";

  // get and set parameters
  Boolean interpolate = XmToggleButtonGetState(InterpolateFlag);
  Boolean runaway = XmToggleButtonGetState(RunawayFlag);
  Point lastPoint;
  int minInterpolate, maxInterpolate, maxInterpolateGaps, runawayMultiInt;
  XtVaGetValues(InterpolateMin, \
      XmNvalue, &minInterpolate,
      NULL); 
  XtVaGetValues(InterpolateMax, \
      XmNvalue, &maxInterpolate,
      NULL); 
  XtVaGetValues(InterpolateGaps, \
      XmNvalue, &maxInterpolateGaps,
      NULL); 
  XtVaGetValues(RunawayMulti, \
      XmNvalue, &runawayMultiInt,
      NULL); 
  double runawayMultiplicator = (double) runawayMultiInt / 10.0;


  // generate point sequence
  if (interpolate != false) {
    Interpolator myInterpolator(minInterpolate, maxInterpolate, maxInterpolateGaps);
    //myInterpolator.setVerboseMode(true);
    res = myInterpolator.apply(punkte);
  } else {
    for (int i = 0; i < punkte.size(); i++) {
      res.insert(res.end(), punkte[i]);      
    };
  };

  // draw lines to image
  theImageBlank = new Image8(theImageLarge->getWidth(), theImageLarge->getHeight());
  theImageBlank->setWhite();
  for (int j = 1; j < res.size(); j++) {
    if (runaway == True) {
      // compare with previous and next position
      if (res[j-1].valid() && res[j].valid() && res[j+1].valid()) {
        double directDist = dist(res[j-1], res[j+1]);
        double normalDist = dist(res[j-1], res[j]) + dist(res[j], res[j+1]);
        if (normalDist <= runawayMultiplicator * directDist) {
          // distance is OK
	} else {
          // actual Point is a runaway
          // -> interpolate between prev and next point
          int x = (unsigned int) ((res[j-1].x+res[j+1].x)/2.0 + 0.5);
          int y = (unsigned int) ((res[j-1].y+res[j+1].y)/2.0 + 0.5);
          res[j] = Point(x,y);
	};
      };
    };

    DrawPointImage(theImageLarge, Point(res[j-1].x-theRect.x1, res[j-1].y-theRect.y1), pointColor);
    // DrawPointImage(theImageBlank, Point(res[j-1].x-theRect.x1, res[j-1].y-theRect.y1), pointColor);

    if (res[j].valid() && res[j-1].valid()) {
      if (theRect.valid()) {
        DrawLineImage(theImageLarge, Point(res[j-1].x-theRect.x1, res[j-1].y-theRect.y1), \
                      Point(res[j].x-theRect.x1, res[j].y-theRect.y1), lineColor);    
        DrawLineImage(theImageBlank, Point(res[j-1].x-theRect.x1, res[j-1].y-theRect.y1), \
                      Point(res[j].x-theRect.x1, res[j].y-theRect.y1), Black);    
      } else {
        DrawLineImage(theImageLarge, res[j-1], res[j], lineColor);
        DrawLineImage(theImageBlank, res[j-1], res[j], Black);
      };
    };
  }; 


  // write images to disk
  PPMWriter myWriter;
  char filename[255];
  strcpy(filename, thePath);
  strcat(filename, "lines_on_blank.ppm");
  myWriter.setFilename(filename);
  myWriter.apply(theImageBlank);
  cout << filename << " ... ";

  strcpy(filename, thePath);
  strcat(filename, "lines_on_image.ppm");
  myWriter.setFilename(filename);
  myWriter.apply(theImageLarge);
  cout << filename << " ... ";
  cout << "done." << endl;

};


void EthzAddWordCallback(Widget w, XtPointer, XtPointer) {
  int pos;
  char *cpos = XmTextFieldGetString(EthzPos); 
  pos = atoi(cpos);
  char *word = XmTextFieldGetString(EthzWord);
  char charitem[255];
  sprintf(charitem, "%04d - %s", pos, word);
  XmString stringitem = XmStringCreateSimple(charitem);
  XmListAddItem(EthzWordList, stringitem, 0);
};

void EthzReplaceWordCallback(Widget w, XtPointer, XtPointer) {
  // check selection
  int *positions;
  int num;
  XmListGetSelectedPos(EthzWordList, &positions, &num);
  if (num == 1) {
    // get new item
    int pos;
    char *cpos = XmTextFieldGetString(EthzPos); 
    pos = atoi(cpos);
    char *word = XmTextFieldGetString(EthzWord);
    char charitem[255];
    sprintf(charitem, "%04d - %s", pos, word);
    XmString stringitem = XmStringCreateSimple(charitem);
    // replace it
    XmString new_items[1];
    new_items[0] = stringitem;
    XmListReplacePositions(EthzWordList, positions, new_items, 1);
  };
};

void EthzClearWordsCallback(Widget w, XtPointer, XtPointer) {
  XmListDeleteAllItems(EthzWordList);
};

void EthzDeleteWordCallback(Widget w, XtPointer, XtPointer) {
  int *positions;
  int num;
  XmListGetSelectedPos(EthzWordList, &positions, &num);
  if (num == 1) {
    XmListDeletePos(EthzWordList, positions[0]);
  };
};

void EthzWordListCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  
  XmListCallbackStruct *cbs = (XmListCallbackStruct*) call_data;
  if (cbs->selected_item_count >= 1) {
    XmString item = cbs->selected_items[0];
    char *string;
    Boolean status = XmStringGetLtoR(item, XmFONTLIST_DEFAULT_TAG, &string);
    char cpos[5];
    strncpy(cpos, string, 4);
    int pos = atoi(cpos);
    int max, min;
    XtVaGetValues(CMDscroll, XmNmaximum, &max, NULL);
    XtVaGetValues(CMDscroll, XmNminimum, &min, NULL);
    if (pos > max) pos = max;
    if (pos < min) pos = min;
    XtVaSetValues(CMDscroll, XmNvalue, pos, NULL);
    ScrollCallback(w, NULL, NULL);   
  };
};

void ScrollCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  int pos;
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  char cpos[10];
  sprintf(cpos, "%d", pos);  
  XtVaSetValues(EthzPos, XmNvalue, cpos, NULL);
  RepaintCallback(w, NULL, NULL);
};

void DrawCallback(Widget w, XtPointer client_data, XtPointer call_data) {
  int max;
  XtVaGetValues(CMDscroll, XmNmaximum, &max, NULL);
  XtVaSetValues(CMDscroll, XmNvalue, max, NULL);
  ScrollCallback(w, NULL, NULL);
};

void NextCallback(Widget w, XtPointer, XtPointer) {
  int pos, max;
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  XtVaGetValues(CMDscroll, XmNmaximum, &max, NULL);
  if (pos < max) pos++;
  XtVaSetValues(CMDscroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void PrevCallback(Widget w, XtPointer, XtPointer) {
  int pos, min;
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  XtVaGetValues(CMDscroll, XmNminimum, &min, NULL);
  if (pos > min) pos--;
  XtVaSetValues(CMDscroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void Next5Callback(Widget w, XtPointer, XtPointer) {
  int pos, max;
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  XtVaGetValues(CMDscroll, XmNmaximum, &max, NULL);
  if (pos+5 < max) pos+=5;
  XtVaSetValues(CMDscroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void Prev5Callback(Widget w, XtPointer, XtPointer) {
  int pos, min;
  XtVaGetValues(CMDscroll, XmNvalue, &pos, NULL);
  XtVaGetValues(CMDscroll, XmNminimum, &min, NULL);
  if (pos-1 > min) pos-=5;
  XtVaSetValues(CMDscroll, XmNvalue, pos, NULL);
  ScrollCallback(w, NULL, NULL);   
};

void ClearCallback(Widget w, XtPointer, XtPointer) {
  XtVaSetValues(CMDscroll, XmNvalue, 0, NULL);
  ScrollCallback(w, NULL, NULL);
};

 

void GenerateCallback(Widget w, XtPointer, XtPointer) {

  // get parameters for interpolation
  Boolean interpolate = XmToggleButtonGetState(InterpolateFlag);
  Boolean runaway = XmToggleButtonGetState(RunawayFlag);
  Point lastPoint;
  int minInterpolate, maxInterpolate, maxInterpolateGaps, runawayMultiInt;
  XtVaGetValues(InterpolateMin, \
      XmNvalue, &minInterpolate,
      NULL); 
  XtVaGetValues(InterpolateMax, \
      XmNvalue, &maxInterpolate,
      NULL); 
  XtVaGetValues(InterpolateGaps, \
      XmNvalue, &maxInterpolateGaps,
      NULL); 
  XtVaGetValues(RunawayMulti, \
      XmNvalue, &runawayMultiInt,
      NULL); 
  double runawayMultiplicator = (double) runawayMultiInt / 10.0;

  // interpolate points
  res.erase(res.begin(), res.end());;
  if (interpolate != false) {
    Interpolator myInterpolator(minInterpolate, maxInterpolate, maxInterpolateGaps);
    //myInterpolator.setVerboseMode(true);
    res = myInterpolator.apply(punkte);
  } else {
    for (int i = 0; i < punkte.size(); i++) {
      res.insert(res.end(), punkte[i]);      
    };
  };

  // generate segments for draw routines
  segments.erase(segments.begin(), segments.end());
  for (int j = 1; j < res.size(); j++) {
    cout << j-1 << ". Point: " << res[j-1] << endl;

    tSegment aSegment;
    aSegment.lineColor = lineColor;
    aSegment.pointColor = pointColor;

    if (runaway == True) {
      // compare with previous and next position
      if (res[j-1].valid() && res[j].valid() && res[j+1].valid()) {
        double directDist = dist(res[j-1], res[j+1]);
        double normalDist = dist(res[j-1], res[j]) + dist(res[j], res[j+1]);
        if (normalDist <= runawayMultiplicator * directDist) {
          // distance is OK
	} else {
          // actual Point is a runaway
          // -> interpolate between prev and next point
          int x = (unsigned int) ((res[j-1].x+res[j+1].x)/2.0 + 0.5);
          int y = (unsigned int) ((res[j-1].y+res[j+1].y)/2.0 + 0.5);
          cout << "(" << res[j-1].x << "/" << res[j-1].y << ")" << endl;
          cout << "Pixel (" << res[j].x << "/" << res[j].y << ") is a runaway";
          res[j] = Point(x,y);
	  cout << " and interpolated to (" << res[j].x << "/" << res[j].y << ")" << endl;
          cout << "(" << res[j+1].x << "/" << res[j+1].y << ")" << endl;
	};
      };
    };
    aSegment.startPoint = Point(res[j-1].x-theRect.x1, res[j-1].y-theRect.y1);
    if (res[j].valid() && res[j-1].valid()) {
       aSegment.endPoint = Point(res[j].x-theRect.x1, res[j].y-theRect.y1);
    };
    segments.push_back(aSegment); 
  }; 

  cout << endl;

  XtVaSetValues(CMDscroll, 
      XmNminimum, 0,
      XmNmaximum, segments.size()-1,
      XmNvalue, segments.size()-1, 
      NULL);

  DrawCallback(w, NULL, NULL);  

};
 


void EthzCallback(Widget w, XtPointer, XtPointer) {

  cout << "Write points in ETHZ format to disk ... ";

  // convert the Widget EthzWordList to a vector
  struct tWord {
    unsigned int pos;  // position of word beginning
    char word[255];    // ascii representation of the word
  };
  vector<tWord> words(0);
  XmStringTable strings;
  XtVaGetValues(EthzWordList, XmNitems, &strings, NULL); 
  int num;
  XtVaGetValues(EthzWordList, XmNitemCount, &num, NULL); 
  for (int i=0; i < num; i++) {
    char *line;
    Boolean status = XmStringGetLtoR(strings[i], XmFONTLIST_DEFAULT_TAG, &line);
    tWord aWord;
    char cpos[5];
    strncpy(cpos, line, 4);
    aWord.pos = atoi(cpos);
    strcpy(aWord.word, &(line[7]));
    words.push_back(aWord);
  };


  // read paramters
  int thePendownStep, thePenupStep;
  XtVaGetValues(EthzPenup, XmNvalue, &thePenupStep, NULL); 
  XtVaGetValues(EthzPendown, XmNvalue, &thePendownStep, NULL); 


  // open file for ETHZ output
  char filename[255];
  strcpy(filename, thePath);
  strcat(filename, "punkte.ethz");
  ofstream ETHZ(filename);
  unsigned int timeETHZ = 0;
  unsigned int numPenups = 0;


  // convert to points to ETHZ format
  // ETHZ << words.size() << endl;
  int word = 0;
  for (int i = 0; i < res.size(); i++) {
    // check if this pos starts a new word
    for (int j = 0; j < words.size(); j++) {
      if (words[j].pos == i) {
        word++;
        if (word > 1) { ETHZ << "]" << endl; };
        ETHZ << "\"" << words[j].word << "\"" << endl;
        ETHZ << "[" << endl;
        break;
      };
    };

    // write point if valid
    if (res[i].valid()) {
      ETHZ << "[ " << res[i].x << "\t" << res[i].y << "\t" << timeETHZ << " ]" << endl;
      timeETHZ += thePendownStep;
    } else {
      // no entry
      timeETHZ += thePenupStep;
    };

  };
  ETHZ << "]" << endl;

  cout << filename << " ... ";
  cout << "done." << endl;

};







