
#include <vista/Vlib.h>
#include <vista/mu.h>
#include <vista/option.h>
#include <vista/VImage.h>

#include <stdio.h>

#include "../vlib/vlib.h"


VImage srcImage ;

typedef struct {
	 long *histogram;
	 int   N;
	 float min, max, mean, var;
} Histrogram;

Histogram HistoImage(src)
VImage src;
{
	 int i;
	 VDouble min, max, mean, variance;
	 VLong *hist;
	 int size;
	 Histrogram H;

	 if (!VImageStats (src, VAllBands, &min, &max, &mean, &variance))
		  exit (1);

	 size = max - min + 1;
	 hist = (VLong *)VCalloc(size, sizeof(VLong));
	 fprintf(stderr, "Histogram size: %d, min : %d, max: %d\n",
			 (int)size, (int)max, (int)min);

	 H->min = min; 
	 H->max = max;
	 H->mean = mean;
	 H->var = variance;
	 H->N   = size;

#define CompHistogram(type)									\
	 {													\
		 type *src_p = & VPixel(src, 0, 0, 0, type);				\
															 \
		 for (i = (int)min; i <= (int)max; i++, src_p++)					\
			 hist[(*src_p)-(int)min] ++;							\
	 }
	 
	 switch(VPixelRepn(src)) {
			   case VBitRepn		:	CompHistogram(VBit);	break;
			   case VUByteRepn	:	CompHistogram(VUByte);	break;
			   case VSByteRepn	:	CompHistogram(VSByte);	break;
			   case VShortRepn	:	CompHistogram(VShort);	break;
			   case VFloatRepn	:	CompHistogram(VShort);	break;
			   case VLongRepn		:	CompHistogram(VLong);	break;
			   default:
		  VError("Undefined type representation");
	 }
	
	 for (i = (int)min; i <= (int)max ; i++)
		  printf("%d %d\n", i, hist[i-(int)min]);
	 
	
	 H->histogram =  hist;

	 return H;
}



VFloat Otsu(image)
VImage image;
{
/* Get Histogram */
	Histogram H = HistoImage(image);

	VLong *histogram = H->histogram;
	int    size = H->N;

	int   i, threshold, bm, fm;
	long  s, r, num_pixels, sum_pixels;
	float m1, m2, q1, bgv, max_bgv;

	for (i = 0,num_pixels = sum_pixels = 0; i < size; i++) 
	{
		 num_pixels += histogram[i];
		 sum_pixels += histogram[i] * i;
	}

	s = histogram[0];
	r = 0;
	if (s) 
		 m1 = (float)r / s;
	else 
		 m1 = 0;

	if (num_pixels == s) 
	{
		 threshold = bm = fm = 0;
		 return  threshold;
	}
	else 
		 m2 = (float)(sum_pixels - r) / (num_pixels - s);

	q1 = (float)s / num_pixels;
	threshold = bm = fm = 0;
	max_bgv = q1 * (1 - q1) * (m1 - m2) * (m1 - m2);
	for (i = 1; i < size; i++) 
	{
		 s += histogram[i];
		 r += histogram[i] * i;
		 q1 = (float)s / num_pixels;
		 if (s) 
		 {
			  m1 = (float)r / s;
			  if (s == num_pixels) m2 = 0;
			  else m2 = (float)(sum_pixels - r) / (num_pixels - s);
		 }
		 else m1 = 0;

		 bgv = q1 * (1 - q1) * (m1 - m2) * (m1 - m2);
		 if (bgv > max_bgv) 
		 {
			  thres = i;
			  bgm = (int)m1;
			  fgm = (int)m2;
			  max_bgv = bgv;
		 }
	}
/*
	*bm = bgm;
	*fm = fgm;
/*

	retrun threshold;
}




int main (int argc, char *argv[])
{
	 FILE *out_file;
	 FILE *in_file;
	 VUByte *img_p;
	 VImage *images;
	 VAttrList list;
	 register int i;
	 VBand band;	

	 VParseFilterCmd(NULL, 0, argc, argv, & in_file, NULL);

/* read src image */
	i = VReadImages(in_file, & list, & images);
	VDestroyAttrList(list);
	srcImage = images[0];
	VFree((VPointer) images);

	 return 0;
} /* end of main () */

