//
// Routinen um Bilder unter X11 darzustellen
//
// taken from XV 3.10a
//

#include <iostream.h>

#include <stdio.h>
#include <stdlib.h>

// X11 Libs
#include <X11/Xos.h>     /* need type declarations immediately */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/Xmd.h>

typedef unsigned char byte;

/* indicies into conv24MB */
#define CONV24_8BIT  0
#define CONV24_24BIT 1
#define CONV24_SEP1  2
#define CONV24_LOCK  3
#define CONV24_SEP2  4
#define CONV24_FAST  5
#define CONV24_SLOW  6
#define CONV24_BEST  7
#define CONV24_MAX   8

/* values 'picType' can take */
#define PIC8  CONV24_8BIT
#define PIC24 CONV24_24BIT

/* values of haveStdCmap */
#define STD_NONE     0        /* no stdcmap currently defined */
#define STD_111      1        /* 1/1/1 stdcmap is available */
#define STD_222      2        /* 2/2/2 stdcmap is available */
#define STD_232      3        /* 2/3/2 stdcmap is available */
#define STD_666      4        /* 6x6x6 stdcmap is available */
#define STD_332      5        /* 3/3/2 stdcmap is available */

/* values of colorMapMode */
#define CM_NORMAL    0        /* normal RO or RW color allocation */
#define CM_PERFECT   1        /* install own cmap if necessary */
#define CM_OWNCMAP   2        /* install own cmap always */
#define CM_STDCMAP   3        /* use stdcmap */

#define DEBUG 1


/* X stuff */
Window        theWindow;
Display       *theDisp;
int           theScreen;
unsigned int  ncells, dispWIDE, dispHIGH, dispDEEP;
unsigned int  vrWIDE, vrHIGH, maxWIDE, maxHIGH;
Colormap      theCmap, LocalCmap;
Window        rootW, mainW, vrootW;
GC            theGC;
u_long        black, white, fg, bg, infofg, infobg;
u_long        hicol, locol;
u_long        blkRGB, whtRGB;
Font          mfont, monofont;
XFontStruct   *mfinfo, *monofinfo;
Visual        *theVisual;
Cursor        arrow, cross, tcross, zoom, inviso;
Pixmap        iconPix, iconmask;
Pixmap        riconPix, riconmask;
int           showzoomcursor;
u_long        xorMasks[8];
// XImage        *theImage;     /* X version of current picture */
// unsigned int  theImageWidth, theImageHeight;

// Creating the X Image & Stuff for the Colormaps
int           directConv[256];    /* used with directColor visuals */
int           ncols;         /* max # of (different) colors to alloc */
byte          stdrdisp[256], stdgdisp[256], stdbdisp[256];
int            nfcols;       /* number of colors to free */
unsigned long  cols[256];    /* maps pic pixel values to X pixel vals */
int            fc2pcol[256]; /* maps freecols into pic pixel values */
int            numcols;      /* # of desired colors in picture */
byte           rdisp[256],gdisp[256],bdisp[256];  /* DISPLAYED colors */
byte           rMap[256],gMap[256],bMap[256];  /* colormap */
unsigned long  stdcols[256];                     /* 3/3/2 -> X colors */
byte           stdr[256], stdg[256], stdb[256];  /* std 3/3/2 cmap */
unsigned long  stdfreecols[256];   /* list of cols to free on exit */
int            stdnfcols;          /* # of cols in stdfreecols[] */
int            colorMapMode; /* CM_NORMAL, CM_PERFECT, CM_OWMCMAP ... */
int            haveStdCmap;  /* STD_NONE, STD_222, STD_666, STD_332 */







/* MACROS */
/* RANGE forces a to be in the range b..c (inclusive) */
#define RANGE(a,b,c) { if (a < b) a = b;  if (a > c) a = c; }

/* XVMISC.C */
void FatalError (char *identifier)
{
  // fprintf(stderr, "%s: %s\n",cmd, identifier);
  // Quit(-1);
  // 
  // replaced by
  //

  fprintf(stderr, "ERROR: %s\n", identifier);
  exit(0);
}




/* XVIMAGE.C */

/********************************************/
byte *Do332ColorDither(byte *pic24, byte *pic8, int w, int h, byte *rmap, byte *gmap, byte *bmap, byte *rdisp, byte *gdisp, byte *bdisp, int maplen)
{
  /* some sort of color dither optimized for the 332 std cmap */

  /* takes a 24 bit picture, of size w*h, dithers with the colors in
     rdisp, gdisp, bdisp (which have already been allocated),
     and generates an 8-bit w*h image, which it returns.  
     ignores input value 'pic8'
     returns NULL on error 

     note: the rdisp,gdisp,bdisp arrays should be the 'displayed' colors,
     not the 'desired' colors

     if pic24 is NULL, uses the passed-in pic8 (an 8-bit image) as
     the source, and the rmap,gmap,bmap arrays as the desired colors */

  byte *np, *ep, *newpic; 
  int r2, g2, b2;
  int *thisline, *nextline, *thisptr, *nextptr, *tmpptr;
  int  i, j, rerr, gerr, berr, pwide3;
  int  imax, jmax;
  long cnt1, cnt2;
  int  fserrmap[512];   /* -255 .. 0 .. +255 */

  /* compute somewhat non-linear floyd-steinberg error mapping table */
  for (i=j=0; i<=0x40; i++,j++) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
  for (     ; i<0x80; i++, j += !(i&1) ? 1 : 0) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
  for (     ; i<=0xff; i++) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }


  cnt1 = cnt2 = 0;
  pwide3 = w*3;  imax = h-1;  jmax = w-1;

  /* attempt to malloc things */
  newpic   = (byte *) malloc((size_t) (w * h));
  thisline = (int *)  malloc(pwide3 * sizeof(int));
  nextline = (int *)  malloc(pwide3 * sizeof(int));
  if (!newpic || !thisline || !nextline) { 
    if (newpic)   free(newpic);
    if (thisline) free(thisline);
    if (nextline) free(nextline);

    return (byte *) NULL;
  }

  np = newpic;
  ep = (pic24) ? pic24 : pic8;


  /* get first line of picture */

  if (pic24) {
    for (j=pwide3, tmpptr=nextline; j; j--, ep++) *tmpptr++ = (int) *ep;
  }
  else {
    for (j=w, tmpptr=nextline; j; j--, ep++) {
      *tmpptr++ = (int) rmap[*ep];
      *tmpptr++ = (int) gmap[*ep];
      *tmpptr++ = (int) bmap[*ep];
    }
  }


  for (i=0; i<h; i++) {
    np = newpic + i*w;
    // ProgressMeter(0, h-1, i, "Dither");
    // if ((i&127) == 0) WaitCursor();

    tmpptr = thisline;  thisline = nextline;  nextline = tmpptr;   /* swap */

    if (i!=imax) {  /* get next line */
      if (!pic24)
	for (j=w, tmpptr=nextline; j; j--, ep++) {
	  *tmpptr++ = (int) rmap[*ep];
	  *tmpptr++ = (int) gmap[*ep];
	  *tmpptr++ = (int) bmap[*ep];
	}
      else
	for (j=pwide3, tmpptr=nextline; j; j--, ep++) *tmpptr++ = (int) *ep;
    }


    /* dither a line, doing odd-lines right-to-left (serpentine) */
    thisptr = (i&1) ? thisline + w*3 - 3 : thisline;
    nextptr = (i&1) ? nextline + w*3 - 3 : nextline;


    if (i&1) np += w-1;


    for (j=0; j<w; j++) {
      int k, d, mind, closest, rb,gb,bb;

      r2 = *thisptr++;  g2 = *thisptr++;  b2 = *thisptr++;
      if (i&1) thisptr -= 6;  /* move left */

      rb = (r2 + 0x10);    /* round top 3 bits */
      RANGE(rb,0,255);
      rb = rb & 0xe0;

      gb = (g2 + 0x10);    /* round 3 bits */
      RANGE(gb,0,255);
      gb = gb & 0xe0;

      bb = (b2 + 0x20);    /* round 2 bits */
      RANGE(bb,0,255);
      bb = bb & 0xc0;
      

      *np = rb | (gb>>3) | (bb>>6);

      /* propogate the error */
      rerr = r2 - rdisp[*np];
      gerr = g2 - gdisp[*np];
      berr = b2 - bdisp[*np];


      RANGE(rerr, -255, 255);
      RANGE(gerr, -255, 255);
      RANGE(berr, -255, 255);
      rerr = fserrmap[256+rerr];
      gerr = fserrmap[256+gerr];
      berr = fserrmap[256+berr];


      if (j!=jmax) {  /* adjust LEFT/RIGHT pixel */
	thisptr[0] += (rerr/2);  rerr -= (rerr/2);
	thisptr[1] += (gerr/2);  gerr -= (gerr/2);
	thisptr[2] += (berr/2);  berr -= (berr/2);
      }

      if (i!=imax) { /* adjust BOTTOM pixel */
	nextptr[0] += rerr;    /* possibly all err if we're at l/r edge */
	nextptr[1] += gerr;
	nextptr[2] += berr;
      }

      if (i&1) { nextptr -= 3;  np--; }
          else { nextptr += 3;  np++; }
    }
  }


  free(thisline);  free(nextline);

  return newpic;
}


/* different type of dithering */
/********************************************/
byte *DoColorDither(byte *pic24, byte *pic8, int w, int h, byte *rmap, \
                    byte *gmap, byte *bmap, byte *rdisp, byte *gdisp, \
                    byte *bdisp, int maplen) 
{
  /* takes a 24 bit picture, of size w*h, dithers with the colors in
     rdisp, gdisp, bdisp (which have already been allocated),
     and generates an 8-bit w*h image, which it returns.  
     ignores input value 'pic8'
     returns NULL on error 

     note: the rdisp,gdisp,bdisp arrays should be the 'displayed' colors,
     not the 'desired' colors

     if pic24 is NULL, uses the passed-in pic8 (an 8-bit image) as
     the source, and the rmap,gmap,bmap arrays as the desired colors */

  byte *np, *ep, *newpic; 
  short *cache;
  int r2, g2, b2;
  int *thisline, *nextline, *thisptr, *nextptr, *tmpptr;
  int  i, j, rerr, gerr, berr, pwide3;
  int  imax, jmax;
  int key;
  long cnt1, cnt2;
  int fserrmap[512];   /* -255 .. 0 .. +255 */

  /* compute somewhat non-linear floyd-steinberg error mapping table */
  for (i=j=0; i<=0x40; i++,j++) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
  for (     ; i<0x80; i++, j += !(i&1) ? 1 : 0) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
  for (     ; i<=0xff; i++) 
    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }


  cnt1 = cnt2 = 0;
  pwide3 = w*3;  imax = h-1;  jmax = w-1;
  ep = (pic24) ? pic24 : pic8;

  /* attempt to malloc things */
  newpic = (byte *)  malloc((size_t) (w * h));
  cache  = (short *) calloc((size_t) (2<<14), sizeof(short));
  thisline = (int *) malloc(pwide3 * sizeof(int));
  nextline = (int *) malloc(pwide3 * sizeof(int));
  if (!cache || !newpic || !thisline || !nextline) { 
    if (newpic)   free(newpic);
    if (cache)    free(cache);
    if (thisline) free(thisline);
    if (nextline) free(nextline);

    return (byte *) NULL;
  }

  np = newpic;

  /* get first line of picture */

  if (pic24) {
    for (j=pwide3, tmpptr=nextline; j; j--, ep++)
      *tmpptr++ = (int) *ep;
  }
  else {
    for (j=w, tmpptr=nextline; j; j--, ep++) {
      *tmpptr++ = (int) rmap[*ep];
      *tmpptr++ = (int) gmap[*ep];
      *tmpptr++ = (int) bmap[*ep];
    }
  }


  for (i=0; i<h; i++) {
    //ProgressMeter(0, h-1, i, "Dither");
    //if ((i&15) == 0) WaitCursor();

    tmpptr = thisline;  thisline = nextline;  nextline = tmpptr;   /* swap */

    if (i!=imax) {  /* get next line */
      if (!pic24)
	for (j=w, tmpptr=nextline; j; j--, ep++) {
	  *tmpptr++ = (int) rmap[*ep];
	  *tmpptr++ = (int) gmap[*ep];
	  *tmpptr++ = (int) bmap[*ep];
	}
      else
	for (j=pwide3, tmpptr=nextline; j; j--, ep++) *tmpptr++ = (int) *ep;
    }

    /* dither a line */
    for (j=0, thisptr=thisline, nextptr=nextline; j<w; j++,np++) {
      int k, d, mind, closest;

      r2 = *thisptr++;  g2 = *thisptr++;  b2 = *thisptr++;

      /* map r2,g2,b2 components (could be outside 0..255 range) 
	 into 0..255 range */
      
      if (r2<0 || g2<0 || b2<0) {   /* are there any negatives in RGB? */
	if (r2<g2) { if (r2<b2) k = 0; else k = 2; }
	else { if (g2<b2) k = 1; else k = 2; }

	switch (k) {
	case 0:  g2 -= r2;  b2 -= r2;  d = (abs(r2) * 3) / 2;    /* RED */
	         r2 = 0;  
	         g2 = (g2>d) ? g2 - d : 0;
	         b2 = (b2>d) ? b2 - d : 0;
	         break;

	case 1:  r2 -= g2;  b2 -= g2;  d = (abs(g2) * 3) / 2;    /* GREEN */
	         r2 = (r2>d) ? r2 - d : 0;
	         g2 = 0;  
	         b2 = (b2>d) ? b2 - d : 0;
	         break;

	case 2:  r2 -= b2;  g2 -= b2;  d = (abs(b2) * 3) / 2;    /* BLUE */
	         r2 = (r2>d) ? r2 - d : 0;
	         g2 = (g2>d) ? g2 - d : 0;
	         b2 = 0;  
	         break;
	}
      }

      if (r2>255 || g2>255 || b2>255) {   /* any overflows in RGB? */
	if (r2>g2) { if (r2>b2) k = 0; else k = 2; }
              else { if (g2>b2) k = 1; else k = 2; }

	switch (k) {
	case 0:   g2 = (g2*255)/r2;  b2 = (b2*255)/r2;  r2=255;  break;
	case 1:   r2 = (r2*255)/g2;  b2 = (b2*255)/g2;  g2=255;  break;
	case 2:   r2 = (r2*255)/b2;  g2 = (g2*255)/b2;  b2=255;  break;
	}
      }

      key = ((r2&0xf8)<<6) | ((g2&0xf8)<<1) | (b2>>4);
      if (key >= (2<<14)) FatalError("'key' overflow in DoColorDither()");

      if (cache[key]) { *np = (byte) (cache[key] - 1);  cnt1++;	}
      else {
	/* not in cache, have to search the colortable */
	cnt2++;

        mind = 10000;
	for (k=closest=0; k<maplen && mind>7; k++) {
	  d = abs(r2 - rdisp[k])
	    + abs(g2 - gdisp[k]) 
	    + abs(b2 - bdisp[k]);
	  if (d<mind) { mind = d;  closest = k; }
	}
	cache[key] = closest + 1;
	*np = closest;
      }


      /* propogate the error */
      rerr = r2 - rdisp[*np];
      gerr = g2 - gdisp[*np];
      berr = b2 - bdisp[*np];


      RANGE(rerr, -255, 255);
      RANGE(gerr, -255, 255);
      RANGE(berr, -255, 255);
      rerr = fserrmap[256+rerr];
      gerr = fserrmap[256+gerr];
      berr = fserrmap[256+berr];



      if (j!=jmax) {  /* adjust RIGHT pixel */
	thisptr[0] += (rerr*7)/16;
	thisptr[1] += (gerr*7)/16;
	thisptr[2] += (berr*7)/16;
      }
      
      if (i!=imax) {	/* do BOTTOM pixel */
	nextptr[0] += (rerr*5)/16;
	nextptr[1] += (gerr*5)/16;
	nextptr[2] += (berr*5)/16;

	if (j>0) {  /* do BOTTOM LEFT pixel */
	  nextptr[-3] += (rerr*3)/16;
	  nextptr[-2] += (gerr*3)/16;
	  nextptr[-1] += (berr*3)/16;
	}

	if (j!=jmax) {  /* do BOTTOM RIGHT pixel */
	  nextptr[3] += rerr/16;
	  nextptr[4] += gerr/16;
	  nextptr[5] += berr/16;
	}
	nextptr += 3;
      }
    }
  }


  free(thisline);  free(nextline);
  free(cache);

  return newpic;
};


/***********************/
static int highbit(unsigned long ul)
{
  /* returns position of highest set bit in 'ul' as an integer (0-31),
   or -1 if none */

  int i;  unsigned long hb;

  hb = 0x80;  hb = hb << 24;   /* hb = 0x80000000UL */
  for (i=31; ((ul & hb) == 0) && i>=0;  i--, ul<<=1);
  return i;
}

/***********************************/
XImage *Pic24ToXImage(byte *pic24, unsigned int wide, unsigned int high)
{
  /*
   * this has to do the none-to-simple bit of converting the data in 'pic24'
   * into something usable by X.
   *
   * There are two major approaches:  if we're displaying on a TrueColor
   * or DirectColor display, we've got all the colors we're going to need,
   * and 'all we have to do' is convert 24-bit RGB pixels into whatever
   * variation of RGB the X device in question wants.  No color allocation
   * is involved.
   *
   * Alternately, if we're on a PseudoColor, GrayScale, StaticColor or 
   * StaticGray display, we're going to continue to operate in an 8-bit 
   * mode.  (In that by this point, a 3/3/2 standard colormap has been
   * created for our use (though all 256 colors may not be unique...), and
   * we're just going to display the 24-bit picture by dithering with those
   * colors
   *
   */

  int     i,j;
  XImage *xim;

  xim     = (XImage *) NULL;

  if (!pic24) return xim;  /* shouldn't happen */


  // 
  // 1bit Visual ignored
  // 
  /* special case: 1-bit display.  Doesn't much matter *what* the visual is */
  if (dispDEEP == 1) {

    FatalError("1-bit visual is not supported!");
    
    // byte  *imagedata;
    //
    // xim = XCreateImage(theDisp, theVisual, dispDEEP, XYPixmap, 0, NULL, wide,  high, 32, 0);
    // if (!xim) FatalError("couldn't create xim!");
    // 
    // imagedata = (byte *) malloc((size_t) (xim->bytes_per_line * high));
    // if (!imagedata) FatalError("couldn't malloc imagedata");
    //
    // xim->data = (char *) imagedata; 
    // floydDitherize1(xim, pic24,PIC24, (int) wide, (int) high, NULL,NULL,NULL);
    //
    // return xim;
    
  };


  if (theVisual->c_class == TrueColor || theVisual->c_class == DirectColor) {

    /************************************************************************/
    /* Non-ColorMapped Visuals:  TrueColor, DirectColor                     */
    /************************************************************************/

    unsigned long r, g, b, rmask, gmask, bmask, xcol;
    int           rshift, gshift, bshift, bperpix, bperline, border, cshift;
    int           maplen;
    byte         *imagedata, *lip, *ip, *pp;


    /* compute various shifting constants that we'll need... */

    rmask = theVisual->red_mask;
    gmask = theVisual->green_mask;
    bmask = theVisual->blue_mask;

    rshift = 7 - highbit(rmask);
    gshift = 7 - highbit(gmask);
    bshift = 7 - highbit(bmask);

    maplen = theVisual->map_entries;
    if (maplen>256) maplen=256;
    cshift = 7 - highbit((u_long) (maplen-1));

    xim = XCreateImage(theDisp, theVisual, dispDEEP, ZPixmap, 0, NULL,
		        wide,  high, 32, 0);
    if (!xim) FatalError("couldn't create X image!");

    bperline = xim->bytes_per_line;
    bperpix  = xim->bits_per_pixel;
    border   = xim->byte_order;

    imagedata = (byte *) malloc((size_t) (high * bperline));
    if (!imagedata) FatalError("couldn't malloc imagedata");

    xim->data = (char *) imagedata;

    if (bperpix != 8 && bperpix != 16 && bperpix != 24 && bperpix != 32) {
      char buf[128];
      sprintf(buf,"Sorry, no code written to handle %d-bit %s",
	      bperpix, "TrueColor/DirectColor displays!");
      FatalError(buf);
    }


    lip = imagedata;  pp = pic24;
    for (i=0; i<high; i++, lip+=bperline) {
      for (j=0, ip=lip; j<wide; j++) {
	r = *pp++;  g = *pp++;  b = *pp++;

	/* shift r,g,b so that high bit of 8-bit color specification is 
	 * aligned with high bit of r,g,b-mask in visual, 
	 * AND each component with its mask,
	 * and OR the three components together
	 */

	if (theVisual->c_class == DirectColor) {
	  r = (u_long) directConv[(r>>cshift) & 0xff] << cshift;
	  g = (u_long) directConv[(g>>cshift) & 0xff] << cshift;
	  b = (u_long) directConv[(b>>cshift) & 0xff] << cshift;
	}


	/* shift the bits around */
	if (rshift<0) r = r << (-rshift);
	         else r = r >> rshift;
	
	if (gshift<0) g = g << (-gshift);
	         else g = g >> gshift;

	if (bshift<0) b = b << (-bshift);
	         else b = b >> bshift;

	r = r & rmask;
	g = g & gmask;
	b = b & bmask;

	xcol = r | g | b;

	if (bperpix == 32) {
	  if (border == MSBFirst) {
	    *ip++ = (xcol>>24) & 0xff;
	    *ip++ = (xcol>>16) & 0xff;
	    *ip++ = (xcol>>8)  & 0xff;
	    *ip++ =  xcol      & 0xff;
	  }
	  else {  /* LSBFirst */
	    *ip++ =  xcol      & 0xff;
	    *ip++ = (xcol>>8)  & 0xff;
	    *ip++ = (xcol>>16) & 0xff;
	    *ip++ = (xcol>>24) & 0xff;
	  }
	}

	else if (bperpix == 24) {
	  if (border == MSBFirst) {
	    *ip++ = (xcol>>16) & 0xff;
	    *ip++ = (xcol>>8)  & 0xff;
	    *ip++ =  xcol      & 0xff;
	  }
	  else {  /* LSBFirst */
	    *ip++ =  xcol      & 0xff;
	    *ip++ = (xcol>>8)  & 0xff;
	    *ip++ = (xcol>>16) & 0xff;
	  }
	}

	else if (bperpix == 16) {
	  if (border == MSBFirst) {
	    *ip++ = (xcol>>8)  & 0xff;
	    *ip++ =  xcol      & 0xff;
	  }
	  else {  /* LSBFirst */
	    *ip++ =  xcol      & 0xff;
	    *ip++ = (xcol>>8)  & 0xff;
	  }
	}

	else if (bperpix == 8) {
	  *ip++ =  xcol      & 0xff;
	}
      }
    }
  }

  else {

    /************************************************************************/
    /* CMapped Visuals:  PseudoColor, GrayScale, StaticGray, StaticColor... */
    /************************************************************************/


    byte *pic8;
    int   bwdith;

    /* in all cases, make an 8-bit version of the image, either using
       'black' and 'white', or the stdcmap */

    bwdith = 0;

    if (ncols == 0 && dispDEEP != 1) {   /* do 'black' and 'white' dither */
      /* note that if dispDEEP > 8, pic8 will just have '0' and '1' instead 
	 of 'black' and 'white' */

      // pic8 = FSDither(pic24, PIC24, (int) wide, (int) high, NULL, NULL, NULL, 
      //		      (int) ((dispDEEP <= 8) ? black : 0), 
      //		      (int) ((dispDEEP <= 8) ? white : 1));
      //      bwdith = 1;

      FatalError("Black and White displaying of Image not supported!");

    }

    else {                               /* do color dither using stdcmap */
      pic8 = DoColorDither(pic24, NULL, (int) wide, (int) high, NULL, NULL, NULL, 
                              stdrdisp, stdgdisp, stdbdisp, 256);
    }

    if (!pic8) FatalError("out of memory in Pic24ToXImage()\n");


    /* DISPLAY-DEPENDENT code follows... */


    switch (dispDEEP) {


    case 8: {
      byte  *imagedata, *ip, *pp;
      int   j, imWIDE, nullCount;
  
      nullCount = (4 - (wide % 4)) & 0x03;  /* # of padding bytes per line */
      imWIDE = wide + nullCount;
 
      /* Now create the image data - pad each scanline as necessary */
      imagedata = (byte *) malloc((size_t) (imWIDE * high));
      if (!imagedata) FatalError("couldn't malloc imagedata");
      
      for (i=0, pp=pic8, ip=imagedata; i<high; i++) {
	// if (((i+1)&0x7f) == 0) WaitCursor();

	if (bwdith)
	  for (j=0; j<wide; j++, ip++, pp++) *ip = *pp;
	else
	  for (j=0; j<wide; j++, ip++, pp++) *ip = stdcols[*pp];

	for (j=0; j<nullCount; j++, ip++)  *ip = 0;
      }

      xim = XCreateImage(theDisp, theVisual, dispDEEP, ZPixmap, 0,
			 (char *) imagedata,  wide,  high, 
			 32, imWIDE);
      if (!xim) FatalError("couldn't create xim!");
    }
      break;


      /*********************************/
      
    case 4: {
      FatalError("4-bit Visual not supported");
      break;
    }


      /*********************************/
      
    case 2: {
      FatalError("2-bit Visual not supported");
      break;
    }
      

      /*********************************/
    
    case 6: { 
      FatalError("6-bit Visual not supported");
      break;
    }
      
      /*********************************/

    case 15:
    case 16: {
      unsigned short  *imagedata, *ip, *lip;
      byte   *pp;
      int     bperline;
      unsigned long xcol;

      imagedata = (unsigned short *) malloc((size_t) (2*wide*high));
      if (!imagedata) FatalError("couldn't malloc imagedata");

      xim = XCreateImage(theDisp,theVisual,dispDEEP,ZPixmap,0,
			 (char *) imagedata, wide, high, 32, 0);
      if (!xim) FatalError("couldn't create xim!");
      bperline = xim->bytes_per_line;

      pp = pic8;

      if (xim->byte_order == MSBFirst) {
	for (i=0, lip=imagedata; i<high; i++, lip+=bperline) {
	  // if (((i+1)&0x7f) == 0) WaitCursor();

	  for (j=0, ip=lip; j<wide; j++,pp++) {
	    *ip++ = ((bwdith) ? *pp : stdcols[*pp]) & 0xffff;
	  }
	}
      }

      else {   /* LSBFirst */
	for (i=0, lip=imagedata; i<high; i++, lip+=bperline) {
	  // if (((i+1)&0x7f) == 0) WaitCursor();

	  for (j=0, ip=lip; j<wide; j++,pp++) {
	    xcol = ((bwdith) ? *pp : stdcols[*pp]) & 0xffff;
	    /* WAS *ip++ = ((xcol>>8) & 0xff) | ((xcol&0xff) << 8);  */
	    *ip++ = (unsigned short) (xcol);
	  }
	}
      }
    }
      break;

      
      /*********************************/

      /* this wouldn't seem likely to happen, but what the heck... */

    case 24:
    case 32: {
      byte  *imagedata, *ip, *pp;
      unsigned long xcol;
      int bperpix;

      imagedata = (byte *) malloc((size_t) (4*wide*high));
      if (!imagedata) FatalError("couldn't malloc imagedata");
      
      xim = XCreateImage(theDisp,theVisual,dispDEEP,ZPixmap,0,
			 (char *) imagedata, wide, high, 32, 0);
      if (!xim) FatalError("couldn't create xim!");

      bperpix = xim->bits_per_pixel;

      pp = pic8;
      
      if (xim->byte_order == MSBFirst) {
	for (i=wide*high, ip=imagedata; i>0; i--,pp++) {
	  // if (((i+1)&0x1ffff) == 0) WaitCursor();
	  xcol = (bwdith) ? *pp : stdcols[*pp];

	  if (bperpix == 32) *ip++ = 0;
	  *ip++ = (xcol>>16) & 0xff;
	  *ip++ = (xcol>>8)  & 0xff;
	  *ip++ =  xcol      & 0xff;
	}
      }

      else {  /* LSBFirst */
	for (i=wide*high, ip=imagedata; i>0; i--,pp++) {
	  xcol = (bwdith) ? *pp : stdcols[*pp];

	  // if (((i+1)&0x1ffff) == 0) WaitCursor();
	  *ip++ =  xcol      & 0xff;
	  *ip++ = (xcol>>8)  & 0xff;
	  *ip++ = (xcol>>16) & 0xff;
	  if (bperpix == 32) *ip++ = 0;
	}
      }
    }     
      break;

    }   /* end of the switch */

    free(pic8);  /* since we ALWAYS make a copy of it into imagedata */
  }

  // probably here could be a memory leak
  // delete[] pic24;

  return xim;
}



/* XV.C */
/*****************************************************/
static void makeDirectCmap()
{
  int    i, cmaplen, numgot;
  byte   origgot[256];
  XColor c;
  u_long rmask, gmask, bmask;
  int    rshift, gshift, bshift;
  

  rmask = theVisual->red_mask;
  gmask = theVisual->green_mask;
  bmask = theVisual->blue_mask;

  rshift = highbit(rmask) - 15;
  gshift = highbit(gmask) - 15;
  bshift = highbit(bmask) - 15;

  if (rshift<0) rmask = rmask << (-rshift);
           else rmask = rmask >> rshift;
  
  if (gshift<0) gmask = gmask << (-gshift);
           else gmask = gmask >> gshift;
  
  if (bshift<0) bmask = bmask << (-bshift);
           else bmask = bmask >> bshift;


  cmaplen = theVisual->map_entries;
  if (cmaplen>256) cmaplen=256;
  

  /* try to alloc a 'cmaplen' long grayscale colormap.  May not get all
     entries for whatever reason.  Build table 'directConv[]' that
     maps range [0..(cmaplen-1)] into set of colors we did get */
  
  for (i=0; i<256; i++) {  origgot[i] = 0;  directConv[i] = 0; }

  for (i=numgot=0; i<cmaplen; i++) {
    c.red = c.green = c.blue = (i * 0xffff) / (cmaplen - 1);
    c.red   = c.red   & rmask;
    c.green = c.green & gmask;
    c.blue  = c.blue  & bmask;
    c.flags = DoRed | DoGreen | DoBlue;

    if (XAllocColor(theDisp, theCmap, &c)) {
      /* fprintf(stderr,"%3d: %4x,%4x,%4x\n", i, c.red,c.green,c.blue); */
      directConv[i] = i;
      origgot[i] = 1;
      numgot++;
    }
  }

  
  if (numgot == 0) FatalError("Got no entries in DirectColor cmap!\n");
  
  /* directConv may or may not have holes in it. */
  for (i=0; i<cmaplen; i++) {
    if (!origgot[i]) {
      int numbak, numfwd;
      numbak = numfwd = 0;
      while ((i - numbak) >= 0       && !origgot[i-numbak]) numbak++;
      while ((i + numfwd) <  cmaplen && !origgot[i+numfwd]) numfwd++;
      
      if (i-numbak<0        || !origgot[i-numbak]) numbak = 999;
      if (i+numfwd>=cmaplen || !origgot[i+numfwd]) numfwd = 999;
      
      if      (numbak<numfwd) directConv[i] = directConv[i-numbak];
      else if (numfwd<999)    directConv[i] = directConv[i+numfwd];
      else FatalError("DirectColor cmap:  can't happen!");
    }
  }
}



/* XVCOLOR */

Status xvAllocColor(Display *dp, Colormap cm, XColor *cdef)
{
  if (theVisual->c_class == TrueColor || theVisual->c_class == DirectColor) {
    unsigned long r, g, b, rmask, gmask, bmask, origr, origg, origb;
    int rshift, gshift, bshift;
    
    /* shift r,g,b so that high bit of 16-bit color specification is 
     * aligned with high bit of r,g,b-mask in visual, 
     * AND each component with its mask,
     * and OR the three components together
     */

    origr = r = cdef->red;  origg = g = cdef->green;  origb = b = cdef->blue;

    if (theVisual->c_class == DirectColor) {
      /* r,g,b are 16-bit values  */
      int maplen = theVisual->map_entries;
      if (maplen>256) maplen = 256;

      rshift = 15 - highbit((unsigned long) (maplen-1));
      r = (u_long) directConv[(r>>rshift) & 0xff];
      g = (u_long) directConv[(g>>rshift) & 0xff];
      b = (u_long) directConv[(b>>rshift) & 0xff];

      /* shift the bits back up */
      r = r << rshift;
      g = g << rshift;
      b = b << rshift;
    }


    rmask = theVisual->red_mask;
    gmask = theVisual->green_mask;
    bmask = theVisual->blue_mask;

    rshift = 15 - highbit(rmask);
    gshift = 15 - highbit(gmask);
    bshift = 15 - highbit(bmask);

    /* shift the bits around */
    if (rshift<0) r = r << (-rshift);
             else r = r >> rshift;

    if (gshift<0) g = g << (-gshift);
             else g = g >> gshift;

    if (bshift<0) b = b << (-bshift);
             else b = b >> bshift;


    r = r & rmask;
    g = g & gmask;
    b = b & bmask;


    cdef->pixel = r | g | b;


    /* put 'exact' colors into red,green,blue fields */
    /* shift the bits BACK to where they were, now that they've been masked */
    if (rshift<0) r = r >> (-rshift);
             else r = r << rshift;

    if (gshift<0) g = g >> (-gshift);
             else g = g << gshift;

    if (bshift<0) b = b >> (-bshift);
             else b = b << bshift;

    cdef->red = r;  cdef->green = g;  cdef->blue = b;


    if (DEBUG > 1) {
      fprintf(stderr,
	      "xvAlloc: col=%04lx,%04lx,%04lx -> exact=%04x,%04x,%04x\n",
	      origr, origg, origb, cdef->red, cdef->green, cdef->blue);
      fprintf(stderr,
	      "         mask=%04lx,%04lx,%04lx  pix=%08lx\n",
	      rmask, gmask, bmask, cdef->pixel);
    }
    
    return 1;
  }
  else {
    return (XAllocColor(dp,cm,cdef));
  }
}

void xvFreeColors(Display *dp, Colormap cm, unsigned long *pixels, int npixels, unsigned long planes)
{
  if (theVisual->c_class != TrueColor && theVisual->c_class != DirectColor)
    XFreeColors(dp, cm, pixels, npixels, planes);
}

#define CDIST(x,y,z)  ((x)*(x) + (y)*(y) + (z)*(z))

/************************************/
static void diverseOrder(byte *rmap, byte *gmap, byte *bmap,int maplen, byte *order)
{
  /* takes a colormap (maxlen 256) and produces an order array that 
     contains the most-diverse order for allocating these colors */

  int dist[256], i, pick, maxv, ocnt, d;

  /* arbitrarily pick the brightest color first */
  pick = 0;  maxv = 0;
  for (i=0; i<maplen; i++) {
    if (CDIST((int)rmap[i], (int)gmap[i], (int)bmap[i]) > maxv) {
      maxv = CDIST((int) rmap[i], (int) gmap[i], (int)bmap[i]);
      pick = i;
    }
  }

  ocnt = 0;
  order[ocnt++] = pick;
  
  /* init dist[] array */
  for (i=0; i<maplen; i++) dist[i] = 1000000;

  while (ocnt < maplen) {
    /* update distances */
    for (i=0; i<maplen; i++) {
      d = CDIST(rmap[pick]-rmap[i], gmap[pick]-gmap[i], bmap[pick]-bmap[i]);
      if (dist[i] > d) dist[i] = d;
    }

    /* pick greatest distance */
    for (i=0, maxv=0, pick=0;  i<maplen; i++) {
      if (dist[i] > maxv) { maxv = dist[i];  pick = i; }
    }

    order[ocnt++] = pick;
  }
}


/***************************************************************/
static void freeStdCmaps()
{
  int i;

  if (DEBUG) fprintf(stderr,"freeStdCmaps:  haveStdCmap = %d\n", haveStdCmap);

  if (haveStdCmap == STD_NONE) return;

  for (i=0; i<stdnfcols; i++)
    xvFreeColors(theDisp, theCmap, &stdfreecols[i], 1, 0L);
  stdnfcols = 0;
  haveStdCmap = STD_NONE;
}

/***************************************************************/
int MakeStdCmaps()
{
  /* produces many things:
   *   stdr,stdg,stdb[256] - a 256-entry, desired 3/3/2 colormap
   *   stdcols[256]        - 256-entry, maps 3/3/2 colors into X pixel values
   *   stdrdisp,stdgdisp, stdbdisp[256]
   *                       - for a given 3/3/2 color, gives the rgb values
   *			     actually displayed.  Since we're not going to
   *			     successfully allocate all 256 colors (generally),
   *			     several 3/3/2 colors may be mapped into the
   *			     same X pixel (and hence, same rgb value)
   *   stdfreecols[256]    - list of colors to free on exit
   *   stdnfcols           - # of colors to free
   *
   * possibly modifies browR, browG, browB, and browcols arrays 
   *     (if !browPerfect)
   */       

  /* returns '1' if the colors were reallocated, '0' otherwise */

  /*
   * if we're on a TrueColor/DirectColor visual, it will attempt to alloc
   *    256 colors
   * otherwise, if colorMapMode == CM_STDCMAP, it will attempt to alloc
   *    6*6*6 = 216 colors, or 4*4*4 = 64, or 2*2*2 = 8 colors, depending
   *    on 'ncols' variable
   */

  /* note:
   *   if (ncols==0) (ie, we're either on, or emulating a b/w display),
   *   build std*[], std*disp[], colormaps, but don't actually 
   *   allocate any colors.
   */

  int i,j,r,g,b, desMode, screwed;
  XColor def;
  byte rmap[256],gmap[256],bmap[256],order[256];
  unsigned long descols[256];
  int des2got[256], failed[256];
  int maplen, exactCnt, nearCnt;
  
  
  /* generate stdr,stdg,stdb cmap.  Same in all cases */

//  for (r=0, i=0; r<8; r++)
//    for (g=0; g<8; g++)
//      for (b=0; b<4; b++,i++) {
//	stdr[i] = (r*255)/7;
//	stdg[i] = (g*255)/7;
//	stdb[i] = (b*255)/3;
//      }
  
for (i=0; i <= 255; i++) {
  stdr[i] = i;
  stdg[i] = i;
  stdb[i] = i;
};


  
  /* determine what size cmap we should build */
  if (theVisual->c_class == TrueColor || 
      theVisual->c_class == DirectColor) desMode = STD_332;
  else if (colorMapMode == CM_STDCMAP) desMode = STD_232;
  else desMode = STD_222;


  /* make sure that we're not exceeding 'ncols' (ignore ncols==0) */
  if (ncols > 0) {
    if      (ncols < 64)  desMode = STD_111;
    else if (ncols < 128) desMode = STD_222;
    else if (ncols < 256 && desMode == STD_332) desMode = STD_232;
  }


  if (DEBUG) fprintf(stderr,"MakeStdCmaps: have=%d, des=%d, ncols=%d\n", 
		     haveStdCmap, desMode, ncols);
  
  if (haveStdCmap != STD_NONE && haveStdCmap == desMode) return 0;
  freeStdCmaps();


  /* init some stuff */
  screwed = 0;
  stdnfcols = 0;
  for (i=0; i<256; i++) stdcols[i] = stdfreecols[i] = 0;
  for (i=0; i<256; i++) des2got[i] = i;
  exactCnt = nearCnt = 0;


  // hack for 8bit color-mode
  //desMode = STD_666;

  
  if (desMode == STD_111) {   /* try to alloc 8 colors */
    /* generate a 1/1/1 desired colormap */
    maplen = 8;
    for (r=0, i=0; r<2; r++)
      for (g=0; g<2; g++)
	for (b=0; b<2; b++,i++) {
	  rmap[i] = (r*255);
	  gmap[i] = (g*255);
	  bmap[i] = (b*255);
	}
  }
  
  else if (desMode == STD_222) {   /* try to alloc 64 colors */
    /* generate a 2/2/2 desired colormap */
    maplen = 64;
    for (r=0, i=0; r<4; r++)
      for (g=0; g<4; g++)
	for (b=0; b<4; b++,i++) {
	  rmap[i] = (r*255)/3;
	  gmap[i] = (g*255)/3;
	  bmap[i] = (b*255)/3;
	}

  }
  
  else if (desMode == STD_232) {   /* try to alloc 128 colors */
    /* generate a 2/3/2 desired colormap */
    maplen = 128;
    for (r=0, i=0; r<4; r++)
      for (g=0; g<8; g++)
 	for (b=0; b<4; b++, i++) {
 	  rmap[i] = (r*255)/3;
 	  gmap[i] = (g*255)/7;
  	  bmap[i] = (b*255)/3;
 	}
  }
  
  else if (desMode == STD_666) {   /* try to alloc 216 colors */
    /* generate a 6*6*6 desired colormap */
    maplen = 216;
    for (r=0, i=0; r<6; r++)
      for (g=0; g<6; g++)
	for (b=0; b<6; b++,i++) {
	  rmap[i] = (r*255)/5;
	  gmap[i] = (g*255)/5;
	  bmap[i] = (b*255)/5;
	}
  }
  
  else {   /* desMode == STD_332 */
    maplen = 256;
    for (i=0; i<maplen; i++) {
      rmap[i] = stdr[i];  gmap[i] = stdg[i];  bmap[i] = stdb[i];
    }
  }

  
  // hack for 8bit greyscale mode
  desMode = STD_232;
  cout << endl << "using hacked greyscale mode ..." << endl;
  maplen = 128;
  for (i=0; i < maplen; i++) {
    rmap[i] = (256/maplen)*i;
    gmap[i] = (256/maplen)*i;
    bmap[i] = (256/maplen)*i;
  };


  /* sort the colors according to the diversity algorithm... */
  diverseOrder(rmap,gmap,bmap,maplen,order);


  for (i=0; i<maplen; i++) failed[i] = 0;

  if (ncols!=0) {
    XColor ctab[256];
    long d, mind;
    int  dc,num,j,rd,gd,bd, numgot;

    numgot = 0;

    /* try to allocate the desired (rmap,gmap,bmap) colormap */
    for (i=0; i<maplen; i++) {
      def.red   = rmap[order[i]] << 8;
      def.green = gmap[order[i]] << 8;
      def.blue  = bmap[order[i]] << 8;
      
      def.flags = DoRed | DoGreen | DoBlue;

      if (xvAllocColor(theDisp, theCmap, &def)) {  /* success */
	numgot++;
	des2got[order[i]] = order[i];
	descols[order[i]] = def.pixel;
	for (j=0; j<stdnfcols && stdfreecols[j]!=def.pixel; j++);
	if (j==stdnfcols) exactCnt++;     /* new and different */

	stdfreecols[stdnfcols++] = def.pixel;
      }
      else failed[order[i]] = 1;
    }


    if (numgot != maplen) {
      /* PHASE 2:  find 'close' colors in colormap, try to alloc those */
      
      /* read entire colormap (or first 256 entries) into 'ctab' */
      dc = (ncells<256) ? ncells : 256;
      if (dc>0) {
	for (i=0; i<dc; i++) ctab[i].pixel = (unsigned long) i;
	XQueryColors(theDisp, theCmap, ctab, dc);
	
	for (i=0; i<maplen; i++) {
	  if (failed[i]) {
	    
	    /* find closest color in colormap, and try to alloc it */
	    mind = 1000000;   /* greater than 3 * (256^2) */
	    for (j=0,num = -1; j<dc; j++) {
	      rd = rmap[i] - (ctab[j].red  >>8);
	      gd = gmap[i] - (ctab[j].green>>8);
	      bd = bmap[i] - (ctab[j].blue >>8);
	      
	      d = CDIST(rd, gd, bd);
	      if (d<mind) { mind = d;  num = j; }
	    }
	    
	    if (num < 0) screwed = 1;
	    else if (xvAllocColor(theDisp, theCmap, &ctab[num])) {  /*success*/
	      des2got[i] = i;
	      descols[i] = ctab[num].pixel;
	      failed[i]  = 0;
	      nearCnt++; 
	      /* for (j=0; j<stdnfcols && stdfreecols[j]!=ctab[num].pixel; 
		 j++); */
	      stdfreecols[stdnfcols++] = ctab[num].pixel;
	    }
	  }
	}
      }
    }
      
    /* PHASE 3:  map remaining unallocated colors into closest we got */  
    
    for (i=0; i<maplen; i++) {
      if (failed[i]) {
	
	/* find closest alloc'd color */
	mind = 1000000;   /* greater than 3 * (256^2) */
	for (j=0,num=0; j<maplen; j++) {
	  if (!failed[j]) {
	    d = CDIST(rmap[i]-rmap[j], gmap[i]-gmap[j], bmap[i]-bmap[j]);
	    if (d<mind) { mind = d;  num = j; }
	  }
	}
	
	if (failed[num]) screwed = 1;
	else {
	  descols[i] = descols[num];
	  des2got[i] = num;
	  failed[i] = 0;
	}
      }
    }
  }


  /* at this point, we have 'descols', a maplen long array of 
     X pixel values that maps 1/1/1, 2/2/2, 6*6*6, or 3/3/2 values 
     into an X pixel value */

  /* build stdcols and stdrdisp,stdgdisp,stdbdisp colormap */
  if (desMode == STD_111) {
    for (r=0; r<8; r++)
      for (g=0; g<8; g++)
	for (b=0; b<4; b++) {
	  int i332, i111;
	  i332 = (r<<5) | (g<<2) | b;
	  i111 = (r&0x04) | ((g&0x04)>>1) | (b>>1);

	  stdrdisp[i332] = rmap[des2got[i111]];
	  stdgdisp[i332] = gmap[des2got[i111]];
	  stdbdisp[i332] = bmap[des2got[i111]];

	  stdcols[i332] = descols[des2got[i111]];
	}
  } 

  else if (desMode == STD_222) {
    for (r=0; r<8; r++)
      for (g=0; g<8; g++)
	for (b=0; b<4; b++) {
	  int i332, i222;
	  i332 = (r<<5) | (g<<2) | b;
	  i222 = ((r&0x06)<<3) | ((g&0x06)<<1) | b;

	  stdrdisp[i332] = rmap[des2got[i222]];
	  stdgdisp[i332] = gmap[des2got[i222]];
	  stdbdisp[i332] = bmap[des2got[i222]];

	  stdcols[i332] = descols[des2got[i222]];
	}
  } 

  else if (desMode == STD_232) {
    for (r=0; r<8; r++)
      for (g=0; g<8; g++)
	for (b=0; b<4; b++) {
	  int i332, i232;
	  i332 = (r<<5) | (g<<2) | b;
	  i232 = ((r&0x06)<<4) | ((g&0x07)<<2) | b;

	  stdrdisp[i332] = rmap[des2got[i232]];
	  stdgdisp[i332] = gmap[des2got[i232]];
	  stdbdisp[i332] = bmap[des2got[i232]];
	  stdcols[i332]  = descols[des2got[i232]];
	}
  } 

  else if (desMode == STD_666) {
    for (r=0,i=0; r<8; r++)
      for (g=0; g<8; g++)
	for (b=0; b<4; b++,i++) {
	  int r6,g6,b6,i666;

	  r6 = (((r*10) + 7) / 14);   /* r6 = round(r*5 / 7) */
	  g6 = (((g*10) + 7) / 14);   /* g6 = round(g*5 / 7) */
	  b6 = (((b*10) + 3) / 6);    /* b6 = round(b*5 / 3) */

	  i666 = (36 * r6) + (6 * g6) + b6;

	  stdrdisp[i] = rmap[des2got[i666]];
	  stdgdisp[i] = gmap[des2got[i666]];
	  stdbdisp[i] = bmap[des2got[i666]];

	  stdcols[i]  = descols[des2got[i666]];
	}
  } 

  else {  /* desMode == STD_332 */
    for (i=0; i<256; i++) {
      stdrdisp[i] = rmap[des2got[i]];
      stdgdisp[i] = gmap[des2got[i]];
      stdbdisp[i] = bmap[des2got[i]];

      stdcols[i] = descols[des2got[i]];
    }
  }


// no browsing needed

//  if (!browPerfect) {  /* we've changed the colors the browser icons used */
//    for (i=0; i<256; i++) {
//      browR[i]    = stdr[i];
//      browG[i]    = stdg[i];
//      browB[i]    = stdb[i];
//      browcols[i] = stdcols[i];
//    }
//  }

  haveStdCmap = desMode;

  if (DEBUG > 1) {
    fprintf(stderr,"MakeStdCmaps:  ncols=%d  maplen=%d\n", ncols, maplen);
    fprintf(stderr,"  std*[]= ");
    for (i=0; i<256; i++) 
      fprintf(stderr,"%02x,%02x,%02x  ",stdr[i],stdg[i],stdb[i]);
    fprintf(stderr,"\n\n");

    fprintf(stderr,"  disp[]= ");
    for (i=0; i<256; i++) 
      fprintf(stderr,"%02x,%02x,%02x  ",stdrdisp[i],stdgdisp[i],stdbdisp[i]);
    fprintf(stderr,"\n\n");

    fprintf(stderr,"  stdcols[]= ");
    for (i=0; i<256; i++) 
      fprintf(stderr,"%02lx ",stdcols[i]);
    fprintf(stderr,"\n\n");

    fprintf(stderr,"  stdfreecols[%d] = ", stdnfcols);
    for (i=0; i<stdnfcols; i++) 
      fprintf(stderr,"%02lx ",stdfreecols[i]);
    fprintf(stderr,"\n\n");
  }

  
  // display results on screen
  if (exactCnt == maplen)
    cout << "Got all " << exactCnt << " colors." << endl;
  else {
    if (nearCnt>0) {
      cout << "Got " << exactCnt << " out of " << maplen << " colors. (";
      cout << nearCnt << " close color(s)" << endl;
    }
    else {
      cout << "Got " << exactCnt << " out of " << maplen << " colors." << endl;
    }
  }

  if (screwed) FatalError("something nasty happened in makeStdCmap()\n");

  if (ncols==0) return 0;   /* as no colors were actually alloc'd */
  return 1;
}





/* other stuff composed from misc sources */

  /*****************************************************/
  /*** X Setup                                       ***/
  /*****************************************************/
  

void XSetup () {

  char *display;
  display = NULL;

  /* open the display */
  if ( (theDisp=XOpenDisplay(display)) == NULL) {
    fprintf(stderr, "Can't open display\n");
    exit(1);
  }
  theScreen = DefaultScreen(theDisp);
  theCmap   = DefaultColormap(theDisp, theScreen);
  rootW     = RootWindow(theDisp,theScreen);
  theGC     = DefaultGC(theDisp,theScreen);
  theVisual = DefaultVisual(theDisp,theScreen);
  ncells    = DisplayCells(theDisp, theScreen);
  dispDEEP  = DisplayPlanes(theDisp,theScreen);
  maxWIDE   = vrWIDE  = dispWIDE  = DisplayWidth(theDisp,theScreen);
  maxHIGH   = vrHIGH  = dispHIGH  = DisplayHeight(theDisp,theScreen);

/* make linear colormap for DirectColor visual */
  if (theVisual->c_class == DirectColor) makeDirectCmap();


  ncols = -1;
  /* if ncols wasn't set, set it to 2^dispDEEP, unless dispDEEP=1, in which
     case ncols = 0;  (ncols = max number of colors allocated.  on 1-bit
     displays, no colors are allocated */
  
  if (ncols == -1) {
    if (dispDEEP>1) ncols = 1 << ((dispDEEP>8) ? 8 : dispDEEP);
    else ncols = 0;
  }
  else if (ncols>256) ncols = 256;       /* so program doesn't blow up */

  // Colormap initialisieren  
  colorMapMode = CM_STDCMAP;

  MakeStdCmaps();

cout << endl << endl << "Xsetup done" << endl << endl; cout.flush();

};


